package org.wow.autodkp.model;



/**
 * Represents the abstract concept of a cost.
 * Costs can be direct floating point values or percentages.
 * 
 * @author jsandusky
 */
public class Cost {
    
    private float mCost;
    private boolean mIsPercent;
    
    public Cost() {
    }
    
    public Cost(float cost, boolean isPercent) {
        mCost = cost;
        mIsPercent = isPercent;
    }

    public float getCost() {
        return mCost;
    }

    public void setCost(float cost) {
        mCost = cost;
    }

    public boolean isPercent() {
        return mIsPercent;
    }

    public void setIsPercent(boolean isPercent) {
        mIsPercent = isPercent;
    }
    
    public static Cost parseCost(String str) {
        return parseCost(str, false);
    }
    
    public static Cost parseCost(String str, boolean allowZeroPercent) {
        if (str != null) {
            str = str.trim();
            boolean isPercent = false;
            if (str.endsWith("%")) {
                isPercent = true;
                str = str.substring(0, str.lastIndexOf('%'));
            }
            try {
                float costValue = Float.parseFloat(str);
                if (isPercent) {
                    if        ( allowZeroPercent && (costValue <  0 || costValue > 100)) {
                        // out of bounds
                        return null;
                    } else if (!allowZeroPercent && (costValue <= 0 || costValue > 100)) {
                        // out of bounds
                        return null;
                    }
                } else if (!isPercent && costValue < 0) {
                    return null;
                }
                Cost cost = new Cost(costValue, isPercent);
                return cost;
            } catch (NumberFormatException nfe) {
                // cannot parse, return null
            }
        }
        return null;
    }
    
    public String toString() {
        if (mIsPercent) {
            String costStr = FormatConstants.NUMBER_FORMAT.format((double) mCost);
            return costStr + "%";
        }
        return String.valueOf(mCost);
    }
}
