package org.wow.autodkp.model;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Date;

import org.wow.autodkp.persistence.Persistence;


/**
 * Handles program logging.
 * 
 * @author jsandusky
 */
public class Logger {
    
    private static Logger mInstance = new Logger();
    private PrintStream mPrintStream;

    private Logger() {
    }

    public static void initialize(boolean toLogFile) {
        if (toLogFile) {
            File logDir = new File(Persistence.LOG_DIR);
            if (!logDir.exists()) {
                logDir.mkdirs();
            }
            
            File logFile = new File(
                logDir.getPath() + 
                File.separator + 
                Persistence.LOG_FILE);
            try {
                mInstance.mPrintStream = new PrintStream(
                        new BufferedOutputStream(new FileOutputStream(logFile, true)));
            } catch(IOException e) {
                e.printStackTrace();
            }
        } else {
            mInstance.mPrintStream = System.err;
        }
    }

    public static void log(Throwable e) {
        mInstance.doLog(e);
    }

    private void doLog(Throwable e) {
        if (mPrintStream != null) {
            mPrintStream.println(new Date());
            e.printStackTrace(mPrintStream);
            mPrintStream.flush();
        }
    }
}
