package org.wow.autodkp.model;

import java.text.ParseException;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.wow.autodkp.model.graph.LuaGraphItem;
import org.wow.autodkp.model.graph.RaidGraphItem;


/**
 * 
 * 
 * @author jsandusky
 */
public class PlayerDataGenerator {

    private RaidGraphItem mCurrentRaidGraphItem;
    
    
    public PlayerData[] generatePlayerData(RaidGraphItem raidGraphItem) {
        mCurrentRaidGraphItem = raidGraphItem;
        
        Map playerNameToDataMap = new HashMap();
        getPlayersAndJoinTime(playerNameToDataMap);
        getPlayerClasses(playerNameToDataMap);
        getLeaveTime(playerNameToDataMap);
        getLoot(playerNameToDataMap);
        
        PlayerData[] data = 
            (PlayerData[]) playerNameToDataMap.values().toArray(new PlayerData[0]);
        Arrays.sort(data);
        
        for (int i=0; i < data.length; i++) {
            PlayerData iterData = data[i];
            // Sort loot by time looted
            iterData.sortLootByTimeLooted();
            // Fix end times, as some players may never have left the raid before
            // the raid was terminated.
            if (!iterData.hasTimeLeave()) {
                iterData.addTimeLeave(mCurrentRaidGraphItem.getEndDate());
            }
        }
        return data;
    }
    
    private void getPlayersAndJoinTime(Map playerNameToDataMap) {
        LuaGraphItem joinGroup = mCurrentRaidGraphItem.getItem(LuaGraphItem.JOIN_GROUP);
        if (joinGroup != null) {
            LuaGraphItem[] children = joinGroup.getChildren();
            if (children != null) {
                for (int i=0; i < children.length; i++) {
                    LuaGraphItem joinItem   = children[i];
                    LuaGraphItem playerItem = joinItem.getItem(LuaGraphItem.JOIN_GROUP_ITEM_PLAYER);
                    LuaGraphItem timeItem   = joinItem.getItem(LuaGraphItem.JOIN_GROUP_ITEM_TIME);
                    if (playerItem != null && timeItem != null) {
                        String timeJoinString = (String) timeItem.getValue();
                        try {
                            Date date = FormatConstants.DATE_PARSE_FORMAT.parse(timeJoinString);
                            String playerName = (String) playerItem.getValue();
                            PlayerData playerData = (PlayerData) playerNameToDataMap.get(playerName);
                            if (
                                    !playerName.equals(PlayerData.UNKNOWN_ENTITY) &&
                                    !playerName.equals(PlayerData.UNKNOWN)) {
                                if (playerData == null) {
                                    playerData = new PlayerData();
                                    playerData.setPlayerName(playerName);
                                    playerNameToDataMap.put(playerName, playerData);
                                    playerData.addTimeJoin(date);
                                } else {
                                    playerData.addTimeJoin(date);
                                }
                            }
                        } catch (ParseException e) {
                            Logger.log(e);
                        }
                    }
                }
            }
        }
    }
    
    private void getPlayerClasses(Map playerNameToDataMap) {
        LuaGraphItem playerInfosGroup = mCurrentRaidGraphItem.getItem(LuaGraphItem.PLAYER_INFOS_GROUP);
        if (playerInfosGroup != null) {
            LuaGraphItem[] children = playerInfosGroup.getChildren();
            if (children != null) {
                for (int i=0; i < children.length; i++) {
                    LuaGraphItem infoItem   = children[i];
                    LuaGraphItem classItem = infoItem.getItem(LuaGraphItem.PLAYER_INFOS_CLASS);
                    if (classItem != null) {
                        String playerName = (String) infoItem.getName();
                        PlayerData playerData = (PlayerData) playerNameToDataMap.get(playerName);
                        if (playerData != null) {
                            String playerClass = (String) classItem.getValue();
                            if (playerClass != null) {
                                playerData.setPlayerClass(playerClass);
                            }
                        }
                    }
                }
            }
        }
    }
    
    private void getLeaveTime(Map playerNameToDataMap) {
        LuaGraphItem leaveGroup = mCurrentRaidGraphItem.getItem(LuaGraphItem.LEAVE_GROUP);
        if (leaveGroup != null) {
            LuaGraphItem[] children = leaveGroup.getChildren();
            if (children != null) {
                for (int i=0; i < children.length; i++) {
                    LuaGraphItem leaveItem   = children[i];
                    LuaGraphItem playerItem = leaveItem.getItem(LuaGraphItem.LEAVE_GROUP_ITEM_PLAYER);
                    LuaGraphItem timeItem   = leaveItem.getItem(LuaGraphItem.LEAVE_GROUP_ITEM_TIME);
                    if (playerItem != null && timeItem != null) {
                        String playerName = (String) playerItem.getValue();
                        PlayerData playerData = (PlayerData) playerNameToDataMap.get(playerName);
                        if (playerData != null) {
                            String timeLeaveString = (String) timeItem.getValue();
                            try {
                                Date date = FormatConstants.DATE_PARSE_FORMAT.parse(timeLeaveString);
                                playerData.addTimeLeave(date);
                            } catch (ParseException e) {
                                Logger.log(e);
                            }
                        }
                    }
                }
            }
        }
    }

    private void getLoot(Map playerNameToDataMap) {
        LuaGraphItem lootGroup = mCurrentRaidGraphItem.getItem(LuaGraphItem.LOOT_GROUP);
        if (lootGroup != null) {
            LuaGraphItem[] children = lootGroup.getChildren();
            if (children != null) {
                for (int i=0; i < children.length; i++) {
                    LuaGraphItem lootItem       = (LuaGraphItem) children[i];
                    LuaGraphItem playerItem     = lootItem.getItem(LuaGraphItem.LOOT_ITEM_PLAYER);
                    LuaGraphItem timeItem       = lootItem.getItem(LuaGraphItem.LOOT_ITEM_TIME);
                    LuaGraphItem lootDetailItem = lootItem.getItem(LuaGraphItem.LOOT_ITEM_DETAIL);
                    LuaGraphItem zoneItem       = lootItem.getItem(LuaGraphItem.LOOT_ITEM_ZONE);
                    LuaGraphItem bossItem       = lootItem.getItem(LuaGraphItem.LOOT_ITEM_BOSS);
                    LuaGraphItem costsItem      = lootItem.getItem(LuaGraphItem.LOOT_ITEM_COSTS);
                    if (
                            lootItem != null &&
                            playerItem != null &&
                            lootDetailItem != null) {
                        String playerName = (String) playerItem.getValue();
                        PlayerData playerData = (PlayerData) playerNameToDataMap.get(playerName);
                        if (playerData != null) {
                            String timeLootString = (String) timeItem.getValue();
                            try {
                                Date date = FormatConstants.DATE_PARSE_FORMAT.parse(timeLootString);
                                LuaGraphItem lootIDItem    = lootDetailItem.getItem(LuaGraphItem.LOOT_ITEM_DETAIL_ID);
                                LuaGraphItem lootNameItem  = lootDetailItem.getItem(LuaGraphItem.LOOT_ITEM_DETAIL_NAME);
                                LuaGraphItem lootColorItem = lootDetailItem.getItem(LuaGraphItem.LOOT_ITEM_DETAIL_COLOR);
                                
                                String instanceName = LootConstants.INSTANCE_NAME_NONE;
                                if (zoneItem != null) {
                                    instanceName = (String) zoneItem.getValue();
                                }
                                
                                String bossName = LootConstants.BOSS_NAME_NONE;
                                if (bossItem != null) {
                                    bossName = (String) bossItem.getValue();
                                }
    
                                Cost auctionValue = null;
                                if (costsItem != null) {
                                    auctionValue = Cost.parseCost((String) costsItem.getValue());
                                }
                                
                                if (lootNameItem != null && lootColorItem != null) {
                                    String itemIDString = (String) lootIDItem.getValue();
                                    int colonIndex = itemIDString.indexOf(":");
                                    if (colonIndex >= 0) {
                                        itemIDString = itemIDString.substring(0, colonIndex);
                                    }
                                    
                                    Loot loot = AutoDKPModel.getInstance().createLoot(
                                                itemIDString,
                                                (String) lootNameItem.getValue(),
                                                (String) lootColorItem.getValue(),
                                                bossName,
                                                instanceName);
                                    playerData.addLoot(loot, date, auctionValue);
                                }
                            } catch (ParseException e) {
                                Logger.log(e);
                            }
                        }
                    }
                }
            }
        }
    }
}
