package org.wow.autodkp.model.graph;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.LineNumberReader;

import org.wow.autodkp.model.Logger;

/**
 * Parses an input Lua variables file
 * and generates an in-memory model from the data.
 * The model is a tree of LuaGraphItem objects.
 * 
 * @author jsandusky
 */
public class LuaGraphGenerator {
    
    private LuaGraphItem mParent;

    
    public LuaGraphGenerator() {
        mParent = new LuaGraphItem(null, "Data", null, true);
    }

    
    public LuaGraphItem generateGraph(File inputFile) {
        if (inputFile.exists()) {
            LineNumberReader reader = null;
            try {
                // To support foreign characters, we parse as UTF8.
                reader = new LineNumberReader(new BufferedReader(
                        new InputStreamReader(new FileInputStream(inputFile), "UTF8")));
                String line = null;
                while ( (line = reader.readLine()) != null) {
                    if (line.length() > 0) {
                        parseLine(line);
                    }
                }
            } catch (IOException ioe) {
                Logger.log(ioe);
            } finally {
                try {
                    reader.close();
                } catch (IOException ioe) {
                    Logger.log(ioe);
                }
            }
        }
        
        return mParent;
    }
    
    public static final String EQUAL         = "=";
    public static final String BRACKET_LEFT  = "{";
    public static final String COMMA         = ",";
    public static final String QUOTE_DOUBLE  = "\"";
    public static final String BRACKET_RIGHT = "}";
    
    private void parseLine(String line) {
        LuaGraphItem newItem = null;
        line = line.trim();
        int equalsIndex = line.indexOf(EQUAL);
        if (equalsIndex >= 0) {
            String name = line.substring(0, equalsIndex).trim();
            String value = line.substring(equalsIndex + 1).trim();
            if (value.equals(BRACKET_LEFT)) {
                // array child
                if (mParent != null && mParent.getType() == LuaGraphItem.RAID_ROOT) {
                    newItem = new RaidGraphItem(mParent, name, null, false);
                } else {
                    newItem = new LuaGraphItem(mParent, name, null, false);
                    newItem.resolveType();
                }
            } else {
                if (value.endsWith(COMMA)) {
                    value = value.substring(0, value.length() - 1);
                }
                if (value.startsWith(QUOTE_DOUBLE) && value.endsWith(QUOTE_DOUBLE)) {
                    value = value.substring(1, value.length() - 1);
                }
                // leaf child
                newItem = new LuaGraphItem(mParent, name, value, true);
                newItem.resolveType();
            }
        }
        
        if (line.startsWith(BRACKET_RIGHT)) {
            if (mParent != null && (LuaGraphItem) mParent.getParent() != null) {
                mParent = (LuaGraphItem) mParent.getParent();
            }
        } else if (newItem != null) {
            if (mParent != null) {
                if (newItem.isData()) {
                    mParent.addItem(newItem);
                }
            }
            if (!newItem.isLeaf()) {
                mParent = newItem;
            }
        }
    }
}
