package org.wow.autodkp.model.points;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.Loot;


/**
 * Representation of a formula cost.
 * The underlying item could be a loot drop or a custom formula item.
 * 
 * @author jsandusky
 */
public class CostItem {
    
    private Object mCostItem;
    private boolean mIsLoot;
    private boolean mIsFiltered;
    private Cost mCost;

    
    public CostItem(Loot loot) {
        mCostItem = loot;
        mIsLoot = true;
    }

    public CostItem(CustomFormulaItem item) {
        mCostItem = item;
        mIsLoot = false;
        mCost = item.getCost();
    }

    
    public void setFiltered(boolean isFiltered) {
        mIsFiltered = isFiltered;
    }

    public boolean isFiltered() {
        return mIsFiltered;
    }

    public boolean isLoot() {
        return mIsLoot;
    }

    public Loot getLoot() {
        return (Loot) mCostItem;
    }

    public CustomFormulaItem getFormulaItem() {
        return (CustomFormulaItem) mCostItem;
    }

    public String getLootID() {
        return getLoot().getID();
    }

    public String getFormulaItemText() {
        return getFormulaItem().getItemText();
    }

    public Cost getCost() {
        return mCost;
    }

    public void setCost(Cost cost) {
        mCost = cost;
    }
}
