package org.wow.autodkp.model.points;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.points.dkp.IFormula;


/**
 * Represents point data for a player.
 * Point data is point-system-specific information.
 * 
 * @author jsandusky
 */
public class DefaultPointSystemPlayerData implements IPointSystemPlayerData {

    private String mPlayerName;
    private String mPlayerClass;
    private float mIncomingTotal;
    private IFormula mFormula;
    private float mFormulaTotal;
    private float mOutgoingTotal;
    private List mExcludedLootIDs;
    private Map mAdjustmentIDToValueMap = new HashMap();
    private List mCustomFormulaItems;
    private List mDistributionExceptions;
    private List mBossKillExceptions;
    private List mTimeExceptions;
    
    
    public DefaultPointSystemPlayerData() {
    }
    
    
    public String getPlayerClass() {
        return mPlayerClass;
    }
    
    public void setPlayerClass(String playerClass) {
        if (playerClass != null) {
            mPlayerClass = playerClass.toLowerCase();
        } else {
            mPlayerClass = "";
        }
    }

    public String getPlayerName() {
        return mPlayerName;
    }
    
    public void setPlayerName(String playerName) {
        mPlayerName = playerName;
    }

    public float getIncomingTotal() {
        return mIncomingTotal;
    }

    public Map getAdjustmentIDToValueMap() {
        return mAdjustmentIDToValueMap;
    }
    
    public float getAdjustment(String adjustmentID) {
        Float adjustment = (Float) mAdjustmentIDToValueMap.get(adjustmentID);
        if (adjustment != null) {
            return adjustment.floatValue();
        }
        return 0;
    }
    
    public void setAdjustment(String adjustmentID, float num) {
        mAdjustmentIDToValueMap.put(adjustmentID, new Float(num));
    }
    
    public float getFormulaTotal() {
        return mFormulaTotal;
    }
    
    public void setFormulaTotal(float num) {
        mFormulaTotal = num;
    }
    
    public void setIncomingTotal(float num) {
        mIncomingTotal = num;
    }
    
    public float getOutgoingTotal() {
        return mOutgoingTotal;
    }
    
    public void setOutgoingTotal(float num) {
        mOutgoingTotal = num;
    }

    public IFormula getFormula() {
        return mFormula;
    }

    public void setFormula(IFormula formula) {
        mFormula = formula;
    }

    public List getExcludedLootIDs() {
        return mExcludedLootIDs;
    }
    
    public void setExcludedLootIDs(List excluded) {
        mExcludedLootIDs = excluded;
    }
    
    public void addExcludedLootID(String lootID) {
        if (mExcludedLootIDs == null) {
            mExcludedLootIDs = new ArrayList();
        }
        mExcludedLootIDs.add(lootID);
    }
    
    public void removeExcludedLootID(String lootID) {
        if (mExcludedLootIDs != null) {
            mExcludedLootIDs.remove(lootID);
        }
    }
    
    public void merge(IPointSystemPlayerData playerData) {
        // the data that has the highest points should override
        if (playerData.getIncomingTotal() > mIncomingTotal) {
            mIncomingTotal = playerData.getIncomingTotal();
            mFormula = playerData.getFormula();
            mFormulaTotal = playerData.getFormulaTotal();
            mOutgoingTotal = playerData.getOutgoingTotal();
            mExcludedLootIDs = playerData.getExcludedLootIDs();
            mAdjustmentIDToValueMap = playerData.getAdjustmentIDToValueMap();
        }
    }

    public void addCustomFormulaItem(CustomFormulaItem item) {
        if (mCustomFormulaItems == null) {
            mCustomFormulaItems = new ArrayList();
        }
        mCustomFormulaItems.add(item);
    }

    public List getCustomFormulaItems() {
        return mCustomFormulaItems;
    }

    public void setCustomFormulaItems(List items) {
        mCustomFormulaItems = items;
    }
    
    public void removeCustomFormulaItem(CustomFormulaItem item) {
        if (mCustomFormulaItems != null) {
            mCustomFormulaItems.remove(item);
        }
    }
    
    public void setDistributionExceptions(List data) {
        mDistributionExceptions = data;
    }
    
    public void setBossKillExceptions(List data) {
        mBossKillExceptions = data;
    }
    
    public void setTimeExceptions(List data) {
        mTimeExceptions = data;
    }
    
    public List getDistributionExceptions() {
        return mDistributionExceptions;
    }
    
    public List getBossKillExceptions() {
        return mBossKillExceptions;
    }
    
    public List getTimeExceptions() {
        return mTimeExceptions;
    }
}
