package org.wow.autodkp.model.points;

import java.util.LinkedHashMap;
import java.util.List;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.dkp.IDKPCalculator;
import org.wow.autodkp.persistence.IItemDB;


/**
 * An AutoDKP Point System.
 * Point systems are used to store rules on how to
 * calculate DKP. A guild will define its own point
 * system, and will then use AutoDKP to apply that
 * point system's rules to the selected raid. The end
 * result is a points file.
 * 
 * @author jsandusky
 */
public interface IPointSystem {
    
    String getID();
    
    String getName();
    void setName(String name);
    
    boolean isTimeBased();
    boolean isBossKill();
    boolean isZeroSum();
    boolean isZeroSumExcludeLooter();
    
    public void setBossKill(boolean isBossKill);
    public void setZeroSum(boolean isZeroSum, boolean isExcludeLooter);
    public void setTimeBased(boolean isTimeBased);
    
    IItemDB getItemDB();

    IPointSystemRaidData getCurrentRaidData();
    void loadFromRaidData(Raid raid);
    void importRaid(Raid raidToImport, IPointSystemRaidData importedRaidData, Raid currentRaid);
    
    boolean isAuctionedLootValues();
    void setAuctionedLootValues(boolean isAuctioned);
    boolean isDirectLootValueEditing();
    void setDirectLootValueEditing(boolean isDirect);
    
    List getLootConstraintKeys();
    List getLootConstraintCosts();
    List getLootConstraintNames();
    String getLootConstraintKey(int index);
    int getLootConstraintIndex(String key);
    void addLootCategory(String id, String name, Cost cost);
    void setLootCategory(String id, String name, Cost cost);
    void removeLootCategory(String id);
    
    IDKPCalculator getCalculator();
    
    List getBossConstraintCosts();
    List getBossConstraintNames();
    float getStandardPointsForBoss();
    void setStandardPointsForBoss(float points);
    float getCustomPointsForBoss(String bossName);
    
    void addBossConstraint(String name, Cost cost);
    void removeBossConstraint(String name);
    void setBossConstraintCost(String name, Cost cost);
    
    float getDurationPoints();
    void setDurationPoints(float points);
    int getDurationMinutes();
    void setDurationMinutes(int minutes);
    TimeThreshold getTimeThreshold();
    void setTimeThreshold(TimeThreshold threshold);
    RollCall getRollCall(String name);
    void addRollCall(RollCall rollCall);
    void addRollCall(int index, RollCall rollCall);
    void removeRollCall(String name);
    List getRollCalls();
    
    LinkedHashMap getAdjustmentMap();
    
    void deleteRaid(Raid raid);
    void saveRaid(Raid raid);
    
    boolean isPointsStarted(Raid raid);
    
    PlayerData[] getNonRemovedPlayerData();
    
    PlayerData[] getNonRemovedRaidMemberPlayerData();
}
