package org.wow.autodkp.model.points;

import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.points.dkp.IFormula;


/**
 * Point system player-specific information.
 * Points for a raid consist of raid-specific and
 * player-specific information.
 * 
 * This interface represents info for one particular player.
 * 
 * @author jsandusky
 */
public interface IPointSystemPlayerData {
    
    String getPlayerName();
    String getPlayerClass();
    void setPlayerName(String playerName);
    void setPlayerClass(String className);
    
    float getIncomingTotal();
    Map getAdjustmentIDToValueMap();
    float getAdjustment(String adjustmentID);
    float getOutgoingTotal();
    float getFormulaTotal();
    
    List getDistributionExceptions();
    List getBossKillExceptions();
    List getTimeExceptions();
    void setDistributionExceptions(List data);
    void setBossKillExceptions(List data);
    void setTimeExceptions(List data);
    
    List getExcludedLootIDs();
    void setExcludedLootIDs(List excluded);
    void addExcludedLootID(String lootID);
    void removeExcludedLootID(String lootID);
    
    void setFormula(IFormula formula);
    IFormula getFormula();
    
    List getCustomFormulaItems();
    void setCustomFormulaItems(List items);
    void addCustomFormulaItem(CustomFormulaItem item);
    void removeCustomFormulaItem(CustomFormulaItem item);
    
    void setFormulaTotal(float num);
    void setOutgoingTotal(float num);
    void setIncomingTotal(float num);
    void setAdjustment(String adjustmentID, float num);
    
    void merge(IPointSystemPlayerData playerData);
}
