package org.wow.autodkp.model.points;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.wow.autodkp.persistence.Persistence;
import org.wow.autodkp.persistence.xml.XMLConstraintsReader;
import org.wow.autodkp.persistence.xml.XMLConstraintsWriter;


/**
 * Loads point systems from their persisted state.
 * 
 * @author jsandusky
 */
public class PointSystemFactory {
    
    public static IPointSystem[] loadConstraints() {
        List constraints = new ArrayList();
        XMLConstraintsReader reader = new XMLConstraintsReader();
        
        File pointSystemDir = new File(Persistence.DEFINITIONS_DIR + File.separator + Persistence.CONSTRAINTS_DIR);
        if (pointSystemDir.exists()) {
            File[] files = pointSystemDir.listFiles();
            for (int i=0; i < files.length; i++) {
                File file = files[i];
                if (file.isFile()) {
                    if (file.getName().endsWith(Persistence.XML_EXTENSION)) {
                        // Must be a constraints save file, load it.
                        IPointSystem constraint = reader.readXML(file);
                        if (constraint != null) {
                            constraints.add(constraint);
                        }
                    }
                }
            }
        } else {
            pointSystemDir.mkdirs();
        }
        
        return (IPointSystem[]) constraints.toArray(new IPointSystem[0]);
    }
    
    public static String getConstraintSaveDirectory() {
        return 
            Persistence.DEFINITIONS_DIR + 
            File.separator + 
            Persistence.CONSTRAINTS_DIR;
    }
    
    public static boolean constraintExists(String id) {
        File saveFile = new File(
                getConstraintSaveDirectory() + 
                File.separator + 
                id + Persistence.XML_EXTENSION);
        return saveFile.exists();
    }
    
    public static void deleteConstraint(IPointSystem pointSystem) {
        File saveFile = new File(
                getConstraintSaveDirectory() + 
                File.separator + 
                pointSystem.getID() + Persistence.XML_EXTENSION);
        if (saveFile.exists()) {
            saveFile.delete();
        }
    }
    
    public static void saveConstraint(IPointSystem pointSystem) {
        String saveDirectory = getConstraintSaveDirectory();
        File saveDir = new File(saveDirectory);
        if (!saveDir.exists()) {
            saveDir.mkdirs();
        }
        File saveFile = new File(
                saveDirectory + 
                File.separator + 
                pointSystem.getID() + Persistence.XML_EXTENSION);
        new XMLConstraintsWriter().writeXML(saveFile, pointSystem);
    }
}
