package org.wow.autodkp.model.points.dkp;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.wow.autodkp.model.FormatConstants;
import org.wow.autodkp.model.points.CostItem;


/**
 * A formula represents all standard deductions a player might
 * have when doing points. This includes the DKP calculations and
 * loot drop expenses.
 * 
 * @author jsandusky
 */
public abstract class AbstractDKPFormula implements IFormula {

    protected List mPointAdjustments;
    protected List mPointIDs;
    protected List mPointTypes;
    protected List mPointNames;
    private List mCostItems;
    protected int mLineCount;
    protected float mRunningTotal;
    private float mRunningDistributions;
    private Map mBossKillDetailMap = new HashMap();
    private Map mDistributionDetailMap = new HashMap();
    private Map mTimeDetailMap = new HashMap();
    private Object mPrintObject;
    
    
    public AbstractDKPFormula(String formulaString) {
        mPointAdjustments = new ArrayList(4);
        mPointIDs = new ArrayList(4);
        mPointTypes = new ArrayList(4);
        mPointNames = new ArrayList(4);
        mCostItems = new ArrayList();
        
        StringTokenizer lineItemParser = new StringTokenizer(formulaString, ";");
        while (lineItemParser.hasMoreTokens()) {
            String lineItem = lineItemParser.nextToken();
            StringTokenizer lineItemElementParser = new StringTokenizer(lineItem, "=");
            if (lineItemElementParser.countTokens() >= 2) {
                String pointStr = lineItemElementParser.nextToken();
                String idStr = lineItemElementParser.nextToken();
                mPointAdjustments.add(pointStr);
                mPointIDs.add(idStr);
                String typeStr = "";
                String nameStr = "";
                if (lineItemElementParser.countTokens() >= 2) {
                    typeStr = lineItemElementParser.nextToken();
                    nameStr = lineItemElementParser.nextToken();
                }
                mPointTypes.add(typeStr);
                mPointNames.add(nameStr);
            }
        }
    }

    public AbstractDKPFormula(int initialTotal)
    {
        mPointAdjustments = new ArrayList(4);
        mPointIDs = new ArrayList(4);
        mPointTypes = new ArrayList(4);
        mPointNames = new ArrayList(4);
        mCostItems = new ArrayList();
        mRunningTotal = initialTotal;
        mRunningDistributions = 0;
    }

    public int getLineCount() {
        return mLineCount;
    }

    public void setLineCount(int lineCount) {
        mLineCount = lineCount;
    }
    
    public void reset(float initialTotal) {
        mRunningTotal = initialTotal;
        mPrintObject = null;
        mCostItems.clear();
        mPointAdjustments.clear();
        mPointIDs.clear();
        mPointTypes.clear();
        mPointNames.clear();
        mLineCount = 0;
        mRunningDistributions = 0;
    }

    public void setCostItems(List costItems) {
        mCostItems = costItems;
    }

    public void addLineItem(float pointAdjustment, 
                            String pointID, 
                            String pointType, 
                            String pointName)
    {
        mRunningTotal += pointAdjustment;
        String pointStr = FormatConstants.NUMBER_FORMAT.format((double) pointAdjustment);
        if (pointAdjustment <= 0) {
            mPointAdjustments.add(pointStr);
        }
        else {
            mPointAdjustments.add("+" + pointStr);
        }
        mPointIDs.add(pointID);
        mPointTypes.add(pointType);
        mPointNames.add(pointName);
    }

    public void updateDistribution(float pointAdjustment) {
        if (pointAdjustment != 0) {
            pointAdjustment = Math.abs(pointAdjustment);
            mRunningDistributions += pointAdjustment;
        }
    }
    
    public float getDistributions() {
        return mRunningDistributions;
    }
    
    public float getRunningTotal() {
        return mRunningTotal;
    }

    public CostItem[] getCostItems() {
        return (CostItem[]) mCostItems.toArray(new CostItem[mCostItems.size()]);
    }

    public String generateFormulaPersistenceString() {
        StringBuffer buf = new StringBuffer();
        int count = mPointAdjustments.size();
        for (int i = 0; i < count; i++) {
            String points = (String) mPointAdjustments.get(i);
            String id     = (String) mPointIDs.get(i);
            String type   = (String) mPointTypes.get(i);
            String name   = (String) mPointNames.get(i);
            buf.append(points + "=" + id + "=" + type + "=" + name);
            if (i + 1 < count) {
                buf.append(";");
            }
        }

        return buf.toString();
    }

    public Object getPrintObject() {
        return mPrintObject;
    }
    
    public void setPrintObject(Object printObject) {
        mPrintObject = printObject;
    }

    public void setBossKillDetailMap(Map map) {
        mBossKillDetailMap = map;
    }
    
    public Map getBossKillDetailMap() {
        return mBossKillDetailMap;
    }
    
    public Map getDistributionDetailMap() {
        return mDistributionDetailMap;
    }
    
    public void setDistributionDetailMap(Map map) {
        mDistributionDetailMap = map;
    }
    
    public Map getTimeDetailMap() {
        return mTimeDetailMap;
    }
    
    public void setTimeDetailMap(Map map) {
        mTimeDetailMap = map;
    }
    
    public List getPointAdjustments() {
        return mPointAdjustments;
    }
    
    public List getPointIDs() {
        return mPointIDs;
    }
    
    public List getPointNames() {
        return mPointNames;
    }
    
    public List getPointTypes() {
        return mPointTypes;
    }
}