package org.wow.autodkp.model.points.dkp;

import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.points.CostItem;


/**
 * A formula represents all standard deductions a player might
 * have when doing points. This includes the DKP calculations and
 * loot drop expenses.
 * 
 * @author jsandusky
 */
public interface IFormula {
    
    String ID_BOSS_POINTS = "boss points";
    String ID_ZERO_SUM = "distribution";
    String ID_TIME_POINTS = "time points";
    String ID_CUSTOM = "custom";
    String TYPE_NONE = "";
    
    String generateFormulaPersistenceString();
    
    // The display object of the formula.
    // The user of this interface can set this arbitrarily.
    Object getPrintObject();
    void setPrintObject(Object printObject);
    
    int getLineCount();
    void setLineCount(int lineCount);
    
    void addLineItem(float cost, String pointID, String pointType, String pointName);

    float getDistributions();
    void updateDistribution(float cost);
    
    float getRunningTotal();

    CostItem[] getCostItems();
    void setCostItems(List list);

    void reset(float initialTotal);
    
    Map getBossKillDetailMap();
    void setBossKillDetailMap(Map map);
    
    Map getDistributionDetailMap();
    void setDistributionDetailMap(Map map);
    
    Map getTimeDetailMap();
    void setTimeDetailMap(Map map);
    
    List getPointAdjustments();
    List getPointIDs();
    List getPointNames();
    List getPointTypes();

}
