package org.wow.autodkp.model.points.dkp;

import java.util.Date;

import org.wow.autodkp.model.points.RollCall;


/**
 * Whether points for a boss kill are to be applied or not.
 * 
 * @author jsandusky
 */
public class TimePointDetail implements Comparable {
    
    private String mID;
    private boolean mIsIncluding;
    private float mCurrentPoints;
    private boolean mIsRollCall;
    private String mRollCallName;
    private Date mRollCallDate;
    private int mRecurrence;
    
    
    public TimePointDetail(
            String id,
            boolean isIncluding, 
            float currentPoints,
            boolean isRollCall,
            String rollCallName,
            Date rollCallDate,
            int recurrence)
    {
        mID = id;
        mIsIncluding = isIncluding;
        mCurrentPoints = currentPoints;
        mIsRollCall = isRollCall;
        mRollCallName = rollCallName;
        mRollCallDate = rollCallDate;
        mRecurrence = recurrence;
    }
    
    public TimePointDetail(String id, boolean isIncluding, float currentPoints)
    {
        this(id, isIncluding, currentPoints, false, null, null, -1);
    }
    
    
    public String getID() {
        return mID;
    }
    
    public boolean isIncluding() {
        return mIsIncluding;
    }
    
    public float getCurrentPoints() {
        return mCurrentPoints;
    }

    public boolean isRollCall() {
        return mIsRollCall;
    }
    
    public void setRollCall(boolean isRollCall) {
        mIsRollCall = isRollCall;
    }
    
    public Date getRollCallDate() {
        return mRollCallDate;
    }
    
    public void setRollCallDate(Date rollCallDate) {
        mRollCallDate = rollCallDate;
    }
    
    public String getRollCallName() {
        return mRollCallName;
    }
    
    public void setRollCallName(String rollCallName) {
        mRollCallName = rollCallName;
    }

    public int getRecurrence() {
        return mRecurrence;
    }
    
    public void setRecurrence(int recurrence) {
        mRecurrence = recurrence;
    }
    
    public boolean hasRecurrence() {
        return mRecurrence > -1;
    }
    
    public int compareTo(Object ob) {
        if (ob instanceof TimePointDetail) {
            TimePointDetail other = (TimePointDetail) ob;
            if (!isRollCall()) {
                // The duration comes first.
                return -1;
            }
            if (!other.isRollCall()) {
                // The duration comes first.
                return 1;
            }
            // Must both be roll-calls.
            int result = getRollCallDate().compareTo(other.getRollCallDate());
            if (result == 0) {
                if        (mRollCallName.equals(RollCall.RAID_END_NAME)) {
                    return 1;
                } else if (other.getRollCallName().equals(RollCall.RAID_END_NAME)) {
                    return -1;
                } else if (mRollCallName.equals(RollCall.RAID_START_NAME)) {
                    return -1;
                } else if (other.getRollCallName().equals(RollCall.RAID_START_NAME)) {
                    return 1;
                }
            }
            return result;
        }
        return 1;
    }
}
