package org.wow.autodkp.persistence;

import java.io.InputStream;
import java.util.Collections;
import java.util.Set;
import java.util.StringTokenizer;

import org.wow.autodkp.model.Loot;


/**
 * An items "database".
 * 
 * @author jsandusky
 */
public abstract class AbstractItemDB extends AbstractPropertiesDB implements IItemDB {
    
    public AbstractItemDB(String propertiesDirectoryPath, String propertiesFilePath) {
        // Used for reading properties file from the file system.
        // This constructor creates a saveable item db.
        super(propertiesDirectoryPath, propertiesFilePath);
    }
    
    public AbstractItemDB(InputStream propertyStream) {
        // Used for reading a temporary properties file from a zip input stream.
        // This item db cannot be saved: read-only.
        super(propertyStream);
    }
    
    public boolean hasItem(String lootID) {
        return mProperties.getProperty(lootID) != null;
    }
    
    protected abstract String getValueString(Loot loot);
    
    public void addLoot(Loot loot) {
        String lootID = loot.getID();
        mProperties.setProperty(lootID, getValueString(loot));
    }
    
    public Set getLootIDs() {
        return Collections.unmodifiableSet(mProperties.keySet());
    }
    
    protected String getElementAtPosition(String value, int position) {
        StringTokenizer tokener = new StringTokenizer(value, ":");
        if (tokener.countTokens() > position) {
            int i=0;
            while (i < position) {
                tokener.nextToken();
                i++;
            }
            return tokener.nextToken();
        }
        return null;
    }
    
    // Avoid using this method unless you really know what you're doing.
    public String getRawLootValue(String lootID) {
        return mProperties.getProperty(lootID);
    }
    
    // Avoid using this method unless you really know what you're doing.
    public void setRawLootValue(String lootID, String rawValue) {
        mProperties.setProperty(lootID, rawValue);
    }
}
