package org.wow.autodkp.persistence;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;

import org.wow.autodkp.model.Logger;


/**
 * A "database" that is based off of a Java properties file.
 * 
 * @author jsandusky
 */
public abstract class AbstractPropertiesDB {
    
    protected Properties mProperties;
    private String mPropertiesFilePath;
    private String mPropertiesDirectoryPath;
    
    
    public AbstractPropertiesDB(String propertiesDirectoryPath, String propertiesFilePath) {
        mPropertiesDirectoryPath = propertiesDirectoryPath;
        mPropertiesFilePath = propertiesDirectoryPath + File.separator + propertiesFilePath;
        mProperties = new Properties();
        
        File itemsDir = new File(mPropertiesDirectoryPath);
        if (!itemsDir.exists()) {
            itemsDir.mkdirs();
        }
        
        File propertiesFile = new File(mPropertiesFilePath);
        if (propertiesFile.exists()) {
            try {
                InputStream fileStream = new FileInputStream(propertiesFile);
                load(fileStream);
            } catch (FileNotFoundException fnfe) {
                Logger.log(fnfe);
            }
        }
    }
    
    public AbstractPropertiesDB(InputStream propertyStream) {
        mProperties = new Properties();
        load(propertyStream);
    }
    
    
    private void load(InputStream inputStream) {
        try {
            inputStream = new BufferedInputStream(inputStream);
            mProperties.load(inputStream);
        } catch (IOException ioe) {
            Logger.log(ioe);
        } finally {
            try {
                inputStream.close();
            } catch (IOException ioe) {
                Logger.log(ioe);
            }
        }
    }
    
    protected abstract String getDBType();
    
    public void save() {
        if (mPropertiesFilePath == null) {
            // read only, cannot save
            return;
        }
        
        File dirPath = new File(mPropertiesDirectoryPath);
        if (!dirPath.exists()) {
            dirPath.mkdirs();
        }
        
        File propertiesFile = new File(mPropertiesFilePath);
        OutputStream outputStream = null;
        try {
            outputStream = new BufferedOutputStream(new FileOutputStream(propertiesFile));
            mProperties.store(outputStream, getDBType());
        } catch (IOException ioe) {
            Logger.log(ioe);
        } finally {
            try {
                outputStream.close();
                outputStream.flush();
            } catch (IOException ioe) {
                Logger.log(ioe);
            }
        }
    }
}
