package org.wow.autodkp.persistence;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.persistence.xml.XMLBossReader;


/**
 * Handles the known bosses within AutoDKP.
 * 
 * @author jsandusky
 */
public class BossManager {
    
    private static final String FILE_SAVE = "bosses";
    
    private List mNames = new ArrayList();
    
    private static BossManager mInstance = new BossManager();
    
    private BossManager() {
        mNames.add(LootConstants.BOSS_NAME_NONE);
        
        // Load from bosses file
        File saveFile = new File(
                Persistence.DEFINITIONS_DIR + 
                File.separator + 
                FILE_SAVE + 
                Persistence.XML_EXTENSION);
        if (saveFile.exists()) {
            mNames = new XMLBossReader().readXML(saveFile);
            Collections.sort(mNames);
        }
    }
    
    public static BossManager getDefault() {
        return mInstance;
    }
    
    public List getNames() {
        return mNames;
    }
    
    public String getName(int index) {
        return (String) mNames.get(index);
    }
    
    public int getNameIndex(String name) {
        int index = mNames.indexOf(name);
        if (index < 0) {
            return LootConstants.BOSS_NONE;
        }
        return index;
    }
}
