package org.wow.autodkp.persistence;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import org.wow.autodkp.model.Logger;


public class FileIOUtilities {

    public static final String NEWLINE = System.getProperty("line.separator");
    
    
    /**
     * Reads a text file, and returns a String object containing the text.
     * This will never return null, though it may return an empty String.
     */
    public static String readFileToString(File file) {
        BufferedReader inputStream = null;
        StringBuffer buf = new StringBuffer();
        try {
            inputStream = new BufferedReader(new FileReader(file));
            String line;
            while ((line = inputStream.readLine()) != null) {
                buf.append(line + NEWLINE);
            }
        } catch (IOException e) {
            Logger.log(e);
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException ioe) {
                    Logger.log(ioe);
                }
            }
        }
        return buf.toString();
    }
    
    
    /**
     * Writes the specified text to the specified file.
     */
    public static boolean writeStringToFile(String str, File file) {
        boolean errorOccurred = false;
        BufferedWriter outputStream = null;
        try {
            outputStream = new BufferedWriter(new FileWriter(file));
            outputStream.write(str);
            outputStream.flush();
        } catch (IOException e) {
            errorOccurred = true;
            Logger.log(e);
        } finally {
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException ioe) {
                    errorOccurred = true;
                    Logger.log(ioe);
                }
            }
        }
        return !errorOccurred;
    }
    
    
    /**
     * Copies the contents of the source file to the target file.
     * The copy is a pure byte-wise copy.
     */
    public static void copyBytes(File source, File target) {
        BufferedInputStream in = null;
        BufferedOutputStream out = null;
        try {
            in  = new BufferedInputStream(new FileInputStream(source));
            out = new BufferedOutputStream(new FileOutputStream(target));
            byte[] bytes = new byte[1024];
            int bytesRead = 0;
            while ((bytesRead = in.read(bytes)) != -1) {
                out.write(bytes, 0, bytesRead);
            }
            out.flush();
        } catch (IOException e) {
            Logger.log(e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException ioe) {
                    Logger.log(ioe);
                }
            }
            if (out != null) {
                try {
                    out.close();
                } catch (IOException ioe) {
                    Logger.log(ioe);
                }
            }
        }
    }
}
