package org.wow.autodkp.persistence;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.persistence.xml.XMLInstanceReader;


/**
 * Handles the known instances within AutoDKP (i.e. Molten Core, Blackwing Lair, etc.)
 * 
 * @author jsandusky
 */
public class InstanceManager {
    
    private static final String SAVE_PATH = 
        Persistence.DEFINITIONS_DIR + 
        File.separator + 
        "instances" + 
        Persistence.XML_EXTENSION;
    
    private static InstanceManager mInstance = new InstanceManager();
    
    private List mNames = new ArrayList();
    
    
    private InstanceManager() {
        mNames.add(LootConstants.INSTANCE_NAME_NONE);
        
        // Load from save file
        File saveFile = new File(SAVE_PATH);
        if (saveFile.exists()) {
            mNames = new XMLInstanceReader().readXML(saveFile);
            Collections.sort(mNames);
        }
    }
    
    public static InstanceManager getDefault() {
        return mInstance;
    }
    
    public List getNames() {
        return mNames;
    }
    
    public String getName(int index) {
        return (String) mNames.get(index);
    }
    
    public int getNameIndex(String name) {
        int index = mNames.indexOf(name);
        if (index < 0) {
            return LootConstants.INSTANCE_NONE;
        }
        return index;
    }
}
