package org.wow.autodkp.persistence;

import java.io.File;
import java.io.InputStream;
import java.util.Iterator;
import java.util.List;

import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.model.points.IPointSystem;


/**
 * The master items database. This "database" stores
 * point-system independent loot information.
 * 
 * @author jsandusky
 */
public class ItemMasterDB extends AbstractItemDB {
    
    private static final ItemMasterDB mInstance = new ItemMasterDB();
    
    private static final int POSITION_QUALITY     = 0;
    private static final int POSITION_NAME        = 1;
    private static final int POSITION_BOSS        = 2;
    private static final int POSITION_INSTANCE    = 3;

    private boolean mUpdateFlag;
    
    
    protected ItemMasterDB(String propertiesDirectoryPath) {
        super(propertiesDirectoryPath, Persistence.MASTER_ITEMS_FILE);
    }
    
    private ItemMasterDB() {
        this(
                Persistence.DEFINITIONS_DIR + 
                File.separator + 
                Persistence.ITEMS_DIR + 
                File.separator);
    }
    
    private ItemMasterDB(InputStream stream) {
        super(stream);
    }
    
    
    public static ItemMasterDB createMasterItemDBGhost(InputStream stream) {
        return new ItemMasterDB(stream);
    }
    
    
    protected String getDBType() {
        return "All Items";
    }
    
    public static ItemMasterDB getDefault() {
        return mInstance;
    }
    
    public void setUpdateFlag(boolean toUpdate) {
        mUpdateFlag = toUpdate;
    }
    
    public boolean isUpdateFlagSet() {
        return mUpdateFlag;
    }
    
    protected String getValueString(Loot loot) {
        StringBuffer buf = new StringBuffer();
        String saveQuality  = LootConstants.QUALITY_KEYS[loot.getQuality()];
        buf.append(saveQuality);
        buf.append(":");
        buf.append(loot.getName());
        buf.append(":");
        buf.append(loot.getBossName());
        buf.append(":");
        buf.append(loot.getInstanceName());
        buf.append(":");
        return buf.toString();
    }
    
    public void saveLoots(List loots, IPointSystem[] constraints) {
        for (Iterator iter=loots.iterator(); iter.hasNext();) {
            Loot loot = (Loot) iter.next();
            addLoot(loot);
            for (int j=0; j < constraints.length; j++) {
                constraints[j].getItemDB().addLoot(loot);
            }
        }
        
        save();
        for (int i=0; i < constraints.length; i++) {
            constraints[i].getItemDB().save();
        }
    }
    
    public String getName(String lootID) {
        String value = mProperties.getProperty(lootID);
        if (value != null) {
            String str = getElementAtPosition(value, POSITION_NAME);
            return str;
        }
        return null;
    }
    
    public String getBossName(String lootID) {
        String value = mProperties.getProperty(lootID);
        if (value != null) {
            String str = getElementAtPosition(value, POSITION_BOSS);
            return str;
        }
        return null;
    }
    
    public String getInstanceName(String lootID) {
        String value = mProperties.getProperty(lootID);
        if (value != null) {
            String str = getElementAtPosition(value, POSITION_INSTANCE);
            return str;
        }
        return null;
    }
    
    public int getQuality(String lootID) {
        String value = mProperties.getProperty(lootID);
        if (value != null) {
            String str = getElementAtPosition(value, POSITION_QUALITY);
            if (str != null) {
                return LootConstants.getQualityValueFromKey(str);
            }
        }
        return LootConstants.QUALITY_EPIC;
    }
}
