package org.wow.autodkp.persistence;

import java.io.File;
import java.io.InputStream;

import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.points.IPointSystem;


/**
 * The database that stores point-system dependent 
 * loot information. Such info would be the value of 
 * a particular loot within a point system.
 * 
 * @author jsandusky
 */
public class ItemPointsDB extends AbstractItemDB {
    
    private IPointSystem mConstraint;
    
    protected ItemPointsDB(String propertiesDirectoryPath, IPointSystem constraint) {
        super(propertiesDirectoryPath, constraint.getID() + ".properties");
        mConstraint = constraint;
    }
    
    public ItemPointsDB(IPointSystem constraint) {
        this(
                Persistence.INSTANCES_DIR + 
                File.separator + 
                Persistence.ITEMS_DIR + 
                File.separator,
                constraint);
    }
    
    
    private ItemPointsDB(InputStream stream) {
        super(stream);
    }
    
    
    public static ItemPointsDB createConstraintsItemDBGhost(InputStream stream) {
        return new ItemPointsDB(stream);
    }

    
    protected String getDBType() {
        return "Point System Item Values";
    }
    
    protected String getValueString(Loot loot) {
        String value = loot.getValueForConstraint(mConstraint);
        if (value != null) {
            return value;
        }
        return "";
    }
    
    public String getValue(String lootID) {
        String value = mProperties.getProperty(lootID);
        if (value != null) {
            String str = getElementAtPosition(value, 0);
            return str;
        }
        return null;
    }
}
