package org.wow.autodkp.persistence.xml;

import java.io.File;
import java.io.InputStream;
import java.util.Map;
import java.util.TreeMap;

import org.wow.autodkp.model.AlternatesGroup;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;


/**
 * 
 * 
 * @author jsandusky
 */
public class XMLAlternatesReader {
    
    public Map readXML(File xmlFile) {
        DataHandler handler = new DataHandler();
        XMLUtilities.parse(xmlFile, handler);
        return handler.getAlternates();
    }
    
    public Map readXML(InputStream xmlStream) {
        DataHandler handler = new DataHandler();
        XMLUtilities.parse(xmlStream, handler);
        return handler.getAlternates();
    }
    
    
    private static class DataHandler extends DefaultHandler {
        private StringBuffer mBuffer = new StringBuffer();
        
        private Map mAllAlternatesGroupings;
        private AlternatesGroup mCurrentAlternatesGroup;
        
        
        public void characters(char[] ch, int start, int length) throws SAXException {
            super.characters(ch, start, length);
            if (mBuffer == null) {
                mBuffer = new StringBuffer();
            }
            mBuffer.append(ch, start, length);
        }

        public void endElement(String uri, String localName, String qName) throws SAXException {
            super.endElement(uri, localName, qName);
            String str = null;
            if (mBuffer != null) {
                str = mBuffer.toString();
            }
            if (localName.equals(XMLUtilities.XML_PRIMARY_NAME)) {
                mCurrentAlternatesGroup.setPrimaryPlayerName(str);
            } else if (localName.equals(XMLUtilities.XML_PRIMARY_CLASS)) {
                mCurrentAlternatesGroup.setPrimaryPlayerClass(str);
            } else if (localName.equals(XMLUtilities.XML_ALTERNATE_PLAYER)) {
                mCurrentAlternatesGroup.addAlternate(str);
            } else if (localName.equals(XMLUtilities.XML_ALTERNATE_GROUP)) {
                if (mCurrentAlternatesGroup != null) {
                    mAllAlternatesGroupings.put(
                            mCurrentAlternatesGroup.getPrimaryPlayerName(),
                            mCurrentAlternatesGroup);
                }
                mCurrentAlternatesGroup = null;
            }
        }

        public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
            super.startElement(uri, localName, qName, attributes);
            mBuffer = null;
            if (localName.equals(XMLUtilities.XML_ALTERNATES)) {
                mAllAlternatesGroupings = new TreeMap();
            } else if (localName.equals(XMLUtilities.XML_ALTERNATE_GROUP)) {
                mCurrentAlternatesGroup = new AlternatesGroup();
            }
        }

        public Map getAlternates() {
            return mAllAlternatesGroupings;
        }
    }
}
