package org.wow.autodkp.persistence.xml;

import java.io.File;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.points.Adjustment;
import org.wow.autodkp.model.points.DefaultPointSystem;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.RollCall;
import org.wow.autodkp.model.points.TimeThreshold;
import org.wow.autodkp.persistence.Persistence;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;


/**
 * 
 * 
 * @author jsandusky
 */
public class XMLConstraintsReader {
    
    public IPointSystem readXML(File xmlFile) {
        ConstraintsHandler handler = new ConstraintsHandler();
        XMLUtilities.parse(xmlFile, handler);
        return handler.getConstraints();
    }
    
    
    private static class ConstraintsHandler extends DefaultHandler {
        private StringBuffer mBuffer = new StringBuffer();
        
        private DefaultPointSystem mPointSystem;
        private String  mCurrentLootConstraintName;
        private String  mCurrentLootConstraintID;
        private Float   mCurrentLootConstraintCostValue;
        private Boolean mCurrentLootConstraintCostIsPercent;
        private String  mCurrentBossConstraintName;
        private Float   mCurrentBossConstraintCostValue;
        private Boolean mCurrentBossConstraintCostIsPercent;
        private String  mCurrentAdjustmentID;
        private String  mCurrentAdjustmentName;
        private float   mCurrentAdjustmentInitial;
        private int     mCurrentThresholdValue = -1;
        private Boolean mCurrentThresholdIsPercent;
        private float   mCurrentThresholdPoints;
        private int     mCurrentThresholdMinutes;
        private String  mCurrentRollCallName;
        private float   mCurrentRollCallPoints;
        private int     mCurrentRollCallRecurrence;
        
        
        public void characters(char[] ch, int start, int length) throws SAXException {
            super.characters(ch, start, length);
            if (mBuffer == null) {
                mBuffer = new StringBuffer();
            }
            mBuffer.append(ch, start, length);
        }

        public void endElement(String uri, String localName, String qName) throws SAXException {
            super.endElement(uri, localName, qName);
            String str = null;
            if (mBuffer != null) {
                str = mBuffer.toString();
            }
            if (localName.equals(XMLUtilities.XML_CONSTRAINT_ID)) {
                mPointSystem.setID(str);
            } else if (localName.equals(XMLUtilities.XML_CONSTRAINT_NAME)) {
                mPointSystem.setName(str);
            } else if (localName.equals(XMLUtilities.XML_CONSTRAINT_DKPTYPE)) {
                if (str.indexOf(Persistence.DKPTYPE_BOSSKILL) >= 0) {
                    mPointSystem.setBossKill(true);
                }
                if (str.indexOf(Persistence.DKPTYPE_TIME_BASED) >= 0) {
                    mPointSystem.setTimeBased(true);
                }
                
                if        (str.indexOf(Persistence.DKPTYPE_ZERO_SUM_DIVIDE_EVENLY) >= 0) {
                    mPointSystem.setZeroSum(true, false);
                } else if (str.indexOf(Persistence.DKPTYPE_ZERO_SUM_EXCLUDE_LOOTER) >= 0) {
                    mPointSystem.setZeroSum(true, true);
                }
            } else if (localName.equals(XMLUtilities.XML_CONSTRAINT_BOSSPOINTS)) {
                float value = XMLUtilities.parseFloat(str);
                mPointSystem.setStandardPointsForBoss(value);
            } else if (localName.equals(XMLUtilities.XML_LOOT_NAME)) {
                mCurrentLootConstraintName = str;
            } else if (localName.equals(XMLUtilities.XML_LOOT_ID)) {
                mCurrentLootConstraintID = str;
            } else if (localName.equals(XMLUtilities.XML_LOOT_COST)) {
                mCurrentLootConstraintCostValue = new Float(XMLUtilities.parseFloat(str));
            } else if (localName.equals(XMLUtilities.XML_LOOT_ISPERCENT)) {
                if (str.equals(XMLUtilities.IS_YES)) {
                    mCurrentLootConstraintCostIsPercent = new Boolean(true);
                } else if (str.equals(XMLUtilities.IS_NO)) {
                    mCurrentLootConstraintCostIsPercent = new Boolean(false);
                }
            } else if (localName.equals(XMLUtilities.XML_CONSTRAINT_LOOT)) {
                if (
                        mCurrentLootConstraintID            != null &&
                        mCurrentLootConstraintName          != null &&
                        mCurrentLootConstraintCostValue     != null &&
                        mCurrentLootConstraintCostIsPercent != null) {
                    Cost cost = new Cost(
                            mCurrentLootConstraintCostValue.floatValue(),
                            mCurrentLootConstraintCostIsPercent.booleanValue());
                    mPointSystem.getLootConstraintKeys().add(mCurrentLootConstraintID);
                    mPointSystem.getLootConstraintNames().add(mCurrentLootConstraintName);
                    mPointSystem.getLootConstraintCosts().add(cost);
                }
                mCurrentLootConstraintID = null;
                mCurrentLootConstraintName = null;
                mCurrentLootConstraintCostValue = null;
                mCurrentLootConstraintCostIsPercent = null;
            } else if (localName.equals(XMLUtilities.XML_BOSS_NAME)) {
                mCurrentBossConstraintName = str;
            } else if (localName.equals(XMLUtilities.XML_BOSS_COST)) {
                float value = XMLUtilities.parseFloat(str);
                mCurrentBossConstraintCostValue = new Float(value);
            } else if (localName.equals(XMLUtilities.XML_BOSS_ISPERCENT)) {
                if (str.equals(XMLUtilities.IS_YES)) {
                    mCurrentBossConstraintCostIsPercent = new Boolean(true);
                } else if (str.equals(XMLUtilities.IS_NO)) {
                    mCurrentBossConstraintCostIsPercent = new Boolean(false);
                }
            } else if (localName.equals(XMLUtilities.XML_CONSTRAINT_BOSS)) {
                if (
                        mCurrentBossConstraintName          != null &&
                        mCurrentBossConstraintCostValue     != null &&
                        mCurrentBossConstraintCostIsPercent != null) {
                    Cost cost = new Cost(
                            mCurrentBossConstraintCostValue.floatValue(),
                            mCurrentBossConstraintCostIsPercent.booleanValue());
                    mPointSystem.getBossConstraintNames().add(mCurrentBossConstraintName);
                    mPointSystem.getBossConstraintCosts().add(cost);
                }
                mCurrentBossConstraintName = null;
                mCurrentBossConstraintCostValue = null;
                mCurrentBossConstraintCostIsPercent = null;
            } else if (localName.equals(XMLUtilities.XML_ADJUSTMENT_ID)) {
                mCurrentAdjustmentID = str;
            } else if (localName.equals(XMLUtilities.XML_ADJUSTMENT_NAME)) {
                mCurrentAdjustmentName = str;
            } else if (localName.equals(XMLUtilities.XML_ADJUSTMENT_INITIAL)) {
                mCurrentAdjustmentInitial = XMLUtilities.parseFloat(str);
            } else if (localName.equals(XMLUtilities.XML_ADJUSTMENT)) {
                if (mCurrentAdjustmentID != null && mCurrentAdjustmentName != null) {
                    Adjustment newAdjustment = new Adjustment(mCurrentAdjustmentName);
                    if (mCurrentAdjustmentInitial > 0) {
                        newAdjustment.setInitialValue(mCurrentAdjustmentInitial);
                    }
                    mPointSystem.getAdjustmentMap().put(mCurrentAdjustmentID, newAdjustment);
                }
                mCurrentAdjustmentID = null;
                mCurrentAdjustmentName = null;
                mCurrentAdjustmentInitial = 0.0f;
            } else if (localName.equals(XMLUtilities.XML_LOOT_IS_DIRECT_EDIT)) {
                if (str.equals(XMLUtilities.IS_YES)) {
                    mPointSystem.setDirectLootValueEditing(true);
                } else if (str.equals(XMLUtilities.IS_NO)) {
                    mPointSystem.setDirectLootValueEditing(false);
                }
            } else if (localName.equals(XMLUtilities.XML_LOOT_IS_AUCTIONED_LOOT)) {
                if (str.equals(XMLUtilities.IS_YES)) {
                    mPointSystem.setAuctionedLootValues(true);
                } else if (str.equals(XMLUtilities.IS_NO)) {
                    mPointSystem.setAuctionedLootValues(false);
                }
            } else if (localName.equals(XMLUtilities.XML_TIMING_DURATION_MINUTES)) {
                int value = XMLUtilities.parseInt(str);
                mPointSystem.setDurationMinutes(value);
            } else if (localName.equals(XMLUtilities.XML_TIMING_DURATION_POINTS)) {
                float value = XMLUtilities.parseFloat(str);
                mPointSystem.setDurationPoints(value);
            } else if (localName.equals(XMLUtilities.XML_TIMING_THRESHOLD_VALUE)) {
                mCurrentThresholdValue = XMLUtilities.parseInt(str);
            } else if (localName.equals(XMLUtilities.XML_TIMING_THRESHOLD_ISPERCENT)) {
                if (str.equals(XMLUtilities.IS_YES)) {
                    mCurrentThresholdIsPercent = new Boolean(true);
                } else if (str.equals(XMLUtilities.IS_NO)) {
                    mCurrentThresholdIsPercent = new Boolean(false);
                }
            } else if (localName.equals(XMLUtilities.XML_TIMING_THRESHOLD_POINTS)) {
                mCurrentThresholdPoints = XMLUtilities.parseFloat(str);
            } else if (localName.equals(XMLUtilities.XML_TIMING_THRESHOLD_MINUTES)) {
                mCurrentThresholdMinutes = XMLUtilities.parseInt(str);
            } else if (localName.equals(XMLUtilities.XML_TIMING_THRESHOLD)) {
                if (
                        mCurrentThresholdIsPercent != null &&
                        mCurrentThresholdValue >= 0) {
                    TimeThreshold threshold = new TimeThreshold(
                            mCurrentThresholdValue,
                            mCurrentThresholdIsPercent.booleanValue(),
                            mCurrentThresholdPoints,
                            mCurrentThresholdMinutes);
                    mPointSystem.setTimeThreshold(threshold);
                }
                mCurrentThresholdValue = -1;
                mCurrentThresholdIsPercent = null;
                mCurrentThresholdPoints = 0.0f;
                mCurrentThresholdMinutes = 0;
            } else if (localName.equals(XMLUtilities.XML_TIMING_ROLL_CALL_NAME)) {
                mCurrentRollCallName = str;
            } else if (localName.equals(XMLUtilities.XML_TIMING_ROLL_CALL_POINTS)) {
                mCurrentRollCallPoints = XMLUtilities.parseFloat(str);
            } else if (localName.equals(XMLUtilities.XML_TIMING_ROLL_CALL_RECUR)) {
                mCurrentRollCallRecurrence = XMLUtilities.parseInt(str);
            } else if (localName.equals(XMLUtilities.XML_TIMING_ROLL_CALL)) {
                if (mCurrentRollCallName != null) {
                    RollCall rollCall = new RollCall(
                            mCurrentRollCallName, 
                            mCurrentRollCallPoints,
                            mCurrentRollCallRecurrence);
                    mPointSystem.addRollCall(rollCall);
                }
                mCurrentRollCallName = null;
                mCurrentRollCallPoints = 0.0f;
                mCurrentRollCallRecurrence = 0;
            } else if (localName.equals(XMLUtilities.XML_RAID_TIMING)) {
                // Verify that the Raid Start and Raid End roll calls exist.
                // Add them if they don't exist.
                if (mPointSystem.getRollCall(RollCall.RAID_START_NAME) == null) {
                    mPointSystem.addRollCall(0, new RollCall(RollCall.RAID_START_NAME, 0.0f, 0));
                }
                if (mPointSystem.getRollCall(RollCall.RAID_END_NAME) == null) {
                    mPointSystem.addRollCall(1, new RollCall(RollCall.RAID_END_NAME, 0.0f, 0));
                }
            }
            
        }

        public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
            super.startElement(uri, localName, qName, attributes);
            mBuffer = null;
            if (localName.equals(XMLUtilities.XML_CONSTRAINT)) {
                mPointSystem = new DefaultPointSystem();
            }
        }
        
        public IPointSystem getConstraints() {
            return mPointSystem;
        }
    }
}
