package org.wow.autodkp.persistence.xml;

import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.points.Adjustment;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.RollCall;
import org.wow.autodkp.model.points.TimeThreshold;
import org.wow.autodkp.persistence.Persistence;


/**
 * 
 * 
 * @author jsandusky
 */
public class XMLConstraintsWriter {    
    
    public void writeXML(File saveFile, IPointSystem constraint) {
        
        StringBuffer buf = new StringBuffer();
        buf.append(XMLUtilities.header());
        buf.append(XMLUtilities.tagStart(XMLUtilities.XML_CONSTRAINT));
        
        buf.append(XMLUtilities.tag(XMLUtilities.XML_CONSTRAINT_NAME, constraint.getName()));
        
        StringBuffer dkpTypeStr = new StringBuffer();
        if (constraint.isBossKill()) {
            dkpTypeStr.append(Persistence.DKPTYPE_BOSSKILL + ",");
        }
        if (constraint.isTimeBased()) {
            dkpTypeStr.append(Persistence.DKPTYPE_TIME_BASED + ",");
        }
        if (constraint.isZeroSum()) {
            if (constraint.isZeroSumExcludeLooter()) {
                dkpTypeStr.append(Persistence.DKPTYPE_ZERO_SUM_EXCLUDE_LOOTER + ",");
            } else {
                dkpTypeStr.append(Persistence.DKPTYPE_ZERO_SUM_DIVIDE_EVENLY + ",");
            }
        }
        
        buf.append(XMLUtilities.tag(XMLUtilities.XML_CONSTRAINT_ID,         constraint.getID()));
        buf.append(XMLUtilities.tag(XMLUtilities.XML_CONSTRAINT_DKPTYPE,    dkpTypeStr.toString()));
        buf.append(XMLUtilities.tag(XMLUtilities.XML_CONSTRAINT_BOSSPOINTS, String.valueOf(constraint.getStandardPointsForBoss())));
        
        
        // MISC LOOT INFO
        boolean isLootDirectEdit = constraint.isDirectLootValueEditing();
        if (isLootDirectEdit) {
            buf.append(XMLUtilities.tag(XMLUtilities.XML_LOOT_IS_DIRECT_EDIT, XMLUtilities.IS_YES));
        } else {
            buf.append(XMLUtilities.tag(XMLUtilities.XML_LOOT_IS_DIRECT_EDIT, XMLUtilities.IS_NO));
        }
        
        boolean isAuctionedLootValues = constraint.isAuctionedLootValues();
        if (isAuctionedLootValues) {
            buf.append(XMLUtilities.tag(XMLUtilities.XML_LOOT_IS_AUCTIONED_LOOT, XMLUtilities.IS_YES));
        } else {
            buf.append(XMLUtilities.tag(XMLUtilities.XML_LOOT_IS_AUCTIONED_LOOT, XMLUtilities.IS_NO));
        }
        
        
        // LOOT CATEGORIES
        buf.append(XMLUtilities.tagStart(XMLUtilities.XML_RAID_CONSTRAINT_LOOTS));
        List lootPointCategoryCosts = constraint.getLootConstraintCosts();
        List lootPointCategoryNames = constraint.getLootConstraintNames();
        List lootPointCategoryKeys  = constraint.getLootConstraintKeys();
        for (Iterator iter=lootPointCategoryKeys.iterator(); iter.hasNext();) {
            String key = (String) iter.next();
            if (key.length() > 0) {
                buf.append(XMLUtilities.tagStart(XMLUtilities.XML_CONSTRAINT_LOOT));
                int index = lootPointCategoryKeys.indexOf(key);
                String name = (String) lootPointCategoryNames.get(index);
                Cost cost   = (Cost)   lootPointCategoryCosts.get(index);
                buf.append(XMLUtilities.tag(XMLUtilities.XML_LOOT_ID,        key));
                buf.append(XMLUtilities.tag(XMLUtilities.XML_LOOT_NAME,      name));
                buf.append(XMLUtilities.tag(XMLUtilities.XML_LOOT_COST,      String.valueOf(cost.getCost())));
                buf.append(XMLUtilities.tag(XMLUtilities.XML_LOOT_ISPERCENT, 
                        cost.isPercent() ? XMLUtilities.IS_YES : XMLUtilities.IS_NO));
                buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_CONSTRAINT_LOOT));
            }
        }
        buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_RAID_CONSTRAINT_LOOTS));
        
        
        // BOSSES
        buf.append(XMLUtilities.tagStart(XMLUtilities.XML_RAID_CONSTRAINT_BOSSES));
        List bossPointCategoryCosts = constraint.getBossConstraintCosts();
        List bossPointCategoryNames = constraint.getBossConstraintNames();
        for (Iterator iter=bossPointCategoryNames.iterator(); iter.hasNext();) {
            String name = (String) iter.next();
            if (name.length() > 0) {
                buf.append(XMLUtilities.tagStart(XMLUtilities.XML_CONSTRAINT_BOSS));
                int index = bossPointCategoryNames.indexOf(name);
                Cost cost = (Cost) bossPointCategoryCosts.get(index);
                buf.append(XMLUtilities.tag(XMLUtilities.XML_BOSS_NAME,      name));
                buf.append(XMLUtilities.tag(XMLUtilities.XML_BOSS_COST,      String.valueOf(cost.getCost())));
                buf.append(XMLUtilities.tag(XMLUtilities.XML_BOSS_ISPERCENT, 
                        cost.isPercent() ? XMLUtilities.IS_YES : XMLUtilities.IS_NO));
                buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_CONSTRAINT_BOSS));
            }
        }
        buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_RAID_CONSTRAINT_BOSSES));

        
        // ADJUSTMENTS
        buf.append(XMLUtilities.tagStart(XMLUtilities.XML_RAID_ADJUSTMENTS));
        Map adjustmentMap = constraint.getAdjustmentMap();
        if (adjustmentMap != null) {
            for (Iterator adjIter=adjustmentMap.keySet().iterator(); adjIter.hasNext();) {
                buf.append(XMLUtilities.tagStart(XMLUtilities.XML_ADJUSTMENT));
                String adjustmentID   = (String) adjIter.next();
                Adjustment adjustment = (Adjustment) adjustmentMap.get(adjustmentID);
                buf.append(XMLUtilities.tag(XMLUtilities.XML_ADJUSTMENT_ID,      adjustmentID));
                buf.append(XMLUtilities.tag(XMLUtilities.XML_ADJUSTMENT_NAME,    adjustment.getName()));
                float initialValue = adjustment.getInitialValue();
                if (initialValue != 0) {
                    buf.append(XMLUtilities.tag(XMLUtilities.XML_ADJUSTMENT_INITIAL, String.valueOf(initialValue)));
                }
                buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_ADJUSTMENT));
            }
        }
        buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_RAID_ADJUSTMENTS));
        
        
        // TIME DKP
        buf.append(XMLUtilities.tagStart(XMLUtilities.XML_RAID_TIMING));
        buf.append(XMLUtilities.tag(
                XMLUtilities.XML_TIMING_DURATION_POINTS, 
                String.valueOf(constraint.getDurationPoints())));
        buf.append(XMLUtilities.tag(
                XMLUtilities.XML_TIMING_DURATION_MINUTES, 
                String.valueOf(constraint.getDurationMinutes())));
        TimeThreshold threshold = constraint.getTimeThreshold();
        if (threshold != null) {
            buf.append(XMLUtilities.tagStart(XMLUtilities.XML_TIMING_THRESHOLD));
            buf.append(XMLUtilities.tag(
                    XMLUtilities.XML_TIMING_THRESHOLD_VALUE, 
                    String.valueOf(threshold.getValue())));
            buf.append(XMLUtilities.tag(
                    XMLUtilities.XML_TIMING_THRESHOLD_ISPERCENT, 
                    threshold.isPercent() ? XMLUtilities.IS_YES : XMLUtilities.IS_NO));
            buf.append(XMLUtilities.tag(
                    XMLUtilities.XML_TIMING_THRESHOLD_POINTS, 
                    String.valueOf(threshold.getPoints())));
            buf.append(XMLUtilities.tag(
                    XMLUtilities.XML_TIMING_THRESHOLD_MINUTES, 
                    String.valueOf(threshold.getMinutes())));
            buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_TIMING_THRESHOLD));
        }
        List rollCalls = constraint.getRollCalls();
        if (rollCalls != null) {
            for (Iterator rollCallIter=rollCalls.iterator(); rollCallIter.hasNext();) {
                buf.append(XMLUtilities.tagStart(XMLUtilities.XML_TIMING_ROLL_CALL));
                RollCall rollCall = (RollCall) rollCallIter.next();
                buf.append(XMLUtilities.tag(
                        XMLUtilities.XML_TIMING_ROLL_CALL_NAME, 
                        rollCall.getName()));
                buf.append(XMLUtilities.tag(
                        XMLUtilities.XML_TIMING_ROLL_CALL_POINTS, 
                        String.valueOf(rollCall.getPoints())));
                buf.append(XMLUtilities.tag(
                        XMLUtilities.XML_TIMING_ROLL_CALL_RECUR, 
                        String.valueOf(rollCall.getRecurrence())));
                buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_TIMING_ROLL_CALL));
            }
        }
        buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_RAID_TIMING));

        
        buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_CONSTRAINT));

        XMLUtilities.store(buf.toString(), saveFile);
    }
}
