package org.wow.autodkp.persistence.xml;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.points.CustomFormulaItem;
import org.wow.autodkp.model.points.DefaultPointSystemPlayerData;
import org.wow.autodkp.model.points.DefaultPointSystemRaidData;
import org.wow.autodkp.model.points.IPointSystemPlayerData;
import org.wow.autodkp.model.points.IPointSystemRaidData;
import org.wow.autodkp.model.points.dkp.IDKPCalculator;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;


/**
 * 
 * 
 * @author jsandusky
 */
public class XMLRaidConstraintsReader {
    
    
    public IPointSystemRaidData readXML(IDKPCalculator calculator, File xmlFile) {
        RaidConstraintsHandler handler = new RaidConstraintsHandler(calculator);
        XMLUtilities.parse(xmlFile, handler);
        return handler.getRaidData();
    }
    
    
    private static class RaidConstraintsHandler extends DefaultHandler {
        
        private IDKPCalculator mCalculator;
        private StringBuffer mBuffer = new StringBuffer();
        private Map mConstraintsPlayerData = new HashMap();
        private DefaultPointSystemPlayerData mCurrentPlayerData;
        private DefaultPointSystemRaidData mRaidData;
        private String mCurrentAdjustmentID;
        private float mCurrentAdjustmentValue;
        private String mCurrentCarryoverPlayerName;
        private String mCurrentCarryoverPlayerClass;
        private float mCurrentCarryoverPoints;
        private CustomFormulaItem mCustomFormulaItem;
        private float mCurrentCustomCostValue;
        private boolean mCurrentCustomCostIsPercent;
        private List mDistributionExceptions;
        private List mBossKillExceptions;
        private List mTimeExceptions;
        private String mCurrentRollCallName;
        private Date mCurrentRollCallDate;
        
        
        public RaidConstraintsHandler(IDKPCalculator calculator) {
            mCalculator = calculator;
        }
        
        public void characters(char[] ch, int start, int length) throws SAXException {
            super.characters(ch, start, length);
            if (mBuffer == null) {
                mBuffer = new StringBuffer();
            }
            mBuffer.append(ch, start, length);
        }

        public void endElement(String uri, String localName, String qName) throws SAXException {
            super.endElement(uri, localName, qName);
            String str = null;
            if (mBuffer != null) {
                str = mBuffer.toString();
            }
            if (localName.equals(XMLUtilities.XML_RAID_DATA_PLAYER)) {
                mConstraintsPlayerData.put(
                        mCurrentPlayerData.getPlayerName(), 
                        mCurrentPlayerData);
                mCurrentPlayerData = null;
            } else if (localName.equals(XMLUtilities.XML_PLAYER_NAME)) {
                mCurrentPlayerData.setPlayerName(str);
            } else if (localName.equals(XMLUtilities.XML_PLAYER_CLASS)) {
                mCurrentPlayerData.setPlayerClass(str);
            } else if (localName.equals(XMLRaidConstraintsWriter.XML_POINTS_INCOMING)) {
                mCurrentPlayerData.setIncomingTotal(XMLUtilities.parseFloat(str));
            } else if (localName.equals(XMLRaidConstraintsWriter.XML_FORMULA)) {
                mCurrentPlayerData.setFormula(mCalculator.generateFormula(str));
            } else if (localName.equals(XMLRaidConstraintsWriter.XML_FORMULA_TOTAL)) {
                mCurrentPlayerData.setFormulaTotal(XMLUtilities.parseFloat(str));
            } else if (localName.equals(XMLUtilities.XML_ADJUSTMENT_ID)) {
                mCurrentAdjustmentID = str;
            } else if (localName.equals(XMLUtilities.XML_ADJUSTMENT_VALUE)) {
                mCurrentAdjustmentValue = XMLUtilities.parseFloat(str);
            } else if (localName.equals(XMLUtilities.XML_ADJUSTMENT)) {
                if (mCurrentAdjustmentID != null) {
                    mCurrentPlayerData.setAdjustment(mCurrentAdjustmentID, mCurrentAdjustmentValue);
                }
                mCurrentAdjustmentID = null;
                mCurrentAdjustmentValue = 0;
            } else if (localName.equals(XMLUtilities.XML_CARRYOVER_PLAYER_NAME)) {
                mCurrentCarryoverPlayerName = str;
            } else if (localName.equals(XMLUtilities.XML_CARRYOVER_PLAYER_CLASS)) {
                mCurrentCarryoverPlayerClass = str;
            } else if (localName.equals(XMLUtilities.XML_CARRYOVER_POINTS)) {
                mCurrentCarryoverPoints = XMLUtilities.parseFloat(str);
            } else if (localName.equals(XMLUtilities.XML_CARRYOVER)) {
                if (mCurrentCarryoverPlayerName != null) {
                    IPointSystemPlayerData newData = new DefaultPointSystemPlayerData();
                    newData.setPlayerName(mCurrentCarryoverPlayerName);
                    newData.setPlayerClass(mCurrentCarryoverPlayerClass);
                    newData.setIncomingTotal(mCurrentCarryoverPoints);
                    newData.setOutgoingTotal(mCurrentCarryoverPoints);
                    newData.setFormula(mCalculator.generateFormula(null));
                    mConstraintsPlayerData.put(mCurrentCarryoverPlayerName, newData);
                }
                mCurrentCarryoverPlayerName = null;
                mCurrentCarryoverPlayerClass = null;
                mCurrentCarryoverPoints = 0;
            } else if (localName.equals(XMLRaidConstraintsWriter.XML_POINTS_OUTGOING)) {
                mCurrentPlayerData.setOutgoingTotal(XMLUtilities.parseFloat(str));
            } else if (localName.equals(XMLRaidConstraintsWriter.XML_LOCK_STATUS)) {
                if (str.equals(XMLRaidConstraintsWriter.IS_LOCKED)) {
                    mRaidData.setLocked(true);
                }
            } else if (localName.equals(XMLUtilities.XML_IMPORTED_RAIDID)) {
                mRaidData.setImportedRaidID(str);
            } else if (localName.equals(XMLUtilities.XML_RAID_DATA)) {
                mRaidData.setConstraintsPlayerData(mConstraintsPlayerData);
            } else if (localName.equals(XMLUtilities.XML_EXCLUDED_LOOTDEF_ID)) {
                mCurrentPlayerData.addExcludedLootID(str);
            } else if (localName.equals(XMLUtilities.XML_CUSTOM_COST_TEXT)) {
                mCustomFormulaItem.setItemText(str);
            } else if (localName.equals(XMLUtilities.XML_CUSTOM_COST_INDEX)) {
                mCustomFormulaItem.setIndex(XMLUtilities.parseInt(str));
            } else if (localName.equals(XMLUtilities.XML_CUSTOM_COST_COLOR)) {
                mCustomFormulaItem.setLootColor(Loot.getColorFromColorString(str));
            } else if (localName.equals(XMLUtilities.XML_CUSTOM_COST_LOOT_TIME)) {
                if (str.length() > 0) {
                    mCustomFormulaItem.setLootTime(XMLUtilities.parseDate(str));
                }
            } else if (localName.equals(XMLUtilities.XML_CUSTOM_COST_COST)) {
                mCurrentCustomCostValue = XMLUtilities.parseFloat(str);
            } else if (localName.equals(XMLUtilities.XML_CUSTOM_COST_ISPERCENT)) {
                if (str.equals(XMLUtilities.IS_YES)) {
                    mCurrentCustomCostIsPercent = true;
                } else if (str.equals(XMLUtilities.IS_NO)) {
                    mCurrentCustomCostIsPercent = false;
                }
            } else if (localName.equals(XMLUtilities.XML_CUSTOM_COST)) {
                if (mCustomFormulaItem != null) {
                    mCustomFormulaItem.setCost(new Cost(
                            mCurrentCustomCostValue, 
                            mCurrentCustomCostIsPercent));
                    mCurrentPlayerData.addCustomFormulaItem(mCustomFormulaItem);
                }
                mCustomFormulaItem = null;
                mCurrentCustomCostValue = 0;
            } else if (localName.equals(XMLUtilities.XML_EXCEPTION_DIST_ID)) {
                if (mDistributionExceptions == null) {
                    mDistributionExceptions = new ArrayList();
                }
                if (!mDistributionExceptions.contains(str)) {
                    mDistributionExceptions.add(str);
                }
            } else if (localName.equals(XMLUtilities.XML_EXCEPTION_DIST)) {
                mCurrentPlayerData.setDistributionExceptions(mDistributionExceptions);
                mDistributionExceptions = null;
            } else if (localName.equals(XMLUtilities.XML_EXCEPTION_BOSS_ID)) {
                if (mBossKillExceptions == null) {
                    mBossKillExceptions = new ArrayList();
                }
                if (!mBossKillExceptions.contains(str)) {
                    mBossKillExceptions.add(str);
                }
            } else if (localName.equals(XMLUtilities.XML_EXCEPTION_BOSS)) {
                mCurrentPlayerData.setBossKillExceptions(mBossKillExceptions);
                mBossKillExceptions = null;
            } else if (localName.equals(XMLUtilities.XML_EXCEPTION_TIME_ID)) {
                if (mTimeExceptions == null) {
                    mTimeExceptions = new ArrayList();
                }
                if (!mTimeExceptions.contains(str)) {
                    mTimeExceptions.add(str);
                }
            } else if (localName.equals(XMLUtilities.XML_EXCEPTION_TIME)) {
                mCurrentPlayerData.setTimeExceptions(mTimeExceptions);
                mTimeExceptions = null;
            } else if (localName.equals(XMLUtilities.XML_CONSTRAINT_ROLL_CALL_NAME)) {
                mCurrentRollCallName = str;
            } else if (localName.equals(XMLUtilities.XML_CONSTRAINT_ROLL_CALL_DATE)) {
                mCurrentRollCallDate = XMLUtilities.parseDate(str);
            } else if (localName.equals(XMLUtilities.XML_CONSTRAINT_ROLL_CALL)) {
                if (mCurrentRollCallName != null && mCurrentRollCallDate != null) {
                    Map rollCallDateMap = mRaidData.getRollCallDateMap();
                    rollCallDateMap.put(mCurrentRollCallName, mCurrentRollCallDate);
                }
                mCurrentRollCallName = null;
                mCurrentRollCallDate = null;
            }
        }

        public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
            super.startElement(uri, localName, qName, attributes);
            mBuffer = null;
            if (localName.equals(XMLUtilities.XML_RAID_DATA_PLAYER)) {
                mCurrentPlayerData = new DefaultPointSystemPlayerData();
            } else if (localName.equals(XMLUtilities.XML_RAID)) {
                mRaidData = new DefaultPointSystemRaidData();
            } else if (localName.equals(XMLUtilities.XML_CUSTOM_COST)) {
                mCustomFormulaItem = new CustomFormulaItem();
            }
        }
        
        public IPointSystemRaidData getRaidData() {
            return mRaidData;
        }
    }
}
