package org.wow.autodkp.persistence.xml;

import java.io.File;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.points.CustomFormulaItem;
import org.wow.autodkp.model.points.IPointSystemPlayerData;
import org.wow.autodkp.model.points.IPointSystemRaidData;


/**
 * 
 * 
 * @author jsandusky
 */
public class XMLRaidConstraintsWriter {
    
    public static final String XML_POINTS_INCOMING   = "PointsIncoming";
    public static final String XML_POINTS_OUTGOING   = "PointsOutgoing";
    public static final String XML_FORMULA           = "Formula";
    public static final String XML_FORMULA_TOTAL     = "FormulaTotal";
    public static final String XML_LOCK_STATUS       = "LockedStatus";
    
    public static final String IS_LOCKED   = "locked";
    public static final String IS_UNLOCKED = "unlocked";
    
    
    public void writeXML(File saveFile, IPointSystemRaidData raidData) {
        
        StringBuffer buf = new StringBuffer();
        buf.append(XMLUtilities.header());
        buf.append(XMLUtilities.tagStart(XMLUtilities.XML_RAID));
        
        if (raidData.isLocked()) {
            buf.append(XMLUtilities.tag(XML_LOCK_STATUS, IS_LOCKED));
        } else {
            buf.append(XMLUtilities.tag(XML_LOCK_STATUS, IS_UNLOCKED));
        }
        
        buf.append(XMLUtilities.tag(XMLUtilities.XML_IMPORTED_RAIDID, raidData.getImportedRaidID()));

        buf.append(XMLUtilities.tagStart(XMLUtilities.XML_RAID_DATA));
        Map allPlayerData = raidData.getPointSystemPlayerDataMap();
        for (Iterator playerIter=allPlayerData.keySet().iterator(); playerIter.hasNext();) {
            String playerName = (String) playerIter.next();
            IPointSystemPlayerData playerDatum = (IPointSystemPlayerData) allPlayerData.get(playerName);
            buf.append(XMLUtilities.tagStart(XMLUtilities.XML_RAID_DATA_PLAYER));
            
            buf.append(XMLUtilities.tag(XMLUtilities.XML_PLAYER_NAME,  playerName));
            buf.append(XMLUtilities.tag(XMLUtilities.XML_PLAYER_CLASS, playerDatum.getPlayerClass()));
            
            buf.append(XMLUtilities.tag(XML_POINTS_INCOMING,   String.valueOf(playerDatum.getIncomingTotal())));
            buf.append(XMLUtilities.tag(XML_FORMULA,           playerDatum.getFormula().generateFormulaPersistenceString()));
            buf.append(XMLUtilities.tag(XML_FORMULA_TOTAL,     String.valueOf(playerDatum.getFormulaTotal())));
            buf.append(XMLUtilities.tag(XML_POINTS_OUTGOING,   String.valueOf(playerDatum.getOutgoingTotal())));
            
            buf.append(XMLUtilities.tagStart(XMLUtilities.XML_RAID_ADJUSTMENTS));
            Map adjustmentIDToValueMap = playerDatum.getAdjustmentIDToValueMap();
            if (adjustmentIDToValueMap != null) {
                for (Iterator adjIter=adjustmentIDToValueMap.keySet().iterator(); adjIter.hasNext();) {
                    buf.append(XMLUtilities.tagStart(XMLUtilities.XML_ADJUSTMENT));
                    String adjustmentID = (String) adjIter.next();
                    Float value = (Float) adjustmentIDToValueMap.get(adjustmentID);
                    buf.append(XMLUtilities.tag(XMLUtilities.XML_ADJUSTMENT_ID,    adjustmentID));
                    buf.append(XMLUtilities.tag(XMLUtilities.XML_ADJUSTMENT_VALUE, value.toString()));
                    buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_ADJUSTMENT));
                }
            }
            buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_RAID_ADJUSTMENTS));

            buf.append(XMLUtilities.tagStart(XMLUtilities.XML_RAID_CUSTOM_COSTS));
            List customFormulaItems = playerDatum.getCustomFormulaItems();
            if (customFormulaItems != null) {
                for (Iterator iter=customFormulaItems.iterator(); iter.hasNext();) {
                    CustomFormulaItem item = (CustomFormulaItem) iter.next();
                    String colorText = Loot.getColorStringFromColor(item.getLootColor());
                    Cost cost = item.getCost();
                    buf.append(XMLUtilities.tagStart(XMLUtilities.XML_CUSTOM_COST));
                    buf.append(XMLUtilities.tag(XMLUtilities.XML_CUSTOM_COST_TEXT, item.getItemText()));
                    buf.append(XMLUtilities.tag(XMLUtilities.XML_CUSTOM_COST_INDEX, String.valueOf(item.getIndex())));
                    buf.append(XMLUtilities.tag(XMLUtilities.XML_CUSTOM_COST_COLOR, colorText));
                    if (item.getLootTime() != null) {
                        buf.append(XMLUtilities.tagDate(XMLUtilities.XML_CUSTOM_COST_LOOT_TIME, item.getLootTime()));
                    }
                    buf.append(XMLUtilities.tag(XMLUtilities.XML_CUSTOM_COST_COST, String.valueOf(cost.getCost())));
                    if (cost.isPercent()) {
                        buf.append(XMLUtilities.tag(XMLUtilities.XML_CUSTOM_COST_ISPERCENT, XMLUtilities.IS_YES));
                    } else {
                        buf.append(XMLUtilities.tag(XMLUtilities.XML_CUSTOM_COST_ISPERCENT, XMLUtilities.IS_NO));
                    }
                    buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_CUSTOM_COST));
                }
            }
            buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_RAID_CUSTOM_COSTS));
            
            buf.append(XMLUtilities.tagStart(XMLUtilities.XML_CONSTRAINT_EXCLUDED_LOOTDEF));
            List excludedLootIDs = playerDatum.getExcludedLootIDs();
            if (excludedLootIDs != null) {
                for (Iterator tagIter=excludedLootIDs.iterator(); tagIter.hasNext();) {
                    String lootID = (String) tagIter.next();
                    buf.append(XMLUtilities.tag(XMLUtilities.XML_EXCLUDED_LOOTDEF_ID, lootID));
                }
            }
            buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_CONSTRAINT_EXCLUDED_LOOTDEF));
            
            List distExceptions = playerDatum.getDistributionExceptions();
            if (distExceptions != null) {
                buf.append(XMLUtilities.tagStart(XMLUtilities.XML_EXCEPTION_DIST));
                for (Iterator distIter=distExceptions.iterator(); distIter.hasNext();) {
                    String id = (String) distIter.next();
                    buf.append(XMLUtilities.tag(XMLUtilities.XML_EXCEPTION_DIST_ID, id));
                }
                buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_EXCEPTION_DIST));
            }
            
            List bossKillExceptions = playerDatum.getBossKillExceptions();
            if (bossKillExceptions != null) {
                buf.append(XMLUtilities.tagStart(XMLUtilities.XML_EXCEPTION_BOSS));
                for (Iterator bossIter=bossKillExceptions.iterator(); bossIter.hasNext();) {
                    String id = (String) bossIter.next();
                    buf.append(XMLUtilities.tag(XMLUtilities.XML_EXCEPTION_BOSS_ID, id));
                }
                buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_EXCEPTION_BOSS));
            }
            
            List timeExceptions = playerDatum.getTimeExceptions();
            if (timeExceptions != null) {
                buf.append(XMLUtilities.tagStart(XMLUtilities.XML_EXCEPTION_TIME));
                for (Iterator timeIter=timeExceptions.iterator(); timeIter.hasNext();) {
                    String id = (String) timeIter.next();
                    buf.append(XMLUtilities.tag(XMLUtilities.XML_EXCEPTION_TIME_ID, id));
                }
                buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_EXCEPTION_TIME));
            }
            
            buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_RAID_DATA_PLAYER));
        }
        buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_RAID_DATA));
        
        buf.append(XMLUtilities.tagStart(XMLUtilities.XML_CONSTRAINT_TIMING));
        Map rollCallDateMap = raidData.getRollCallDateMap();
        for (Iterator rollCallIter=rollCallDateMap.keySet().iterator(); rollCallIter.hasNext();) {
            buf.append(XMLUtilities.tagStart(XMLUtilities.XML_CONSTRAINT_ROLL_CALL));
            String name = (String) rollCallIter.next();
            Date date = (Date) rollCallDateMap.get(name);
            buf.append(XMLUtilities.tag(XMLUtilities.XML_CONSTRAINT_ROLL_CALL_NAME, name));
            if (date != null) {
                buf.append(XMLUtilities.tagDate(XMLUtilities.XML_CONSTRAINT_ROLL_CALL_DATE, date));
            }
            buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_CONSTRAINT_ROLL_CALL));
        }
        buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_CONSTRAINT_TIMING));
        
        buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_RAID));
        
        XMLUtilities.store(buf.toString(), saveFile);
    }
}
