package org.wow.autodkp.persistence.xml;

import java.io.File;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.model.PlayerData;



/**
 * 
 * 
 * @author jsandusky
 */
public class XMLRaidWriter {
    
    public static final String XML_RAID_DATE_START    = "RaidDateStart";
    public static final String XML_RAID_DATE_END      = "RaidDateEnd";
    
    public static final String XML_PLAYER_JOIN_DATES  = "JoinDates";
    public static final String XML_PLAYER_LEAVE_DATES = "LeaveDates";
    public static final String XML_PLAYER_JOIN_DATE   = "JoinDate";
    public static final String XML_PLAYER_LEAVE_DATE  = "LeaveDate";
    
    public static final String XML_RAID_DATA_PLAYER_LOOT    = "LootData";
    public static final String XML_LOOT                     = "Loot";
    public static final String XML_LOOT_ID                  = "ID";
    public static final String XML_LOOT_DATE                = "TimeLooted";
    public static final String XML_LOOT_AUCTION             = "Auction";
    public static final String XML_LOOT_AUCTION_VALUE       = "AuctionValue";
    public static final String XML_LOOT_AUCTION_ISPERCENT   = "AuctionValueIsPercent";
    
    public static final String XML_LOOT_NAME                = "LootName";
    public static final String XML_LOOT_QUALITY             = "LootQuality";
    public static final String XML_LOOT_INSTANCE            = "LootInstance";
    public static final String XML_LOOT_BOSS                = "LootBoss";
    
    public static final String XML_RAID_DATA_BOSS_KILLS  = "BossKillDates";
    public static final String XML_RAID_DATA_BOSS        = "BossKillDate";
    public static final String XML_BOSS_NAME             = "BossName";
    public static final String XML_BOSS_KILL_DATE        = "KillDate";

    
    public void writeXML(File saveFile, 
                         Date startDate, 
                         Date endDate, 
                         Map bossKeyToKillDateMap,
                         List instanceKeys,
                         PlayerData[] data) {
        
        StringBuffer buf = new StringBuffer();
        buf.append(XMLUtilities.header());
        buf.append(XMLUtilities.tagStart(XMLUtilities.XML_RAID));
        
        buf.append(XMLUtilities.tagDate(XML_RAID_DATE_START, startDate));
        if (endDate != null) {
            buf.append(XMLUtilities.tagDate(XML_RAID_DATE_END, endDate));
        }
        
        buf.append(XMLUtilities.tagStart(XML_RAID_DATA_BOSS_KILLS));
        for (Iterator iter=bossKeyToKillDateMap.keySet().iterator(); iter.hasNext();) {
            buf.append(XMLUtilities.tagStart(XML_RAID_DATA_BOSS));
            String bossKey = (String) iter.next();
            Date date = (Date) bossKeyToKillDateMap.get(bossKey);
            buf.append(XMLUtilities.tag(XML_BOSS_NAME, bossKey));
            buf.append(XMLUtilities.tagDate(XML_BOSS_KILL_DATE, date));
            buf.append(XMLUtilities.tagEnd(XML_RAID_DATA_BOSS));
        }
        buf.append(XMLUtilities.tagEnd(XML_RAID_DATA_BOSS_KILLS));
        
        buf.append(XMLUtilities.tagStart(XMLUtilities.XML_INSTANCES));
        for (Iterator iter=instanceKeys.iterator(); iter.hasNext();) {
            String key = (String) iter.next();
            buf.append(XMLUtilities.tag(XMLUtilities.XML_INSTANCES_NAME, key));
        }
        buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_INSTANCES));
        
        buf.append(XMLUtilities.tagStart(XMLUtilities.XML_RAID_DATA));
        for (int i=0; i < data.length; i++) {
            PlayerData playerData = data[i];
            if (playerData.isNonRaidMember()) {
                // do not save non-raid members
                continue;
            }
            buf.append(XMLUtilities.tagStart(XMLUtilities.XML_RAID_DATA_PLAYER));
            
            buf.append(XMLUtilities.tag(XMLUtilities.XML_PLAYER_NAME, playerData.getPlayerName()));
            buf.append(XMLUtilities.tag(XMLUtilities.XML_PLAYER_CLASS, playerData.getPlayerClass()));
            buf.append(XMLUtilities.tagDates(XML_PLAYER_JOIN_DATES, XML_PLAYER_JOIN_DATE, playerData.getJoinTimes(), playerData));
            buf.append(XMLUtilities.tagDates(XML_PLAYER_LEAVE_DATES, XML_PLAYER_LEAVE_DATE, playerData.getLeaveTimes(), playerData));
            
            buf.append(XMLUtilities.tagStart(XML_RAID_DATA_PLAYER_LOOT));
            List loots = playerData.getLoots();
            if (loots != null) {
                for (Iterator iter=loots.iterator(); iter.hasNext();) {
                    buf.append(tagLoot((Loot) iter.next(), playerData));
                }
            }
            buf.append(XMLUtilities.tagEnd(XML_RAID_DATA_PLAYER_LOOT));
            
            buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_RAID_DATA_PLAYER));
        }
        buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_RAID_DATA));
        buf.append(XMLUtilities.tagEnd(XMLUtilities.XML_RAID));

        XMLUtilities.store(buf.toString(), saveFile);
    }
    
    
    private static String tagLoot(Loot loot, PlayerData playerData) {
        StringBuffer buf = new StringBuffer();
        buf.append(XMLUtilities.tagStart(XML_LOOT));
        buf.append(XMLUtilities.tag(XML_LOOT_ID,       loot.getID()));
        
        // Save the name, quality, instance, and boss so that if an AutoDKP
        // attempts to load this raid that does not have the item in its
        // database, it will still have all the item information.
        String quality = LootConstants.QUALITY_KEYS[loot.getQuality()];
        buf.append(XMLUtilities.tag(XML_LOOT_NAME,     loot.getName()));
        buf.append(XMLUtilities.tag(XML_LOOT_QUALITY,  quality));
        buf.append(XMLUtilities.tag(XML_LOOT_INSTANCE, loot.getInstanceName()));
        buf.append(XMLUtilities.tag(XML_LOOT_BOSS,     loot.getBossName()));
        
        buf.append(XMLUtilities.tagDate(XML_LOOT_DATE, playerData.getLootTimeForLoot(loot)));
        Cost auctionValue = playerData.getAuctionedLootValue(loot);
        if (auctionValue != null && auctionValue.getCost() != 0) {
            buf.append(XMLUtilities.tagStart(XML_LOOT_AUCTION));
            buf.append(XMLUtilities.tag(XML_LOOT_AUCTION_VALUE, String.valueOf(auctionValue.getCost())));
            if (auctionValue.isPercent()) {
                buf.append(XMLUtilities.tag(XML_LOOT_AUCTION_ISPERCENT, XMLUtilities.IS_YES));
            } else {
                buf.append(XMLUtilities.tag(XML_LOOT_AUCTION_ISPERCENT, XMLUtilities.IS_NO));
            }
            buf.append(XMLUtilities.tagEnd(XML_LOOT_AUCTION));
        }
        buf.append(XMLUtilities.tagEnd(XML_LOOT));
        return buf.toString();
    }
}
