package org.wow.autodkp.persistence.xml;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.Reader;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.wow.autodkp.model.FormatConstants;
import org.wow.autodkp.model.Logger;
import org.wow.autodkp.model.PlayerData;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;


/**
 * 
 * 
 * @author jsandusky
 */
public class XMLUtilities {
    
    public static final String NEWLINE = System.getProperty("line.separator");

    public static final String XML_HEADER = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
    
    public static final String XML_RAID               = "Raid";
    public static final String XML_RAID_DATA          = "RaidData";
    public static final String XML_RAID_DATA_PLAYER   = "Player";
    public static final String XML_PLAYER_NAME        = "PlayerName";
    public static final String XML_PLAYER_CLASS       = "PlayerClass";
    
    public static final String XML_IMPORTED_RAIDID    = "ImportedRaidID";
    public static final String XML_BOSSES             = "Bosses";
    public static final String XML_BOSSES_NAME        = "BossName";
    
    public static final String XML_INSTANCES          = "Instances";
    public static final String XML_INSTANCES_NAME     = "InstanceName";
    
    public static final String XML_ALTERNATES         = "Alternates";
    public static final String XML_ALTERNATE_GROUP    = "AlternatesGroup";
    public static final String XML_PRIMARY_NAME       = "PrimaryPlayerName";
    public static final String XML_PRIMARY_CLASS      = "PrimaryPlayerClass";
    public static final String XML_PLAYER_ALTERNATES  = "PlayerAlternates";
    public static final String XML_ALTERNATE_PLAYER   = "Alternate";
    
    public static final String XML_CONSTRAINT              = "PointSystem";
    public static final String XML_CONSTRAINT_ID           = "ID";
    public static final String XML_CONSTRAINT_NAME         = "Name";
    public static final String XML_CONSTRAINT_DKPTYPE      = "DKPType";
    public static final String XML_CONSTRAINT_BOSSPOINTS   = "StandardBossPoints";
    
    public static final String XML_RAID_CONSTRAINT_LOOTS   = "RaidLootCategories";
    public static final String XML_CONSTRAINT_LOOT         = "LootCategory";
    public static final String XML_LOOT_NAME               = "LootCategoryName";
    public static final String XML_LOOT_ID                 = "LootCategoryID";
    public static final String XML_LOOT_COST               = "LootCost";
    public static final String XML_LOOT_ISPERCENT          = "LootCostIsPercent";
    
    public static final String XML_RAID_CONSTRAINT_BOSSES  = "RaidBossCategories";
    public static final String XML_CONSTRAINT_BOSS         = "BossCategory";
    public static final String XML_BOSS_NAME               = "BossCategoryName";
    public static final String XML_BOSS_COST               = "BossCost";
    public static final String XML_BOSS_ISPERCENT          = "BossCostIsPercent";
    
    public static final String XML_RAID_ADJUSTMENTS        = "RaidAdjustments";
    public static final String XML_ADJUSTMENT              = "Adjustment";
    public static final String XML_ADJUSTMENT_ID           = "AdjustmentID";
    public static final String XML_ADJUSTMENT_NAME         = "AdjustmentName";
    public static final String XML_ADJUSTMENT_VALUE        = "AdjustmentValue";
    public static final String XML_ADJUSTMENT_INITIAL      = "AdjustmentInitialValue";
    
    public static final String XML_RAID_TIMING             = "RaidTiming";
    public static final String XML_TIMING_DURATION_POINTS  = "DurationPoints";
    public static final String XML_TIMING_DURATION_MINUTES = "DurationMinutes";
    
    public static final String XML_TIMING_THRESHOLD           = "Threshold";
    public static final String XML_TIMING_THRESHOLD_VALUE     = "ThresholdValue";
    public static final String XML_TIMING_THRESHOLD_ISPERCENT = "ThresholdIsPercent";
    public static final String XML_TIMING_THRESHOLD_POINTS    = "ThresholdPoints";
    public static final String XML_TIMING_THRESHOLD_MINUTES   = "ThresholdMinutes";
    
    public static final String XML_TIMING_ROLL_CALL        = "RollCall";
    public static final String XML_TIMING_ROLL_CALL_NAME   = "RollCallName";
    public static final String XML_TIMING_ROLL_CALL_POINTS = "RollCallPoints";
    public static final String XML_TIMING_ROLL_CALL_RECUR  = "RollCallRecurrence";
    
    public static final String XML_RAID_CARRYOVERS         = "RaidCarryovers";
    public static final String XML_CARRYOVER               = "Carryover";
    public static final String XML_CARRYOVER_PLAYER_NAME   = "CarryoverPlayer";
    public static final String XML_CARRYOVER_PLAYER_CLASS  = "CarryoverPlayerClass";
    public static final String XML_CARRYOVER_POINTS        = "CarryoverPoints";
    
    public static final String XML_RAID_CUSTOM_COSTS       = "CustomFormulaItems";
    public static final String XML_CUSTOM_COST             = "CustomItem";
    public static final String XML_CUSTOM_COST_TEXT        = "CustomText";
    public static final String XML_CUSTOM_COST_INDEX       = "CustomIndex";
    public static final String XML_CUSTOM_COST_COLOR       = "CustomColor";
    public static final String XML_CUSTOM_COST_LOOT_TIME   = "CustomLootTime";
    public static final String XML_CUSTOM_COST_COST        = "CustomCost";
    public static final String XML_CUSTOM_COST_ISPERCENT   = "CustomCostIsPercent";
    
    public static final String XML_LOOT_IS_DIRECT_EDIT     = "LootIsDirectEdit";
    public static final String XML_LOOT_IS_AUCTIONED_LOOT  = "LootIsAuctioned";
    
    public static final String XML_EXCEPTION_DIST          = "DistributionExceptions";
    public static final String XML_EXCEPTION_DIST_ID       = "DistributionExceptionID";
    public static final String XML_EXCEPTION_BOSS          = "BossKillExceptions";
    public static final String XML_EXCEPTION_BOSS_ID       = "BossKillExceptionID";
    public static final String XML_EXCEPTION_TIME          = "TimeExceptions";
    public static final String XML_EXCEPTION_TIME_ID       = "TimeExceptionID";
    
    public static final String XML_CONSTRAINT_EXCLUDED_LOOTDEF      = "ExcludedLootIDs";
    public static final String XML_EXCLUDED_LOOTDEF_ID              = "ExcludedID";
    
    public static final String XML_CONSTRAINT_TIMING         = "RaidTiming";
    public static final String XML_CONSTRAINT_ROLL_CALL      = "RollCall";
    public static final String XML_CONSTRAINT_ROLL_CALL_NAME = "RollCallName";
    public static final String XML_CONSTRAINT_ROLL_CALL_DATE = "RollCallDate";
    
    public static final String IS_YES   = "yes";
    public static final String IS_NO    = "no";
    public static final String IS_FALSE = "false";

    
    public static boolean isValidString(String str) {
        if (str.length() == 0) {
            return true;
        }
        
        if (
                str.indexOf('&')  >= 0 || 
                str.indexOf('<')  >= 0 ||
                str.indexOf('>')  >= 0 ||
                str.indexOf('"')  >= 0 ||
                str.indexOf('\'') >= 0 ) {
            return false;
        }
        
        return true;
    }
    
    public static String header() {
        return XML_HEADER + NEWLINE;
    }
    
    public static String tag(String tag, String value) {
        return "<" + tag + ">" + value + "</" + tag + ">" + NEWLINE;
    }
    
    public static String tagStart(String tag) {
        return "<" + tag + ">" + NEWLINE;
    }

    public static String tagEnd(String tag) {
        return "</" + tag + ">" + NEWLINE;
    }
    
    public static String tagDate(String tag, Date date) {
        return tag(tag, FormatConstants.FORMAT_ID.format(date));
    }
    
    public static String tagDates(String headerTag, String tag, List dates, PlayerData playerData) {
        StringBuffer buf = new StringBuffer();
        buf.append(tagStart(headerTag));
        if (dates != null) {
            for (Iterator iter=dates.iterator(); iter.hasNext();) {
                Date date = (Date) iter.next();
                if (date != null) {
                    buf.append(tagDate(tag, date));
                }
            }
        }
        buf.append(tagEnd(headerTag));
        return buf.toString();
    }
    
    public static void parse(File xmlFile, DefaultHandler handler) {
        try {
            parse(new FileInputStream(xmlFile), handler);
        } catch (FileNotFoundException fnfe) {
            Logger.log(fnfe);
        }
    }
    
    public static void parse(InputStream xmlStream, DefaultHandler handler) {
        Reader reader = null;
        try {
            XMLReader parser = XMLReaderFactory.createXMLReader();
            parser.setContentHandler(handler);
            // To support foreign characters, we read and store as UTF8.
            reader = new BufferedReader(
                    new InputStreamReader(xmlStream, "UTF8"));
            parser.parse(new InputSource(reader));
        } catch (SAXException e) {
            Logger.log(e);
        } catch (IOException e) {
            Logger.log(e);
        } finally {
            try {
                reader.close();
            } catch (IOException e) {
                Logger.log(e);
            }
        }
    }
    
    public static void store(String text, File saveFile) {
        PrintWriter writer = null;
        try {
            // To support foreign characters, we read and store as UTF8.
            writer = new PrintWriter(new BufferedWriter(
                    new OutputStreamWriter(new FileOutputStream(saveFile), "UTF8")));
            writer.println(text);
        } catch (IOException ioe) {
            Logger.log(ioe);
        } finally {
            writer.flush();
            writer.close();
        }
    }
    
    public static float parseFloat(String str) {
        try {
            float number = Float.parseFloat(str);
            return number;
        } catch (Exception e) {
            Logger.log(e);
        }
        return 0;
    }
    
    public static Date parseDate(String str) {
        try {
            Date date = FormatConstants.FORMAT_ID.parse(str);
            return date;
        } catch (Exception e) {
            Logger.log(e);
        }
        return null;
    }
    
    public static int parseInt(String str) {
        try {
            int number = Integer.parseInt(str);
            return number;
        } catch (Exception e) {
            Logger.log(e);
        }
        return 0;
    }
}
