package org.wow.autodkp.view;

import java.io.File;

import org.wow.autodkp.model.Logger;
import org.wow.autodkp.persistence.Persistence;
import org.wow.autodkp.view.skin.AutoDKPLookAndFeelSupport;


/**
 * The main method. Program starts here.
 * 
 * @author jsandusky
 */
public class AutoDKP {
    
    public static final String PROGRAM_NAME    = "AutoDKP";
    public static final String PROGRAM_VERSION = "2.0";
    
    public static final String RT_MIN_VERSION = "1.25.8";
    public static final String VERSION_XX_MIN  = "8";
    
    
    public static void main(String[] args) {
        boolean isLoggingToFile = true;
        if (args.length > 0 && args[0].equals("nolog")) {
            isLoggingToFile = false;
        }
        Logger.initialize(isLoggingToFile);

        AutoDKPLookAndFeelSupport.installSkin();

        GeneralProperties props = GeneralProperties.getDefault();
        String raidProp = props.getStringProperty(props.RAID_TRACKER_FILE);
        if (
                (raidProp == null) || 
                (!raidProp.equals(Persistence.IGNORE_RT_FILE) && !(new File(raidProp).exists()))) {
            CTRaidTrackerLocator locator = new CTRaidTrackerLocator(
                    getListener(), "First Time Installation");
            String autoGuessPath = autoGuessRTLocation();
            if (autoGuessPath != null) {
                // We automatically found the RT file path.
                // Show the locator with the pre-filled in field.
                // Remember we still need to do version checking and whatnot.
                locator.setAutoGuessPath(autoGuessPath);
            }
            locator.setVisible(true);
            return;
        }
        AutoDKPView.unloadReload();
    }
    
    private static String autoGuessRTLocation() {
        // Attempt to quickly auto-guess where the raid tracker file is.
        if (System.getProperties().getProperty("os.name").toLowerCase().indexOf("windows") >= 0) {
            File testAccountFile = new File(
                "C:" + File.separator + 
                "Program Files" + File.separator + 
                "World of Warcraft" + File.separator + 
                "WTF" + File.separator + 
                "Account");
            if (testAccountFile.exists()) {
                // Proceed if there is only one account.
                String accountPath = null;
                int accountCount = 0;
                File[] accounts = testAccountFile.listFiles();
                for (int i=0; i < accounts.length; i++) {
                    if (accounts[i].isDirectory()) {
                        accountCount++;
                        accountPath = accounts[i].getName();
                    }
                }
                if (accountCount == 1) {
                    File testRTFile = new File(
                            testAccountFile.getPath() + File.separator + 
                            accountPath + File.separator + 
                            "SavedVariables" + File.separator + 
                            Persistence.RT_FILE);
                    if (testRTFile.exists()) {
                        // Found it! Default to this path.
                        return testRTFile.getPath();
                    }
                }
            }
        }
        
        return null;
    }
    
    private static IChangeCommitListener getListener() {
        return new IChangeCommitListener() {
            public void changesCancelled() {
            }
            public void changesCommitted() {
                AutoDKPView.unloadReload();
            }
        };
    }
}
