package org.wow.autodkp.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;


/**
 * Generic window helper class.
 * 
 * @author jsandusky
 */
public abstract class BannerWindow {

    protected IChangeCommitListener mListener;
    protected Window mWindow;
    protected Container mContentPane;
    protected JRootPane mRootPane;
    private JButton mOKButton;

    
    // Frame
    public BannerWindow(String title) {
        JFrame frame = new JFrame();
        mWindow = frame;
        mRootPane = frame.getRootPane();
        
        Image windowIcon = ImageUtilities.loadJarImage("castle.gif");
        frame.setIconImage(windowIcon);
        
        frame.setTitle(title);
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        mContentPane = frame.getContentPane();
        
        initUI();
    }
    
    
    // Dialog
    public BannerWindow(JFrame owner, String title) {
        JDialog dialog = new JDialog(owner);
        mWindow = dialog;
        mRootPane = dialog.getRootPane();
        
        dialog.setTitle(title);
        dialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        mContentPane = dialog.getContentPane();
        
        initUI();
    }
    
    
    public boolean isVisible() {
        return mWindow.isVisible();
    }

    public void setVisible(boolean isVisible) {
        mWindow.setVisible(isVisible);
    }
    
    public int getX() {
        return mWindow.getX();
    }
    
    public int getY() {
        return mWindow.getY();
    }
    
    public int getWidth() {
        return mWindow.getWidth();
    }
    
    public int getHeight() {
        return mWindow.getHeight();
    }
    
    public Dimension getSize() {
        return mWindow.getSize();
    }

    public void setSize(int width, int height) {
        mWindow.setSize(width, height);
    }
    
    public void setLocation(int x, int y) {
        mWindow.setLocation(x, y);
    }
    
    public void toFront() {
        mWindow.toFront();
    }
    
    public void setBackground(Color color) {
        mContentPane.setBackground(color);
    }
    
    public void setResizable(boolean isResizable) {
        if (mWindow instanceof JFrame) {
            ((JFrame) mWindow).setResizable(isResizable);
        } else if (mWindow instanceof JDialog) {
            ((JDialog) mWindow).setResizable(isResizable);
        }
    }
    
    private void initUI() {
        JPanel buttonPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttonPane.setBorder(BorderFactory.createEmptyBorder(10, 5, 10, 5));
        JButton cancelButton = new JButton("Cancel");
        mOKButton = new JButton(getOKButtonText());
        mOKButton.setEnabled(isOKInitiallyEnabled());
        
        mOKButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (applyChanges()) {
                    mWindow.dispose();
                    if (getChangeCommitListener() != null) {
                        getChangeCommitListener().changesCommitted();
                    }
                    closing();
                }
            }
        });
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mWindow.dispose();
                if (getChangeCommitListener() != null) {
                    getChangeCommitListener().changesCancelled();
                }
                closing();
            }
        });
        
        buttonPane.add(cancelButton);
        buttonPane.add(mOKButton);
        
        JComponent bannerCustomizer = getBannerCustomizer();
        if (bannerCustomizer != null) {
            mContentPane.add(bannerCustomizer, BorderLayout.NORTH);
        }
        mContentPane.add(getComponent(), BorderLayout.CENTER);
        mContentPane.add(buttonPane,     BorderLayout.SOUTH);
        
        mWindow.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                super.windowClosing(e);
                if (getChangeCommitListener() != null) {
                    getChangeCommitListener().changesCancelled();
                }
                closing();
            }
        });
        
        mRootPane.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
                KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
                "escape");        
        mRootPane.getActionMap().put("escape",
            new AbstractAction() {
                public void actionPerformed(ActionEvent e) {
                    mWindow.dispose();
                    if (getChangeCommitListener() != null) {
                        getChangeCommitListener().changesCancelled();
                    }
                    closing();
                }
            });
    }
    
    protected void closing() {}
    
    protected abstract JComponent getComponent();
    
    protected abstract boolean applyChanges();
    
    protected boolean isOKInitiallyEnabled() {
        return true;
    }
    
    protected void setOKEnabled(boolean isEnabled) {
        mOKButton.setEnabled(isEnabled);
    }
    
    protected String getOKButtonText() {
        return "Save";
    }
    
    private IChangeCommitListener getChangeCommitListener() {
        return mListener;
    }
    
    public void setChangeCommitListener(IChangeCommitListener listener) {
        mListener = listener;
    }
    
    protected JComponent getBannerCustomizer() {
        return null;
    }
}
