package org.wow.autodkp.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.StringTokenizer;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.wow.autodkp.model.graph.LuaGraphGenerator;
import org.wow.autodkp.model.graph.LuaGraphItem;
import org.wow.autodkp.persistence.Persistence;


/**
 * Allows the user to specify where his/her CT Raid Tracker
 * saved variables file is. Performs verification upon this file.
 * 
 * @author jsandusky
 */
public class CTRaidTrackerLocator extends BannerWindow {
    
    private Box mFileFieldPane;
    private JTextField mFileField;
    private JFileChooser mFileChooser;
    private JLabel mStatusLabel;
    private JLabel mHeaderLabel;
    
    
    public CTRaidTrackerLocator(IChangeCommitListener listener, String title) {
        super(title);
        mListener = listener;
        
        setBackground(Color.white);
        
        setSize(650, 400);
        
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension windowSize = getSize();
        setLocation(
                (screenSize.width  - windowSize.width)  / 2, 
                (screenSize.height - windowSize.height) / 2);
        
        setResizable(false);
    }
    
    public void setAutoGuessPath(String path) {
        setOriginalPath(path);
        mStatusLabel.setForeground(Color.BLUE);
        mStatusLabel.setText(
                "<HTML>" + 
                "<P>AutoDKP automatically found your " + Persistence.RT_FILE + " file and entered it below!<BR>" + 
                "Click the \"Save\" button to proceed.<P>" + 
                "</HTML>");
    }
    
    public void setOriginalPath(String path) {
        if (path.equals(Persistence.IGNORE_RT_FILE)) {
            path = "";
        }
        mFileField.setText(path);
        getFileChooser().setSelectedFile(new File(path));
    }
    
    private JFileChooser getFileChooser() {
        if (mFileChooser == null) {
            mFileChooser = new JFileChooser();
        }
        return mFileChooser;
    }
    
    protected JComponent getComponent() {
        mFileField = new JTextField(40);
        mFileField.setMaximumSize(mFileField.getPreferredSize());
        
        JButton browseButton = new JButton("Browse");
        browseButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mFileField.getText().length() > 0) {
                    File currentFile = new File(mFileField.getText());
                    getFileChooser().setSelectedFile(currentFile);
                }
                getFileChooser().showDialog(mWindow, "Select");
                if (getFileChooser().getSelectedFile() != null) {
                    mFileField.setText(getFileChooser().getSelectedFile().getPath());
                }
            }
        });
        
        mFileFieldPane = Box.createHorizontalBox();
        mFileFieldPane.setBorder(BorderFactory.createEmptyBorder(5, 10, 5, 5));
        mFileFieldPane.add(new JLabel("File:"));
        mFileFieldPane.add(Box.createHorizontalStrut(10)); 
        mFileFieldPane.add(mFileField);
        mFileFieldPane.add(Box.createHorizontalStrut(10)); 
        mFileFieldPane.add(browseButton);
        
        return mFileFieldPane;
    }

    protected JComponent getBannerCustomizer() {
        JPanel banner = new JPanel(new BorderLayout());
        mHeaderLabel = new JLabel();
        mStatusLabel = new JLabel();
        mHeaderLabel.setBorder(BorderFactory.createEmptyBorder(0, 10, 5, 5));
        mHeaderLabel.setFont(new Font("Arial", Font.PLAIN, 12));
        mStatusLabel.setBorder(BorderFactory.createEmptyBorder(0, 10, 5, 5));
        mStatusLabel.setFont(new Font("Arial", Font.BOLD, 12));
        banner.add(mHeaderLabel, BorderLayout.CENTER);
        banner.add(mStatusLabel, BorderLayout.SOUTH);
        
        mHeaderLabel.setText(
                "<HTML>" +
                "<H2>" + Persistence.RT_FILE + "</H2>" +
                "<P>" + AutoDKP.PROGRAM_NAME + " needs to know " +
                "the location of your Raid Tracker saved variables file.<BR>" +
                "It can be found at the following location (starting from your World of Warcraft folder):</P>" + 
                "<PRE><CODE>World of Warcraft/WTF/Account/<I>(AccountName)</I>/SavedVariables/" + Persistence.RT_FILE + "</CODE></PRE>" +
                "</HTML>");
        
        return banner;
    }
    
    protected boolean applyChanges() {
        File newRaidFile = new File(mFileField.getText());
        boolean isDisconnected = false;
        
        if (mFileField.getText().length() == 0) {
            int returnVal = JOptionPane.showConfirmDialog(
                    mWindow, 
                    "No " + Persistence.RT_FILE + " file has been specified!\n" + 
                    "AutoDKP will no longer obtain raid info from Raid Tracker.\n" + 
                    "If you proceed, you can always change this option later.\n" +
                    "Are you sure you want to do this?",
                    "Ignore Raid Tracker",
                    JOptionPane.YES_NO_OPTION,
                    JOptionPane.WARNING_MESSAGE);
            if (returnVal == JOptionPane.NO_OPTION) {
                return false;
            } else {
                isDisconnected = true;
            }
        }
        else {
            if (!newRaidFile.exists()) {
                mStatusLabel.setText(
                    "<HTML>" + 
                    "<P>File does not exist:<BR><I>" + mFileField.getText() + "</I><BR>" + 
                    "Please check the file path and try again.<P>" + 
                    "</HTML>");
                mStatusLabel.setForeground(Color.RED);
                return false;
            }
            
            if (!newRaidFile.getName().equals(Persistence.RT_FILE)) {
                mStatusLabel.setText(
                        "<HTML>" + 
                        "<P>Incorrect selection:<BR><I>" + mFileField.getText() + "</I><BR>" + 
                        "Please locate your <U>" + Persistence.RT_FILE + "</U> file.<P>" + 
                        "</HTML>");
                mStatusLabel.setForeground(Color.RED);
                return false;
            }
            
            String parent1 = newRaidFile.getParentFile().getName();
            if (parent1 != null && parent1.equals("CT_RaidTracker")) {
                String parent2 = newRaidFile.getParentFile().getParentFile().getName();
                if (parent2 != null && parent2.equals("AddOns")) {
                    String parent3 = newRaidFile.getParentFile().getParentFile().getParentFile().getName();
                    if (parent3 != null && parent3.equals("Interface")) {
                        mStatusLabel.setText(
                                "<HTML>" + 
                                "<P>You have selected the <U>AddOns</U> script file of the same name, which is the <U>wrong</U> file.<BR>" +
                                "The correct variables file should be located at:<BR>" + 
                                "<CODE>World of Warcraft / WTF / Account / <I>(AccountName)</I> / SavedVariables / " + Persistence.RT_FILE + "</CODE></P>" +
                                "</HTML>");
                        mStatusLabel.setForeground(Color.RED);
                        return false;
                    }
                }
            }
        }
        
        if (!checkVersion(newRaidFile)) {
            return false;
        }
        
        GeneralProperties props = GeneralProperties.getDefault();
        if (isDisconnected) {
            props.setStringProperty(props.RAID_TRACKER_FILE, Persistence.IGNORE_RT_FILE);
        } else {
            props.setStringProperty(props.RAID_TRACKER_FILE, newRaidFile.getPath());
        }
        props.save();
        
        return true;
    }
    
    private boolean checkVersion(File logFile) {
        File scriptFile = logFile;
        for (int i=0; i < 5; i++) {
            scriptFile = scriptFile.getParentFile();
            if (scriptFile == null) {
                // Only perform the version check if we can find the world of warcraft folder
                return true;
            }
        }
        if (!scriptFile.getName().equals("World of Warcraft")) {
            // Only perform the version check if we can find the world of warcraft folder
            return true;
        }
        scriptFile = new File(
                scriptFile.getPath() + File.separator + 
                "Interface"          + File.separator + 
                "AddOns"             + File.separator + 
                "CT_RaidTracker"     + File.separator +
                Persistence.RT_FILE);
        
        if (scriptFile.exists()) {
            LuaGraphItem versionRoot = null;
            LuaGraphItem graphRoot = new LuaGraphGenerator().generateGraph(scriptFile);
            if (graphRoot != null) {
                LuaGraphItem[] children = graphRoot.getChildren();
                if (children != null && children.length > 0) {
                    LuaGraphItem raidInfo = children[0];
                    if (raidInfo.getType() == LuaGraphItem.VERSION) {
                        versionRoot = raidInfo;
                    }
                }
            }
            
            if (versionRoot != null) {
                if (versionRoot.getValue() instanceof String) {
                    String versionText = (String) versionRoot.getValue();
                    if (versionText.length() > 0) {
                        versionText = versionText.replaceAll("[\"]", "").trim();
                        if (versionText.length() > 0) {
                            versionText = versionText.substring(1);
                            StringTokenizer tokener = new StringTokenizer(versionText);
                            if (tokener.hasMoreTokens()) {
                                versionText = tokener.nextToken();
                                CTRaidTrackerVersionInfo currentVersion = new CTRaidTrackerVersionInfo(versionText);
                                CTRaidTrackerVersionInfo requiredVersion = new CTRaidTrackerVersionInfo(AutoDKP.RT_MIN_VERSION);
                                if (currentVersion.meets(requiredVersion)) {
                                    return true;
                                }
                            }
                        }
                    }
                }
            }
        }
        
        mStatusLabel.setText(
                "<HTML>" + 
                "<P>The Raid Tracker you are using is not the correct CT-modified version.<BR>" +
                "The version number should be 1.25.xx, where xx is " + AutoDKP.VERSION_XX_MIN + " or higher.<BR>" + 
                "Only use a Raid Tracker whose version starts with \"1.25\", i.e. " + AutoDKP.RT_MIN_VERSION + ".</P>" +
                "</HTML>");
        mStatusLabel.setForeground(Color.RED);
        
        return false;
    }
}
