package org.wow.autodkp.view;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

/**
 * Version checking utility for CT Raid Tracker.
 * 
 * @author jsandusky
 */
public class CTRaidTrackerVersionInfo {
    
    private List mVersionParts = new ArrayList();
    
    public CTRaidTrackerVersionInfo(String text) {
        // we should have something like 1.xx.xx, or 1.xx
        StringTokenizer tokener = new StringTokenizer(text, ".");
        while (tokener.hasMoreTokens()) {
            String part = tokener.nextToken();
            try {
                double result = Double.parseDouble(part);
                mVersionParts.add(new Double(result));
            } catch (NumberFormatException nfe) {
                // just add a version number of 0
                mVersionParts.add(new Double(0));
            }
        }
    }
    
    public boolean meets(CTRaidTrackerVersionInfo otherVersion) {
        int index = 0;
        for (Iterator iter=otherVersion.mVersionParts.iterator(); iter.hasNext(); index++) {
            Double otherPart = (Double) iter.next();
            if (this.hasVersionPart(index)) {
                Double part = this.getVersionPart(index);
                if (part.doubleValue() > otherPart.doubleValue()) {
                    return true;
                } else if (part.doubleValue() == otherPart.doubleValue()) {
                    continue;
                } else {
                    return false;
                }
            }
        }
        if (this.getPartCount() > otherVersion.getPartCount()) {
            return true;
        }
        return false;
    }
    
    public boolean hasVersionPart(int index) {
        return index < mVersionParts.size();
    }
    
    public int getPartCount() {
        return mVersionParts.size();
    }
    
    public Double getVersionPart(int index) {
        return (Double) mVersionParts.get(index);
    }
}
