package org.wow.autodkp.view;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.text.NumberFormat;
import java.util.Calendar;
import java.util.Date;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import org.wow.autodkp.model.FormatConstants;


public class DateControls {
    
    private JComponent mComponent;
    
    private JFormattedTextField mMonthField;
    private JFormattedTextField mDayField;
    private JFormattedTextField mYearField;
    private JFormattedTextField mHourField;
    private JFormattedTextField mMinuteField;
    private JComboBox  mTimeCombo = new JComboBox(new String[] { "AM", "PM" });

    
    public DateControls() {
        NumberFormat format = NumberFormat.getIntegerInstance();
        format.setGroupingUsed(false);
        
        mMonthField = new JFormattedTextField(format);
        mDayField = new JFormattedTextField(format);
        mYearField = new JFormattedTextField(format);
        mHourField = new JFormattedTextField(format);
        mMinuteField = new JFormattedTextField(format);
        mTimeCombo.setSelectedIndex(1);
        
        Action focusTabNextAction = new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                Component component = (Component) e.getSource();
                Container container = component.getParent();
                Component nextComponent = 
                    mComponent.getFocusTraversalPolicy().getComponentAfter(container, component);
                if (component instanceof JFormattedTextField) {
                    JFormattedTextField textField = (JFormattedTextField) component;
                    if (textField.getText().trim().length() == 0) {
                        // Clear out the value of an empty field, otherwise formatting
                        // will fail to parse, and the value will revert back. 
                        textField.setValue(null);
                    }
                }
                nextComponent.requestFocusInWindow();
            }
        };
        Action focusTabPreviousAction = new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                Component component = (Component) e.getSource();
                Container container = component.getParent();
                Component nextComponent = 
                    mComponent.getFocusTraversalPolicy().getComponentBefore(container, component);
                if (component instanceof JFormattedTextField) {
                    JFormattedTextField textField = (JFormattedTextField) component;
                    if (textField.getText().trim().length() == 0) {
                        // Clear out the value of an empty field, otherwise formatting
                        // will fail to parse, and the value will revert back. 
                        textField.setValue(null);
                    }
                }
                nextComponent.requestFocusInWindow();
            }
        };
        setFocusActions(mMonthField, focusTabNextAction, focusTabPreviousAction);
        setFocusActions(mDayField, focusTabNextAction, focusTabPreviousAction);
        setFocusActions(mYearField, focusTabNextAction, focusTabPreviousAction);
        setFocusActions(mHourField, focusTabNextAction, focusTabPreviousAction);
        setFocusActions(mMinuteField, focusTabNextAction, focusTabPreviousAction);
        setFocusActions(mTimeCombo, focusTabNextAction, focusTabPreviousAction);
        
        mMonthField.setHorizontalAlignment(JTextField.CENTER);
        mDayField.setHorizontalAlignment(JTextField.CENTER);
        mYearField.setHorizontalAlignment(JTextField.CENTER);
        mHourField.setHorizontalAlignment(JTextField.CENTER);
        mMinuteField.setHorizontalAlignment(JTextField.CENTER);
        
        mMonthField.setColumns(2);
        mDayField.setColumns(2);
        mYearField.setColumns(4);
        mHourField.setColumns(2);
        mMinuteField.setColumns(2);
        
        Dimension twoDigitSize = new Dimension(
                mMonthField.getPreferredSize().width, 
                mMonthField.getPreferredSize().height);
        Dimension fourDigitSize = new Dimension(twoDigitSize.width * 2, twoDigitSize.height);
        mMonthField.setPreferredSize(twoDigitSize);
        mMonthField.setMaximumSize(twoDigitSize);
        mDayField.setPreferredSize(twoDigitSize);
        mDayField.setMaximumSize(twoDigitSize);
        mYearField.setPreferredSize(fourDigitSize);
        mYearField.setMaximumSize(fourDigitSize);
        mHourField.setPreferredSize(twoDigitSize);
        mHourField.setMaximumSize(twoDigitSize);
        mMinuteField.setPreferredSize(twoDigitSize);
        mMinuteField.setMaximumSize(twoDigitSize);
        mTimeCombo.setPreferredSize(fourDigitSize);
        mTimeCombo.setMaximumSize(fourDigitSize);
        
        // Display the date/time files in the order
        // recommended by International Standard ISO 8601.
        // (YYYY-MM-DD)
        Box pane = Box.createHorizontalBox();
        pane.add(mYearField);
        pane.add(new JLabel("-"));
        pane.add(mMonthField);
        pane.add(new JLabel("-"));
        pane.add(mDayField);
        
        pane.add(Box.createHorizontalStrut(10));
        pane.add(mHourField);
        pane.add(new JLabel(":"));
        pane.add(mMinuteField);
        pane.add(mTimeCombo);
        pane.add(Box.createHorizontalGlue());
        
        mComponent = pane;
        mComponent.setFocusCycleRoot(true); 
    }

    
    private void setFocusActions(JComponent component, Action tabNext, Action tabPrevious) {
        component.getActionMap().put("tabNext", tabNext);
        component.getActionMap().put("tabPrevious", tabPrevious);
        component.getInputMap().put(
                KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0), 
                "tabNext");
        component.getInputMap().put(
                KeyStroke.getKeyStroke(KeyEvent.VK_TAB, InputEvent.SHIFT_DOWN_MASK), 
                "tabPrevious");
    }
    
    public JComponent getComponent() {
        return mComponent;
    }
    
    public void initializeDateAndTime(Date date) {
        initializeDate(date);
        if (date != null) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            mHourField.setText(String.valueOf(calendar.get(Calendar.HOUR)));
            mMinuteField.setText(String.valueOf(calendar.get(Calendar.MINUTE)));
            boolean isPM = calendar.get(Calendar.AM_PM) == Calendar.PM;
            if (isPM) {
                mTimeCombo.setSelectedIndex(1);
            } else {
                mTimeCombo.setSelectedIndex(0);
            }
        } else {
            mHourField.setText("");
            mHourField.setValue(null);
            mMinuteField.setText("");
            mMinuteField.setValue(null);
            mTimeCombo.setSelectedIndex(1);
        }
    }
    
    public void initializeDate(Date date) {
        if (date != null) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            mMonthField.setText(String.valueOf(calendar.get(Calendar.MONTH) + 1));
            mDayField.setText(String.valueOf(calendar.get(Calendar.DAY_OF_MONTH)));
            mYearField.setText(String.valueOf(calendar.get(Calendar.YEAR)));
            mHourField.setText("");
            mHourField.setValue(null);
            mMinuteField.setText("");
            mMinuteField.setValue(null);
            mTimeCombo.setSelectedIndex(1);
        } else {
            mMonthField.setText("");
            mMonthField.setValue(null);
            mDayField.setText("");
            mDayField.setValue(null);
            mYearField.setText("");
            mYearField.setValue(null);
        }
    }
    
    public boolean isDateEntered() {
        if (
                mMonthField.getText().length()  > 0 ||
                mDayField.getText().length()    > 0 ||
                mYearField.getText().length()   > 0 ||
                mHourField.getText().length()   > 0 ||
                mMinuteField.getText().length() > 0) {
            return true;
        }
        return false;
    }

    public boolean isTimeEntered() {
        if (
                mHourField.getText().length()   > 0 ||
                mMinuteField.getText().length() > 0) {
            return true;
        }
        return false;
    }
    
    public void setEditable(boolean isEditable) {
        mMonthField.setEditable(isEditable);
        mDayField.setEditable(isEditable);
        mYearField.setEditable(isEditable);
        mHourField.setEditable(isEditable);
        mMinuteField.setEditable(isEditable);
        mTimeCombo.setEnabled(isEditable);
    }
    
    public void setControlForeground(Color color) {
        mMonthField.setForeground(color);
        mDayField.setForeground(color);
        mYearField.setForeground(color);
        mHourField.setForeground(color);
        mMinuteField.setForeground(color);
    }
    
    public Date getDate() {
        try {
            int hour = Integer.parseInt(mHourField.getText());
            if (hour <= 12 && mTimeCombo.getSelectedIndex() == 1) {
                // convert hour to 24-hour format
                hour += 12;
            }
            StringBuffer buf = new StringBuffer();
            buf.append(mMonthField.getText() + "/");
            buf.append(mDayField.getText() + "/");
            buf.append(mYearField.getText() + " ");
            buf.append(hour + ":");
            buf.append(mMinuteField.getText() + ":00");
            return FormatConstants.DATE_PARSE_FORMAT.parse(buf.toString());
        } catch (Exception pe) {
            // this is ok, just allow user to change it later
        }
        return null;
    }
    
    public void addInputFieldCommitListener(ActionListener listener) {
        mMonthField.addActionListener(listener);
        mDayField.addActionListener(listener);
        mYearField.addActionListener(listener);
        mHourField.addActionListener(listener);
        mMinuteField.addActionListener(listener);
    }
}
