package org.wow.autodkp.view;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.Window;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.TimeZone;

import org.wow.autodkp.model.FormatConstants;
import org.wow.autodkp.model.Logger;


/**
 * AutoDKP program properties (i.e. window sizes, etc.).
 * 
 * @author jsandusky
 */
public class GeneralProperties {
    
    public static final String RAID_TRACKER_FILE            = "CTRaidTrackerFile";
    public static final String MAIN_WINDOW_WIDTH            = "MainWindowWidth";
    public static final String MAIN_WINDOW_HEIGHT           = "MainWindowHeight";
    public static final String MAIN_WINDOW_X                = "MainWindowX";
    public static final String MAIN_WINDOW_Y                = "MainWindowY";
    public static final String ITEM_WINDOW_WIDTH            = "ItemWindowWidth";
    public static final String ITEM_WINDOW_HEIGHT           = "ItemWindowHeight";
    public static final String ITEM_WINDOW_X                = "ItemWindowX";
    public static final String ITEM_WINDOW_Y                = "ItemWindowY";
    public static final String MAIN_SPLIT                   = "VerticalSplitLocation";
    public static final String RAID_SPLIT                   = "HorizontalSplitLocation";
    public static final String RAID_COLUMN_WIDTH_SHORT      = "RaidTableColumnWidthShort_";
    public static final String RAID_COLUMN_WIDTH_LONG       = "RaidTableColumnWidthLong_";
    public static final String ITEM_COLUMN_WIDTH            = "ItemTableColumnWidth_";
    public static final String ITEM_FILTER1                 = "ItemFilter1";
    public static final String ITEM_FILTER2                 = "ItemFilter2";
    public static final String FORMULA_ITEM_FILTER1         = "FormulaItemFilter1";
    public static final String FORMULA_ITEM_FILTER2         = "FormulaItemFilter2";
    public static final String FORMULA_ITEM_SHOWING         = "FormulaItemChooserShowing";
    public static final String RAID_LOOT_FILTER             = "RaidLootFilter";
    public static final String RAID_COLUMN_SORTING          = "RaidColumnSorting";
    public static final String CUSTOMIZER_WINDOW_X          = "CustomizerX";
    public static final String CUSTOMIZER_WINDOW_Y          = "CustomizerY";
    public static final String CUSTOMIZER_WINDOW_WIDTH      = "CustomizerWidth";
    public static final String CUSTOMIZER_WINDOW_HEIGHT     = "CustomizerHeight";
    public static final String EXPORTER_WINDOW_X            = "ExporterX";
    public static final String EXPORTER_WINDOW_Y            = "ExporterY";
    public static final String EXPORTER_WINDOW_WIDTH        = "ExporterWidth";
    public static final String EXPORTER_WINDOW_HEIGHT       = "ExporterHeight";
    public static final String EXPORT_LOCATION              = "ExportLocation";
    public static final String SHOW_NON_RAID_MEMBERS        = "ShowCarryovers";
    public static final String SHOW_LOOT_TIMES              = "ShowLootTimes";
    public static final String HTML_WINDOW_WIDTH            = "HTMLWindowWidth";
    public static final String HTML_WINDOW_HEIGHT           = "HTMLWindowHeight";
    public static final String HTML_WINDOW_X                = "HTMLWindowX";
    public static final String HTML_WINDOW_Y                = "HTMLWindowY";
    public static final String HTML_INCLUDE_RAID_DATE       = "HTMLIncludeRaidDate";
    public static final String CSS_WINDOW_WIDTH             = "CSSWindowWidth";
    public static final String CSS_WINDOW_HEIGHT            = "CSSWindowHeight";
    public static final String CSS_WINDOW_X                 = "CSSWindowX";
    public static final String CSS_WINDOW_Y                 = "CSSWindowY";
    public static final String DATE_GUIDE_WINDOW_WIDTH      = "DateGuideWindowWidth";
    public static final String DATE_GUIDE_WINDOW_HEIGHT     = "DateGuideWindowHeight";
    public static final String DATE_GUIDE_WINDOW_X          = "DateGuideWindowX";
    public static final String DATE_GUIDE_WINDOW_Y          = "DateGuideWindowY";
    public static final String FORMULA_EDITOR_WINDOW_X      = "FormulaEditorX";
    public static final String FORMULA_EDITOR_WINDOW_Y      = "FormulaEditorY";
    public static final String FORMULA_EDITOR_WINDOW_WIDTH  = "FormulaEditorWidth";
    public static final String FORMULA_EDITOR_WINDOW_HEIGHT = "FormulaEditorHeight";
    public static final String PREFS_WINDOW_X               = "PreferencesWindowX";
    public static final String PREFS_WINDOW_Y               = "PreferencesWindowY";
    public static final String PREFS_WINDOW_WIDTH           = "PreferencesWindowWidth";
    public static final String PREFS_WINDOW_HEIGHT          = "PreferencesWindowHeight";
    public static final String PREFS_SELECTED_TAB_MAIN      = "PreferencesSelectedTabMain";
    public static final String PREFS_SELECTED_TAB_POINTS    = "PreferencesSelectedTabPointSystem";
    public static final String PREFS_SELECTED_POINTS        = "PreferencesSelectedPointSystem";
    public static final String HTML_VIEWER_MODE             = "HTMLViewerMode";
    
    public static final String COLUMN_TRANSLATION_NAME      = "ColumnTranslationName";
    public static final String COLUMN_TRANSLATION_CLASS     = "ColumnTranslationClass";
    public static final String COLUMN_TRANSLATION_JOINS     = "ColumnTranslationJoins";
    public static final String COLUMN_TRANSLATION_LEAVES    = "ColumnTranslationLeaves";
    public static final String COLUMN_TRANSLATION_LOOT      = "ColumnTranslationLoot";
    public static final String COLUMN_TRANSLATION_PTS       = "ColumnTranslationPts";
    public static final String COLUMN_TRANSLATION_FORMULA   = "ColumnTranslationFormula";
    public static final String COLUMN_TRANSLATION_TOTAL     = "ColumnTranslationTotal";
    
    public static final String DATE_FORMAT_SHORT            = "FormatDateShort";
    public static final String DATE_FORMAT_LONG             = "FormatDateLong";
    public static final String DATE_TIME_ZONE               = "FormatTimeZone";
    
    public static final String SKINNING_ENABLED             = "IsSkinningEnabled";
    public static final String SKIN_THEME_MAIN              = "SkinThemeMain";
    public static final String SKIN_THEME_SECONDARY         = "SkinThemeSecondary";
    public static final String SKIN_WATERMARK               = "SkinWatermark";
    
    public static final String SKIN_THEME_DEFAULT_MAIN      = "Aqua";
    public static final String SKIN_THEME_DEFAULT_SECONDARY = "Orchid Alloy";
    public static final String SKIN_WATERMARK_DEFAULT       = "Stripes";
    
    public static final int DEFAULT_MAIN_WINDOW_WIDTH       = 1000;
    public static final int DEFAULT_MAIN_WINDOW_HEIGHT      =  700;
    public static final int DEFAULT_ITEM_WINDOW_WIDTH       =  700;
    public static final int DEFAULT_ITEM_WINDOW_HEIGHT      =  700;
    public static final int DEFAULT_MAIN_SPLIT              =  250;
    public static final int DEFAULT_RAID_SPLIT              =  300;
    public static final int DEFAULT_FORMULA_EDITOR_WIDTH    =  400;
    public static final int DEFAULT_FORMULA_EDITOR_HEIGHT   =  500;
    public static final int DEFAULT_CUSTOMIZER_WIDTH        =  600;
    public static final int DEFAULT_CUSTOMIZER_HEIGHT       =  600;
    public static final int DEFAULT_PREFERENCES_WIDTH       =  660;
    public static final int DEFAULT_PREFERENCES_HEIGHT      =  600;
    
    
    private static final String SAVE_FILE = "general.properties";
    private static final GeneralProperties mInstance = new GeneralProperties();
    
    // Default String Property values
    private static final Map DEFAULT_DATE_PATTERNS = new HashMap();
    static {
        DEFAULT_DATE_PATTERNS.put(DATE_FORMAT_SHORT, "EE hh:mma");
        DEFAULT_DATE_PATTERNS.put(DATE_FORMAT_LONG, "EE MM/dd/yy hh:mm:ss a");
    }
    
    private Map mDateFormatCache = new HashMap();
    private Properties mProperties;
    private Set mSaveOnExitProperties = new HashSet();
    
    
    private GeneralProperties() {
        mProperties = new Properties();
        load();
        
        // clean up deprecated properties
        // 05/29/06 - renamed raid tracker log file location property
        //            in order to force people to have their log files double checked
        if (mProperties.containsKey("RaidTrackerFile")) {
            mProperties.remove("RaidTrackerFile");
        }
    }
    
    
    public static GeneralProperties getDefault() {
        return mInstance;
    }
    
    private void load() {
        File propertiesFile = new File(SAVE_FILE);
        if (propertiesFile.exists()) {
            InputStream inputStream = null;
            try {
                inputStream = new BufferedInputStream(new FileInputStream(propertiesFile));
                mProperties.load(inputStream);
            } catch (IOException ioe) {
                Logger.log(ioe);
            } finally {
                try {
                    inputStream.close();
                } catch (IOException ioe) {
                    Logger.log(ioe);
                }
            }
        }
    }
    
    public void removeProperty(String key) {
        mProperties.remove(key);
    }
    
    public SimpleDateFormat getDateFormatProperty(String key) {
        SimpleDateFormat format = (SimpleDateFormat) mDateFormatCache.get(key);
        if (format == null) {
            // Cache the date format.
            String pattern = getStringProperty(key);
            if (pattern == null || pattern.length() == 0) {
                // Load the default.
                pattern = (String) DEFAULT_DATE_PATTERNS.get(key);
            }
            format = getTimeZoneDateFormat(pattern);
            mDateFormatCache.put(key, format);
        }
        return format;
    }
    
    public void setDateFormatProperty(String key, String pattern) {
        mDateFormatCache.put(key, getTimeZoneDateFormat(pattern));
        setStringProperty(key, pattern);
    }
    
    public void resetDateFormats() {
        mDateFormatCache.clear();
    }
    
    private SimpleDateFormat getTimeZoneDateFormat(String pattern) {
        String timeZoneID = getStringProperty(DATE_TIME_ZONE);
        if (timeZoneID != null) {
            TimeZone timeZone = TimeZone.getTimeZone(timeZoneID);
            SimpleDateFormat originalFormat   = 
                new SimpleDateFormat(pattern);
            originalFormat.setTimeZone(TimeZone.getTimeZone(FormatConstants.TIME_ZONE_SERVER));
            SimpleDateFormat translatedFormat = 
                new SimpleDateFormat(originalFormat.toPattern());
            translatedFormat.setTimeZone(timeZone);
            return translatedFormat;
        }
        return new SimpleDateFormat(pattern);
    }
    
    public void setStringProperty(String key, String value) {
        mProperties.setProperty(key, value);
    }
    
    public String getStringProperty(String key) {
        return mProperties.getProperty(key);
    }
    
    public void setBooleanProperty(String key, boolean value) {
        mProperties.setProperty(key, String.valueOf(value));
    }
    
    public void setBooleanProperty(String key, Boolean value) {
        mProperties.setProperty(key, value.toString());
    }
    
    public boolean hasProperty(String key) {
        return mProperties.getProperty(key) != null;
    }
    
    public boolean getBooleanProperty(String key) {
        Boolean value = null;
        String property = mProperties.getProperty(key);
        if (property != null) {
            value = new Boolean(property);
            return value.booleanValue();
        }
        return false;
    }

    public void setIntegerProperty(String key, int value) {
        mProperties.setProperty(key, String.valueOf(value));
    }
    
    public void setIntegerProperty(String key, Integer value) {
        mProperties.setProperty(key, value.toString());
    }
    
    public Integer getIntegerProperty(String key) {
        Integer value   = null;
        String property = mProperties.getProperty(key);
        if (property != null) {
            try {
                value = new Integer(property);
            } catch (NumberFormatException nfe) {
                // ignore, user may have removed properties file,
                // or properties file may not yet exist
            }
        }
        return value;
    }
    
    public Set getSaveOnExitProperties() {
        return mSaveOnExitProperties;
    }
    
    public void addSaveOnExitProperty(SaveOnExitProperty property) {
        mSaveOnExitProperties.add(property);
    }
    
    public void save() {
        File propertiesFile = new File(SAVE_FILE);
        OutputStream outputStream = null;
        try {
            outputStream = new BufferedOutputStream(new FileOutputStream(propertiesFile));
            mProperties.store(outputStream, "General Properties");
        } catch (IOException ioe) {
            Logger.log(ioe);
        } finally {
            try {
                outputStream.flush();
                outputStream.close();
            } catch (IOException ioe) {
                Logger.log(ioe);
            }
        }
    }
    
    public void loadWindowRect(final Window win, String px, String py, String pw, String ph, int dw, int dh) {
        Integer x      = getIntegerProperty(px);
        Integer y      = getIntegerProperty(py);
        Integer width  = getIntegerProperty(pw);
        Integer height = getIntegerProperty(ph);
        if (width == null || height == null) {
            win.setSize(dw, dh);
        } else {
            win.setSize(width.intValue(), height.intValue());
        }
        if (x == null || y == null) {
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            Dimension windowSize = win.getSize();
            win.setLocation(
                    (screenSize.width  - windowSize.width)  / 2, 
                    (screenSize.height - windowSize.height) / 2);
        } else {
            win.setLocation(x.intValue(), y.intValue());
            
            // Fix bug where if the window is too large, it can no longer
            // be resized. This will limit it to the viewable area.
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            Rectangle screenRect = new Rectangle(screenSize);
            if (!screenRect.contains(win.getBounds())) {
                win.setBounds(screenRect.intersection(win.getBounds()));
            }
        }
        
        addSaveOnExitProperty(new SaveOnExitProperty(px) {
            public Object getValue() { return new Integer(win.getX()); }
        });
        addSaveOnExitProperty(new SaveOnExitProperty(py) {
            public Object getValue() { return new Integer(win.getY()); }
        });
        addSaveOnExitProperty(new SaveOnExitProperty(pw) {
            public Object getValue() { return new Integer(win.getWidth()); }
        });
        addSaveOnExitProperty(new SaveOnExitProperty(ph) {
            public Object getValue() { return new Integer(win.getHeight()); }
        });
    }
}
