package org.wow.autodkp.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.WindowConstants;

import org.wow.autodkp.persistence.FileIOUtilities;


/**
 * Dialog to show program help info.
 * 
 * @author jsandusky
 */
public class HelpDialog extends BasicDialog {
    
    private String mHelpPath;
    private String mXKey;
    private String mYKey;
    private String mWKey;
    private String mHKey;
    
    
    public HelpDialog(
            JFrame owner,
            String title, 
            String helpPath, 
            String xPropertyKey,
            String yPropertyKey,
            String widthPropertyKey,
            String heightPropertyKey,
            boolean isModal)
    {
        super(owner, title, isModal);
        initialize(helpPath, xPropertyKey, yPropertyKey, widthPropertyKey, heightPropertyKey);
    }
    
    public HelpDialog(
            JDialog owner,
            String title, 
            String helpPath, 
            String xPropertyKey,
            String yPropertyKey,
            String widthPropertyKey,
            String heightPropertyKey,
            boolean isModal)
    {
        super(owner, title, isModal);
        initialize(helpPath, xPropertyKey, yPropertyKey, widthPropertyKey, heightPropertyKey);
    }
    
    
    private void initialize(
            String helpPath, 
            String xPropertyKey,
            String yPropertyKey,
            String widthPropertyKey,
            String heightPropertyKey)
    {
        addBasicSupport();
        
        mHelpPath = helpPath;
        
        mXKey = xPropertyKey;
        mYKey = yPropertyKey;
        mWKey = widthPropertyKey;
        mHKey = heightPropertyKey;
        
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
        setLayout(new BorderLayout());
        getContentPane().add(getMainPane(),   BorderLayout.CENTER);
        getContentPane().add(getButtonPane(), BorderLayout.SOUTH);

        GeneralProperties props = GeneralProperties.getDefault();
        Integer width  = null;
        Integer height = null;
        Integer x      = null;
        Integer y      = null;
        
        if (xPropertyKey != null) {
            x      = props.getIntegerProperty(props.DATE_GUIDE_WINDOW_X);
            y      = props.getIntegerProperty(props.DATE_GUIDE_WINDOW_Y);
            width  = props.getIntegerProperty(props.DATE_GUIDE_WINDOW_WIDTH);
            height = props.getIntegerProperty(props.DATE_GUIDE_WINDOW_HEIGHT);
            props.addSaveOnExitProperty(new SaveOnExitProperty(xPropertyKey) {
                public Object getValue() { return new Integer(getX()); }
            });
            props.addSaveOnExitProperty(new SaveOnExitProperty(yPropertyKey) {
                public Object getValue() { return new Integer(getY()); }
            });
            props.addSaveOnExitProperty(new SaveOnExitProperty(widthPropertyKey) {
                public Object getValue() { return new Integer(getWidth()); }
            });
            props.addSaveOnExitProperty(new SaveOnExitProperty(heightPropertyKey) {
                public Object getValue() { return new Integer(getHeight()); }
            });
        }
        
        if (width == null || height == null) {
            setSize(props.DEFAULT_CUSTOMIZER_WIDTH, props.DEFAULT_CUSTOMIZER_WIDTH);
        } else {
            setSize(width.intValue(), height.intValue());
        }
        if (x == null || y == null) {
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            Dimension windowSize = getSize();
            setLocation(
                    (screenSize.width  - windowSize.width)  / 2, 
                    (screenSize.height - windowSize.height) / 2);
        } else {
            setLocation(x.intValue(), y.intValue());
        }
    }
    
    protected void closing() {
        GeneralProperties props = GeneralProperties.getDefault();
        if (mXKey != null) {
            props.setIntegerProperty(mXKey, getX());
            props.setIntegerProperty(mYKey, getY());
            props.setIntegerProperty(mWKey, getWidth());
            props.setIntegerProperty(mHKey, getHeight());
        }
    }
    
    protected JComponent getMainPane() {
        JEditorPane aboutShower = new JEditorPane("text/html", loadContent());
        aboutShower.setEditable(false);
        JScrollPane scroller = new JScrollPane(aboutShower);
        aboutShower.setCaretPosition(0);
        
        JPanel mainPane = new JPanel(new BorderLayout());
        mainPane.add(scroller, BorderLayout.CENTER);
        mainPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        return mainPane;
    }
    
    protected JComponent getButtonPane() {
        JButton applyButton = new JButton("Close");
        applyButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closing();
                dispose();
            }
        });
        
        JPanel buttonPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttonPane.add(applyButton);
        buttonPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        return buttonPane;
    }
    
    protected String loadContent() {
        File readme = new File(mHelpPath);
        if (readme.exists()) {
            String helpString = FileIOUtilities.readFileToString(readme);
            if (helpString.length() > 0) {
                return helpString;
            }
        }
        return 
            "<HTML>" +
            "<DIV ALIGN=CENTER>" +
            "<H2>" + AutoDKP.PROGRAM_NAME + "</H2>" +
            "<P>version " + AutoDKP.PROGRAM_VERSION + "</P>" +
            "<P>by Josh Sandusky</P><BR>" +
            "</DIV>" +
            "</HTML>"; 
    }
}
