package org.wow.autodkp.view;

import java.awt.Component;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.io.File;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.wow.autodkp.model.FormatConstants;
import org.wow.autodkp.model.Logger;
import org.wow.autodkp.persistence.Persistence;


/**
 * Utility class.
 * 
 * @author jsandusky
 */
public class ImageUtilities {
    
    private static final String ICON_DIR = 
        Persistence.DEFINITIONS_DIR + 
        File.separator + 
        Persistence.ICONS_DIR;

    private static Map mInstanceIconMap;
    
    
    public static Image loadJarImage(String imagePath) {
        URL imageURL = ImageUtilities.class.getResource(imagePath);
        Image image = Toolkit.getDefaultToolkit().getImage(imageURL);
        return image;
    }
    
    public static Image loadJarImage(Class clazz, String imagePath) {
        URL imageURL = clazz.getResource(imagePath);
        Image image = Toolkit.getDefaultToolkit().getImage(imageURL);
        return image;
    }
    
    public static Image loadImageFile(Component component, String imagePath) {
        MediaTracker fMediaTracker = new MediaTracker(component);
        Image image = Toolkit.getDefaultToolkit().getImage(imagePath);

        fMediaTracker.addImage(image, 0);
        try {
            fMediaTracker.waitForID(0);
        } catch (InterruptedException ie) {
            Logger.log(ie);
        }
        
        return image;
    }
    
    public static Image getInstanceIcon(Component component, String instanceName) {
        if (mInstanceIconMap == null) {
            mInstanceIconMap = new HashMap();
        }
        Image image = (Image) mInstanceIconMap.get(instanceName);
        if (image == null) {
            String iconName = FormatConstants.generateID(instanceName).toLowerCase();
            File iconFile = new File(ICON_DIR + File.separator + iconName + ".jpg");
            if (iconFile.exists()) {
                image = ImageUtilities.loadImageFile(component, iconFile.getPath());
                mInstanceIconMap.put(instanceName, image);
            }
        }
        return image;
    }
}
