package org.wow.autodkp.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.GrayFilter;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.DefaultPointSystemPlayerData;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.IPointSystemPlayerData;
import org.wow.autodkp.view.filters.ItemFilter;
import org.wow.autodkp.view.filters.QualityItemFilter;
import org.wow.autodkp.view.preferences.AlternateItem;
import org.wow.autodkp.view.preferences.DialogPlayerNameAndClass;
import org.wow.autodkp.view.summary.PointSummaryPane;
import org.wow.autodkp.view.summary.SummaryPane;
import org.wow.autodkp.view.table.customizers.bosskill.BossKillsEditor;
import org.wow.autodkp.view.table.customizers.time.TimeEditor;
import org.wow.autodkp.view.table.customizers.time.TimeTableModel;
import org.wow.autodkp.view.table.customizers.zerosum.DistributionEditor;
import org.wow.autodkp.view.table.customizers.zerosum.DistributionsTableModel;
import org.wow.autodkp.view.table.players.PlayerTable;


/**
 * The raid and point system list panels.
 * 
 * @author jsandusky
 */
public class MainSelector {
    
    private JSplitPane mComponent;
    private JPanel mTableControl;
    private JList mRaidList;
    private JList mPointSystemList;
    
    private AbstractButton mAddNonRaidMemberButton;
    private AbstractButton mLootTimesButton;
    
    private JButton mDistributionsExceptionsButton;
    private JButton mBossKillExceptionsButton;
    private JButton mTimeExceptionsButton;
    
    private PlayerTable mPlayerTable;
    private Raid mCurrentRaid;
    private IPointSystem mCurrentPointSystem;
    private ListSelectionListener mRaidSelectionListener;
    private ListSelectionListener mPointSystemSelectionListener;
    private SummaryPane mSummaryPane;
    private RaidCellRenderer mRaidCellRenderer;
    
    private JComboBox mLootFilter;
    private static ItemFilter[] FILTERS1;
    
    
    static {
        List filters = new ArrayList();
        for (int i=1; i < LootConstants.QUALITY_KEYS.length; i++) {
            int quality = i;
            filters.add(new QualityItemFilter(quality));
        }
        FILTERS1 = (ItemFilter[]) filters.toArray(new ItemFilter[0]);
    }
    
    private static Font FONT_CONTROL = new Font("Arial", Font.PLAIN, 11);
    
    
    public MainSelector(PlayerTable playerTable, 
                        SummaryPane constraintsPane) {
        mPlayerTable = playerTable;
        mSummaryPane = constraintsPane;
        
        mRaidSelectionListener = new RaidSelectionListener();
        mRaidCellRenderer = new RaidCellRenderer();
        
        mRaidList = new JList(AutoDKPModel.getInstance().getRaidItems().toArray());
        mRaidList.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 0));
        mRaidList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        mRaidList.addListSelectionListener(mRaidSelectionListener);
        mRaidList.setCellRenderer(mRaidCellRenderer);
        JScrollPane raidScroller = new JScrollPane(mRaidList);
        raidScroller.setBorder(BorderFactory.createTitledBorder("Raids"));
        raidScroller.setBackground(Color.WHITE);
        mRaidList.setBackground(Color.WHITE);
        
        mPointSystemSelectionListener = new PointSystemSelectionListener();
        mPointSystemList = new JList(AutoDKPModel.getInstance().getConstraints());
        mPointSystemList.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 0));
        mPointSystemList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        mPointSystemList.addListSelectionListener(mPointSystemSelectionListener);
        mPointSystemList.setCellRenderer(new PointSystemCellRenderer());
        JScrollPane pointSystemScroller = new JScrollPane(mPointSystemList);
        pointSystemScroller.setBorder(BorderFactory.createTitledBorder("Point Systems"));
        pointSystemScroller.setBackground(Color.WHITE);
        mPointSystemList.setBackground(Color.WHITE);
        
        mComponent = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        mComponent.setBackground(Color.WHITE);
        mComponent.setLeftComponent(raidScroller);
        mComponent.setRightComponent(pointSystemScroller);
        
        mAddNonRaidMemberButton = new JCheckBox("All Players?");
        mAddNonRaidMemberButton.setToolTipText("Show Non-Raid Members?");
        mAddNonRaidMemberButton.setOpaque(false);
        mAddNonRaidMemberButton.setFont(FONT_CONTROL);
        mAddNonRaidMemberButton.setBorder(BorderFactory.createEmptyBorder(3, 5, 2, 0));
        mAddNonRaidMemberButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                AutoDKPView.getInstance().reloadRaids();
            }
        });
        
        mLootTimesButton = new JCheckBox("Loot Times?");
        mLootTimesButton.setToolTipText("Show Loot Times with Loot?");
        mLootTimesButton.setOpaque(false);
        mLootTimesButton.setFont(FONT_CONTROL);
        mLootTimesButton.setBorder(BorderFactory.createEmptyBorder(3, 5, 2, 5));
        mLootTimesButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                AutoDKPView.getInstance().reloadRaids();
            }
        });
        
        JLabel filterLabel = new JLabel("Loot:");
        mLootFilter = new JComboBox(FILTERS1);
        ActionListener filterListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                ItemFilter filter = (ItemFilter) mLootFilter.getSelectedItem();
                mPlayerTable.setLootFilter(filter);
                mPlayerTable.fireTableDataChanged();
                mPlayerTable.adjustRowHeights();
                GeneralProperties props = GeneralProperties.getDefault();
                props.setIntegerProperty(props.RAID_LOOT_FILTER, mLootFilter.getSelectedIndex());
            }
        };
        mLootFilter.addActionListener(filterListener);
        Box filterPane = Box.createHorizontalBox();
        filterPane.add(filterLabel);
        filterPane.add(Box.createHorizontalStrut(5));
        filterPane.add(mLootFilter);
        filterLabel.setFont(FONT_CONTROL);
        mLootFilter.setFont(FONT_CONTROL);

        mDistributionsExceptionsButton = new JButton("");
        mDistributionsExceptionsButton.setToolTipText("Zero Sum Distributions Customizer");
        Image zeroSumImage = ImageUtilities.loadJarImage("zeroSum.gif");
        ImageIcon zeroSumIcon = new ImageIcon(zeroSumImage);
        Image disabledZeroSumImage = GrayFilter.createDisabledImage(zeroSumImage);
        mDistributionsExceptionsButton.setDisabledIcon(new ImageIcon(disabledZeroSumImage));
        mDistributionsExceptionsButton.setIcon(zeroSumIcon);
        mDistributionsExceptionsButton.setRolloverIcon(zeroSumIcon);
        mDistributionsExceptionsButton.setVisible(false);
        mDistributionsExceptionsButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (!DistributionsTableModel.hasDistributionData(mCurrentPointSystem)) {
                    JOptionPane.showMessageDialog(
                            mDistributionsExceptionsButton, 
                            "No zero sum distribution information for this raid.");
                    return;
                }
                new DistributionEditor(
                        mPlayerTable, 
                        mCurrentRaid, 
                        mCurrentPointSystem).setVisible(true);
            }
        });
        
        mBossKillExceptionsButton = new JButton("");
        mBossKillExceptionsButton.setToolTipText("Boss Kill Points Customizer");
        Image bossKillImage = ImageUtilities.loadJarImage("bossKill.gif");
        ImageIcon bossKillIcon = new ImageIcon(bossKillImage);
        Image disabledBossKillImage = GrayFilter.createDisabledImage(bossKillImage);
        mBossKillExceptionsButton.setDisabledIcon(new ImageIcon(disabledBossKillImage));
        mBossKillExceptionsButton.setIcon(bossKillIcon);
        mBossKillExceptionsButton.setRolloverIcon(bossKillIcon);
        mBossKillExceptionsButton.setVisible(false);
        mBossKillExceptionsButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mCurrentRaid.getBossNameToKillDateMap().size() == 0) {
                    JOptionPane.showMessageDialog(
                            mBossKillExceptionsButton, 
                            "No boss kill information for this raid.");
                    return;
                }
                new BossKillsEditor(
                        mPlayerTable, 
                        mCurrentRaid, 
                        mCurrentPointSystem).setVisible(true);
            }
        });
        
        mTimeExceptionsButton = new JButton("");
        mTimeExceptionsButton.setToolTipText("Time Points Customizer");
        Image timeImage = ImageUtilities.loadJarImage("timeBased.gif");
        ImageIcon timeIcon = new ImageIcon(timeImage);
        Image disabledTimeImage = GrayFilter.createDisabledImage(timeImage);
        mTimeExceptionsButton.setDisabledIcon(new ImageIcon(disabledTimeImage));
        mTimeExceptionsButton.setIcon(timeIcon);
        mTimeExceptionsButton.setRolloverIcon(timeIcon);
        mTimeExceptionsButton.setVisible(false);
        mTimeExceptionsButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (!TimeTableModel.hasTimeData(mCurrentPointSystem)) {
                    JOptionPane.showMessageDialog(
                            mTimeExceptionsButton, 
                            "No time DKP information for this raid.");
                    return;
                }
                new TimeEditor(
                        mPlayerTable, 
                        mCurrentRaid, 
                        mCurrentPointSystem).setVisible(true);
            }
        });
        
        mDistributionsExceptionsButton.setFont(FONT_CONTROL);
        mBossKillExceptionsButton.setFont(FONT_CONTROL);
        mTimeExceptionsButton.setFont(FONT_CONTROL);
        
        JPanel leftControls  = new JPanel(new FlowLayout(FlowLayout.LEFT,  10, 0));
        JPanel rightControls = new JPanel(new FlowLayout(FlowLayout.RIGHT, 10, 0));
        leftControls.setOpaque(false);
        rightControls.setOpaque(false);
        leftControls.add(mAddNonRaidMemberButton);
        leftControls.add(mLootTimesButton);
        leftControls.add(filterPane);
        rightControls.add(mDistributionsExceptionsButton);
        rightControls.add(mBossKillExceptionsButton);
        rightControls.add(mTimeExceptionsButton);
        mTableControl = new JPanel(new BorderLayout(0, 0));
        mTableControl.setBorder(BorderFactory.createEmptyBorder(2, 4, 2, 4));
        mTableControl.setBackground(PointSummaryPane.BG_COLOR);
        mTableControl.add(leftControls,  BorderLayout.CENTER);
        mTableControl.add(rightControls, BorderLayout.EAST);
        mTableControl.setPreferredSize(new Dimension(mComponent.getPreferredSize().width, 30));
        
        GeneralProperties props = GeneralProperties.getDefault();
        Integer raidSplit = props.getIntegerProperty(props.RAID_SPLIT);
        if (raidSplit != null) {
            mComponent.setDividerLocation(raidSplit.intValue());
        } else {
            mComponent.setDividerLocation(props.DEFAULT_RAID_SPLIT);
        }
        boolean isShowingNonRaidMembers = props.getBooleanProperty(props.SHOW_NON_RAID_MEMBERS);
        mAddNonRaidMemberButton.setSelected(isShowingNonRaidMembers);
        boolean isShowingLootTimes = props.getBooleanProperty(props.SHOW_LOOT_TIMES);
        mLootTimesButton.setSelected(isShowingLootTimes);
        
        Integer filterProp = props.getIntegerProperty(props.RAID_LOOT_FILTER);
        if (filterProp != null) {
            int index = filterProp.intValue();
            if (index < mLootFilter.getItemCount()) {
                mLootFilter.setSelectedIndex(index);
            }
        }
        
        props.addSaveOnExitProperty(new SaveOnExitProperty(props.RAID_SPLIT) {
            public Object getValue() { return new Integer(mComponent.getDividerLocation()); }
        });
        props.addSaveOnExitProperty(new SaveOnExitProperty(props.SHOW_NON_RAID_MEMBERS) {
            public Object getValue() { return new Boolean(mAddNonRaidMemberButton.isSelected()); }
        });
        props.addSaveOnExitProperty(new SaveOnExitProperty(props.SHOW_LOOT_TIMES) {
            public Object getValue() { return new Boolean(mLootTimesButton.isSelected()); }
        });
    }
    
    
    public boolean addNonRaidMember() {
        if (mRaidList.getSelectedIndex() >= 0 && mPointSystemList.getSelectedIndex() >= 0)
        {
            if(!mCurrentPointSystem.isPointsStarted(mCurrentRaid))
            {
                JOptionPane.showMessageDialog(
                        mComponent, 
                        "Points are not started for this raid, nothing added.", 
                        "Add Non-Raid Member", 
                        JOptionPane.ERROR_MESSAGE);
                return false;
            }
            
            DialogPlayerNameAndClass diag = new DialogPlayerNameAndClass(
                    AutoDKPView.getInstance().getFrame(),
                    "Add Non-Raid Member",
                    null);
            diag.setVisible(true);
            AlternateItem item = diag.getAlternateItem();
            if (item != null) {
                String playerName  = item.getPlayerName();
                String playerClass = item.getPlayerClass();
                Map constraintsPlayerData = 
                    mCurrentPointSystem.getCurrentRaidData().getPointSystemPlayerDataMap();
                if (constraintsPlayerData.containsKey(playerName)) {
                    JOptionPane.showMessageDialog(
                            mComponent, 
                            "Player " + playerName + " is already in the raid.",
                            "Add Non-Raid Member", 
                            JOptionPane.ERROR_MESSAGE);
                    return false;
                }
                IPointSystemPlayerData newPlayerDatum = new DefaultPointSystemPlayerData();
                newPlayerDatum.setPlayerName(playerName);
                newPlayerDatum.setPlayerClass(playerClass);
                newPlayerDatum.setFormula(mCurrentPointSystem.getCalculator().generateFormula(null));
                constraintsPlayerData.put(playerName, newPlayerDatum);
                mCurrentPointSystem.saveRaid(mCurrentRaid);
                List newPlayerData = new ArrayList();
                newPlayerData.add(new PlayerData(playerName, playerClass));
                mCurrentRaid.addPlayerData(newPlayerData);
                return true;
            }

        } else {
            JOptionPane.showMessageDialog(
                    mComponent, 
                    "A raid and/or point system was not selected, nothing added.", 
                    "Add Non-Raid Member", 
                    JOptionPane.ERROR_MESSAGE);
        }
        return false;
    }
    
    public boolean removeSelectedPlayers() {
        if (mRaidList.getSelectedIndex() >= 0 && mPointSystemList.getSelectedIndex() >= 0)
        {
            if(!mCurrentPointSystem.isPointsStarted(mCurrentRaid))
            {
                JOptionPane.showMessageDialog(
                        mComponent, 
                        "Points are not started for this raid, nothing removed.", 
                        "Remove Player Points", 
                        JOptionPane.ERROR_MESSAGE);
                return false;
            }
            boolean selectionMade = mPlayerTable.removeSelectedPlayers();
            if (selectionMade) {
                return true;
            }
            JOptionPane.showMessageDialog(
                    mComponent, 
                    "No player was selected in the table, nothing removed.", 
                    "Remove Player Points", 
                    JOptionPane.ERROR_MESSAGE);
        } else {
            JOptionPane.showMessageDialog(
                    mComponent, 
                    "A raid and/or point system was not selected, nothing removed.", 
                    "Remove Player Points", 
                    JOptionPane.ERROR_MESSAGE);
        }
        return false;
    }
    
    public void reloadRaids() {
        int summaryTabIndex = mSummaryPane.getSelectedIndex();
        int index = mRaidList.getSelectedIndex();
        mRaidList.removeListSelectionListener(mRaidSelectionListener);
        mRaidList.clearSelection();
        mRaidList.setListData(AutoDKPModel.getInstance().getRaidItems().toArray());
        mRaidList.addListSelectionListener(mRaidSelectionListener);
        mRaidList.setSelectedIndex(index);
        if (summaryTabIndex < mSummaryPane.getTabCount()) {
            mSummaryPane.setSelectedIndex(summaryTabIndex);
        }
    }
    
    public String generateHTMLReport() {
        if (mCurrentRaid == null) {
            JOptionPane.showMessageDialog(
                    mComponent, 
                    "No raid selected, please select a raid.", 
                    "HTML Report", 
                    JOptionPane.ERROR_MESSAGE);
            return null;
        } else if (mCurrentPointSystem == null) {
            JOptionPane.showMessageDialog(
                    mComponent, 
                    "No point system selected, please select a point system.", 
                    "HTML Report", 
                    JOptionPane.ERROR_MESSAGE);
            return null;
        }
        return mPlayerTable.generateHTMLReport();
    }
    
    public Component getComponent() {
        return mComponent;
    }
    
    public Component getTableControl() {
        return mTableControl;
    }
    
    private void installIntoRaid(Raid currentRaid, IPointSystem currentSystem) {
        mCurrentRaid = currentRaid;
        mCurrentPointSystem = currentSystem;
        
        boolean isStarted = false;
        if (currentRaid != null) {
            if (currentSystem != null) {
                isStarted = currentSystem.isPointsStarted(currentRaid);
                currentSystem.loadFromRaidData(mCurrentRaid);
            }
        }
        mSummaryPane.loadConstraintsPane(currentRaid, currentSystem, isStarted);
        
        if (currentRaid != null) {
            mPlayerTable.switchRaid(
                    currentRaid, 
                    currentSystem, 
                    isStarted, 
                    mAddNonRaidMemberButton.isSelected(),
                    mLootTimesButton.isSelected());
        }
        
        if (
                currentRaid == null || 
                currentSystem == null ||
                !currentSystem.isPointsStarted(currentRaid)) {
            mBossKillExceptionsButton.setVisible(false);
            mDistributionsExceptionsButton.setVisible(false);
            mTimeExceptionsButton.setVisible(false);
        } else {
            boolean isEnabled = !currentSystem.getCurrentRaidData().isLocked();
            mBossKillExceptionsButton.setEnabled(isEnabled);
            mDistributionsExceptionsButton.setEnabled(isEnabled);
            mTimeExceptionsButton.setEnabled(isEnabled);
            mBossKillExceptionsButton.setVisible(currentSystem.isBossKill());
            mDistributionsExceptionsButton.setVisible(currentSystem.isZeroSum());
            mTimeExceptionsButton.setVisible(currentSystem.isTimeBased());
        }
    }
    
    private class RaidSelectionListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
            if (!e.getValueIsAdjusting()) {
                IPointSystem currentSystem = null;
                if (mPointSystemList.getSelectedIndex() >= 0) {
                    currentSystem = 
                        (IPointSystem) mPointSystemList.getModel().getElementAt(mPointSystemList.getSelectedIndex());
                }
                Raid currentRaid = 
                    (Raid) mRaidList.getModel().getElementAt(mRaidList.getSelectedIndex());
                if (currentRaid != null) {
                    AutoDKPModel.getInstance().setSelectedRaid(currentRaid);
                    installIntoRaid(currentRaid, currentSystem);
                }
            }
        }
    }
    
    private class PointSystemSelectionListener implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
            if (!e.getValueIsAdjusting()) {
                Raid currentRaid = null;
                if (mRaidList.getSelectedIndex() >= 0) {
                    currentRaid = 
                        (Raid) mRaidList.getModel().getElementAt(mRaidList.getSelectedIndex());
                }
                IPointSystem currentSystem = 
                    (IPointSystem) mPointSystemList.getModel().getElementAt(mPointSystemList.getSelectedIndex());
                mRaidCellRenderer.setPointSystem(currentSystem);
                mRaidList.repaint();
                installIntoRaid(currentRaid, currentSystem);
            }
        }
    }
}
