package org.wow.autodkp.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.util.Iterator;
import java.util.List;

import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.MatteBorder;

import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.IPointSystem;


/**
 * 
 * 
 * @author jsandusky
 */
public class RaidCellRenderer extends JPanel implements ListCellRenderer {
    
    private static final Font RAID_LIST_FONT = new Font("Monospaced", Font.BOLD, 14);
    private static final Image STAR   = ImageUtilities.loadJarImage("link.png");
    private static final Image NOSTAR = new BufferedImage(20, 20, BufferedImage.TRANSLUCENT);
    
    protected static Border noFocusBorder;
    
    private IPointSystem mPointSystem;
    private Box mBox;
    private JLabel mStarLabel;
    private JLabel mNoStarLabel;
    private JLabel mRaidLabel;

    
    public RaidCellRenderer() {
        
        if (noFocusBorder == null) {
            noFocusBorder = new EmptyBorder(1, 1, 1, 1);
        }
        setOpaque(true);
        setBorder(noFocusBorder);

        mStarLabel = new JLabel(new ImageIcon(STAR));
        mStarLabel.setBorder(null);
        mNoStarLabel = new JLabel(new ImageIcon(NOSTAR));
        mNoStarLabel.setBorder(null);

        mRaidLabel = new JLabel();
        mRaidLabel.setFont(RAID_LIST_FONT);
        mRaidLabel.setOpaque(false);
        mRaidLabel.setBorder(null);
        
        setLayout(new BorderLayout(0, 0));
        mBox = Box.createHorizontalBox();
        mBox.setOpaque(false);
        mBox.setBorder(new MatteBorder(1, 0, 1, 0, Color.LIGHT_GRAY));
        
        add(mBox, BorderLayout.CENTER);
    }
    
    public void setPointSystem(IPointSystem pointSystem) {
        mPointSystem = pointSystem;
    }
    
    public Component getListCellRendererComponent(JList list, 
                                                  Object value, 
                                                  int index,
                                                  boolean isSelected, 
                                                  boolean cellHasFocus)
    {
        
        if (isSelected) {
            setBackground(list.getSelectionBackground());
            setForeground(list.getSelectionForeground());
        }
        else {
            setBackground(list.getBackground());
            setForeground(list.getForeground());
        }

        setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);

        if (value instanceof Raid) {
            Raid raid = (Raid) value;
            
            mBox.removeAll();
            
            if (mPointSystem != null && mPointSystem.isPointsStarted(raid)) {
                mBox.add(mStarLabel);
                mBox.add(Box.createHorizontalStrut(5));
            } else {
                mBox.add(mNoStarLabel);
                mBox.add(Box.createHorizontalStrut(5));
            }

            StringBuffer buf = new StringBuffer();
            
            GeneralProperties props = GeneralProperties.getDefault();
            String formattedStart = 
                props.getDateFormatProperty(props.DATE_FORMAT_LONG).format(raid.getStartTime());
            buf.append(formattedStart);
            
            mRaidLabel.setText(buf.toString());
            mBox.add(mRaidLabel);
            mBox.add(Box.createHorizontalStrut(5));
            
            int maxHeight = getPreferredSize().height;
            List instanceNames = raid.getInstanceNames();
            if (instanceNames != null) {
                for (Iterator iter=instanceNames.iterator(); iter.hasNext();) {
                    String instanceName = (String) iter.next();
                    Image icon = ImageUtilities.getInstanceIcon(this, instanceName);
                    if (icon != null) {
                        ImageIcon imageIcon = new ImageIcon(icon);
                        JLabel imageLabel = new JLabel(imageIcon);
                        imageLabel.setSize(imageIcon.getIconWidth(), imageIcon.getIconHeight());
                        mBox.add(imageLabel);
                        mBox.add(Box.createHorizontalStrut(5));
                        maxHeight = Math.max(maxHeight, imageLabel.getPreferredSize().height);
                    }
                }
            }
            
            if (maxHeight > getPreferredSize().height) {
                setPreferredSize(new Dimension(getPreferredSize().width, maxHeight));
            }
        }
        
        return this;
    }
    
}
