package org.wow.autodkp.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.IPointSystem;



/**
 * 
 * 
 * @author jsandusky
 */
public class RaidLinker extends BasicDialog {

    private boolean mIsRaidSelectionApplied;
    private JButton mOKButton;
    private Raid mSelectedRaid;

    
    public RaidLinker(Frame owner, IPointSystem pointSystem, Raid[] raids) {
        super(owner, "Raid Linker", true);
        addBasicSupport();
        
        setLayout(new BorderLayout());

        final JList raidList = new JList(raids);
        raidList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        RaidCellRenderer renderer = new RaidCellRenderer();
        renderer.setPointSystem(pointSystem);
        raidList.setCellRenderer(renderer);
        raidList.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    if (!mOKButton.isEnabled()) {
                        mOKButton.setEnabled(true);
                    }
                    mSelectedRaid = (Raid) raidList.getSelectedValue();
                }
            }
        });
        JScrollPane listScroller = new JScrollPane(raidList);
        listScroller.setBorder(BorderFactory.createTitledBorder("Select Raid to Link"));
        raidList.setBackground(Color.WHITE);
        listScroller.setBackground(Color.WHITE);
        
        mOKButton = new JButton("OK");
        mOKButton.setEnabled(false);
        mOKButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mIsRaidSelectionApplied = true;
                dispose();
            }
        });
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        JPanel buttonPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttonPane.add(cancelButton);
        buttonPane.add(mOKButton);
        buttonPane.setBorder(BorderFactory.createEmptyBorder(10, 5, 10, 5));

        getContentPane().add(listScroller, BorderLayout.CENTER);
        getContentPane().add(buttonPane, BorderLayout.SOUTH);

        setSize(500, 600);
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension windowSize = getSize();
        setLocation(
                (screenSize.width  - windowSize.width)  / 2, 
                (screenSize.height - windowSize.height) / 2);
        
        setVisible(true);
    }

    
    public boolean isRaidSelectionApplied() {
        return mIsRaidSelectionApplied;
    }

    public Raid getSelectedRaid() {
        return mSelectedRaid;
    }
}
