package org.wow.autodkp.view;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.KeyStroke;
import javax.swing.text.JTextComponent;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;


public class UndoRedoUtilities {

    public static void addUndoRedo(JTextComponent textArea) {
        UndoManager manager = new UndoManager();
        textArea.getDocument().addUndoableEditListener(manager);
        Action undoAction = new UndoAction(manager);
        Action redoAction = new RedoAction(manager);
        
        // Assign the actions to keys
        textArea.getActionMap().put("undo", undoAction);
        textArea.getActionMap().put("redo", redoAction);
        
        textArea.getInputMap().put(KeyStroke.getKeyStroke(
            KeyEvent.VK_Z, InputEvent.CTRL_MASK), "undo");
        textArea.getInputMap().put(KeyStroke.getKeyStroke(
                KeyEvent.VK_Y, InputEvent.CTRL_MASK), "redo");
    }
    
    
    // The Undo action
    private static class UndoAction extends AbstractAction {
      public UndoAction(UndoManager manager) {
        this.manager = manager;
      }

      public void actionPerformed(ActionEvent evt) {
        try {
          manager.undo();
        } catch (CannotUndoException e) {
          Toolkit.getDefaultToolkit().beep();
        }
      }

      private UndoManager manager;
    }

    // The Redo action
    private static class RedoAction extends AbstractAction {
      public RedoAction(UndoManager manager) {
        this.manager = manager;
      }

      public void actionPerformed(ActionEvent evt) {
        try {
          manager.redo();
        } catch (CannotRedoException e) {
          Toolkit.getDefaultToolkit().beep();
        }
      }

      private UndoManager manager;
    }
}
