package org.wow.autodkp.view.actions;

import java.awt.event.ActionEvent;
import java.util.Iterator;

import javax.swing.JOptionPane;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.view.AutoDKP;
import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.GeneralProperties;


/**
 * Deletes the selected raid.
 * Note: if there is no AutoDKP raid file for the selected
 * raid, then nothing will be deleted. If the AutoDKP raid
 * file is deleted but the raid data also exists within
 * Raid Tracker, then the raid will still show up in AutoDKP.
 * 
 * @author jsandusky
 */
public class DeleteSelectedRaid extends AbstractAutoDKPAction
{
    public DeleteSelectedRaid() {
        super("Delete Selected Raid");
        setIcon("cancel.png");
    }

    public void actionPerformed(ActionEvent e) {
        Raid selectedRaid = AutoDKPModel.getInstance().getSelectedRaid();
        GeneralProperties props = GeneralProperties.getDefault();
        String dateString = 
            props.getDateFormatProperty(props.DATE_FORMAT_LONG).format(
                    selectedRaid.getStartTime());
        if (selectedRaid != null) {
            if (!selectedRaid.isPromoted()) {
                JOptionPane.showMessageDialog(
                        AutoDKPView.getInstance().getFrame(), 
                        dateString + "\n" +
                        "This raid is not promoted, and cannot be deleted within " + AutoDKP.PROGRAM_NAME + ".\n" + 
                        "To delete this raid permanently, remove it from within CT Raid Tracker.",
                        "Delete Raid",
                        JOptionPane.ERROR_MESSAGE);
            } else {
                int result = JOptionPane.showConfirmDialog(
                        AutoDKPView.getInstance().getFrame(), 
                        dateString + "\n" +
                        "Are you sure you wish to delete this raid?\n" + 
                        "This cannot be undone.",
                        "Delete Raid",
                        JOptionPane.OK_CANCEL_OPTION,
                        JOptionPane.WARNING_MESSAGE);
                if (result == JOptionPane.YES_OPTION) {
                    // delete the raid
                    selectedRaid.delete();
                    AutoDKPView.unloadReload();
                    String message = dateString + "\n" + "Raid deleted.";
                    for (Iterator iter=AutoDKPModel.getInstance().getRaidItems().iterator(); iter.hasNext();) {
                        Raid iterRaid = (Raid) iter.next();
                        if (iterRaid.getID().equals(selectedRaid.getID())) {
                            message = dateString + "\n" +
                                "The promoted version of this raid was deleted.\n" +
                                "However, the raid still exists within CT Raid Tracker.\n" +
                                "To delete this raid permanently, remove it from within CT Raid Tracker.";
                            break;
                        }
                    }
                    JOptionPane.showMessageDialog(
                            AutoDKPView.getInstance().getFrame(), 
                            message, 
                            "Delete Raid",
                            JOptionPane.INFORMATION_MESSAGE);
                }
            }
        } else {
            JOptionPane.showMessageDialog(
                    AutoDKPView.getInstance().getFrame(), 
                    "No raid selected, nothing deleted.", 
                    "Delete Raid",
                    JOptionPane.ERROR_MESSAGE);
        }
    }
}