package org.wow.autodkp.view.actions;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.WindowConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.BasicDialog;
import org.wow.autodkp.view.RaidCellRenderer;


/**
 * Dialog for merging two raids into one.
 * There is no difference as to which raid is selected in
 * the left vs. right JList.
 * 
 * @author jsandusky
 */
public class DialogMergeRaids extends BasicDialog {
    
    private JList mRaid1List;
    private JList mRaid2List;
    private JButton mMergeButton;
    
    
    public DialogMergeRaids(
            Frame owner, 
            String title) {
        super(owner, title, true);
        addBasicSupport();
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
        setLayout(new BorderLayout());
        getContentPane().add(getMainPane(), BorderLayout.CENTER);
        getContentPane().add(getButtonPane(getMergeActionListener()), BorderLayout.SOUTH);

        setResizable(true);
        setSize(700, 400);
        setLocationRelativeTo(owner);
    }
    
    protected ActionListener getMergeActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Raid raid1 = (Raid) mRaid1List.getSelectedValue();
                Raid raid2 = (Raid) mRaid2List.getSelectedValue();
                Raid mergedRaid = raid1.merge(raid2);
                if (mergedRaid != null) {
                    mergedRaid.promote();
                    AutoDKPView.unloadReload();
                    dispose();
                } else {
                    JOptionPane.showMessageDialog(
                            DialogMergeRaids.this, 
                            "Error occurred during merging of raids.", 
                            "Merge Raids", 
                            JOptionPane.ERROR_MESSAGE);
                }
            }
        };
    }
    
    private JComponent initRaidPane(JList list, String title) {
        list.setListData(AutoDKPModel.getInstance().getRaidItems().toArray());
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        list.setCellRenderer(new RaidCellRenderer());
        JScrollPane raidScroller = new JScrollPane(list);
        raidScroller.setBorder(BorderFactory.createTitledBorder(title));
        raidScroller.setBackground(Color.WHITE);
        list.setBackground(Color.WHITE);
        return raidScroller;
    }
    
    protected JComponent getMainPane() {
        mRaid1List = new JList();
        mRaid2List = new JList();
        JComponent raid1Pane = initRaidPane(mRaid1List, "First Raid");
        JComponent raid2Pane = initRaidPane(mRaid2List, "Second Raid");
        
        ListSelectionListener raidSelectionListener = new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    if (
                            !mRaid1List.isSelectionEmpty() && 
                            !mRaid2List.isSelectionEmpty()) {
                        mMergeButton.setEnabled(true);
                    }
                }
            }
        };
        mRaid1List.addListSelectionListener(raidSelectionListener);
        mRaid2List.addListSelectionListener(raidSelectionListener);
        
        JTextArea headerText = new JTextArea(
                "Select two raids to merge.\n" +
                "A new raid will be created that is the sum of the first and second raids.\n" +
                "When raids are merged: players, loot, and join/leave-times are combined.\n" +
                "The new raid will have a start date of 1 second before the earlier of the two raids.");
        headerText.setOpaque(false);
        headerText.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(10, 10, 10, 10),
                BorderFactory.createEtchedBorder()));
        headerText.setEditable(false);
        headerText.setFont(headerText.getFont().deriveFont(Font.BOLD));
        
        Box hBox = Box.createHorizontalBox();
        hBox.add(raid1Pane);
        hBox.add(raid2Pane);
        hBox.setBorder(BorderFactory.createEtchedBorder());
        JPanel mainPane = new JPanel();
        mainPane.setLayout(new BorderLayout());
        mainPane.add(headerText, BorderLayout.NORTH);
        mainPane.add(hBox, BorderLayout.CENTER);
        
        mainPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 5, 10));
        
        return mainPane;
    }
    
    protected JComponent getButtonPane(ActionListener actionListener) {
        mMergeButton = new JButton();
        mMergeButton.setEnabled(false);
        mMergeButton.setText("Merge");
        mMergeButton.addActionListener(actionListener);
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        
        JPanel buttonPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttonPane.add(mMergeButton);
        buttonPane.add(cancelButton);
        buttonPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        return buttonPane;
    }
}
