package org.wow.autodkp.view.actions;

import java.awt.event.ActionEvent;

import javax.swing.JOptionPane;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.GeneralProperties;


/**
 * Promote the selected raid.
 * Promoting a raid will save the in-memory model of a raid into
 * an AutoDKP raid file. This raid data from this file takes precedence
 * over the corresponding raid within Raid Tracker (if one exists).
 * The word "promote" is used instead of "save" to emphasize the fact
 * that the Raid Tracker data for a particular raid will be ignored
 * once that raid has been "promoted" within AutoDKP.
 * 
 * @author jsandusky
 */
public class PromoteAction extends AbstractAutoDKPAction {
    
    public PromoteAction() {
        super("Promote Selected Raid");
        setIcon("promote.png");
    }

    public void actionPerformed(ActionEvent e) {
        Raid selectedRaid = AutoDKPModel.getInstance().getSelectedRaid();
        if (selectedRaid != null) {
            selectedRaid.promote();
            GeneralProperties props = GeneralProperties.getDefault();
            JOptionPane.showMessageDialog(
                    AutoDKPView.getInstance().getFrame(), 
                    props.getDateFormatProperty(props.DATE_FORMAT_LONG).format(
                            selectedRaid.getStartTime()) + "\n" +
                    "Raid promoted.", 
                    "Promote Raid",
                    JOptionPane.INFORMATION_MESSAGE);
        } else {
            JOptionPane.showMessageDialog(
                    AutoDKPView.getInstance().getFrame(), 
                    "No raid selected, nothing promoted.", 
                    "Promote Raid",
                    JOptionPane.ERROR_MESSAGE);
        }
    }
}
