package org.wow.autodkp.view.handlers;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import org.wow.autodkp.model.BossKillInfo;
import org.wow.autodkp.persistence.BossManager;
import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.BasicDialog;
import org.wow.autodkp.view.DateControls;


/**
 * Dialog for entering boss kill info.
 * 
 * @author jsandusky
 */
public class DialogBossKill extends BasicDialog {
    
    private BossKillInfo mBossKillInfo;
    
    private DateControls mDateControls;
    private JLabel mNameLabel;
    private JLabel mDateLabel;
    private JComboBox mNameCombo;
    private Date mRaidStart;
    
    
    public DialogBossKill(
            String title, 
            BossKillInfo bossKillInfo,
            Date raidStart) {
        super(AutoDKPView.getInstance().getFrame(), title, true);
        addBasicSupport();
        
        mBossKillInfo = bossKillInfo;
        mRaidStart = raidStart;
        
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
        ActionListener listener = null;
        if (mBossKillInfo == null) {
            listener = getAddActionListener();
        } else {
            listener = getEditActionListener();
        }
        setLayout(new BorderLayout());
        getContentPane().add(getMainPane(),           BorderLayout.CENTER);
        getContentPane().add(getButtonPane(listener), BorderLayout.SOUTH);

        setResizable(false);
        pack();
        setLocationRelativeTo(getOwner());
    }
    
    protected ActionListener getAddActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mDateLabel.setForeground(Color.BLACK);
                String name = (String) mNameCombo.getSelectedItem();
                boolean isValid = true;
                Date killDate = mDateControls.getDate();
                if (killDate == null) {
                    // incorrect date format
                    isValid = false;
                    mDateLabel.setForeground(Color.RED);
                }
                if (isValid) {
                    mBossKillInfo = new BossKillInfo(name, killDate);
                    dispose();
                }
            }
        };
    }
    
    protected ActionListener getEditActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mDateLabel.setForeground(Color.BLACK);
                String name = (String) mNameCombo.getSelectedItem();
                boolean isValid = true;
                Date killDate = mDateControls.getDate();
                if (killDate == null) {
                    // incorrect date format
                    isValid = false;
                    mDateLabel.setForeground(Color.RED);
                }
                if (isValid) {
                    mBossKillInfo = new BossKillInfo(name, killDate);
                    dispose();
                }
            }
        };
    }
    
    protected JComponent getMainPane() {
        mNameLabel = new JLabel("Boss:");
        mNameLabel.setBorder(BorderFactory.createEmptyBorder(15, 0, 10, 5));
        List filteredNames = new ArrayList(BossManager.getDefault().getNames());
        mNameCombo = new JComboBox(filteredNames.toArray());

        mDateLabel = new JLabel("Kill Time:");
        mDateLabel.setBorder(BorderFactory.createEmptyBorder(15, 0, 0, 5));
        
        mDateControls = new DateControls();
        if (mBossKillInfo != null && mBossKillInfo.getKillDate() != null) {
            mDateControls.initializeDateAndTime(mBossKillInfo.getKillDate());
        } else {
            mDateControls.initializeDate(mRaidStart);
        }
		
        if (mBossKillInfo != null) {
            mNameCombo.setSelectedItem(mBossKillInfo.getBossName());
        }
        
        JPanel mainPane = new JPanel();
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints gdc = new GridBagConstraints();
        mainPane.setLayout(gridBag);
        gdc.anchor = GridBagConstraints.SOUTHWEST;
        
        int y = -1;
        gdc.weighty = 0;
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mNameLabel, gdc);
        mainPane.add(mNameLabel);
        gdc.gridx = 1;
        gdc.weightx = 1;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mNameCombo, gdc);
        mainPane.add(mNameCombo);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mDateLabel, gdc);
        mainPane.add(mDateLabel);
        gdc.gridx = 1;
        gdc.weightx = 1;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mDateControls.getComponent(), gdc);
        mainPane.add(mDateControls.getComponent());
        
        mainPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));

        return mainPane;
    }
    
    protected JComponent getButtonPane(ActionListener actionListener) {
        JButton applyButton = new JButton();
        if (mBossKillInfo == null) {
            applyButton.setText("Add");
        } else {
            applyButton.setText("Edit");
        }
        applyButton.addActionListener(actionListener);
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        
        getRootPane().setDefaultButton(applyButton);
        
        JPanel buttonPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttonPane.add(applyButton);
        buttonPane.add(cancelButton);
        buttonPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        return buttonPane;
    }

    public BossKillInfo getBossKillInfo() {
        return mBossKillInfo;
    }
}
