package org.wow.autodkp.view.html;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.filechooser.FileFilter;

import org.wow.autodkp.persistence.FileIOUtilities;
import org.wow.autodkp.view.BasicDialog;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.UndoRedoUtilities;


/**
 * Displays a CSS file.
 * 
 * @author jsandusky
 */
public class CSSStyleEditor extends BasicDialog {
    
    private static final String SUFFIX_CSS = ".css";
    
    private JTextArea mTextArea;
    private Action mSaveAction;
    private String mOriginalText;
    
    
    public CSSStyleEditor(JDialog owner) {
        super(owner, "CSS Style Editor", true);
        initialize(StyleUtilities.INTERNAL_STYLE_SHEET);
    }
    
    public CSSStyleEditor(JFrame owner) {
        super(owner, "CSS Style Editor", true);
        initialize(StyleUtilities.INTERNAL_STYLE_SHEET);
    }
    
    
    private void initialize(final String text) {
        mOriginalText = text;
        addBasicSupport();
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        
        mTextArea = new JTextArea(text);
        final JScrollPane scroller = new JScrollPane(mTextArea);
        mTextArea.setLineWrap(false);
        mTextArea.setCaretPosition(0);
        mTextArea.setFont(new Font("Monospaced", Font.PLAIN, 12));
        
        mTextArea.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent e) {
            }
            public void removeUpdate(DocumentEvent e) {
                mSaveAction.setEnabled(true);
            }
            public void insertUpdate(DocumentEvent e) {
                mSaveAction.setEnabled(true);
            }
        });
        
        UndoRedoUtilities.addUndoRedo(mTextArea);
        
        scroller.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(10, 10, 5, 10),
                scroller.getBorder()));
        
        JButton loadButton = new JButton("Load Style Sheet");
        loadButton.setToolTipText("Open a CSS Style Sheet in the Editor");
        loadButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                importSheet();
            }
        });
        JButton revertButton = new JButton("Revert");
        revertButton.setToolTipText("Revert Style Sheet Modifications.");
        revertButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                revert();
            }
        });
        
        mSaveAction = new AbstractAction("Save") {
            public void actionPerformed(ActionEvent e) {
                save();
            }
        };
        mSaveAction.setEnabled(false);
        JButton saveButton = new JButton(mSaveAction);
        
        mTextArea.getActionMap().put("save", mSaveAction);
        mTextArea.getInputMap().put(
                KeyStroke.getKeyStroke(KeyEvent.VK_S, KeyEvent.CTRL_DOWN_MASK),
                "save");
        
        JButton closeButton = new JButton("Close");
        JPanel buttonPanel = new JPanel(new BorderLayout());
        JPanel rightButtonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        rightButtonPanel.add(saveButton);
        rightButtonPanel.add(closeButton);
        JPanel leftButtonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        leftButtonPanel.add(loadButton);
        leftButtonPanel.add(revertButton);

        buttonPanel.add(rightButtonPanel, BorderLayout.EAST);
        buttonPanel.add(leftButtonPanel,  BorderLayout.WEST);
        buttonPanel.setBorder(BorderFactory.createEmptyBorder(0, 5, 5, 5));
        closeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                doClose();
            }
        });
        
        getContentPane().add(scroller,    BorderLayout.CENTER);
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);

        GeneralProperties props = GeneralProperties.getDefault();
        props.loadWindowRect(
                this, 
                props.CSS_WINDOW_X,
                props.CSS_WINDOW_Y,
                props.CSS_WINDOW_WIDTH,
                props.CSS_WINDOW_HEIGHT,
                props.DEFAULT_CUSTOMIZER_WIDTH,
                props.DEFAULT_CUSTOMIZER_HEIGHT);
    }
    
    protected WindowListener getWindowClosingListener() {
        return new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                super.windowClosing(e);
                doClose();
            }
        };
    }
    
    protected void doClose() {
        if (canClose()) {
            closing();
            dispose();
        }
    }
    
    protected void closing() {
        GeneralProperties props = GeneralProperties.getDefault();
        props.setIntegerProperty(props.CSS_WINDOW_WIDTH,  getWidth());
        props.setIntegerProperty(props.CSS_WINDOW_HEIGHT, getHeight());
        props.setIntegerProperty(props.CSS_WINDOW_X,      getX());
        props.setIntegerProperty(props.CSS_WINDOW_Y,      getY());
    }
    
    protected boolean canClose() {
        if (!mTextArea.getText().equals(mOriginalText)) {
            // Change made, ask the user if he/she wishes
            // to abandon these changes.
            int response = JOptionPane.showConfirmDialog(
                    this, 
                    "Style sheet has been modified. Save changes?",
                    "Save Style Sheet",
                    JOptionPane.YES_NO_CANCEL_OPTION,
                    JOptionPane.QUESTION_MESSAGE);
            if (response == JOptionPane.YES_OPTION) {
                // Save and close.
                save();
            } else if (response == JOptionPane.CANCEL_OPTION) {
                // Don't close.
                return false;
            }
        }
        return true;
    }
    
    private void revert() {
        int response = JOptionPane.showConfirmDialog(
                this, 
                "Revert all style modifications to AutoDKP defaults?\n" +
                "This will permanently delete your HTML style changes.",
                "Revert Style Sheet",
                JOptionPane.YES_NO_OPTION,
                JOptionPane.QUESTION_MESSAGE);
        if (response == JOptionPane.YES_OPTION) {
            // Revert changes (deletes the custom sheet).
            StyleUtilities.revertStyles();
            mTextArea.setText(StyleUtilities.INTERNAL_STYLE_SHEET);
            mTextArea.setCaretPosition(0);
        } else if (response == JOptionPane.CANCEL_OPTION) {
            return;
        }
    }
    
    private void save() {
        mOriginalText = mTextArea.getText();
        if (!StyleUtilities.updateInternalStyleSheet(mOriginalText)) {
            JOptionPane.showMessageDialog(
                    this, 
                    "Error occurred during save of custom style sheet.",
                    "Save Style Sheet",
                    JOptionPane.ERROR_MESSAGE);
            mSaveAction.setEnabled(true);
        } else {
            mSaveAction.setEnabled(false);
        }
    }
    
    private void importSheet() {
        String initialPath = System.getProperty("user.dir");
        JFileChooser chooser = new JFileChooser(initialPath);
        chooser.setFileFilter(getCSSFilter());
        chooser.setDialogTitle("Select Style Sheet to Load");
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        chooser.setMultiSelectionEnabled(false);
        chooser.showDialog(this, "Load");
        File selectedFile = chooser.getSelectedFile();
        if (selectedFile != null) {
            String fileText = FileIOUtilities.readFileToString(selectedFile);
            if (fileText.length() > 0) {
                mTextArea.setText(fileText);
                mTextArea.setCaretPosition(0);
            } else {
                JOptionPane.showMessageDialog(
                        this, 
                        "Cannot read from " + selectedFile.getName() + ".\n" +
                        "File is empty or invalid.",
                        "Load Style Sheet",
                        JOptionPane.ERROR_MESSAGE);
            }
        }
    }
    
    private FileFilter getCSSFilter() {
        FileFilter filter = new FileFilter() {
            public String getDescription() {
                return "CSS Files";
            }
            public boolean accept(File f) {
                String name = f.getName().toLowerCase();
                if (f.isDirectory() || name.endsWith(SUFFIX_CSS)) {
                    return true;
                }
                return false;
            }
        };
        return filter;
    }
}
