package org.wow.autodkp.view.html;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToggleButton;

import org.wow.autodkp.view.UndoRedoUtilities;


public class HTMLViewPane extends JPanel {
    
    public static final String MODE_TEXT = "text";
    public static final String MODE_HTML = "html";
    
    private JEditorPane mTextComponent;
    private String mEditorText;
    private JToggleButton mPreviewButton;
    private JToggleButton mTextButton;

    
    public HTMLViewPane(final String text) {
        
        super(new BorderLayout());
        
        mEditorText = text;
        
        mTextComponent = new JEditorPane("text/plain", mEditorText);
        final JScrollPane scroller = new JScrollPane(mTextComponent);
        mTextComponent.setCaretPosition(0);
        mTextComponent.setFont(new Font("Monospaced", Font.PLAIN, 12));
        
        UndoRedoUtilities.addUndoRedo(mTextComponent);
        
        scroller.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(0, 10, 5, 10),
                scroller.getBorder()));
        
        mPreviewButton = new JToggleButton("Preview");
        mTextButton = new JToggleButton("Text");
        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(mTextButton);
        buttonGroup.add(mPreviewButton);
        mTextButton.setSelected(true);
        mTextButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (!mTextComponent.getContentType().equals("text/plain")) {
                    mTextComponent.setEditable(true);
                    mTextComponent.setContentType("text/plain");
                    mTextComponent.setText(mEditorText);
                    mTextComponent.setCaretPosition(0);
                    UndoRedoUtilities.addUndoRedo(mTextComponent);
                }
            }
        });
        mPreviewButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (!mTextComponent.getContentType().equals("text/html")) {
                    mTextComponent.setEditable(false);
                    mEditorText = mTextComponent.getText();
                    mTextComponent.setContentType("text/html");
                    mTextComponent.setText(mEditorText);
                    mTextComponent.setCaretPosition(0);
                }
            }
        });
        JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        buttonPanel.add(mTextButton);
        buttonPanel.add(mPreviewButton);
        buttonPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        
        add(buttonPanel, BorderLayout.NORTH);
        add(scroller,    BorderLayout.CENTER);
    }
    
    
    public void initializeForText(String text) {
        mEditorText = text;
        mTextComponent.setContentType("text/plain");
        mTextComponent.setText(mEditorText);
        mTextComponent.setCaretPosition(0);
        UndoRedoUtilities.addUndoRedo(mTextComponent);
    }
    
    public String getText() {
        if (mTextComponent.getContentType().equals("text/html")) {
            // Cannot edit in HTML mode, so return the last state
            // of the text from the plain editor.
            return mEditorText;
        }
        // Text may have been modified in the plain editor,
        // request the text directly from the component.
        return mTextComponent.getText();
    }
    
    public void switchModes() {
        if (mTextButton.isSelected()) {
            mPreviewButton.doClick();
        } else {
            mTextButton.doClick();
        }
    }
    
    public String getMode() {
        if (mTextButton.isSelected()) {
            return MODE_TEXT; 
        }
        return MODE_HTML;
    }
}
