package org.wow.autodkp.view.html;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.BasicDialog;
import org.wow.autodkp.view.GeneralProperties;


/**
 * Displays an HTML file.
 * 
 * @author jsandusky
 */
public class HTMLViewer extends BasicDialog {
    
    private HTMLViewPane mViewer;
    
    
    public HTMLViewer(String text) {
        super(AutoDKPView.getInstance().getFrame(), "HTML Viewer", true);
        addBasicSupport();
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
        mViewer = new HTMLViewPane(text);
        
        JLabel infoLabel = 
            new JLabel("Note: This HTML Viewer may not support certain CSS styles.");
        infoLabel.setFont(new Font("Dialog", Font.BOLD, 12));
        
        JButton copyButton = new JButton("Copy to Clipboard");
        copyButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
                StringSelection transferable = new StringSelection(mViewer.getText());
                clipboard.setContents(transferable, null);
            }
        });
        JButton okButton = new JButton("Close");
        JPanel buttonPanel = new JPanel(new BorderLayout());
        JPanel leftButtonPanel  = new JPanel(new FlowLayout(FlowLayout.LEFT));
        JPanel rightButtonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        leftButtonPanel.add(infoLabel);
        rightButtonPanel.add(copyButton);
        rightButtonPanel.add(okButton);
        buttonPanel.add(leftButtonPanel,  BorderLayout.WEST);
        buttonPanel.add(rightButtonPanel, BorderLayout.EAST);
        buttonPanel.setBorder(BorderFactory.createEmptyBorder(0, 5, 10, 5));
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closing();
                dispose();
            }
        });
        
        getContentPane().add(mViewer,      BorderLayout.CENTER);
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);

        GeneralProperties props = GeneralProperties.getDefault();
        props.loadWindowRect(
                this, 
                props.HTML_WINDOW_X,
                props.HTML_WINDOW_Y,
                props.HTML_WINDOW_WIDTH,
                props.HTML_WINDOW_HEIGHT,
                props.DEFAULT_CUSTOMIZER_WIDTH,
                props.DEFAULT_CUSTOMIZER_HEIGHT);
        
        String modeProp = props.getStringProperty(props.HTML_VIEWER_MODE);
        if (modeProp != null && modeProp.equals(HTMLViewPane.MODE_HTML)) {
            // we start in text mode, so switch to the html mode
            mViewer.switchModes();
        }
    }
    
    protected void closing() {
        GeneralProperties props = GeneralProperties.getDefault();
        props.setIntegerProperty(props.HTML_WINDOW_WIDTH,  getWidth());
        props.setIntegerProperty(props.HTML_WINDOW_HEIGHT, getHeight());
        props.setIntegerProperty(props.HTML_WINDOW_X,      getX());
        props.setIntegerProperty(props.HTML_WINDOW_Y,      getY());
        props.setStringProperty(props.HTML_VIEWER_MODE,    mViewer.getMode());
    }
}
