package org.wow.autodkp.view.html;

import java.io.File;

import org.wow.autodkp.persistence.FileIOUtilities;
import org.wow.autodkp.persistence.Persistence;


/**
 * HTML css style tags.
 * 
 * @author jsandusky
 */
public abstract class StyleUtilities {
    
    public static String INTERNAL_STYLE_SHEET;
    static {
        initializeInternalStyleSheet();
    }
    
    
    // Internal styles.
    public static final String STYLE_GENERAL_DATE         = "\"raid-date\"";
    public static final String STYLE_GENERAL_NOTES        = "\"general-notes\"";
    public static final String STYLE_POINT_SYSTEM_NOTES   = "\"points-notes\"";
    public static final String STYLE_GENERAL_TABLE        = "\"main-table\"";
    
    public static final String STYLE_LOOT_TIME            = "\"loot-time\"";
    
    public static final String STYLE_TABLE_MAIN           = "\"main\"";
    public static final String STYLE_TABLE_FORMULA        = "\"formula\"";
    public static final String STYLE_TABLE_LOOT           = "\"loot\"";

    public static final String STYLE_TH_MAIN_PLAYER_NAME  = "\"main-player_name\"";
    public static final String STYLE_TH_MAIN_PLAYER_CLASS = "\"main-player_class\"";
    public static final String STYLE_TH_MAIN_JOINS        = "\"main-joins\"";
    public static final String STYLE_TH_MAIN_LEAVES       = "\"main-leaves\"";
    public static final String STYLE_TH_MAIN_LOOT         = "\"main-loot\"";
    public static final String STYLE_TH_MAIN_FORMULA      = "\"main-formula\"";
    public static final String STYLE_TH_MAIN_POINTS       = "\"main-points\"";

    public static final String STYLE_TD_MAIN_PLAYER_NAME  = "\"main-player_name\"";
    public static final String STYLE_TD_MAIN_PLAYER_CLASS = "\"main-player_class\"";
    public static final String STYLE_TD_MAIN_JOINS        = "\"main-joins\"";
    public static final String STYLE_TD_MAIN_LEAVES       = "\"main-leaves\"";
    public static final String STYLE_TD_MAIN_LOOT         = "\"main-loot\"";
    public static final String STYLE_TD_MAIN_FORMULA      = "\"main-formula\"";
    public static final String STYLE_TD_MAIN_POINTS       = "\"main-points\"";
    public static final String STYLE_TD_MAIN_POINTS_TOTAL = "\"main-points_total\"";

    public static final String STYLE_TR_FORMULA_ROW       = "\"formula-row\"";
    public static final String STYLE_TD_FORMULA_VALUE     = "\"formula-value\"";
    public static final String STYLE_TD_FORMULA_TYPE_DKP  = "\"formula-type_dkp\"";
    public static final String STYLE_TD_FORMULA_TYPE_COST = "\"formula-type_cost\"";
    
    
    // Used for inline styles.
    // (Inline styles are only used within Swing components,
    //  the actual HTML output uses an internal sytle sheet
    //  read in from an actual css file.)
    public static final String FONT_RAID_TABLE_CELLS   = "font-size: x-small;";
    public static final String ALIGN_LEFT              = "text-align: left;";
    public static final String MARGIN_SMALL            = "margin: 1px;";
    
    
    public static void initializeInternalStyleSheet() {
        File templateFile = new File(Persistence.TEMPLATE_OUTPUT_CSS_FILE);
        File customFile   = new File(Persistence.CUSTOM_OUTPUT_CSS_FILE);
        if (customFile.exists()) {
            INTERNAL_STYLE_SHEET = FileIOUtilities.readFileToString(customFile);
        } else if (templateFile.exists()) {
            // The custom file may not exist,
            // so just read from the template file.
            INTERNAL_STYLE_SHEET = FileIOUtilities.readFileToString(templateFile);
        }
    }
    
    public static void revertStyles() {
        // The user can always revert style changes by deleting the custom
        // style sheet entirely.
        File customFile = new File(Persistence.CUSTOM_OUTPUT_CSS_FILE);
        if (customFile.exists()) {
            customFile.delete();
            initializeInternalStyleSheet();
        }
    }
    
    public static boolean updateInternalStyleSheet(String text) {
        // We have customized our style sheet.
        // We save to a custom css file (or create one if it does not exist).
        INTERNAL_STYLE_SHEET = text;
        File customFile = new File(Persistence.CUSTOM_OUTPUT_CSS_FILE);
        return FileIOUtilities.writeStringToFile(INTERNAL_STYLE_SHEET, customFile);
    }
}
