package org.wow.autodkp.view.preferences;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import org.wow.autodkp.view.BasicDialog;


/**
 * 
 * 
 * @author jsandusky
 */
public abstract class AbstractPreferencesDialog extends BasicDialog {
    
    protected PreferencesItem mPreferencesItem;
    
    
    public AbstractPreferencesDialog(Frame owner, String title, PreferencesItem preferencesItem) {
        super(owner, title, true);
        initialize(owner, preferencesItem);
    }
    
    public AbstractPreferencesDialog(Dialog owner, String title, PreferencesItem preferencesItem) {
        super(owner, title, true);
        initialize(owner, preferencesItem);
    }
    
    protected void initialize(Component c, PreferencesItem preferencesItem) {
        mPreferencesItem = preferencesItem;
        addBasicSupport();
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
        ActionListener listener = null;
        if (mPreferencesItem == null) {
            listener = getAddActionListener();
        } else {
            listener = getEditActionListener();
        }
        
        setLayout(new BorderLayout());
        getContentPane().add(getMainPane(), BorderLayout.CENTER);
        getContentPane().add(getButtonPane(listener), BorderLayout.SOUTH);

        setResizable(false);
        setSize(400, 150);
        setLocationRelativeTo(c);
    }
    
    protected abstract ActionListener getAddActionListener();
    
    protected abstract ActionListener getEditActionListener();
    
    protected abstract JComponent getMainPane();
    
    protected JComponent getButtonPane(ActionListener actionListener) {
        JButton applyButton = new JButton();
        if (mPreferencesItem == null) {
            applyButton.setText("Add");
        } else {
            applyButton.setText("Edit");
        }
        applyButton.addActionListener(actionListener);
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        
        JPanel buttonPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttonPane.add(applyButton);
        buttonPane.add(cancelButton);
        buttonPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        
        getRootPane().setDefaultButton(applyButton);
        
        return buttonPane;
    }
}
