package org.wow.autodkp.view.preferences;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.wow.autodkp.model.AlternatesGroup;
import org.wow.autodkp.persistence.AlternatesManager;

/**
 * 
 * 
 * @author jsandusky
 */
public class AlternatesTab extends JPanel implements IPreferencesTab {

    private JDialog mOwner;
    private JList mPlayersList;
    private JList mAlternatesList;
    
    private String mSelectedPlayer;
    private String mSelectedAlternate;
    private boolean mIsModified;
    
    private JSplitPane mMainPane;
    private JPanel mDetailPane;

    
    public AlternatesTab(JDialog owner) {
        mOwner = owner;
        
        List items = new ArrayList();
        Collection allGroupings = AlternatesManager.getDefault().getAllAlternatesGroupings();
        for (Iterator iter=allGroupings.iterator(); iter.hasNext();) {
            AlternatesGroup group = (AlternatesGroup) iter.next();
            items.add(new AlternateItem(group.getPrimaryPlayerName(), group.getPrimaryPlayerClass()));
        }
        mPlayersList = new JList(items.toArray());
        JComponent constraintsPane = PreferencesWindow.generatePane(
                "Players", 
                getAddPlayerListener(),
                null,
                getDeletePlayerListener(),
                getPlayerSelectionListener(),
                mPlayersList);

        mAlternatesList = new JList();
        JComponent alterantesPane = PreferencesWindow.generatePane(
                "Alternates", 
                getAddAlternateListener(),
                null,
                getDeleteAlternateListener(),
                getAlternateSelectionListener(),
                mAlternatesList);
        
        mDetailPane = new JPanel(new BorderLayout());
        mDetailPane.add(alterantesPane, BorderLayout.CENTER);
        mDetailPane.setVisible(false);
        
        mMainPane = new JSplitPane();
        mMainPane.setLeftComponent(constraintsPane);
        mMainPane.setRightComponent(mDetailPane);
        mMainPane.setDividerLocation(250);
        mMainPane.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(5, 5, 5, 10),
                mMainPane.getBorder()));

        setLayout(new BorderLayout(0, 0));
        add(mMainPane, BorderLayout.CENTER);
    }
    
    
    public String getName() {
        return "Alternates";
    }
    
    public boolean shouldReload() {
        return mIsModified;
    }
    
    public void closing() {
        if (mIsModified) {
            AlternatesManager.getDefault().save();
        }
    }

    public JComponent getComponent() {
        return this;
    }

    private void updateAlternates() {
        List groupings = AlternatesManager.getDefault().getAlternatesForPlayer(mSelectedPlayer);
        if (groupings != null) {
            mAlternatesList.setListData(groupings.toArray());
        } else {
            mAlternatesList.setListData(new Object[0]);
        }
        mSelectedAlternate = null;
    }
    
    private ActionListener getAddPlayerListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DialogPlayerNameAndClass diag = new DialogPlayerNameAndClass(
                        mOwner,
                        "Add Primary Player",
                        null);
                diag.setVisible(true);
                AlternateItem item = diag.getAlternateItem();
                if (item != null) {
                    String newPrimaryPlayerName  = item.getPlayerName();
                    String newPrimaryPlayerClass = item.getPlayerClass();
                    AlternatesManager.getDefault().addPrimaryPlayer(
                            newPrimaryPlayerName, 
                            newPrimaryPlayerClass);
                    mIsModified = true;
                    
                    ListModel model = mPlayersList.getModel();
                    List playersList = new ArrayList();
                    for (int i=0; i < model.getSize(); i++) {
                        AlternateItem iterItem = (AlternateItem) model.getElementAt(i);
                        String iterName  = iterItem.getPlayerName();
                        if (iterName.equals(newPrimaryPlayerName)) {
                            return;
                        }
                        playersList.add(iterItem);
                    }
                    playersList.add(new AlternateItem(
                            newPrimaryPlayerName,
                            newPrimaryPlayerClass));
                    mPlayersList.setListData(playersList.toArray());
                    mPlayersList.setSelectedIndex(mPlayersList.getModel().getSize() - 1);
                }
            }
        };
    }
    
    private ActionListener getDeletePlayerListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mSelectedPlayer != null) {
                    int result = JOptionPane.showConfirmDialog(
                            mOwner, 
                            "Are you sure you wish to delete Alternates settings for " + mSelectedPlayer + "?\n" + 
                            "This cannot be undone.",
                            "Delete Alternates Settings",
                            JOptionPane.OK_CANCEL_OPTION,
                            JOptionPane.WARNING_MESSAGE);
                    if (result == JOptionPane.YES_OPTION) {
                        AlternatesManager.getDefault().removePrimaryPlayer(mSelectedPlayer);
                        mIsModified = true;
                        
                        ListModel model = mPlayersList.getModel();
                        List playersList = new ArrayList();
                        for (int i=0; i < model.getSize(); i++) {
                            AlternateItem iterItem = (AlternateItem) model.getElementAt(i);
                            String iterName  = iterItem.getPlayerName();
                            if (iterName.equals(mSelectedPlayer)) {
                                continue;
                            }
                            playersList.add(iterItem);
                        }
                        mPlayersList.setListData(playersList.toArray());
                        
                        mDetailPane.setVisible(false);
                    }
                }
            }
        };
    }
    
    private ListSelectionListener getPlayerSelectionListener() {
        return new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    AlternateItem item = (AlternateItem) mPlayersList.getSelectedValue();
                    if (item != null) {
                        mSelectedPlayer = item.getPlayerName();
                        updateAlternates();
                        mDetailPane.setVisible(true);
                        mMainPane.setDividerLocation(250);
                    } else {
                        mSelectedPlayer = null;
                    }
                }
            }
        };
    }
    
    private ActionListener getAddAlternateListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DialogAlternate diag = new DialogAlternate(
                        mOwner,
                        "Add Alternate",
                        null);
                diag.setVisible(true);
                AlternateItem item = diag.getAlternateItem();
                if (item != null) {
                    AlternatesManager.getDefault().addAlternateForPlayer(
                            mSelectedPlayer,
                            item.getPlayerName());
                    mIsModified = true;
                    updateAlternates();
                }
            }
        };
    }
    
    private ActionListener getDeleteAlternateListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mSelectedAlternate != null) {
                    AlternatesManager.getDefault().removeAlternateForPlayer(
                            mSelectedPlayer,
                            mSelectedAlternate);
                    mIsModified = true;
                    updateAlternates();
                }
            }
        };
    }
    
    private ListSelectionListener getAlternateSelectionListener() {
        return new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    mSelectedAlternate = (String) mAlternatesList.getSelectedValue();
                }
            }
        };
    }
}
