package org.wow.autodkp.view.preferences;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;

import org.wow.autodkp.persistence.FileIOUtilities;
import org.wow.autodkp.persistence.Persistence;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.html.CSSStyleEditor;
import org.wow.autodkp.view.html.HTMLViewPane;


/**
 * 
 * 
 * @author jsandusky
 */
public class HTMLTab extends JPanel implements IPreferencesTab {

    private JDialog mOwner;
    private PropertyTableModel mTableModel;
    private JTable mTable;
    private String mOriginalNotes = "";
    private HTMLViewPane mNotesEditor;
    private boolean mIsRaidDateModified;

    
    public HTMLTab(JDialog owner) {
        mOwner = owner;
        
        setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        
        initTablePane();
        JPanel tablePane = new JPanel(new BorderLayout());
        tablePane.add(new JScrollPane(mTable), BorderLayout.CENTER);
        tablePane.add(getRaidDatePane(), BorderLayout.SOUTH);
        
        JComponent stylePane = getStylePane();
        JComponent generalNotesPane = getGeneralNotesPane();
        
        setLayout(new BorderLayout(0, 10));
        add(stylePane,        BorderLayout.NORTH);
        add(tablePane,        BorderLayout.CENTER);
        add(generalNotesPane, BorderLayout.SOUTH);
    }
    
    
    private JComponent getRaidDatePane() {
        final GeneralProperties props = GeneralProperties.getDefault();
        
        final JCheckBox dateCheckBox = new JCheckBox("Include Raid Creation Date?");
        dateCheckBox.setOpaque(false);
        dateCheckBox.setBorder(BorderFactory.createEmptyBorder(10, 5, 5, 5));
        dateCheckBox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                props.setBooleanProperty(props.HTML_INCLUDE_RAID_DATE, 
                        dateCheckBox.isSelected());
                mIsRaidDateModified = true;
            }
        });
        
        if (!props.hasProperty(props.HTML_INCLUDE_RAID_DATE)) {
            props.setBooleanProperty(props.HTML_INCLUDE_RAID_DATE, new Boolean(true));
            mIsRaidDateModified = true;
        }
        dateCheckBox.setSelected(props.getBooleanProperty(props.HTML_INCLUDE_RAID_DATE));

        return dateCheckBox;
    }
    
    private void initTablePane() {
        GeneralProperties props = GeneralProperties.getDefault();
        List propertyItems = new ArrayList();
        propertyItems.add(new PropertyItem(props.COLUMN_TRANSLATION_NAME,    "Player"));
        propertyItems.add(new PropertyItem(props.COLUMN_TRANSLATION_CLASS,   "Class"));
        propertyItems.add(new PropertyItem(props.COLUMN_TRANSLATION_JOINS,   "Joins"));
        propertyItems.add(new PropertyItem(props.COLUMN_TRANSLATION_LEAVES,  "Leaves"));
        propertyItems.add(new PropertyItem(props.COLUMN_TRANSLATION_LOOT,    "Loot"));
        propertyItems.add(new PropertyItem(props.COLUMN_TRANSLATION_PTS,     "Pts"));
        propertyItems.add(new PropertyItem(props.COLUMN_TRANSLATION_FORMULA, "Formula"));
        propertyItems.add(new PropertyItem(props.COLUMN_TRANSLATION_TOTAL,   "Total"));
        mTableModel = new PropertyTableModel(propertyItems, "Name", "Alternate Name");
        mTable = new JTable(mTableModel);
        
        JLabel renderer = (JLabel) mTable.getDefaultRenderer(String.class);
        renderer.setHorizontalAlignment(SwingConstants.CENTER);
        
        mTable.getColumnModel().getColumn(0).setPreferredWidth(100);
        mTable.getColumnModel().getColumn(1).setPreferredWidth(200);
    }
    
    private JComponent getStylePane() {
        JButton styleButton = new JButton("Open CSS Style Editor");
        styleButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                CSSStyleEditor editor = new CSSStyleEditor(mOwner);
                editor.setVisible(true);
            }
        });
        
        Box styleBox = Box.createHorizontalBox();
        styleBox.add(new JLabel("HTML Output Styling (in Cascading Style Sheet format):"));
        styleBox.add(Box.createHorizontalStrut(10));
        styleBox.add(styleButton);
        styleBox.add(Box.createHorizontalGlue());
        
        JPanel stylePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        stylePanel.add(styleBox);
        
        return stylePanel;
    }
    
    private JComponent getGeneralNotesPane() {
        File notesFile = new File(
                Persistence.DEFINITIONS_DIR + 
                File.separator + 
                Persistence.NOTES_DIR + 
                File.separator + 
                Persistence.HTML_NOTES_FILE);
        if (notesFile.exists()) {
            String notes = FileIOUtilities.readFileToString(notesFile);
            if (notes != null && notes.length() > 0) {
                mOriginalNotes = notes;
            }
        }
        mNotesEditor = new HTMLViewPane(mOriginalNotes);
        mNotesEditor.setPreferredSize(new Dimension(
                mNotesEditor.getPreferredSize().width, 
                180));
        
        JPanel panel = new JPanel(new BorderLayout());
        panel.add(new JScrollPane(mNotesEditor), BorderLayout.CENTER);
        panel.setBorder(BorderFactory.createTitledBorder("General Notes"));
        return panel;
    }
    

    public JComponent getComponent() {
        return this;
    }
    
    public String getName() {
        return "HTML";
    }
    
    public void closing() {
        String newNotes = mNotesEditor.getText();
        if (!newNotes.equals(mOriginalNotes)) {
            // User changed the notes. Save the changes.
            File notesFile = new File(
                    Persistence.DEFINITIONS_DIR + 
                    File.separator + 
                    Persistence.NOTES_DIR + 
                    File.separator + 
                    Persistence.HTML_NOTES_FILE);
            notesFile.getParentFile().mkdirs();
            boolean canWrite = 
                FileIOUtilities.writeStringToFile(newNotes, notesFile);
            if (!canWrite) {
                JOptionPane.showMessageDialog(
                        this,
                        "Error occurred while saving notes.",
                        "General Notes",
                        JOptionPane.ERROR_MESSAGE);
            }
        }
        if (mIsRaidDateModified) {
            GeneralProperties.getDefault().save();
        }
    }
    
    public boolean shouldReload() {
        return mTableModel.isModified();
    }
}
