package org.wow.autodkp.view.preferences;

import java.util.List;

import javax.swing.table.AbstractTableModel;


/**
 * 
 * 
 * @author jsandusky
 */
public class PropertyTableModel extends AbstractTableModel {

    private List mPropertyItems;
    private boolean mIsModified;
    private String mNameColumnName;
    private String mValueColumnName;

    
    public PropertyTableModel(
            List propertyItems, 
            String nameColumnName, 
            String valueColumnName)
    {
        mPropertyItems = propertyItems;
        mNameColumnName = nameColumnName;
        mValueColumnName = valueColumnName;
    }

    
    public int getColumnCount() {
        return 2;
    }

    public int getRowCount() {
        return mPropertyItems.size();
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return columnIndex == 1;
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        PropertyItem propertyItem = (PropertyItem) mPropertyItems.get(rowIndex);
        if (columnIndex == 0) {
            return propertyItem.getName();
        }
        return getProperty(propertyItem);
    }

    public String getColumnName(int column) {
        if (column == 0) {
            return mNameColumnName;
        }
        return mValueColumnName;
    }

    public boolean isModified() {
        return mIsModified;
    }

    protected String getProperty(PropertyItem propertyItem) {
        return propertyItem.getValue();
    }
    
    protected void setProperty(PropertyItem propertyItem, String newValue) {
        propertyItem.setProperty(newValue.trim());
    }
    
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        mIsModified = true;
        PropertyItem propertyItem = (PropertyItem) mPropertyItems.get(rowIndex);
        String newValue = (String) aValue;
        setProperty(propertyItem, newValue.trim());
    }
}
