package org.wow.autodkp.view.preferences.pointsystems;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JList;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.wow.autodkp.model.points.Adjustment;
import org.wow.autodkp.view.preferences.PointSystemsTab;
import org.wow.autodkp.view.preferences.PreferencesWindow;


public class AdjustmentsSection extends AbstractPointSystemsSection {

    private JList mAdjustmentList;
    private AdjustmentItem mCurrentAdjustmentItem;
    private ActionListener mEditAdjustmentListener;

    
    public AdjustmentsSection(PointSystemsTab tab) {
        super(tab);
        
        mAdjustmentList = new JList();
        mAdjustmentList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() > 1 && mAdjustmentList.isEnabled()) {
                    Point point = e.getPoint();
                    int index = mAdjustmentList.locationToIndex(point);
                    if (index >= 0) {
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                getEditAdjustmentListener().actionPerformed(null);
                            }
                        });
                    }
                }
            }
        });
        
        mTabComponent = PreferencesWindow.generatePane(
                "Adjustments", 
                getAddAdjustmentListener(),
                getEditAdjustmentListener(),
                getDeleteAdjustmentListener(),
                getAdjustmentSelectionListener(),
                mAdjustmentList);
    }
    
    public void pointSystemChanged() {
        updateAdjustments();
    }
    
    private ActionListener getAddAdjustmentListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DialogAdjustment diag = new DialogAdjustment(
                        mPointSystemsTab.getOwner(),
                        "Add New Adjustment Column",
                        null);
                diag.setVisible(true);
                AdjustmentItem adjustment = diag.getAdjustmentItem();
                if (adjustment != null) {
                    Map adjustmentMap = 
                        mPointSystemsTab.getSelectedPointSystem().getAdjustmentMap();
                    adjustmentMap.put(
                            adjustment.getID(),
                            adjustment.getAdjustment());
                    mPointSystemsTab.setCurrentPointSystemModified(true);
                    updateAdjustments();
                }
            }
        };
    }
    
    private ActionListener getEditAdjustmentListener() {
        if (mEditAdjustmentListener == null) {
            mEditAdjustmentListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (mCurrentAdjustmentItem != null) {
                        DialogAdjustment diag = new DialogAdjustment(
                                mPointSystemsTab.getOwner(),
                                "Edit Adjustment Column",
                                mCurrentAdjustmentItem);
                        diag.setVisible(true);
                        AdjustmentItem adjustment = diag.getAdjustmentItem();
                        if (adjustment != null) {
                            Map adjustmentMap = 
                                mPointSystemsTab.getSelectedPointSystem().getAdjustmentMap();
                            adjustmentMap.put(
                                    adjustment.getID(),
                                    adjustment.getAdjustment());
                            mPointSystemsTab.setCurrentPointSystemModified(true);
                            updateAdjustments();
                        }
                    }
                }
            };
        }
        return mEditAdjustmentListener;
    }
    
    private ActionListener getDeleteAdjustmentListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mCurrentAdjustmentItem != null) {
                    mPointSystemsTab.getSelectedPointSystem().getAdjustmentMap().remove(
                            mCurrentAdjustmentItem.getID());
                    mPointSystemsTab.setCurrentPointSystemModified(true);
                    updateAdjustments();
                }
            }
        };
    }
    
    private ListSelectionListener getAdjustmentSelectionListener() {
        return new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    mCurrentAdjustmentItem = 
                        (AdjustmentItem) mAdjustmentList.getSelectedValue();
                }
            }
        };
    }
    
    private void updateAdjustments() {
        Map adjustmentMap = mPointSystemsTab.getSelectedPointSystem().getAdjustmentMap();
        AdjustmentItem[] adjustmentItems = new AdjustmentItem[adjustmentMap.size()];
        int i=0;
        for (Iterator iter=adjustmentMap.keySet().iterator(); iter.hasNext(); i++) {
            String id = (String) iter.next();
            Adjustment adjustment = (Adjustment) adjustmentMap.get(id);
            adjustmentItems[i] = new AdjustmentItem(
                    id, 
                    adjustment.getName(), 
                    adjustment.getInitialValue());
        }
        mAdjustmentList.setListData(adjustmentItems);
        mCurrentAdjustmentItem = null;
    }
    
}
