package org.wow.autodkp.view.preferences.pointsystems;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.view.preferences.PointSystemsTab;
import org.wow.autodkp.view.preferences.PreferencesWindow;


public class BossesSection extends AbstractPointSystemsSection {

    private JList mBossPointList;
    private BossItem mCurrentBossItem;
    private ActionListener mEditBossPointListener;
    private JSpinner mGenericBossPointsSpinner;


    public BossesSection(PointSystemsTab tab) {
        super(tab);
        
        mBossPointList = new JList();
        mBossPointList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() > 1 && mBossPointList.isEnabled()) {
                    Point point = e.getPoint();
                    int index = mBossPointList.locationToIndex(point);
                    if (index >= 0) {
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                getEditBossPointListener().actionPerformed(null);
                            }
                        });
                    }
                }
            }
        });
        
        mTabComponent = PreferencesWindow.generatePane(
                "Custom Boss Points", 
                getAddBossPointListener(),
                getEditBossPointListener(),
                getDeleteBossPointListener(),
                getBossPointSelectionListener(),
                mBossPointList);
        JLabel genericBossPointsLabel = new JLabel("Generic Boss Points:  ");
        mGenericBossPointsSpinner = new JSpinner();
        Dimension spinnerSize = new Dimension(
                80, 
                mGenericBossPointsSpinner.getPreferredSize().height);
        mGenericBossPointsSpinner.setPreferredSize(spinnerSize);
        mGenericBossPointsSpinner.setMaximumSize(spinnerSize);
        mGenericBossPointsSpinner.setModel(new SpinnerNumberModel(0.0, 0.0, Double.MAX_VALUE, 1.0));
        mGenericBossPointsSpinner.addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent e) {
                Number value = (Number) mGenericBossPointsSpinner.getValue();
                mPointSystemsTab.getSelectedPointSystem().setStandardPointsForBoss(
                        value.floatValue());
                mPointSystemsTab.setCurrentPointSystemModified(true);
            }
        });
        Box bossBox = Box.createHorizontalBox();
        bossBox.add(genericBossPointsLabel);
        bossBox.add(Box.createHorizontalStrut(5));
        bossBox.add(mGenericBossPointsSpinner);
        bossBox.add(Box.createHorizontalGlue());
        bossBox.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        mTabComponent.add(bossBox, BorderLayout.SOUTH);
    }
    
    public void pointSystemChanged() {
        IPointSystem pointSystem = mPointSystemsTab.getSelectedPointSystem();
        mGenericBossPointsSpinner.setValue(new Double(
                pointSystem.getStandardPointsForBoss()));
        updateBosses();
    }

    private ActionListener getAddBossPointListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DialogBossPoint diag = new DialogBossPoint(
                        mPointSystemsTab.getOwner(),
                        "Add Custom Boss Points",
                        null);
                diag.setVisible(true);
                BossItem bossItem = diag.getBossItem();
                if (bossItem != null) {
                    mPointSystemsTab.getSelectedPointSystem().addBossConstraint(
                            bossItem.getName(), 
                            bossItem.getCost());
                    mPointSystemsTab.setCurrentPointSystemModified(true);
                    updateBosses();
                }
            }
        };
    }
    
    private ActionListener getEditBossPointListener() {
        if (mEditBossPointListener == null) {
            mEditBossPointListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (mCurrentBossItem != null) {
                        DialogBossPoint diag = new DialogBossPoint(
                                mPointSystemsTab.getOwner(),
                                "Edit Custom Boss Points",
                                mCurrentBossItem);
                        diag.setVisible(true);
                        BossItem bossItem = diag.getBossItem();
                        if (bossItem != null) {
                            mPointSystemsTab.getSelectedPointSystem().setBossConstraintCost(
                                    bossItem.getName(), 
                                    bossItem.getCost());
                            mPointSystemsTab.setCurrentPointSystemModified(true);
                            updateBosses();
                        }
                    }
                }
            };
        }
        return mEditBossPointListener;
    }
    
    private ActionListener getDeleteBossPointListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mCurrentBossItem != null) {
                    mPointSystemsTab.getSelectedPointSystem().removeBossConstraint(
                            mCurrentBossItem.getName());
                    mPointSystemsTab.setCurrentPointSystemModified(true);
                    updateBosses();
                }
            }
        };
    }
    
    private ListSelectionListener getBossPointSelectionListener() {
        return new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    mCurrentBossItem = 
                        (BossItem) mBossPointList.getSelectedValue();
                }
            }
        };
    }
    
    private void updateBosses() {
        IPointSystem pointSystem = mPointSystemsTab.getSelectedPointSystem();
        int itemCount = pointSystem.getBossConstraintNames().size();
        BossItem[] items = new BossItem[itemCount];
        for (int i=0; i < itemCount; i++) {
            items[i] = new BossItem(
                    (String) pointSystem.getBossConstraintNames().get(i),
                    (Cost)   pointSystem.getBossConstraintCosts().get(i));
        }
        mBossPointList.setListData(items);
        mCurrentBossItem = null;
    }
}
