package org.wow.autodkp.view.preferences.pointsystems;

import java.awt.Color;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;

import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.persistence.xml.XMLUtilities;
import org.wow.autodkp.view.preferences.AbstractPreferencesDialog;

/**
 * 
 * 
 * @author jsandusky
 */
public class DialogAdjustment extends AbstractPreferencesDialog {
    
    private JLabel mNameLabel;
    private JLabel mInitialValueLabel;
    private JTextField mNameField;
    private JSpinner mInitialValueSpinner;
    private AdjustmentItem mResult;
    
    
    public DialogAdjustment(Dialog owner, String title, AdjustmentItem adjustmentItem) {
        super(owner, title, adjustmentItem);
    }
    
    protected ActionListener getAddActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String name = mNameField.getText().trim();
                if (name.length() == 0 || !XMLUtilities.isValidString(name)) {
                    mNameLabel.setForeground(Color.RED);
                    return;
                }
                String id = LootConstants.generateTimeID();
                Number initialValue = (Number) mInitialValueSpinner.getValue();
                if (initialValue != null && initialValue.floatValue() > 0) {
                    mResult = new AdjustmentItem(id, name, initialValue.floatValue());
                } else {
                    mResult = new AdjustmentItem(id, name, 0.0f);
                }
                dispose();
            }
        };
    }
    
    protected ActionListener getEditActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String name = mNameField.getText().trim();
                if (name.length() == 0 || !XMLUtilities.isValidString(name)) {
                    mNameLabel.setForeground(Color.RED);
                    return;
                }
                String id = ((AdjustmentItem) mPreferencesItem).getID();
                Number initialValue = (Number) mInitialValueSpinner.getValue();
                if (initialValue != null && initialValue.floatValue() > 0) {
                    mResult = new AdjustmentItem(id, name, initialValue.floatValue());
                } else {
                    mResult = new AdjustmentItem(id, name, 0.0f);
                }
                dispose();
            }
        };
    }

    protected JComponent getMainPane() {
        
        mNameLabel = new JLabel("Column Name:");
        mNameLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
        mNameField = new JTextField(20);
        mInitialValueLabel = new JLabel("Initial Value");
        mInitialValueLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
        mInitialValueSpinner = new JSpinner(new SpinnerNumberModel(0.0, 0.0, Double.MAX_VALUE, 1.0));
        mInitialValueSpinner.setPreferredSize(
                new Dimension(80, mInitialValueSpinner.getPreferredSize().height));
        
        if (mPreferencesItem != null) {
            AdjustmentItem item = (AdjustmentItem) mPreferencesItem;
            String name = item.getName();
            mNameField.setText(name);
            mInitialValueSpinner.setValue(new Double(item.getInitialValue()));
        } else {
            mInitialValueSpinner.setValue(new Double(0));
        }
        
        JPanel mainPane = new JPanel();
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints gdc = new GridBagConstraints();
        mainPane.setLayout(gridBag);
        gdc.anchor = GridBagConstraints.SOUTHWEST;
        
        int y = -1;
        gdc.weighty = 1;
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mNameLabel, gdc);
        mainPane.add(mNameLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mNameField, gdc);
        mainPane.add(mNameField);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mInitialValueLabel, gdc);
        mainPane.add(mInitialValueLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mInitialValueSpinner, gdc);
        mainPane.add(mInitialValueSpinner);
        
        mainPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 5, 10));
        
        return mainPane;
    }
    
    public AdjustmentItem getAdjustmentItem() {
        return mResult;
    }
}
