package org.wow.autodkp.view.preferences.pointsystems;

import java.awt.Color;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.Border;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.persistence.BossManager;
import org.wow.autodkp.view.preferences.AbstractPreferencesDialog;

/**
 * 
 * 
 * @author jsandusky
 */
public class DialogBossPoint extends AbstractPreferencesDialog {
    
    private JLabel mBossLabel;
    private JComboBox mBossCombo;
    private JSpinner mBossPoints;
    private BossItem mResult;
    
    
    public DialogBossPoint(Dialog owner, String title, BossItem bossItem) {
        super(owner, title, bossItem);
    }
    
    protected ActionListener getAddActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String name = (String) mBossCombo.getSelectedItem();
                Number points = (Number) mBossPoints.getValue();
                mResult = new BossItem(name, new Cost(points.floatValue(), false));
                dispose();
            }
        };
    }
    
    protected ActionListener getEditActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String name = mBossLabel.getText();
                Number points = (Number) mBossPoints.getValue();
                mResult = new BossItem(name, new Cost(points.floatValue(), false));
                dispose();
            }
        };
    }

    protected JComponent getMainPane() {
        
        JLabel nameLabel = new JLabel("Boss:");
        JLabel pointsLabel = new JLabel("Points:");
        List filteredNames = new ArrayList(BossManager.getDefault().getNames());
        mBossPoints = new JSpinner();
        mBossPoints.setModel(new SpinnerNumberModel(0.0, 0.0, Double.MAX_VALUE, 1.0));
        mBossPoints.setPreferredSize(new Dimension(80, mBossPoints.getPreferredSize().height));
        BossItem bossItem = (BossItem) mPreferencesItem;
        JComponent bossNameComponent = null;
        if (mPreferencesItem != null) {
            mBossLabel = new JLabel(bossItem.getName());
            Border valueBorder = BorderFactory.createCompoundBorder(
                    BorderFactory.createEmptyBorder(0, 0, 0, 5),
                    BorderFactory.createMatteBorder(0, 0, 1, 0, Color.BLACK));
            mBossLabel.setBorder(valueBorder);
            bossNameComponent = mBossLabel;
            float value = bossItem.getCost().getCost();
            mBossPoints.setValue(new Double(value));
        } else {
            mBossCombo = new JComboBox(filteredNames.toArray());
            bossNameComponent = mBossCombo;
        }
        
        JPanel mainPane = new JPanel(new FlowLayout(FlowLayout.LEFT));
        mainPane.add(nameLabel);
        mainPane.add(bossNameComponent);
        JSeparator spacer = new JSeparator();
        spacer.setSize(10, 5);
        mainPane.add(spacer);
        mainPane.add(pointsLabel);
        mainPane.add(mBossPoints);
        mainPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        return mainPane;
    }
    
    public BossItem getBossItem() {
        return mResult;
    }
}
