package org.wow.autodkp.view.preferences.pointsystems;

import java.awt.Color;
import java.awt.Dialog;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.persistence.xml.XMLUtilities;
import org.wow.autodkp.view.preferences.AbstractPreferencesDialog;

/**
 * 
 * 
 * @author jsandusky
 */
public class DialogLootCategory extends AbstractPreferencesDialog {
    
    private JLabel mNameLabel;
    private JLabel mPointsLabel;
    private JTextField mNameField;
    private JTextField mCostField;
    private LootCategoryItem mResult;
    
    
    public DialogLootCategory(Dialog owner, String title, LootCategoryItem lootCategoryItem) {
        super(owner, title, lootCategoryItem);
    }
    
    protected ActionListener getAddActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mNameLabel.setForeground(Color.BLACK);
                mPointsLabel.setForeground(Color.BLACK);
                String name = mNameField.getText().trim();
                Cost cost = Cost.parseCost(mCostField.getText().trim());
                boolean isValid = true;
                if (name.length() == 0 || !XMLUtilities.isValidString(name)) {
                    isValid = false;
                    mNameLabel.setForeground(Color.RED);
                }
                if (cost == null) {
                    isValid = false;
                    mPointsLabel.setForeground(Color.RED);
                }
                if (isValid) {
                    String id = LootConstants.generateTimeID();
                    mResult = new LootCategoryItem(id, name, cost);
                    dispose();
                }
            }
        };
    }
    
    protected ActionListener getEditActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mNameLabel.setForeground(Color.BLACK);
                mPointsLabel.setForeground(Color.BLACK);
                String name = mNameField.getText().trim();
                Cost cost = Cost.parseCost(mCostField.getText().trim());
                boolean isValid = true;
                if (name.length() == 0 || !XMLUtilities.isValidString(name)) {
                    isValid = false;
                    mNameLabel.setForeground(Color.RED);
                }
                if (cost == null) {
                    isValid = false;
                    mPointsLabel.setForeground(Color.RED);
                }
                if (isValid) {
                    String id = ((LootCategoryItem) mPreferencesItem).getID();
                    mResult = new LootCategoryItem(id, name, cost);
                    dispose();
                }
            }
        };
    }

    protected JComponent getMainPane() {
        
        mNameLabel = new JLabel("Category Name:");
        mNameLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
        mPointsLabel = new JLabel("Cost:");
        mPointsLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
        mNameField = new JTextField(20);
        mCostField = new JTextField(10);
        if (mPreferencesItem != null) {
            LootCategoryItem lootCategoryItem = (LootCategoryItem) mPreferencesItem;
            mNameField.setText(lootCategoryItem.getCategoryName());
            mCostField.setText(lootCategoryItem.getCost().toString());
        }
        
        JPanel mainPane = new JPanel();
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints gdc = new GridBagConstraints();
        mainPane.setLayout(gridBag);
        gdc.anchor = GridBagConstraints.SOUTHWEST;
        
        int y = -1;
        gdc.weighty = 1;
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mNameLabel, gdc);
        mainPane.add(mNameLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mNameField, gdc);
        mainPane.add(mNameField);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mPointsLabel, gdc);
        mainPane.add(mPointsLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mCostField, gdc);
        mainPane.add(mCostField);
        
        mainPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 5, 10));
        
        return mainPane;
    }
    
    public LootCategoryItem getLootCategoryItem() {
        return mResult;
    }
}
