package org.wow.autodkp.view.preferences.pointsystems;

import java.awt.Color;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;

import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.RollCall;
import org.wow.autodkp.persistence.xml.XMLUtilities;
import org.wow.autodkp.view.preferences.AbstractPreferencesDialog;

/**
 * 
 * 
 * @author jsandusky
 */
public class DialogRollCall extends AbstractPreferencesDialog {
    
    private static final int MIN_RECURRENCE = 5;
    
    private IPointSystem mPointSystem;
    private JLabel mNameLabel;
    private JLabel mPointsLabel;
    private JLabel mRecurrenceLabel;
    private JTextField mNameField;
    private JSpinner mPointsSpinner;
    private JSpinner mRecurrenceSpinner;
    private RollCallItem mResult;
    private JCheckBox mHasRecurrence;
    
    
    public DialogRollCall(
            Dialog owner, 
            String title, 
            RollCallItem rollCallItem,
            IPointSystem pointSystem)
    {
        super(owner, title, rollCallItem);
        mPointSystem = pointSystem;
        
        if (rollCallItem != null) {
            String name = rollCallItem.getName();
            if (
                    !name.equals(RollCall.RAID_START_NAME) &&
                    !name.equals(RollCall.RAID_END_NAME)) {
                setSize(400, 180);
            }
        } else {
            setSize(400, 180);
        }
    }
    
    protected ActionListener getAddActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String name = mNameField.getText().trim();
                if (name.length() == 0 || !XMLUtilities.isValidString(name)) {
                    mNameLabel.setForeground(Color.RED);
                    return;
                }
                if (mPointSystem.getRollCall(name) != null) {
                    // Roll Call by this name already exists
                    mNameLabel.setForeground(Color.RED);
                    return;
                }
                Number pVal = (Number) mPointsSpinner.getValue();
                Number rVal = null;
                if (mHasRecurrence.isSelected()) {
                    rVal = (Number) mRecurrenceSpinner.getValue();
                }
                float points = 
                    pVal != null && pVal.floatValue() > 0 ? pVal.floatValue() : 0.0f;
                int recurrence = 
                    rVal != null && rVal.intValue() > 0 ? rVal.intValue() : 0;
                mResult = new RollCallItem(name, points, recurrence);
                dispose();
            }
        };
    }
    
    protected ActionListener getEditActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String name = mNameField.getText().trim();
                if (name.length() == 0 || !XMLUtilities.isValidString(name)) {
                    mNameLabel.setForeground(Color.RED);
                    return;
                }
                RollCallItem previousRollCallValue = (RollCallItem) mPreferencesItem;
                if (!name.equals(previousRollCallValue.getName())) {
                    if (mPointSystem.getRollCall(name) != null) {
                        // Roll Call by this name already exists
                        mNameLabel.setForeground(Color.RED);
                        return;
                    }
                }
                Number pVal = (Number) mPointsSpinner.getValue();
                Number rVal = null;
                if (mHasRecurrence.isSelected()) {
                    rVal = (Number) mRecurrenceSpinner.getValue();
                }
                float points = 
                    pVal != null && pVal.floatValue() > 0 ? pVal.floatValue() : 0.0f;
                int recurrence = 
                    rVal != null && rVal.intValue() > 0 ? rVal.intValue() : 0;
                mResult = new RollCallItem(name, points, recurrence);
                dispose();
            }
        };
    }

    protected JComponent getMainPane() {
        
        mNameLabel = new JLabel("Name:");
        mNameLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
        mNameField = new JTextField(20);
        mPointsLabel = new JLabel("Points:");
        mPointsLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
        mRecurrenceLabel = new JLabel("Recurrence:");
        mRecurrenceLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
        
        mPointsSpinner = new JSpinner(new SpinnerNumberModel(0.0, 0.0, Double.MAX_VALUE, 1.0));
        mPointsSpinner.setPreferredSize(
                new Dimension(60, mPointsSpinner.getPreferredSize().height));
        mRecurrenceSpinner = new JSpinner(new SpinnerNumberModel(
                MIN_RECURRENCE, 
                MIN_RECURRENCE, 
                Integer.MAX_VALUE, 
                1));
        mRecurrenceSpinner.setPreferredSize(
                new Dimension(60, mPointsSpinner.getPreferredSize().height));
        mHasRecurrence = new JCheckBox("Recurring Roll Call?");
        mHasRecurrence.setBorder(BorderFactory.createEmptyBorder(5, 0, 0, 0));
        mHasRecurrence.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mRecurrenceSpinner.setEnabled(mHasRecurrence.isSelected());
                mRecurrenceLabel.setEnabled(mHasRecurrence.isSelected());
            }
        });
        
        if (mPreferencesItem != null) {
            RollCallItem item = (RollCallItem) mPreferencesItem;
            String name = item.getName();
            mNameField.setText(name);
            mPointsSpinner.setValue(new Double(item.getPoints()));
            Integer recurrenceValue = new Integer(item.getRecurrence());
            if (recurrenceValue.intValue() < MIN_RECURRENCE) {
                recurrenceValue = new Integer(MIN_RECURRENCE);
            }
            mRecurrenceSpinner.setValue(recurrenceValue);
            mHasRecurrence.setSelected(item.hasRecurrence());
            mRecurrenceSpinner.setEnabled(item.hasRecurrence());
            mRecurrenceLabel.setEnabled(item.hasRecurrence());
            if (
                    name.equals(RollCall.RAID_START_NAME) ||
                    name.equals(RollCall.RAID_END_NAME)) {
                mNameField.setEditable(false);
                mRecurrenceLabel.setVisible(false);
                mHasRecurrence.setVisible(false);
                mRecurrenceSpinner.setVisible(false);
            }
        } else {
            mPointsSpinner.setValue(new Double(0));
            mRecurrenceSpinner.setValue(new Integer(MIN_RECURRENCE));
            mHasRecurrence.setSelected(false);
            mRecurrenceSpinner.setEnabled(false);
            mRecurrenceLabel.setEnabled(false);
        }
        
        JPanel mainPane = new JPanel();
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints gdc = new GridBagConstraints();
        mainPane.setLayout(gridBag);
        gdc.anchor = GridBagConstraints.SOUTHWEST;
        
        int y = -1;
        gdc.weighty = 1;
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mNameLabel, gdc);
        mainPane.add(mNameLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mNameField, gdc);
        mainPane.add(mNameField);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mPointsLabel, gdc);
        mainPane.add(mPointsLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mPointsSpinner, gdc);
        mainPane.add(mPointsSpinner);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 4;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mHasRecurrence, gdc);
        mainPane.add(mHasRecurrence);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mRecurrenceLabel, gdc);
        mainPane.add(mRecurrenceLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mRecurrenceSpinner, gdc);
        mainPane.add(mRecurrenceSpinner);
        
        mainPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 5, 10));
        
        return mainPane;
    }
    
    public RollCallItem getRollCallItem() {
        return mResult;
    }
}
