package org.wow.autodkp.view.preferences.pointsystems;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.persistence.xml.XMLUtilities;
import org.wow.autodkp.view.ImageUtilities;
import org.wow.autodkp.view.preferences.PointSystemsTab;


public class GeneralSection extends AbstractPointSystemsSection {

    private JTextField mNameField = new JTextField();
    private JLabel mKeyValue = new JLabel();
    
    private JCheckBox mDKPBossKill;
    private JCheckBox mDKPTimeBased;
    private JCheckBox mDKPZeroSum;
    private JRadioButton mDKPZeroSumDivideEvenly;
    private JRadioButton mDKPZeroSumExcludeLooter;

    
    public GeneralSection(PointSystemsTab tab) {
        super(tab);
        
        JLabel nameLabel = new JLabel("Name:");
        nameLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 10));
        mNameField = new JTextField();
        mNameField.getDocument().addDocumentListener(new DocumentListener() {
            public void removeUpdate(DocumentEvent e) {
                String text = mNameField.getText();
                if (XMLUtilities.isValidString(text)) {
                    mPointSystemsTab.getSelectedPointSystem().setName(mNameField.getText());
                    mNameField.setForeground(Color.BLACK);
                } else {
                    mNameField.setForeground(Color.RED);
                }
                mPointSystemsTab.setCurrentPointSystemModified(true);
            }
            public void insertUpdate(DocumentEvent e) {
                String text = mNameField.getText();
                if (XMLUtilities.isValidString(text)) {
                    mPointSystemsTab.getSelectedPointSystem().setName(mNameField.getText());
                    mNameField.setForeground(Color.BLACK);
                } else {
                    mNameField.setForeground(Color.RED);
                }
                mPointSystemsTab.setCurrentPointSystemModified(true);
            }
            public void changedUpdate(DocumentEvent e) {
            }
        });
        JLabel keyLabel = new JLabel("Key");
        keyLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 10));
        mKeyValue = new JLabel();
        JLabel typeLabel = new JLabel("Calculations:");
        typeLabel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 10));
        
        JComponent dkpPane = getDKPPane();
        
        Border valueBorder = BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(0, 0, 0, 5),
                BorderFactory.createMatteBorder(0, 0, 1, 0, Color.BLACK));
        mKeyValue.setBorder(valueBorder);
        
        mTabComponent = new JPanel();
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints gdc = new GridBagConstraints();
        mTabComponent.setLayout(gridBag);
        gdc.anchor = GridBagConstraints.SOUTHWEST;
        
        int y = -1;
        gdc.weighty = 1;
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(nameLabel, gdc);
        mTabComponent.add(nameLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mNameField, gdc);
        mTabComponent.add(mNameField);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(keyLabel, gdc);
        mTabComponent.add(keyLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mKeyValue, gdc);
        mTabComponent.add(mKeyValue);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 1;
        gdc.gridwidth = 4;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(dkpPane, gdc);
        mTabComponent.add(dkpPane);
        
        JSeparator separator = new JSeparator(JSeparator.VERTICAL);
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 1;
        gdc.gridwidth = 4;
        gdc.weighty = 8;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(separator, gdc);
        mTabComponent.add(separator);
        
        mTabComponent.setBorder(BorderFactory.createEmptyBorder(10, 5, 5, 5));
    }
    
    public void pointSystemChanged() {
        IPointSystem pointSystem = mPointSystemsTab.getSelectedPointSystem();
        mNameField.setText(pointSystem.getName());
        mKeyValue.setText(pointSystem.getID());
        
        mDKPBossKill.setSelected(pointSystem.isBossKill());
        mDKPTimeBased.setSelected(pointSystem.isTimeBased());
        boolean isZeroSum = pointSystem.isZeroSum();
        mDKPZeroSum.setSelected(isZeroSum);
        mDKPZeroSumExcludeLooter.setSelected(
                pointSystem.isZeroSumExcludeLooter());
        mDKPZeroSumExcludeLooter.setEnabled(isZeroSum);
        mDKPZeroSumDivideEvenly.setEnabled(isZeroSum);
        updateBossesTab(pointSystem.isBossKill());
        updateTimeTab(pointSystem.isTimeBased());
    }
    
    private JComponent getDKPPane() {
        mDKPBossKill  = new JCheckBox("Boss Kill");
        mDKPTimeBased = new JCheckBox("Time-based");
        mDKPZeroSum   = new JCheckBox("Zero Sum");
        mDKPZeroSumDivideEvenly  = new JRadioButton("Divide Evenly");
        mDKPZeroSumExcludeLooter = new JRadioButton("Exclude Looter");
        mDKPBossKill.setSelected(true);
        mDKPZeroSumDivideEvenly.setSelected(true);

        Image bossKillImage  = ImageUtilities.loadJarImage("bossKill.gif");
        Image timeBasedImage = ImageUtilities.loadJarImage("timeBased.gif");
        Image zeroSumImage   = ImageUtilities.loadJarImage("zeroSum.gif");
        JLabel bossKillLabel  = new JLabel(new ImageIcon(bossKillImage));
        JLabel timeBasedLabel = new JLabel(new ImageIcon(timeBasedImage));
        JLabel zeroSumLabel   = new JLabel(new ImageIcon(zeroSumImage));
        Dimension labelSize = new Dimension(20, 20);
        bossKillLabel.setPreferredSize(labelSize);
        bossKillLabel.setMinimumSize(labelSize);
        timeBasedLabel.setPreferredSize(labelSize);
        timeBasedLabel.setMinimumSize(labelSize);
        zeroSumLabel.setPreferredSize(labelSize);
        zeroSumLabel.setMinimumSize(labelSize);
        
        ButtonGroup zeroSumGroup = new ButtonGroup();
        zeroSumGroup.add(mDKPZeroSumDivideEvenly);
        zeroSumGroup.add(mDKPZeroSumExcludeLooter);
        
        Box bossKillPane = Box.createHorizontalBox();
        bossKillPane.add(bossKillLabel);
        bossKillPane.add(Box.createHorizontalStrut(5));
        bossKillPane.add(mDKPBossKill);
        // add extra space to allow for our label resizing above
        bossKillPane.add(Box.createHorizontalStrut(10));
        
        Box timeBasedPane = Box.createHorizontalBox();
        timeBasedPane.add(timeBasedLabel);
        timeBasedPane.add(Box.createHorizontalStrut(5));
        timeBasedPane.add(mDKPTimeBased);
        // add extra space to allow for our label resizing above
        timeBasedPane.add(Box.createHorizontalStrut(10));
        
        Box zeroSumPane = Box.createHorizontalBox();
        zeroSumPane.add(zeroSumLabel);
        zeroSumPane.add(Box.createHorizontalStrut(5));
        zeroSumPane.add(mDKPZeroSum);
        zeroSumPane.add(Box.createHorizontalStrut(10));
        zeroSumPane.add(mDKPZeroSumDivideEvenly);
        zeroSumPane.add(mDKPZeroSumExcludeLooter);
        // add extra space to allow for our label resizing above
        zeroSumPane.add(Box.createHorizontalStrut(10));
        
        Box dkpPane = Box.createVerticalBox();
        bossKillPane.setAlignmentX(JComponent.LEFT_ALIGNMENT);
        timeBasedPane.setAlignmentX(JComponent.LEFT_ALIGNMENT);
        zeroSumPane.setAlignmentX(JComponent.LEFT_ALIGNMENT);
        dkpPane.add(Box.createVerticalStrut(5));
        dkpPane.add(bossKillPane);
        dkpPane.add(Box.createVerticalStrut(5));
        dkpPane.add(timeBasedPane);
        dkpPane.add(Box.createVerticalStrut(5));
        dkpPane.add(zeroSumPane);
        dkpPane.setBorder(BorderFactory.createTitledBorder("DKP Systems"));
        
        mDKPBossKill.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean isSelected = mDKPBossKill.isSelected();
                if (!isSelected && !mDKPTimeBased.isSelected() && !mDKPZeroSum.isSelected()) {
                    // cannot allow for no DKP type
                    mDKPBossKill.setSelected(true);
                    showMinimumDKPDialog();
                    return;
                }
                updateBossesTab(isSelected);
                mPointSystemsTab.getSelectedPointSystem().setBossKill(isSelected);
                mPointSystemsTab.setCurrentPointSystemModified(true);
            }
        });
        mDKPTimeBased.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean isSelected = mDKPTimeBased.isSelected();
                if (!isSelected && !mDKPBossKill.isSelected() && !mDKPZeroSum.isSelected()) {
                    // cannot allow for no DKP type
                    mDKPTimeBased.setSelected(true);
                    showMinimumDKPDialog();
                    return;
                }
                updateTimeTab(isSelected);
                mPointSystemsTab.getSelectedPointSystem().setTimeBased(isSelected);
                mPointSystemsTab.setCurrentPointSystemModified(true);
            }
        });
        mDKPZeroSum.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                boolean isSelected = mDKPZeroSum.isSelected();
                if (!isSelected && !mDKPBossKill.isSelected() && !mDKPTimeBased.isSelected()) {
                    // cannot allow for no DKP type
                    mDKPZeroSum.setSelected(true);
                    showMinimumDKPDialog();
                    return;
                }
                mPointSystemsTab.getSelectedPointSystem().setZeroSum(
                        isSelected,
                        mDKPZeroSumExcludeLooter.isSelected());
                mDKPZeroSumExcludeLooter.setEnabled(isSelected);
                mDKPZeroSumDivideEvenly.setEnabled(isSelected);
                mPointSystemsTab.setCurrentPointSystemModified(true);
            }
        });
        
        return dkpPane;
    }
    
    private void updateBossesTab(boolean isShowing) {
        updateTab(
                PointSystemsTab.TAB_TITLE_BOSSES, 
                mPointSystemsTab.getBossesSection().getTabComponent(), 
                isShowing);
    }
    
    private void updateTimeTab(boolean isShowing) {
        updateTab(
                PointSystemsTab.TAB_TITLE_TIME, 
                mPointSystemsTab.getTimeSection().getTabComponent(), 
                isShowing);
    }
    
    private void updateTab(String title, JComponent pane, boolean isShowing) {
        int tabIndex = -1;
        for (int i=0; i < mPointSystemsTab.getTabbedPane().getTabCount(); i++) {
            String iterTitle = mPointSystemsTab.getTabbedPane().getTitleAt(i);
            if (iterTitle.equals(title)) {
                tabIndex = i;
                break;
            }
        }
        
        if (!isShowing && tabIndex >= 0) {
            mPointSystemsTab.getTabbedPane().removeTabAt(tabIndex);
        } else if (isShowing && tabIndex < 0) {
            mPointSystemsTab.getTabbedPane().addTab(title, pane);
        }
    }

    private void showMinimumDKPDialog() {
        JOptionPane.showMessageDialog(
                mPointSystemsTab, 
                "You must have at least one DKP System selected.", 
                "DKP Systems", 
                JOptionPane.INFORMATION_MESSAGE);
    }
}
