package org.wow.autodkp.view.preferences.pointsystems;

import java.awt.BorderLayout;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.persistence.FileIOUtilities;
import org.wow.autodkp.persistence.Persistence;
import org.wow.autodkp.view.html.HTMLViewPane;
import org.wow.autodkp.view.preferences.PointSystemsTab;


/**
 * 
 * 
 * @author jsandusky
 */
public class PointsHTMLTab extends AbstractPointSystemsSection {

    private String mOriginalNotes = "";
    private HTMLViewPane mNotesEditor;
    private IPointSystem mPreviousPointSystem;

    
    public PointsHTMLTab(PointSystemsTab tab) {
        super(tab);
        
        mTabComponent = new JPanel();
        mTabComponent.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        
        JComponent notesPane = getNotesPane();
        mTabComponent.setLayout(new BorderLayout(0, 10));
        mTabComponent.add(notesPane, BorderLayout.CENTER);
    }
    
    
    private JComponent getNotesPane() {
        
        mNotesEditor = new HTMLViewPane("");
        
        JPanel panel = new JPanel(new BorderLayout());
        panel.add(mNotesEditor, BorderLayout.CENTER);
        panel.setBorder(BorderFactory.createTitledBorder("Point System Notes"));
        return panel;
    }
    

    public void saveNotes(IPointSystem system) {
        String newNotes = mNotesEditor.getText();
        if (!newNotes.equals(mOriginalNotes)) {
            // User changed the notes. Save the changes.
            File notesFile = new File(
                    Persistence.DEFINITIONS_DIR + 
                    File.separator + 
                    Persistence.NOTES_DIR + 
                    File.separator + 
                    system.getID() + "_notes.txt");
            notesFile.getParentFile().mkdirs();
            boolean canWrite = 
                FileIOUtilities.writeStringToFile(newNotes, notesFile);
            if (!canWrite) {
                JOptionPane.showMessageDialog(
                        mPointSystemsTab,
                        "Error occurred while saving notes.",
                        "Point System Notes",
                        JOptionPane.ERROR_MESSAGE);
            }
        }
    }
    
    public void closing() {
        if (mPreviousPointSystem != null) {
            saveNotes(mPreviousPointSystem);
        }
    }
    
    public void pointSystemChanged() {
        if (mPreviousPointSystem != null) {
            saveNotes(mPreviousPointSystem);
        }
        IPointSystem system = mPointSystemsTab.getSelectedPointSystem();
        mPreviousPointSystem = system;
        
        File notesFile = new File(
                Persistence.DEFINITIONS_DIR + 
                File.separator + 
                Persistence.NOTES_DIR + 
                File.separator + 
                system.getID() + "_notes.txt");
        if (notesFile.exists()) {
            String notes = FileIOUtilities.readFileToString(notesFile);
            if (notes != null && notes.length() > 0) {
                mOriginalNotes = notes;
                mNotesEditor.initializeForText(notes);
            }
        } else {
            mOriginalNotes = "";
            mNotesEditor.initializeForText(mOriginalNotes);
        }
    }
}
