package org.wow.autodkp.view.preferences.pointsystems;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.RollCall;
import org.wow.autodkp.model.points.TimeThreshold;
import org.wow.autodkp.view.preferences.PointSystemsTab;
import org.wow.autodkp.view.preferences.PreferencesWindow;


public class PointsTimeSection extends AbstractPointSystemsSection {
    
    private static final String TYPE_PERCENT = "Percent of Raid Time";
    private static final String TYPE_MINUTES = "Minutes into the Raid";
    
    private RollCallItem mCurrentRollCallItem;
    private ActionListener mEditRollCallListener;
    
    private JList mRollCallsList;
    private JSpinner mPreThresholdPointsSpinner;
    private JSpinner mPreThresholdMinutesSpinner;
    private JTextField mThresholdField;
    private JSpinner mPostThresholdPointsSpinner;
    private JSpinner mPostThresholdMinutesSpinner;

    
    public PointsTimeSection(PointSystemsTab tab) {
        super(tab);
        
        mRollCallsList = new JList();
        mRollCallsList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() > 1 && mRollCallsList.isEnabled()) {
                    Point point = e.getPoint();
                    int index = mRollCallsList.locationToIndex(point);
                    if (index >= 0) {
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                getEditRollCallListener().actionPerformed(null);
                            }
                        });
                    }
                }
            }
        });
        
        mTabComponent = PreferencesWindow.generatePane(
                "Roll Call Points", 
                getAddRollCallListener(),
                getEditRollCallListener(),
                getDeleteRollCallListener(),
                getRollCallSelectionListener(),
                mRollCallsList);
        
        final JLabel thresholdLabel              = new JLabel("Threshold:");
        final JLabel preThresholdLabel           = new JLabel("Standard Points:");
        final JLabel thresholdDetailLabel        = new JLabel("Post-Threshold Points:");
        final JComboBox thresholdFieldTypeCombo  = new JComboBox(
                new String[] { TYPE_MINUTES, TYPE_PERCENT });
        final JLabel postThresholdPointsPerLabel = new JLabel("point(s) per");
        final JLabel postThresholdMinutesLabel   = new JLabel("minute(s)");
        preThresholdLabel.setPreferredSize(thresholdDetailLabel.getPreferredSize());
        thresholdLabel.setPreferredSize(thresholdDetailLabel.getPreferredSize());
        
        thresholdFieldTypeCombo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (e.getSource() == thresholdFieldTypeCombo) {
                    TimeThreshold threshold =
                        mPointSystemsTab.getSelectedPointSystem().getTimeThreshold();
                    if (thresholdFieldTypeCombo.getSelectedItem().equals(TYPE_PERCENT)) {
                        if (threshold == null) {
                            mThresholdField.setText("0%");
                        } else {
                            if (!threshold.isPercent()) {
                                threshold.setIsPercent(true);
                                mThresholdField.setText(mThresholdField.getText() + "%");
                            }
                        }
                    } else {
                        if (threshold == null) {
                            mThresholdField.setText("0");
                        } else {
                            if (threshold.isPercent()) {
                                threshold.setIsPercent(false);
                                String noPercent = mThresholdField.getText().replace("%", "");
                                mThresholdField.setText(noPercent);
                            }
                        }
                    }
                }
            }
        });
        
        mPreThresholdPointsSpinner = new JSpinner();
        mPreThresholdPointsSpinner.setPreferredSize(new Dimension(
                50, 
                mPreThresholdPointsSpinner.getPreferredSize().height));
        mPreThresholdPointsSpinner.setModel(new SpinnerNumberModel(0.0, 0.0, Double.MAX_VALUE, 1.0));
        mPreThresholdPointsSpinner.addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent e) {
                Number value = (Number) mPreThresholdPointsSpinner.getValue();
                mPointSystemsTab.getSelectedPointSystem().setDurationPoints(
                        value.floatValue());
                mPointSystemsTab.setCurrentPointSystemModified(true);
            }
        });

        mPreThresholdMinutesSpinner = new JSpinner();
        mPreThresholdMinutesSpinner.setPreferredSize(new Dimension(
                50, 
                mPreThresholdMinutesSpinner.getPreferredSize().height));
        mPreThresholdMinutesSpinner.setModel(new SpinnerNumberModel(0, 0, Integer.MAX_VALUE, 1));
        mPreThresholdMinutesSpinner.addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent e) {
                Number value = (Number) mPreThresholdMinutesSpinner.getValue();
                mPointSystemsTab.getSelectedPointSystem().setDurationMinutes(
                        value.intValue());
                mPointSystemsTab.setCurrentPointSystemModified(true);
            }
        });
        
        Box durationBox = Box.createHorizontalBox();
        durationBox.add(preThresholdLabel);
        durationBox.add(Box.createHorizontalStrut(10));
        durationBox.add(mPreThresholdPointsSpinner);
        durationBox.add(Box.createHorizontalStrut(5));
        durationBox.add(new JLabel("point(s) per"));
        durationBox.add(Box.createHorizontalStrut(10));
        durationBox.add(mPreThresholdMinutesSpinner);
        durationBox.add(Box.createHorizontalStrut(5));
        durationBox.add(new JLabel("minute(s)"));
        
        JPanel durationPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        durationPanel.add(durationBox);
        
        
        mThresholdField = new JTextField(4);
        mThresholdField.setDocument(new PlainDocument() {
            public void insertString(int offs, String str, AttributeSet a)
            throws BadLocationException {
                if (str.equals(".")) {
                    return;
                }
                super.insertString(offs, str, a);
            }
        });
        mThresholdField.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent e) {
            }
            public void removeUpdate(DocumentEvent e) {
                updateThreshold();
            }
            public void insertUpdate(DocumentEvent e) {
                updateThreshold();
            }
            private void updateThreshold() {
                mThresholdField.setForeground(Color.BLACK);
                thresholdLabel.setForeground(Color.BLACK);
                Cost cost = Cost.parseCost(mThresholdField.getText(), true);
                if (cost == null) {
                    mThresholdField.setForeground(Color.RED);
                    thresholdLabel.setForeground(Color.RED);
                } else {
                    int intValue = new Float(cost.getCost()).intValue();
                    TimeThreshold threshold = 
                        mPointSystemsTab.getSelectedPointSystem().getTimeThreshold();
                    if (threshold == null) {
                        threshold = new TimeThreshold();
                        mPointSystemsTab.getSelectedPointSystem().setTimeThreshold(threshold);
                    }
                    threshold.setValue(intValue);
                    threshold.setIsPercent(cost.isPercent());
                    if (cost.isPercent()) {
                        thresholdFieldTypeCombo.setSelectedItem(TYPE_PERCENT);
                    } else {
                        thresholdFieldTypeCombo.setSelectedItem(TYPE_MINUTES);
                    }
                    
                    boolean hasThreshold = cost.getCost() > 0;
                    thresholdDetailLabel.setVisible(hasThreshold);
                    mPostThresholdPointsSpinner.setVisible(hasThreshold);
                    postThresholdPointsPerLabel.setVisible(hasThreshold);
                    mPostThresholdMinutesSpinner.setVisible(hasThreshold);
                    postThresholdMinutesLabel.setVisible(hasThreshold);
                    if (hasThreshold) {
                        preThresholdLabel.setText("Pre-Threshold Points:");
                    } else {
                        preThresholdLabel.setText("Standard Points:");
                    }
                    mPointSystemsTab.setCurrentPointSystemModified(true);
                }
            }
        });
        
        mPostThresholdPointsSpinner = new JSpinner();
        mPostThresholdPointsSpinner.setPreferredSize(new Dimension(
                50, 
                mPostThresholdPointsSpinner.getPreferredSize().height));
        mPostThresholdPointsSpinner.setModel(new SpinnerNumberModel(0.0, 0.0, Double.MAX_VALUE, 1.0));
        mPostThresholdPointsSpinner.addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent e) {
                Number value = (Number) mPostThresholdPointsSpinner.getValue();
                TimeThreshold threshold = 
                    mPointSystemsTab.getSelectedPointSystem().getTimeThreshold();
                if (threshold == null) {
                    threshold = new TimeThreshold();
                    mPointSystemsTab.getSelectedPointSystem().setTimeThreshold(threshold);
                }
                threshold.setPoints(value.floatValue());
                mPointSystemsTab.setCurrentPointSystemModified(true);
            }
        });

        mPostThresholdMinutesSpinner = new JSpinner();
        mPostThresholdMinutesSpinner.setPreferredSize(new Dimension(
                50, 
                mPostThresholdMinutesSpinner.getPreferredSize().height));
        mPostThresholdMinutesSpinner.setModel(new SpinnerNumberModel(0, 0, Integer.MAX_VALUE, 1));
        mPostThresholdMinutesSpinner.addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent e) {
                Number value = (Number) mPostThresholdMinutesSpinner.getValue();
                TimeThreshold threshold = 
                    mPointSystemsTab.getSelectedPointSystem().getTimeThreshold();
                if (threshold == null) {
                    threshold = new TimeThreshold();
                    mPointSystemsTab.getSelectedPointSystem().setTimeThreshold(threshold);
                }
                threshold.setMinutes(value.intValue());
                mPointSystemsTab.setCurrentPointSystemModified(true);
            }
        });
        
        Box thresholdValueBox = Box.createHorizontalBox();
        thresholdValueBox.add(thresholdLabel);
        thresholdValueBox.add(Box.createHorizontalStrut(10));
        thresholdValueBox.add(mThresholdField);
        thresholdValueBox.add(Box.createHorizontalStrut(10));
        thresholdValueBox.add(thresholdFieldTypeCombo);
        
        JPanel thresholdValuePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        thresholdValuePanel.add(thresholdValueBox);
        
        Box thresholdDetailBox = Box.createHorizontalBox();
        thresholdDetailBox.add(thresholdDetailLabel);
        thresholdDetailBox.add(Box.createHorizontalStrut(10));
        thresholdDetailBox.add(mPostThresholdPointsSpinner);
        thresholdDetailBox.add(Box.createHorizontalStrut(5));
        thresholdDetailBox.add(postThresholdPointsPerLabel);
        thresholdDetailBox.add(Box.createHorizontalStrut(10));
        thresholdDetailBox.add(mPostThresholdMinutesSpinner);
        thresholdDetailBox.add(Box.createHorizontalStrut(5));
        thresholdDetailBox.add(postThresholdMinutesLabel);
        
        JPanel thresholdDetailPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        thresholdDetailPanel.add(thresholdDetailBox);

        
        Box vBox = Box.createVerticalBox();
        vBox.add(Box.createVerticalGlue());
        vBox.add(durationPanel);
        vBox.add(thresholdValuePanel);
        vBox.add(thresholdDetailPanel);
        vBox.add(Box.createVerticalGlue());
        vBox.setBorder(BorderFactory.createTitledBorder("Duration Points"));

        mTabComponent.add(vBox, BorderLayout.SOUTH);
    }
    
    public void pointSystemChanged() {
        IPointSystem pointSystem = mPointSystemsTab.getSelectedPointSystem();
        mPreThresholdPointsSpinner.setValue(new Double(pointSystem.getDurationPoints()));
        mPreThresholdMinutesSpinner.setValue(new Integer(pointSystem.getDurationMinutes()));
        TimeThreshold threshold = pointSystem.getTimeThreshold();
        if (threshold != null) {
            String str = String.valueOf(threshold.getValue());
            if (threshold.isPercent()) {
                str = str + "%";
            }
            mThresholdField.setText(str);
            mPostThresholdPointsSpinner.setValue(new Double(threshold.getPoints()));
            mPostThresholdMinutesSpinner.setValue(new Integer(threshold.getMinutes()));
        } else {
            mThresholdField.setText("0");
            mPostThresholdPointsSpinner.setValue(new Double(0));
            mPostThresholdMinutesSpinner.setValue(new Integer(0));
        }
        updateRollCalls();
    }

    private ActionListener getAddRollCallListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DialogRollCall diag = new DialogRollCall(
                        mPointSystemsTab.getOwner(),
                        "Add Roll Call",
                        null,
                        mPointSystemsTab.getSelectedPointSystem());
                diag.setVisible(true);
                RollCallItem rollCallItem = diag.getRollCallItem();
                if (rollCallItem != null) {
                    mPointSystemsTab.getSelectedPointSystem().addRollCall(
                            rollCallItem.getRollCall());
                    mPointSystemsTab.setCurrentPointSystemModified(true);
                    updateRollCalls();
                }
            }
        };
    }
    
    private ActionListener getEditRollCallListener() {
        if (mEditRollCallListener == null) {
            mEditRollCallListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (mCurrentRollCallItem != null) {
                        DialogRollCall diag = new DialogRollCall(
                                mPointSystemsTab.getOwner(),
                                "Edit Roll Call",
                                mCurrentRollCallItem,
                                mPointSystemsTab.getSelectedPointSystem());
                        diag.setVisible(true);
                        RollCallItem rollCallItem = diag.getRollCallItem();
                        if (rollCallItem != null) {
                            RollCall rollCallToChange = 
                                mPointSystemsTab.getSelectedPointSystem().getRollCall(
                                        mCurrentRollCallItem.getName());
                            rollCallToChange.setName(rollCallItem.getName());
                            rollCallToChange.setPoints(rollCallItem.getPoints());
                            rollCallToChange.setRecurrence(rollCallItem.getRecurrence());
                            mPointSystemsTab.setCurrentPointSystemModified(true);
                            updateRollCalls();
                        }
                    }
                }
            };
        }
        return mEditRollCallListener;
    }
    
    private ActionListener getDeleteRollCallListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mCurrentRollCallItem != null) {
                    String name = mCurrentRollCallItem.getName();
                    if (
                            name.equals(RollCall.RAID_START_NAME) ||
                            name.equals(RollCall.RAID_END_NAME)) {
                        // Cannot delete these special roll calls.
                        return;
                    }
                    mPointSystemsTab.getSelectedPointSystem().removeRollCall(
                            mCurrentRollCallItem.getName());
                    mPointSystemsTab.setCurrentPointSystemModified(true);
                    updateRollCalls();
                }
            }
        };
    }
    
    private ListSelectionListener getRollCallSelectionListener() {
        return new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    mCurrentRollCallItem = 
                        (RollCallItem) mRollCallsList.getSelectedValue();
                }
            }
        };
    }
    
    private void updateRollCalls() {
        IPointSystem pointSystem = mPointSystemsTab.getSelectedPointSystem();
        List rollCalls = pointSystem.getRollCalls();
        if (rollCalls != null) {
            int itemCount = rollCalls.size();
            RollCallItem[] items = new RollCallItem[itemCount];
            for (int i=0; i < itemCount; i++) {
                RollCall iterRollCall = (RollCall) rollCalls.get(i);
                items[i] = new RollCallItem(
                        iterRollCall.getName(), 
                        iterRollCall.getPoints(),
                        iterRollCall.getRecurrence());
            }
            mRollCallsList.setListData(items);
        }
        mCurrentRollCallItem = null;
    }
}
