package org.wow.autodkp.view.share;

import java.io.File;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.tree.DefaultMutableTreeNode;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.persistence.Persistence;
import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.GeneralProperties;


/**
 * 
 * 
 * @author jsandusky
 */
public class ExportDialog extends ShareDialog {

    private Map mPointSystemsToItemConstraintsMap;
    private File mExportedZipFile;
    
    
    public ExportDialog() {
        super(AutoDKPView.getInstance().getFrame(), "Exporter");
    }
    
    protected void closing() {
        GeneralProperties props = GeneralProperties.getDefault();
        props.setStringProperty(props.EXPORT_LOCATION, mLocationField.getText());
        super.closing();
        
        if (mExportedZipFile != null) {
            JOptionPane.showMessageDialog(
                getComponent(),
                "Data exported to " + mExportedZipFile.getPath(),
                "Exporter",
                JOptionPane.INFORMATION_MESSAGE);
        }
    }
    
    protected void loadNodes() {
        File pointSystemsParent     = new File(Persistence.DEFINITIONS_DIR + File.separator + Persistence.CONSTRAINTS_DIR);
        File pointsParent           = new File(Persistence.INSTANCES_DIR + File.separator + Persistence.CONSTRAINTS_DIR);
        ExportNode raidsNode        = new ExportNode(AutoDKPModel.getInstance().getRaidItems(), true, "All Raids");
        ExportNode pointsNode       = new ExportNode(pointsParent,       true, ExportNode.TYPE_POINTS,        "All Points");
        ExportNode pointSystemsNode = new ExportNode(pointSystemsParent, true, ExportNode.TYPE_POINT_SYSTEM,  "All Point Systems");
        
        ExportNode otherNode = new ExportNode("Other");
        otherNode.add(new ExportNode(
                new File(
                        Persistence.DEFINITIONS_DIR + File.separator + 
                        Persistence.ALTERNATES_FILE), 
                false,
                ExportNode.TYPE_ALTERNATES,
                "Alternates"));
        otherNode.add(new ExportNode(
                new File(
                        Persistence.DEFINITIONS_DIR + File.separator + 
                        Persistence.ITEMS_DIR + File.separator + 
                        Persistence.MASTER_ITEMS_FILE), 
                false,
                ExportNode.TYPE_ITEMS_MASTER,
                "Items"));
        File cssCustomOutputFile = new File(Persistence.CUSTOM_OUTPUT_CSS_FILE);
        if (cssCustomOutputFile.exists()) {
            otherNode.add(new ExportNode(
                    cssCustomOutputFile, 
                    false,
                    ExportNode.TYPE_CSS_OUTPUT,
                    "HTML Styles"));
        }
        
        File itemConstraintParent = new File(Persistence.INSTANCES_DIR + File.separator + Persistence.ITEMS_DIR);
        ExportNode itemsNode = new ExportNode(itemConstraintParent, true, ExportNode.TYPE_ITEM_CONSTRAINTS, null);
        mPointSystemsToItemConstraintsMap = 
            ShareNode.mapPointSystemsNodesToItemConstraintsNodes(itemsNode);
        
        mRootNode.add(raidsNode);
        mRootNode.add(pointsNode);
        mRootNode.add(pointSystemsNode);
        mRootNode.add(otherNode);
    }
    
    protected JComponent getInputPane() {
        return super.createInputPane(
                "Export Location:", 
                GeneralProperties.EXPORT_LOCATION, 
                JFileChooser.DIRECTORIES_ONLY,
                "Select Directory to Export To",
                null);
    }
    
    protected JComponent getNodesPane() {
        return super.createNodesPane(true);
    }
    
    protected void locationEntered() {
        // do nothing
    }
    
    protected boolean applyChanges() {
    	String exportPath = mLocationField.getText().trim();
    	if (exportPath.length() == 0) {
            mStatusScroller.setVisible(true);
            mMainPane.revalidate();
    		mStatusArea.setText(
    				"You must specify an Export Location.\n" + 
    				"When you export, everything you select is stored in a zip file.\n" +
    				"The Export Location is the directory where this zip file will be put.");
            return false;
    	}
    	
    	File exportDirectory = new File(exportPath);
    	if (!(new File(exportPath).exists())) {
            mStatusScroller.setVisible(true);
            mMainPane.revalidate();
    		mStatusArea.setText(
    				"Export location \"" + exportPath + "\" does not exist.\n" + 
    				"Please fix your Export Location and try again.");
            return false;
    	}
    	
        Set exportList = new LinkedHashSet();
        ShareNode.buildShareList(
                (DefaultMutableTreeNode) mTree.getModel().getRoot(), 
                exportList,
                mPointSystemsToItemConstraintsMap);
        
        if (exportList.size() == 0) {
            mStatusScroller.setVisible(true);
            mMainPane.revalidate();
            mStatusArea.setText(
                    "Please select something to export and try again.");
            return false;
        }
        
        mExportedZipFile = ExportUtils.writeZip(exportDirectory, exportList);
        
    	return true;
    }
}
