package org.wow.autodkp.view.share;

import java.io.File;
import java.util.Iterator;
import java.util.List;

import org.wow.autodkp.model.Raid;
import org.wow.autodkp.persistence.Persistence;

/**
 * 
 * 
 * @author jsandusky
 */
public class ExportNode extends ShareNode {
	
	
    public ExportNode(List raidList, boolean isHeader, String text) {
        super(null, isHeader, text);
        mIsSharable = false;
        mNodeType = ShareNode.TYPE_RAID;
        for (Iterator iter=raidList.iterator(); iter.hasNext();) {
            Raid raid = (Raid) iter.next();
            String iterText = "raid, " + getRaidDescription(raid.getID());
            add(new ExportNode(raid, false, iterText));
        }
    }
    
    public ExportNode(Raid raid, boolean isHeader, String text) {
        super(raid, isHeader, text);
        mNodeType = ShareNode.TYPE_RAID;
        mIsSharable = true;
    }
    
    public ExportNode(String text) {
        super(null, true, text);
        mIsSharable = false;
    }
    
	public ExportNode(File file, boolean isHeader, int nodeType, String text) {
		super(file, isHeader, text);
        mNodeType = nodeType;
        mIsSharable = !file.isDirectory();
		if (file.isDirectory()) {
			File[] files = file.listFiles();
			for (int i=0; i < files.length; i++) {
                String iterText = null;
                if (nodeType == TYPE_POINT_SYSTEM) {
                    iterText = getPointSystemDescription(files[i].getName()) +  " Point System Definition";
                } else if (nodeType == TYPE_POINTS) {
                    if (files[i].isDirectory()) {
                        iterText = getPointSystemDescription(files[i].getName());
                    } else {
                        iterText = "points, " + getRaidDescription(files[i].getName());
                    }
                }
				add(new ExportNode(files[i], false, nodeType, iterText));
			}
		}
	}

    public String getFileID() {
        if (getUserObject() instanceof File) {
            return stripExtension(((File) getUserObject()).getName());
        }
        return null;
    }
    
    public File getFileToExport() {
        Object ob = getUserObject();
        if (ob instanceof File) {
            return (File) ob;
        } else if (ob instanceof Raid) {
            // export the raid to a temporary directory first
            Raid raid = (Raid) ob;
            File tmpRaidsDir = new File(
                    Persistence.TMP_DIR + File.separator + 
                    Persistence.SAVE_RAID_DIR);
            if (!tmpRaidsDir.exists()) {
                tmpRaidsDir.mkdirs();
            }
            File exportedRaid = raid.export(tmpRaidsDir);
            exportedRaid.deleteOnExit();
            return exportedRaid;
        }
        return null;
    }
    
    public String getZipPath(File file) {
        Object ob = getUserObject();
        if (ob instanceof File) {
            if        (mNodeType == TYPE_POINT_SYSTEM) {
                // definitions/points/<file>
                String pointsPathName      = file.getParentFile().getName();
                String definitionsPathName = file.getParentFile().getParentFile().getName();
                return definitionsPathName + "/" + pointsPathName + "/" + file.getName();
            } else if (mNodeType == TYPE_POINTS) {
                // save/points/pointSystem/<file>
                String systemPathName = file.getParentFile().getName();
                String pointsPathName = file.getParentFile().getParentFile().getName();
                String savePathName   = file.getParentFile().getParentFile().getParentFile().getName();
                return savePathName + "/" + pointsPathName + "/" + systemPathName + "/" + file.getName();
            } else if (mNodeType == TYPE_ITEM_CONSTRAINTS) {
                // save/items/<file>
                String itemsPathName = file.getParentFile().getName();
                String savePathName  = file.getParentFile().getParentFile().getName();
                return savePathName + "/" + itemsPathName + "/" + file.getName();
            } else if (mNodeType == TYPE_ALTERNATES) {
                // definitions/<file>
                String definitionsPath = file.getParentFile().getName();
                return definitionsPath + "/" + file.getName();
            } else if (mNodeType == TYPE_ITEMS_MASTER) {
                // definitions/items/<file>
                String itemsPathName = file.getParentFile().getName();
                String definitionsPathName  = file.getParentFile().getParentFile().getName();
                return definitionsPathName + "/" + itemsPathName + "/" + file.getName();
            } else if (mNodeType == TYPE_CSS_OUTPUT) {
                // <file>
                return file.getName();
            }
        } else if (ob instanceof Raid) {
            // tmp/raids/<file> - we substitute INSTANCES_DIR for tmp
            String raidsPathName = file.getParentFile().getName();
            return Persistence.INSTANCES_DIR + "/" + raidsPathName + "/" + file.getName();
        }
        return null;
    }
}
