package org.wow.autodkp.view.share;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Iterator;
import java.util.Set;
import java.util.zip.Deflater;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.wow.autodkp.model.FormatConstants;
import org.wow.autodkp.model.Logger;

/**
 * 
 * 
 * @author jsandusky
 */
public class ExportUtils {

    public static final String ZIP_PREFIX = "dkpExport_";
    public static final String ZIP_SUFFIX = ".zip";
    
    public static String generateZipName() {
        Date date = new Date();
        String id = FormatConstants.FORMAT_ID.format(date);
        return ZIP_PREFIX + id + ZIP_SUFFIX;
    }

    public static File writeZip(File targetZipDirectory, Set exportList) {

        File targetZipFile = new File(
                targetZipDirectory.getPath() + File.separator + generateZipName());

        try {
            // Create the ZIP file
            ZipOutputStream out = new ZipOutputStream(new FileOutputStream(targetZipFile));
            out.setLevel(Deflater.BEST_SPEED);

            // Create a buffer for reading the files
            byte[] buf = new byte[1024];

            // Compress the files
            for (Iterator iter = exportList.iterator(); iter.hasNext();) {
                ExportNode exportNode = (ExportNode) iter.next();
                File file = exportNode.getFileToExport();
                if (file != null && file.exists()) {
                    
                    // Add ZIP entry to output stream.
                    String entryName = exportNode.getZipPath(file);
                    out.putNextEntry(new ZipEntry(entryName));
                    
                    FileInputStream in = new FileInputStream(file);
                    // Transfer bytes from the file to the ZIP file
                    int len;
                    while ((len = in.read(buf)) > 0) {
                        out.write(buf, 0, len);
                    }
                    in.close();

                    // Complete the entry
                    out.closeEntry();
                }
            }

            // Complete the ZIP file
            out.close();

        } catch (IOException e) {
            Logger.log(e);
        }
        
        return targetZipFile;
    }

}
