package org.wow.autodkp.view.share;

import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.tree.DefaultMutableTreeNode;

import org.wow.autodkp.view.AutoDKPView;


/**
 * 
 * 
 * @author jsandusky
 */
public class ImportDialog extends ShareDialog {

    private Importer mImporter;
    
    
    public ImportDialog() {
        super(AutoDKPView.getInstance().getFrame(), "Importer");
    }
    

    protected boolean isOKInitiallyEnabled() {
        return false;
    }

    private boolean checkInputField() {
        String inputPath = mLocationField.getText().trim();
        if (inputPath.length() == 0) {
            setOKEnabled(false);
            mStatusScroller.setVisible(true);
            mMainPane.revalidate();
            mStatusArea.setText("You must specify a file to import.");
            return false;
        }
        
        File inputFile = new File(inputPath);
        if (!(inputFile.exists())) {
            setOKEnabled(false);
            mStatusScroller.setVisible(true);
            mMainPane.revalidate();
            mStatusArea.setText("Input file \"" + inputPath + "\" does not exist.");
            return false;
        }
        
        return true;
    }
    
    protected void loadNodes() {
        
        if (!checkInputField()) {
            return;
        }
        
        String inputPath = mLocationField.getText().trim();
        File inputFile = new File(inputPath);

        mImporter = new Importer();
        List importNodes = mImporter.createZipNodes(inputFile);
        if (importNodes != null) {
            for (Iterator iter=importNodes.iterator(); iter.hasNext();) {
                mRootNode.add((ImportNode) iter.next());
            }
            if (importNodes.size() > 0) {
                reloadTree();
                setTreeExpanded(true);
            }
        }
        
        if (importNodes == null) {
            setOKEnabled(false);
            mStatusScroller.setVisible(true);
            mMainPane.revalidate();
            mStatusArea.setText(
                    "The file you specified is invalid.\n" + 
                    "Check that you selected the correct file, or try a different export file.");
        } else if (importNodes.size() == 0) {
            setOKEnabled(false);
            mStatusScroller.setVisible(true);
            mMainPane.revalidate();
            mStatusArea.setText("The file you specified is empty.");
        }
    }
    
    protected JComponent getNodesPane() {
        return super.createNodesPane(false);
    }
    
    protected JComponent getInputPane() {
        FileFilter locationBrowseFilter = new FileFilter() {
            public String getDescription() {
                return "AutoDKP Export Files";
            }
            public boolean accept(File f) {
                String name = f.getName().toLowerCase();
                if (f.isDirectory() || name.endsWith(ExportUtils.ZIP_SUFFIX)) {
                    return true;
                }
                return false;
            }
        };
        
        JComponent inputPane = super.createInputPane(
                "File to Import:", 
                null, 
                JFileChooser.FILES_ONLY,
                "Select File To Import",
                locationBrowseFilter);
        
        return inputPane;
    }
    
    protected void locationEntered() {
        loadNodes();
    }
    
    protected boolean applyChanges() {
        
        if (!checkInputField()) {
            return false;
        }
        if (mImporter == null) {
            setOKEnabled(false);
            mStatusScroller.setVisible(true);
            mMainPane.revalidate();
            mStatusArea.setText("Error occurred during import.");
            return false;
        }
        
        Set importList = mImporter.getImportList(
                (DefaultMutableTreeNode) mTree.getModel().getRoot());
        if (importList.size() == 0) {
            mStatusScroller.setVisible(true);
            mMainPane.revalidate();
            mStatusArea.setText(
                    "Please select something to import and try again.");
            return false;
        }
        
        mImporter.readZip(importList);
        
    	return true;
    }
}
