package org.wow.autodkp.view.share;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import javax.swing.JOptionPane;
import javax.swing.tree.DefaultMutableTreeNode;

import org.wow.autodkp.model.AlternatesGroup;
import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.Logger;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.persistence.AlternatesManager;
import org.wow.autodkp.persistence.IItemDB;
import org.wow.autodkp.persistence.ItemMasterDB;
import org.wow.autodkp.persistence.ItemPointsDB;
import org.wow.autodkp.persistence.Persistence;
import org.wow.autodkp.persistence.xml.XMLAlternatesReader;
import org.wow.autodkp.view.AutoDKPView;


/**
 * 
 * 
 * @author jsandusky
 */
public class Importer {
    
    private static final int MINIMUM_BYTES_TO_SHOW_PROGRESS = 125000;
    
    private static final String ENTRY_RAIDS = 
        Persistence.INSTANCES_DIR + "/" + Persistence.SAVE_RAID_DIR;
    private static final String ENTRY_POINTS = 
        Persistence.INSTANCES_DIR + "/" + Persistence.CONSTRAINTS_DIR;
    private static final String ENTRY_SYSTEMS = 
        Persistence.DEFINITIONS_DIR + "/" + Persistence.CONSTRAINTS_DIR;
    private static final String ENTRY_ALTERNATES = 
        Persistence.DEFINITIONS_DIR + "/" + Persistence.ALTERNATES_FILE;
    private static final String ENTRY_ITEMS_MASTER = 
        Persistence.DEFINITIONS_DIR + "/" + Persistence.ITEMS_DIR;
    private static final String ENTRY_CSS_OUTPUT = 
        Persistence.CUSTOM_OUTPUT_CSS_FILE;
    private static final String ENTRY_ITEMS_CONSTRAINTS = 
        Persistence.INSTANCES_DIR + "/" + Persistence.ITEMS_DIR;
    
    private ZipFile mZipFile;
    private Map mPointSystemsToItemConstraintsMap;
    
    
    public List createZipNodes(File importFile) {
        List importNodes = null;
        try {
            mZipFile = new ZipFile(importFile);
            Enumeration entries = mZipFile.entries();

            importNodes = new ArrayList();
            ImportNode allRaidsNode = null;
            ImportNode allPointsNode = null;
            ImportNode allPointSystemsNode = null;
            ImportNode otherNode = null;
            Map systemNodesMap = new HashMap();
            ImportNode allItemConstraintsNode = null;
            
            while (entries.hasMoreElements()) {
                ZipEntry ze = (ZipEntry) entries.nextElement();
                if (ze.getSize() > 0) {
                    String entryPath = ze.getName();
                    int lastSeparatorIndex = entryPath.lastIndexOf("/");
                    if (lastSeparatorIndex > 0 && lastSeparatorIndex + 1 < entryPath.length()) {
                        // Entries with at least one parent directory.
                        String dirPath  = entryPath.substring(0, lastSeparatorIndex);
                        String name     = entryPath.substring(lastSeparatorIndex + 1);
                        if (dirPath.startsWith(ENTRY_RAIDS)) {
                            if (allRaidsNode == null) {
                                allRaidsNode = new ImportNode("All Raids", ShareNode.TYPE_RAID, true);
                                importNodes.add(allRaidsNode);
                            }
                            allRaidsNode.add(new ImportNode(ze, ShareNode.TYPE_RAID, dirPath, name));
                        } else if (dirPath.startsWith(ENTRY_POINTS)) {
                            if (allPointsNode == null) {
                                allPointsNode = new ImportNode("All Points", ShareNode.TYPE_POINTS, true);
                                importNodes.add(allPointsNode);
                            }
                            int lastSlashIndex = dirPath.lastIndexOf("/");
                            if (lastSlashIndex > 0 && lastSlashIndex + 1 < dirPath.length()) {
                                String systemName  = dirPath.substring(lastSlashIndex + 1);
                                ImportNode parentNode = (ImportNode) systemNodesMap.get(systemName);
                                if (parentNode == null) {
                                    parentNode = new ImportNode(ImportNode.getPointSystemDescription(systemName), ShareNode.TYPE_POINTS, false);
                                    allPointsNode.add(parentNode);
                                    systemNodesMap.put(systemName, parentNode);
                                }
                                parentNode.add(new ImportNode(ze, ShareNode.TYPE_POINTS, dirPath, name));
                            }
                        } else if (dirPath.startsWith(ENTRY_SYSTEMS)) {
                            if (allPointSystemsNode == null) {
                                allPointSystemsNode = new ImportNode("All Point Systems", ShareNode.TYPE_POINT_SYSTEM, true);
                                importNodes.add(allPointSystemsNode);
                            }
                            allPointSystemsNode.add(new ImportNode(ze, ShareNode.TYPE_POINT_SYSTEM, dirPath, name));
                        } else if (entryPath.startsWith(ENTRY_ALTERNATES) ) {
                            if (otherNode == null) {
                                otherNode = new ImportNode("Other", ShareNode.TYPE_NONE, true);
                                importNodes.add(otherNode);
                            }
                            otherNode.add(new ImportNode(ze, ShareNode.TYPE_ALTERNATES, dirPath, name));
                        } else if (dirPath.startsWith(ENTRY_ITEMS_MASTER)) {
                            if (otherNode == null) {
                                otherNode = new ImportNode("Other", ShareNode.TYPE_NONE, true);
                                importNodes.add(otherNode);
                            }
                            otherNode.add(new ImportNode(ze, ShareNode.TYPE_ITEMS_MASTER, dirPath, name));
                        } else if (dirPath.startsWith(ENTRY_ITEMS_CONSTRAINTS)) {
                            if (allItemConstraintsNode == null) {
                                allItemConstraintsNode = new ImportNode(null, ShareNode.TYPE_ITEM_CONSTRAINTS, true);
                            }
                            allItemConstraintsNode.add(new ImportNode(ze, ShareNode.TYPE_ITEM_CONSTRAINTS, dirPath, name));
                        }
                    } else {
                        // Entries on the root path.
                        if (entryPath.startsWith(ENTRY_CSS_OUTPUT) ) {
                            if (otherNode == null) {
                                otherNode = new ImportNode("Other", ShareNode.TYPE_NONE, true);
                                importNodes.add(otherNode);
                            }
                            otherNode.add(new ImportNode(ze, ShareNode.TYPE_CSS_OUTPUT, "", entryPath));
                        }
                    }
                }
            }
            
            mPointSystemsToItemConstraintsMap = 
                ShareNode.mapPointSystemsNodesToItemConstraintsNodes(allItemConstraintsNode);
            
        } catch (IOException e) {
            // user may have entered in an invalid zip file
        }
        
        return importNodes;
    }
    
    public Set getImportList(DefaultMutableTreeNode rootNode) {
        Set exportList = new LinkedHashSet();
        ShareNode.buildShareList(rootNode, exportList, mPointSystemsToItemConstraintsMap);
        return exportList;
    }
    
    public void readZip(final Set importList) {
        
        int totalBytes = 0;
        for (Iterator iter=importList.iterator(); iter.hasNext();) {
            ImportNode importNode = (ImportNode) iter.next();
            totalBytes += importNode.getZipEntry().getSize();
        }

        final ImportProgressDialog progress = new ImportProgressDialog(totalBytes);
        
        Runnable importRunner = new Runnable() {
            public void run() {
                try {
                    // first copy point systems, and reload
                    copyPointSystems(importList);
                    // then copy the other data
                    copyFiles(importList, progress);
                } catch (IOException ioe) {
                    Logger.log(ioe);
                    progress.dispose();
                    JOptionPane.showMessageDialog(
                            AutoDKPView.getInstance().getFrame(), 
                            "Error occurred during import.\n" + 
                            "See AutoDKP log file for more information.",
                            "Import Error",
                            JOptionPane.ERROR_MESSAGE);
                } finally {
                    try {
                        if (mZipFile != null) {
                            mZipFile.close();
                        }
                    } catch (IOException ioe) {
                        Logger.log(ioe);
                    }
                    progress.dispose();
                    
                }
            }
        };
        new Thread(importRunner).start();
        
        if (totalBytes > MINIMUM_BYTES_TO_SHOW_PROGRESS) {
            Runnable progressRunner = new Runnable() {
                public void run() {
                    progress.setVisible(true);
                }
            };
            new Thread(progressRunner).start();
        }
    }
    
    private void copyPointSystems(Set importList)
    throws IOException {
        
        boolean isReloadNeeded = false;
        byte[] buf = new byte[1024];
        for (Iterator iter=importList.iterator(); iter.hasNext();) {
            ImportNode node = (ImportNode) iter.next();
            if (node.getNodeType() == ShareNode.TYPE_POINT_SYSTEM) {
                // simple overwrite
                isReloadNeeded = true;
                copyEntryBytes(node, buf);
            }
        }
        
        if (isReloadNeeded) {
            AutoDKPView.unloadReload();
        }
    }
    
    private void copyFiles(Set importList, ImportProgressDialog progress)
    throws IOException {
        
        boolean isReloadNeeded = false;
        int totalBytesWritten = 0;
        byte[] buf = new byte[1024];
        for (Iterator iter=importList.iterator(); iter.hasNext();) {
            ImportNode node = (ImportNode) iter.next();
            switch(node.getNodeType()) {
            case ShareNode.TYPE_ALTERNATES:
                isReloadNeeded = true;
                copyAlternates(node, buf);
                break;
            case ShareNode.TYPE_ITEM_CONSTRAINTS:
                isReloadNeeded = true;
                copyConstraintItems(node, buf);
                break;
            case ShareNode.TYPE_ITEMS_MASTER:
                isReloadNeeded = true;
                copyMasterItems(node, buf);
                break;
            case ShareNode.TYPE_CSS_OUTPUT:
            case ShareNode.TYPE_POINTS:
            case ShareNode.TYPE_RAID:
                // simple overwrite
                isReloadNeeded = true;
                totalBytesWritten += copyEntryBytes(node, buf);
                progress.updateWritten(node.getFileName(), totalBytesWritten);
                break;
            }
        }
        
        if (isReloadNeeded) {
            AutoDKPView.unloadReload();
        }
    }
    
    private int copyEntryBytes(ImportNode node, byte[] buf)
    throws IOException {
        String dirPath  = node.getDirectoryPath();
        String fileName = node.getFileName();
        if (dirPath == null || fileName == null) {
            return 0;
        }
        String localDirPath = dirPath.replace('/', File.separatorChar);;
        File destFile = null;
        if (localDirPath.length() > 0) {
            destFile = new File(localDirPath + File.separator + fileName);
        } else {
            destFile = new File(fileName);
        }
        
        // ensure the target directories exist
        new File(localDirPath).mkdirs();

        InputStream in = mZipFile.getInputStream(node.getZipEntry());
        FileOutputStream out = new FileOutputStream(destFile);
        
        BufferedInputStream zipInput = new BufferedInputStream(in);
        // Transfer bytes from the file to the ZIP file
        int totalKWritten = 0;
        int len;
        while ((len = zipInput.read(buf)) > 0) {
            totalKWritten += len;
            out.write(buf, 0, len);
        }
        zipInput.close();
        
        out.flush();
        out.close();
        
        return totalKWritten;
    }

    private void copyAlternates(ImportNode node, byte[] buf) 
    throws IOException {
        XMLAlternatesReader altReader = new XMLAlternatesReader();
        Map alternatesMap = altReader.readXML(mZipFile.getInputStream(node.getZipEntry()));
        
        boolean isModified = false;
        AlternatesManager myManager = AlternatesManager.getDefault();
        for (Iterator inIter=alternatesMap.values().iterator(); inIter.hasNext();) {
            AlternatesGroup inGroup = (AlternatesGroup) inIter.next();
            String primaryName = inGroup.getPrimaryPlayerName();
            if (!myManager.getAllPrimaryPlayers().contains(primaryName)) {
                myManager.addPrimaryPlayer(primaryName, inGroup.getPrimaryPlayerClass());
                isModified = true;
            }
            List inAlternates = inGroup.getAlternates();
            if (inAlternates != null) {
                for (Iterator inAltIter=inAlternates.iterator(); inAltIter.hasNext();) {
                    String altName = (String) inAltIter.next();
                    List myAlternates = myManager.getAlternatesForPlayer(primaryName);
                    if (myAlternates != null) {
                        if (!myAlternates.contains(altName)) {
                            myManager.addAlternateForPlayer(primaryName, altName);
                            isModified = true;
                        }
                    } else {
                        myManager.addAlternateForPlayer(primaryName, altName);
                        isModified = true;
                    }
                }
            }
        }
        
        if (isModified) {
            myManager.save();
        }
    }

    private void copyMasterItems(ImportNode node, byte[] buf) 
    throws IOException {
        ItemMasterDB inItemDB = ItemMasterDB.createMasterItemDBGhost(
                mZipFile.getInputStream(node.getZipEntry()));
        
        ItemMasterDB myItemDB = ItemMasterDB.getDefault();
        
        boolean isModified = false;
        for (Iterator inIter=inItemDB.getLootIDs().iterator(); inIter.hasNext();) {
            String iterID = (String) inIter.next();
            String inRawValue = inItemDB.getRawLootValue(iterID);
            if (inRawValue == null || inRawValue.trim().length() == 0) {
                // imported data is empty, do not import
                continue;
            }
            
            String myRawValue = myItemDB.getRawLootValue(iterID);
            if (myItemDB.hasItem(iterID) && myRawValue != null && myRawValue.trim().length() > 0) {
                // we have the imported item, and it has data
                // only update if our boss/instance data needs updating
                String newBossName = null;
                String myBossName = myItemDB.getBossName(iterID);
                if (myBossName == null || myBossName.length() == 0) {
                    String inBossName = inItemDB.getBossName(iterID);
                    if (inBossName != null && inBossName.length() > 0) {
                        newBossName = inBossName;
                    }
                }
                String newInstanceName = null;
                String myInstanceName = myItemDB.getInstanceName(iterID);
                if (myInstanceName == null || myInstanceName.length() == 0) {
                    String inInstanceName = inItemDB.getInstanceName(iterID);
                    if (inInstanceName != null && inInstanceName.length() > 0) {
                        newInstanceName = inInstanceName;
                    }
                }
                
                if (newBossName != null || newInstanceName != null) {
                    // we need to update our boss/instance info
                    myItemDB.setRawLootValue(iterID, inRawValue);
                    isModified = true;
                }
            } else {
                // overwrite
                myItemDB.setRawLootValue(iterID, inRawValue);
                isModified = true;
            }
        }
        
        if (isModified) {
            myItemDB.save();
        }
    }
    
    private void copyConstraintItems(ImportNode node, byte[] buf) 
    throws IOException {
        ItemPointsDB inItemDB = ItemPointsDB.createConstraintsItemDBGhost(
                mZipFile.getInputStream(node.getZipEntry()));
        
        IItemDB myItemDB = null;
        String pointSystemID = node.getFileID();
        IPointSystem[] mySystems = AutoDKPModel.getInstance().getConstraints();
        for (int i=0; i < mySystems.length; i++) {
            if (mySystems[i].getID().equals(pointSystemID)) {
                myItemDB = mySystems[i].getItemDB();
            }
        }
        
        if (myItemDB != null) {
            boolean isModified = false;
            for (Iterator inIter=inItemDB.getLootIDs().iterator(); inIter.hasNext();) {
                String iterID = (String) inIter.next();
                String inRawValue = inItemDB.getRawLootValue(iterID);
                if (inRawValue == null || inRawValue.trim().length() == 0) {
                    // imported data is empty, do not import
                    continue;
                }
                
                // update our data
                String rawValue = inItemDB.getRawLootValue(iterID);
                myItemDB.setRawLootValue(iterID, rawValue);
                isModified = true;
            }
            
            if (isModified) {
                myItemDB.save();
            }
        }
    }
}
