package org.wow.autodkp.view.share;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import org.wow.autodkp.view.BannerWindow;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.IChangeCommitListener;
import org.wow.autodkp.view.ImageUtilities;


/**
 * 
 * 
 * @author jsandusky
 */
public abstract class ShareDialog extends BannerWindow {

    protected JTree mTree;
    protected JComponent mMainPane;
    protected JTextArea mStatusArea;
    protected JTextField mLocationField;
    protected JScrollPane mStatusScroller;
    
    protected DefaultMutableTreeNode mRootNode;
    
    
    public ShareDialog(JFrame owner, String title) {
        super(owner, title);
        
        mListener = new IChangeCommitListener() {
            public void changesCancelled() {
            }
            public void changesCommitted() {
            }
        };
        
        GeneralProperties props = GeneralProperties.getDefault();
        props.loadWindowRect(
                mWindow, 
                props.EXPORTER_WINDOW_X,
                props.EXPORTER_WINDOW_Y,
                props.EXPORTER_WINDOW_WIDTH,
                props.EXPORTER_WINDOW_HEIGHT,
                props.DEFAULT_CUSTOMIZER_WIDTH,
                props.DEFAULT_CUSTOMIZER_HEIGHT);
    }
    
    protected void closing() {
        GeneralProperties props = GeneralProperties.getDefault();
        props.setIntegerProperty(props.EXPORTER_WINDOW_X, getX());
        props.setIntegerProperty(props.EXPORTER_WINDOW_Y, getY());
        props.setIntegerProperty(props.EXPORTER_WINDOW_WIDTH,  getWidth());
        props.setIntegerProperty(props.EXPORTER_WINDOW_HEIGHT, getHeight());
        props.save();
    }
    
    protected JComponent getStatusPane() {
        mStatusArea = new JTextArea(3, 20);
        mStatusArea.setFont(new Font("Serif", Font.BOLD, 16));
        mStatusArea.setForeground(Color.RED);
        mStatusArea.setEditable(false);
        mStatusArea.setOpaque(false);
        mStatusArea.setBorder(null);
        mStatusArea.setWrapStyleWord(true);
        mStatusArea.setLineWrap(true);
        mStatusScroller = new JScrollPane(mStatusArea);
        mStatusScroller.setBorder(null);
        mStatusScroller.setVisible(false);
        return mStatusScroller;
    }
    
    protected JComponent createNodesPane(boolean isInitialLoad) {
        mRootNode = new DefaultMutableTreeNode();
        
        if (isInitialLoad) {
            loadNodes();
        }
        
        mTree = new JTree(mRootNode);
        mTree.setCellRenderer(new ShareNodeTreeCellRenderer());
        mTree.setRootVisible(false);
        mTree.setShowsRootHandles(true);
        JScrollPane treeScroller = new JScrollPane(mTree);
        
        mTree.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                Point point = e.getPoint();
                TreePath treePath = mTree.getPathForLocation(point.x, point.y);
                if (treePath != null) {
                    ShareNode shareNode = (ShareNode) treePath.getLastPathComponent();
                    shareNode.setSelectedForExport(!shareNode.isSelectedForExport());
                    mTree.repaint();
                }
            }
        });
        
        if (isInitialLoad) {
            setTreeExpanded(true);
        }
        
        Image img = null;
        Dimension quickSelectSize = new Dimension(32, 32);
        
        img = ImageUtilities.loadJarImage("ok.png");
        JButton selectAllButton = new JButton(new ImageIcon(img));
        selectAllButton.setPreferredSize(quickSelectSize);
        selectAllButton.setToolTipText("Select All");
        selectAllButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setAllNodesSelected(true);
            }
        });
        img = ImageUtilities.loadJarImage("cancel.png");
        JButton deselectAllButton = new JButton(new ImageIcon(img));
        deselectAllButton.setPreferredSize(quickSelectSize);
        deselectAllButton.setToolTipText("Select None");
        deselectAllButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setAllNodesSelected(false);
            }
        });
        img = ImageUtilities.loadJarImage("zoom-in.png");
        JButton expandAllButton = new JButton(new ImageIcon(img));
        expandAllButton.setPreferredSize(quickSelectSize);
        expandAllButton.setToolTipText("Expand All");
        expandAllButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setTreeExpanded(true);
            }
        });
        img = ImageUtilities.loadJarImage("zoom-out.png");
        JButton collapseAllButton = new JButton(new ImageIcon(img));
        collapseAllButton.setPreferredSize(quickSelectSize);
        collapseAllButton.setToolTipText("Collapse All");
        collapseAllButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setTreeExpanded(false);
            }
        });
        JPanel quickSelectPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        quickSelectPanel.add(expandAllButton);
        quickSelectPanel.add(collapseAllButton);
        quickSelectPanel.add(new JSeparator());
        quickSelectPanel.add(selectAllButton);
        quickSelectPanel.add(deselectAllButton);
        
        JPanel nodesPane = new JPanel(new BorderLayout());
        nodesPane.add(quickSelectPanel, BorderLayout.NORTH);
        nodesPane.add(treeScroller, BorderLayout.CENTER);
        
        return nodesPane;
    }
    
    private void setAllNodesSelected(boolean isSelected) {
        for (int i=0; i < mRootNode.getChildCount(); i++) {
            ShareNode parentNode = (ShareNode) mRootNode.getChildAt(i);
            parentNode.setSelectedForExport(isSelected);
        }
        mTree.repaint();
    }
    
    protected JComponent createInputPane(String locationLabel, 
                                         final String locationProperty, 
                                         final int locationFileMode,
                                         final String locationBrowseTitle,
                                         final FileFilter fileFilter) {
        JLabel exportLabel = new JLabel(locationLabel);
        mLocationField = new JTextField();
        JButton browseButton = new JButton("...");
        
        if (locationProperty != null) {
            GeneralProperties props = GeneralProperties.getDefault();
            String exportPath = props.getStringProperty(locationProperty);
            if (exportPath != null) {
                mLocationField.setText(exportPath);
            }
        }
        
        mLocationField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                locationEntered();
            }
        });
        
        mLocationField.getDocument().addDocumentListener(new DocumentListener() {
            public void changedUpdate(DocumentEvent e) {}
            public void removeUpdate(DocumentEvent e) {
                setOKEnabled(mLocationField.getText().trim().length() > 0);
                mStatusArea.setText("");
                mStatusScroller.setVisible(false);
                mMainPane.revalidate();
            }
            public void insertUpdate(DocumentEvent e) {
                setOKEnabled(mLocationField.getText().trim().length() > 0);
                mStatusArea.setText("");
                mStatusScroller.setVisible(false);
                mMainPane.revalidate();
            }
        });
        
        browseButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JFileChooser chooser = new JFileChooser();
                if (locationProperty != null) {
                    GeneralProperties props = GeneralProperties.getDefault();
                    String exportPath = props.getStringProperty(locationProperty);
                    if (exportPath != null) {
                        File exportDirectory = new File(exportPath);
                        if (exportDirectory.exists()) {
                            chooser.setCurrentDirectory(exportDirectory);
                        }
                    }
                }
                chooser.setFileSelectionMode(locationFileMode);
                chooser.setMultiSelectionEnabled(false);
                chooser.setDialogTitle(locationBrowseTitle);
                if (fileFilter != null) {
                    chooser.setFileFilter(fileFilter);
                }
                chooser.showDialog(mWindow, "Select");
                File selectedFile = chooser.getSelectedFile();
                if (selectedFile != null) {
                    String path = selectedFile.getPath();
                    if (locationProperty != null) {
                        GeneralProperties props = GeneralProperties.getDefault();
                        props.setStringProperty(locationProperty, path);
                    }
                    mLocationField.setText(path);
                    locationEntered();
                }
            }
        });
        
        Box exportLocationPane = Box.createHorizontalBox();
        exportLocationPane.add(exportLabel);
        exportLocationPane.add(Box.createHorizontalStrut(5));
        exportLocationPane.add(mLocationField);
        exportLocationPane.add(Box.createHorizontalStrut(5));
        exportLocationPane.add(browseButton);
        exportLocationPane.add(Box.createHorizontalGlue());
        exportLocationPane.setBorder(BorderFactory.createEmptyBorder(0, 0, 5, 0));
        
        return exportLocationPane;
    }
    
    protected JComponent getComponent() {
        mMainPane = new JPanel(new BorderLayout(5, 5));
        
        JComponent inputPane  = getInputPane();
        if (inputPane != null) {
            mMainPane.add(inputPane, BorderLayout.NORTH);
        }
        
        mMainPane.add(getNodesPane(), BorderLayout.CENTER);
        
        JComponent statusPane = getStatusPane();
        if (statusPane != null) {
            mMainPane.add(statusPane, BorderLayout.SOUTH);
        }
        
        mMainPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        return mMainPane;
    }
    
    protected void reloadTree() {
        mTree.setModel(new DefaultTreeModel(mRootNode));
    }
    
    protected void setTreeExpanded(boolean isExpanded) {
        for (int i=0; i < mTree.getRowCount(); i++) {
            if (isExpanded) {
                mTree.expandRow(i);
            } else {
                mTree.collapseRow(i);
            }
        }
    }
    
    protected String getOKButtonText() {
        return "OK";
    }
    
    protected abstract JComponent getNodesPane();
    
    protected abstract JComponent getInputPane();

    protected abstract void locationEntered();
    
    protected abstract void loadNodes();
    
}
