package org.wow.autodkp.view.share;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.swing.tree.DefaultMutableTreeNode;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.FormatConstants;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.view.GeneralProperties;


/**
 * 
 * 
 * @author jsandusky
 */
public abstract class ShareNode extends DefaultMutableTreeNode {

    public static final int TYPE_NONE             = 0;
    public static final int TYPE_POINT_SYSTEM     = 1;
    public static final int TYPE_POINTS           = 2;
    public static final int TYPE_ITEM_CONSTRAINTS = 3;
    public static final int TYPE_ALTERNATES       = 4;
    public static final int TYPE_ITEMS_MASTER     = 5;
    public static final int TYPE_RAID             = 6;
    public static final int TYPE_CSS_OUTPUT       = 7;
    
    protected int mNodeType = TYPE_NONE;
    protected String mText;
    protected boolean mIsSharable;
    
    private boolean mIsHeader;
    private boolean mIsSelectedForExport;

    
    protected ShareNode(Object userObject, boolean isHeader) {
        super(userObject);
        mIsHeader = isHeader;
    }
    
    protected ShareNode(Object userObject, boolean isHeader, String text) {
        super(userObject);
        mIsHeader = isHeader;
        mText = text;
    }
    
    public String getText() {
        return mText;
    }
    
    public boolean isSelectedForExport() {
        return mIsSelectedForExport;
    }
    
    public void setSelectedForExport(boolean isSelectedForExport) {
        mIsSelectedForExport = isSelectedForExport;
        int childCount = getChildCount();
        if (childCount > 0) {
            for (int i=0; i < childCount; i++) {
                ShareNode iterNode = (ShareNode) getChildAt(i);
                iterNode.setSelectedForExport(isSelectedForExport);
            }
        }
    }
    
    public boolean isSharable() {
        return mIsSharable;
    }
    
    public boolean isHeader() {
        return mIsHeader;
    }
    
    public int getNodeType() {
        return mNodeType;
    }
    
    protected static String stripExtension(String fileName) {
        int dotIndex = fileName.indexOf(".");
        if (dotIndex >= 0) {
            fileName = fileName.substring(0, dotIndex);
        }
        return fileName;
    }
    
    public static String getPointSystemDescription(String fileName) {
        fileName = stripExtension(fileName);
        IPointSystem[] systems = AutoDKPModel.getInstance().getConstraints();
        for (int i=0; i < systems.length; i++) {
            if (systems[i].getID().equals(fileName)) {
                return systems[i].getName();
            }
        }
        return fileName;
    }
    
    protected static String getRaidDescription(String name) {
        name = stripExtension(name);
        try {
            GeneralProperties props = GeneralProperties.getDefault();
            Date date = FormatConstants.FORMAT_ID.parse(name);
            return props.getDateFormatProperty(props.DATE_FORMAT_LONG).format(date);
        } catch (Exception e) {
            return name;
        }
    }
    
    public static Map mapPointSystemsNodesToItemConstraintsNodes(ShareNode itemsNode) {
        Map pointSystemsToItemConstraintsMap = new HashMap();
        if (itemsNode != null) {
            for (int i = 0; i < itemsNode.getChildCount(); i++) {
                ShareNode itemChild = (ShareNode) itemsNode.getChildAt(i);
                String pointSystemID = itemChild.getFileID();
                pointSystemsToItemConstraintsMap.put(pointSystemID, itemChild);
            }
        }
        return pointSystemsToItemConstraintsMap;
    }

    public static void buildShareList(DefaultMutableTreeNode parentNode, 
                                      Set nodeList, 
                                      Map itemConstraintMap) {
        int childCount = parentNode.getChildCount();
        for (int i = 0; i < childCount; i++) {
            ShareNode exportNode = (ShareNode) parentNode.getChildAt(i);
            if (exportNode.isSelectedForExport() && exportNode.isSharable()) {
                nodeList.add(exportNode);
                
                // Share the item constraints whenever sharing point systems or
                // whenever sharing the items master db.
                if (exportNode.getNodeType() == ExportNode.TYPE_POINT_SYSTEM) {
                    String pointSystemID = exportNode.getFileID();
                    ShareNode itemsNode = (ShareNode) itemConstraintMap.get(pointSystemID);
                    nodeList.add(itemsNode);
                } else if (exportNode.getNodeType() == ExportNode.TYPE_ITEMS_MASTER) {
                    nodeList.addAll(itemConstraintMap.values());
                }
            }
            buildShareList(exportNode, nodeList, itemConstraintMap);
        }
    }

    protected abstract String getFileID();
}
