package org.wow.autodkp.view.share;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;

import javax.swing.JCheckBox;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.tree.TreeCellRenderer;

/**
 * 
 * 
 * @author jsandusky
 */
public class ShareNodeTreeCellRenderer implements TreeCellRenderer {

    private JCheckBox leafRenderer = new JCheckBox();

    private Font  mHeaderFont;
    private Font  mNonLeafFont;
    private Font  mLeafFont;
    
    private Color selectionForeground;
    private Color selectionBackground;
    private Color textForeground;
    private Color textBackground;

    public ShareNodeTreeCellRenderer() {
        mLeafFont = UIManager.getFont("Tree.font");
        if (mLeafFont != null) {
            mNonLeafFont = mLeafFont.deriveFont(Font.BOLD);
            mHeaderFont  = mLeafFont.deriveFont(Font.BOLD, 14);
        } else {
            mLeafFont    = new Font("Serif", Font.PLAIN, 10);
            mNonLeafFont = new Font("Serif", Font.BOLD,  10);
            mHeaderFont  = new Font("Serif", Font.BOLD,  14);
        }
        
        Boolean focusBorder = (Boolean) UIManager.get("Tree.drawsFocusBorderAroundIcon");
        leafRenderer.setFocusPainted(focusBorder != null && focusBorder.booleanValue());

        selectionForeground = UIManager.getColor("Tree.selectionForeground");
        selectionBackground = UIManager.getColor("Tree.selectionBackground");
        textForeground = UIManager.getColor("Tree.textForeground");
        textBackground = UIManager.getColor("Tree.textBackground");
    }

    protected JCheckBox getLeafRenderer() {
        return leafRenderer;
    }

    public Component getTreeCellRendererComponent(JTree tree, 
                                                  Object value,
                                                  boolean selected, 
                                                  boolean expanded, 
                                                  boolean leaf, 
                                                  int row,
                                                  boolean hasFocus) {
        leafRenderer.setText("");
        leafRenderer.setSelected(false);
        leafRenderer.setEnabled(tree.isEnabled());

        if (selected) {
            leafRenderer.setForeground(selectionForeground);
            leafRenderer.setBackground(selectionBackground);
        } else {
            leafRenderer.setForeground(textForeground);
            leafRenderer.setBackground(textBackground);
        }

        if (value != null && value instanceof ShareNode) {
            ShareNode shareNode = (ShareNode) value;
            if (leaf) {
                leafRenderer.setFont(mLeafFont);
            } else if (shareNode.isHeader()) {
                leafRenderer.setFont(mHeaderFont);
            } else {
                leafRenderer.setFont(mNonLeafFont);
            }
            
            switch(shareNode.getNodeType()) {
            case ShareNode.TYPE_RAID:
                leafRenderer.setForeground(Color.BLACK);
                break;
            case ShareNode.TYPE_POINTS:
                leafRenderer.setForeground(Color.BLUE);
                break;
            case ShareNode.TYPE_POINT_SYSTEM:
                leafRenderer.setForeground(Color.RED);
                break;
            default:
                leafRenderer.setForeground(Color.MAGENTA);
                break;
            }
            
            leafRenderer.setText(shareNode.getText());
            leafRenderer.setSelected(shareNode.isSelectedForExport());
        }
        
        return leafRenderer;
    }
}
