package org.wow.autodkp.view.skin;

import java.awt.Color;
import java.util.Map;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.table.DefaultTableCellRenderer;

import org.jvnet.substance.SubstanceDefaultTableCellRenderer;
import org.jvnet.substance.SubstanceLookAndFeel;
import org.jvnet.substance.theme.SubstanceMixBiTheme;
import org.jvnet.substance.theme.SubstanceTheme;
import org.jvnet.substance.theme.ThemeInfo;
import org.jvnet.substance.utils.SubstanceCoreUtilities;
import org.jvnet.substance.watermark.WatermarkInfo;
import org.wow.autodkp.model.Logger;
import org.wow.autodkp.persistence.xml.XMLUtilities;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.preferences.IPreferencesTab;


public class AutoDKPLookAndFeelSupport {

    public static final Color COLOR_TOOLBAR = new Color(230, 255, 230);
    
    private static boolean ENABLED;
    
    
    public static void installSkin() {
        
        boolean needsPropertySave = false;
        GeneralProperties props = GeneralProperties.getDefault();
        
        ENABLED = true;
        String enabledStr = props.getStringProperty(props.SKINNING_ENABLED);
        if (enabledStr == null || enabledStr.length() == 0) {
            props.setStringProperty(props.SKINNING_ENABLED, XMLUtilities.IS_YES);
            needsPropertySave = true;
        } else {
            enabledStr = enabledStr.trim().toLowerCase();
            if (
                    enabledStr.equals(XMLUtilities.IS_NO) || 
                    enabledStr.equals(XMLUtilities.IS_FALSE))
            {
                ENABLED = false;
            }
        }
        if (!ENABLED) {
            return;
        }
        
        JFrame.setDefaultLookAndFeelDecorated(true);
        JDialog.setDefaultLookAndFeelDecorated(true);
        System.setProperty("sun.awt.noerasebackground", "true");
        try {
            UIManager.setLookAndFeel(new SubstanceLookAndFeel());
        } catch (UnsupportedLookAndFeelException e) {
            Logger.log(e);
        }
        
        String theme1Name = props.getStringProperty(props.SKIN_THEME_MAIN);
        String theme2Name = props.getStringProperty(props.SKIN_THEME_SECONDARY);
        if (theme1Name == null) {
            theme1Name = props.SKIN_THEME_DEFAULT_MAIN;
            props.setStringProperty(props.SKIN_THEME_MAIN, theme1Name);
            theme2Name = props.SKIN_THEME_DEFAULT_SECONDARY;
            props.setStringProperty(props.SKIN_THEME_SECONDARY, theme2Name);
            needsPropertySave = true;
        }
        updateThemes();
        
        String watermarkName = props.getStringProperty(props.SKIN_WATERMARK);
        if (watermarkName == null) {
            watermarkName = props.SKIN_WATERMARK_DEFAULT;
            props.setStringProperty(props.SKIN_WATERMARK, watermarkName);
            needsPropertySave = true;
        }
        updateWatermark();
        
        if (needsPropertySave) {
            props.save();
        }
        
        // Set up UI defaults.
        UIDefaults uiDefaults = UIManager.getDefaults();
        // The check boxes don't quite paint correctly, replace with standard L&F.
        uiDefaults.put("CheckBoxUI", "javax.swing.plaf.metal.MetalCheckBoxUI");
    }
    
    
    public static boolean updateThemes() {
        if (!ENABLED) {
            return false;
        }

        GeneralProperties props = GeneralProperties.getDefault();
        String theme1Name = props.getStringProperty(props.SKIN_THEME_MAIN);
        String theme2Name = props.getStringProperty(props.SKIN_THEME_SECONDARY);
        Map allThemes = SubstanceLookAndFeel.getAllThemes();
        ThemeInfo info1 = (ThemeInfo) allThemes.get(theme1Name);
        ThemeInfo info2 = null;
        if (theme2Name != null) {
            info2 = (ThemeInfo) allThemes.get(theme2Name);
        }
        if (info1 != null && info2 == null) {
            SubstanceLookAndFeel.setCurrentTheme(info1);
            return true;
        } else if (info1 != null && info2 != null) {
            SubstanceTheme theme1 = SubstanceTheme.createInstance(info1);
            SubstanceTheme theme2 = SubstanceTheme.createInstance(info2);
            SubstanceLookAndFeel.setCurrentTheme(new SubstanceMixBiTheme(theme1, theme2));
            return true;
        }
        return false;
    }
    
    
    public static boolean updateWatermark() {
        if (!ENABLED) {
            return false;
        }

        GeneralProperties props = GeneralProperties.getDefault();
        String name = props.getStringProperty(props.SKIN_WATERMARK);
        if (name != null) {
            WatermarkInfo info = 
                (WatermarkInfo) SubstanceLookAndFeel.getAllWatermarks().get(name);
            SubstanceLookAndFeel.setCurrentWatermark(info.getWatermarkClassName());
            return true;
        }
        return false;
    }
    
    
    public static DefaultTableCellRenderer getDefaultTableCellRenderer() {
        if (!ENABLED) {
            return new DefaultTableCellRenderer();
        }
        return new SubstanceDefaultTableCellRenderer();
    }
    
    
    public static Color getDarkColor() {
        if (!ENABLED) {
            return Color.GRAY;
        }
        return SubstanceLookAndFeel.getColorScheme().getDarkColor();
    }
    
    
    public static Color getLightColor() {
        if (!ENABLED) {
            return Color.LIGHT_GRAY;
        }
        return SubstanceLookAndFeel.getColorScheme().getLightColor();
    }
    
    
    public static Color getPrimaryThemeColor() {
        if (!ENABLED) {
            return COLOR_TOOLBAR;
        }
        return SubstanceLookAndFeel.getCurrentTheme().getWindowTitleBackground();
    }
    
    
    public static Color getUnselectedBackgroundColor(
            int index, 
            Color background)
    {
        if (!ENABLED) {
            return background;
        }
        if (index % 2 != 0) {
            int r = background.getRed();
            int g = background.getGreen();
            int b = background.getBlue();
            double coef = SubstanceCoreUtilities
                    .isThemeDark(SubstanceLookAndFeel.getTheme()) ? 0.80
                    : 0.96;
            background = 
                new Color((int) (coef * r), (int) (coef * g), (int) (coef * b));
        }
        return background;
    }
    
    
    public static IPreferencesTab getSkinningControls() {
        if (!ENABLED) {
            return new EmptySkinningControls();
        }
        return new SkinningControls();
    }
}
