package org.wow.autodkp.view.skin;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.jvnet.substance.SubstanceLookAndFeel;
import org.wow.autodkp.persistence.xml.XMLUtilities;
import org.wow.autodkp.view.AutoDKP;
import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.preferences.IPreferencesTab;


public class SkinningControls extends JPanel implements IPreferencesTab {
    
    private JCheckBox mEnableSkinningCheckbox;
    private JCheckBox mOptionalThemeCheckbox;
    private JComboBox mTheme1;
    private JComboBox mTheme2;
    private JComboBox mWatermark;
    
    private Box mSecondaryThemeBox;
    private boolean mIsModified;
    
    
    public SkinningControls() {
        super(new BorderLayout());
        
        List allThemeNames = new ArrayList();
        List allWatermarkNames = new ArrayList();
        
        for (Iterator iter=SubstanceLookAndFeel.getAllThemes().keySet().iterator(); iter.hasNext();) {
            allThemeNames.add(iter.next());
        }
        Collections.sort(allThemeNames);
        
        for (Iterator iter=SubstanceLookAndFeel.getAllWatermarks().keySet().iterator(); iter.hasNext();) {
            allWatermarkNames.add(iter.next());
        }
        Collections.sort(allWatermarkNames);
        
        mTheme1 = new JComboBox(allThemeNames.toArray());
        mTheme2 = new JComboBox(allThemeNames.toArray());
        mWatermark = new JComboBox(allWatermarkNames.toArray());
        mOptionalThemeCheckbox = new JCheckBox("Use Secondary Theme?");
        
        final GeneralProperties props = GeneralProperties.getDefault();
        
        ActionListener theme1Listener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String name = (String) mTheme1.getSelectedItem();
                props.setStringProperty(props.SKIN_THEME_MAIN, name);
                mIsModified = true;
                if (AutoDKPLookAndFeelSupport.updateThemes()) {
                    refreshScreen();
                }
            }
        };
        ActionListener theme2Listener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mOptionalThemeCheckbox.isSelected()) {
                    String name = (String) mTheme2.getSelectedItem();
                    props.setStringProperty(props.SKIN_THEME_SECONDARY, name);
                    mIsModified = true;
                    if (AutoDKPLookAndFeelSupport.updateThemes()) {
                        refreshScreen();
                    }
                }
            }
        };
        ActionListener useTheme2Listener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mOptionalThemeCheckbox.isSelected()) {
                    mSecondaryThemeBox.setVisible(true);
                    String name = (String) mTheme2.getSelectedItem();
                    props.setStringProperty(props.SKIN_THEME_SECONDARY, name);
                    mIsModified = true;
                } else {
                    mSecondaryThemeBox.setVisible(false);
                    props.removeProperty(props.SKIN_THEME_SECONDARY);
                    mIsModified = true;
                }
                if (AutoDKPLookAndFeelSupport.updateThemes()) {
                    refreshScreen();
                }
            }
        };
        ActionListener watermarkListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String name = (String) mWatermark.getSelectedItem();
                props.setStringProperty(props.SKIN_WATERMARK, name);
                mIsModified = true;
                if (AutoDKPLookAndFeelSupport.updateWatermark()) {
                    refreshScreen();
                }
            }
        };
        
        mOptionalThemeCheckbox.setBorder(null);
        mOptionalThemeCheckbox.setOpaque(false);

        JLabel themeLabel = new JLabel("Main Theme:");
        JLabel optionalThemeLabel = new JLabel("Secondary Theme:");
        themeLabel.setPreferredSize(optionalThemeLabel.getPreferredSize());
        JLabel watermarkLabel = new JLabel("Watermark:");
        
        Box theme1Box = Box.createHorizontalBox();
        theme1Box.add(themeLabel);
        theme1Box.add(Box.createHorizontalStrut(5));
        theme1Box.add(mTheme1);
        
        mSecondaryThemeBox = Box.createHorizontalBox();
        mSecondaryThemeBox.add(optionalThemeLabel);
        mSecondaryThemeBox.add(Box.createHorizontalStrut(5));
        mSecondaryThemeBox.add(mTheme2);
        
        Box watermarkHBox = Box.createHorizontalBox();
        watermarkHBox.add(watermarkLabel);
        watermarkHBox.add(Box.createHorizontalStrut(5));
        watermarkHBox.add(mWatermark);

        JPanel checkboxPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
        checkboxPanel.add(mOptionalThemeCheckbox);
        
        Box themeBox = Box.createVerticalBox();
        themeBox.add(Box.createVerticalGlue());
        themeBox.add(theme1Box);
        themeBox.add(Box.createVerticalStrut(30));
        themeBox.add(checkboxPanel);
        themeBox.add(Box.createVerticalStrut(5));
        themeBox.add(mSecondaryThemeBox);
        themeBox.add(Box.createVerticalGlue());
        
        JPanel themePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        themePanel.add(themeBox);
        themePanel.setBorder(BorderFactory.createTitledBorder("Theme"));
        
        JPanel watermarkPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        watermarkPanel.add(watermarkHBox);
        watermarkPanel.setBorder(BorderFactory.createTitledBorder("Watermark"));
        
        mEnableSkinningCheckbox = new JCheckBox("Skinning Enabled?");
        mEnableSkinningCheckbox.setBorder(null);
        mEnableSkinningCheckbox.setOpaque(false);
        
        JPanel enableSkinningPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        enableSkinningPanel.add(mEnableSkinningCheckbox);
        
        mEnableSkinningCheckbox.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String str = mEnableSkinningCheckbox.isSelected() ?
                        XMLUtilities.IS_YES : XMLUtilities.IS_NO;
                props.setStringProperty(props.SKINNING_ENABLED, str);
                mIsModified = true;
                JOptionPane.showMessageDialog(
                        SkinningControls.this,
                        "You must restart " + AutoDKP.PROGRAM_NAME + " for your changes to take effect.",
                        "Skinning",
                        JOptionPane.INFORMATION_MESSAGE);
            }
        });
        String str = props.getStringProperty(props.SKINNING_ENABLED);
        boolean isEnabled = 
            str == null ||
            (!str.equals(XMLUtilities.IS_NO) && !str.equals(XMLUtilities.IS_FALSE));
        mEnableSkinningCheckbox.setSelected(isEnabled);
        
        Box vBox = Box.createVerticalBox();
        vBox.add(Box.createVerticalGlue());
        vBox.add(themePanel);
        vBox.add(Box.createVerticalStrut(10));
        vBox.add(watermarkPanel);
        vBox.add(Box.createVerticalStrut(10));
        vBox.add(enableSkinningPanel);
        vBox.add(Box.createVerticalGlue());
        
        add(vBox, BorderLayout.CENTER);
        
        String theme1Name = props.getStringProperty(props.SKIN_THEME_MAIN);
        String theme2Name = props.getStringProperty(props.SKIN_THEME_SECONDARY);
        if (theme1Name == null) {
            theme1Name = props.SKIN_THEME_DEFAULT_MAIN;
            props.setStringProperty(props.SKIN_THEME_MAIN, theme1Name);
            theme2Name = props.SKIN_THEME_DEFAULT_SECONDARY;
            props.setStringProperty(props.SKIN_THEME_SECONDARY, theme2Name);
            mIsModified = true;
        }
        
        mTheme1.setSelectedItem(theme1Name);
        
        if (theme2Name == null) {
            mOptionalThemeCheckbox.setSelected(false);
            mSecondaryThemeBox.setVisible(false);
        } else {
            mOptionalThemeCheckbox.setSelected(true);
            mTheme2.setSelectedItem(theme2Name);
        }
        
        String watermarkName = props.getStringProperty(props.SKIN_WATERMARK);
        if (watermarkName != null) {
            mWatermark.setSelectedItem(watermarkName);
        }
        
        mTheme1.addActionListener(theme1Listener);
        mTheme2.addActionListener(theme2Listener);
        mOptionalThemeCheckbox.addActionListener(useTheme2Listener);
        mWatermark.addActionListener(watermarkListener);
    }
    
    
    private void refreshScreen() {
        AutoDKPView.getInstance().getFrame().repaint();
        Component root = SwingUtilities.getRoot(this);
        if (root != null) {
            root.repaint();
        }
    }


    public void closing() {
        if (mIsModified) {
            GeneralProperties.getDefault().save();
        }
    }

    public JComponent getComponent() {
        return this;
    }

    public boolean shouldReload() {
        return false;
    }
}
