package org.wow.autodkp.view.summary;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.wow.autodkp.model.BossKillInfo;
import org.wow.autodkp.view.handlers.DialogBossKill;
import org.wow.autodkp.view.preferences.PreferencesWindow;


/**
 * Shows boss kills for the selected raid.
 * 
 * @author jsandusky
 */
public class BossSummarySection extends AbstractSummarySection {

    private JList mBossList;
    private BossKillInfo mCurrentBossKillInfo;

    
    public BossSummarySection(SummaryPane tab) {
        super(tab);
        
        mBossList = new JList();
        mTabComponent = PreferencesWindow.generatePane(
                null, 
                getAddBossListener(),
                getEditBossListener(),
                getDeleteBossListener(),
                getBossSelectionListener(),
                mBossList);
    }
    
    public void sectionChange() {
        updateBossList();
    }
    
    private ActionListener getAddBossListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DialogBossKill diag = new DialogBossKill(
                        "Add Boss Kill",
                        null,
                        mSummaryPane.getCurrentRaid().getStartTime());
                diag.setVisible(true);
                BossKillInfo bossKillInfo = diag.getBossKillInfo();
                if (bossKillInfo != null) {
                    mSummaryPane.getCurrentRaid().setBossKill(bossKillInfo);
                    mSummaryPane.getCurrentRaid().promote();
                    updateBossList();
                    if (mSummaryPane.getCurrentPointSystem() != null) {
                        mSummaryPane.getPlayerTable().fireTableDataChanged();
                    }
                }
            }
        };
    }
    
    private ActionListener getEditBossListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mCurrentBossKillInfo != null) {
                    DialogBossKill diag = new DialogBossKill(
                            "Edit Custom Boss Points", 
                            mCurrentBossKillInfo,
                            mSummaryPane.getCurrentRaid().getStartTime());
                    diag.setVisible(true);
                    BossKillInfo bossKillInfo = diag.getBossKillInfo();
                    if (bossKillInfo != null) {
                        mSummaryPane.getCurrentRaid().setBossKill(bossKillInfo);
                        mSummaryPane.getCurrentRaid().promote();
                        updateBossList();
                        if (mSummaryPane.getCurrentPointSystem() != null) {
                            mSummaryPane.getPlayerTable().fireTableDataChanged();
                        }
                    }
                }
            }
        };
    }
    
    private ActionListener getDeleteBossListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mCurrentBossKillInfo != null) {
                    int result = JOptionPane.showConfirmDialog(
                            mSummaryPane, 
                            mCurrentBossKillInfo.getBossName() + "\n" +
                            "Are you sure you wish to delete this boss kill from the raid?\n", 
                            "Delete Boss Kill",
                            JOptionPane.OK_CANCEL_OPTION,
                            JOptionPane.WARNING_MESSAGE);
                    if (result == JOptionPane.YES_OPTION) {
                        mSummaryPane.getCurrentRaid().removeBossKill(
                                mCurrentBossKillInfo.getBossName());
                        mSummaryPane.getCurrentRaid().promote();
                        updateBossList();
                        if (mSummaryPane.getCurrentPointSystem() != null) {
                            mSummaryPane.getPlayerTable().fireTableDataChanged();
                        }
                    }
                }
            }
        };
    }
    
    private ListSelectionListener getBossSelectionListener() {
        return new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    mCurrentBossKillInfo = (BossKillInfo) mBossList.getSelectedValue();
                }
            }
        };
    }

    private void updateBossList() {
        Map bossKillMap = mSummaryPane.getCurrentRaid().getBossNameToKillDateMap();
        TreeMap bossData = new TreeMap();
        int i = 0;
        for (Iterator iter=bossKillMap.keySet().iterator(); iter.hasNext(); i++) {
            String bossName = (String) iter.next();
            Date killDate = (Date) bossKillMap.get(bossName);
            bossData.put(killDate, new BossKillInfo(bossName, killDate));
        }
        mBossList.setListData(bossData.values().toArray());
    }
}
