package org.wow.autodkp.view.summary;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.persistence.InstanceManager;
import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.BasicDialog;
import org.wow.autodkp.view.skin.AutoDKPLookAndFeelSupport;


/**
 * Dialog for selecting/de-selecting instances (i.e. Molten Core, etc.)
 * 
 * @author jsandusky
 */
public class DialogModifyInstances extends BasicDialog {
    
    private JList mInstancesList;
    private Map mInstanceItemMap;
    
    
    public DialogModifyInstances(
            Frame owner, 
            String title) {
        super(owner, title, true);
        addBasicSupport();
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
        setLayout(new BorderLayout());
        getContentPane().add(getMainPane(), BorderLayout.CENTER);
        getContentPane().add(getButtonPane(getApplyActionListener()), BorderLayout.SOUTH);

        setResizable(false);
        setSize(300, 300);
        setLocationRelativeTo(owner);
    }
    
    protected ActionListener getApplyActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                List newInstanceNames = new ArrayList();
                for (Iterator iter=mInstanceItemMap.values().iterator(); iter.hasNext();) {
                    InstanceItem instanceItem = (InstanceItem) iter.next();
                    if (instanceItem.isSelected()) {
                        newInstanceNames.add(instanceItem.getInstanceName());
                    }
                }
                Raid selectedRaid = AutoDKPModel.getInstance().getSelectedRaid();
                selectedRaid.setInstanceNames(newInstanceNames);
                selectedRaid.promote();
                AutoDKPView.getInstance().reloadRaids();
                dispose();
            }
        };
    }
    
    protected JComponent getMainPane() {
        mInstanceItemMap = new HashMap();
        Object[] allInstanceNames = InstanceManager.getDefault().getNames().toArray();
        for (int i=0; i < allInstanceNames.length; i++) {
            mInstanceItemMap.put(allInstanceNames[i], new InstanceItem((String) allInstanceNames[i]));
        }
        mInstancesList = new JList(mInstanceItemMap.values().toArray());
        mInstancesList.setCellRenderer(new InstanceCellRenderer());
        
        mInstancesList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() > 1) {
                    Point point = e.getPoint();
                    int index = mInstancesList.locationToIndex(point);
                    if (index >= 0) {
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                InstanceItem instanceItem = 
                                    (InstanceItem) mInstancesList.getSelectedValue();
                                instanceItem.setSelected(!instanceItem.isSelected());
                                mInstancesList.repaint();
                            }
                        });
                    }
                }
            }
        });
        
        mInstancesList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JScrollPane scroller = new JScrollPane(mInstancesList);
        scroller.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(10, 5, 5, 5),
                scroller.getBorder()));
        
        List preSelectedNames = AutoDKPModel.getInstance().getSelectedRaid().getInstanceNames();
        for (Iterator iter=preSelectedNames.iterator(); iter.hasNext();) {
            String preSelectedName = (String) iter.next();
            InstanceItem instanceItem = (InstanceItem) mInstanceItemMap.get(preSelectedName);
            if (instanceItem != null) {
                instanceItem.setSelected(true);
            }
        }
        
        JPanel mainPane = new JPanel();
        mainPane.setLayout(new BorderLayout());
        mainPane.add(scroller, BorderLayout.CENTER);
        
        mainPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 5, 10));
        
        return mainPane;
    }
    
    protected JComponent getButtonPane(ActionListener actionListener) {
        JButton applyButton  = new JButton();
        applyButton.setText("Save");
        applyButton.addActionListener(actionListener);
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        
        JPanel buttonPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttonPane.add(applyButton);
        buttonPane.add(cancelButton);
        buttonPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        return buttonPane;
    }
    
    
    private class InstanceCellRenderer implements ListCellRenderer {
        
        protected Border noFocusBorder;
        private JCheckBox mCheck;
        
        public InstanceCellRenderer() {
            mCheck = new JCheckBox();
            mCheck.setOpaque(true);
            mCheck.setEnabled(true);
            if (noFocusBorder == null) {
                noFocusBorder = new EmptyBorder(1, 1, 1, 1);
            }
            mCheck.setBorder(noFocusBorder);
        }
        
        public Component getListCellRendererComponent(
                JList list, 
                Object value, 
                int index,
                boolean isSelected, 
                boolean cellHasFocus)
        {
            mCheck.setComponentOrientation(list.getComponentOrientation());
            
            if (isSelected) {
                mCheck.setBackground(list.getSelectionBackground());
                mCheck.setForeground(list.getSelectionForeground());
            }
            else {
                mCheck.setForeground(list.getForeground());
                Color background = 
                    AutoDKPLookAndFeelSupport.getUnselectedBackgroundColor(
                            index, 
                            list.getBackground());
                mCheck.setBackground(background);
            }

            mCheck.setFont(list.getFont());
            mCheck.setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);

            InstanceItem instanceItem = (InstanceItem) value;
            mCheck.setText(instanceItem.getInstanceName());
            mCheck.setSelected(instanceItem.isSelected());
            
            return mCheck;
        }
    }
}
