package org.wow.autodkp.view.summary;

import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JToggleButton;
import javax.swing.border.Border;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.FormatConstants;
import org.wow.autodkp.model.Logger;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.DefaultPointSystem;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.IPointSystemRaidData;
import org.wow.autodkp.model.points.RollCall;
import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.ImageUtilities;
import org.wow.autodkp.view.RaidLinker;
import org.wow.autodkp.view.table.players.PlayerTable;


/**
 * Shows combined summary information for a particular raid and
 * point system. Contains controls for starting points, raid linking, etc.
 * 
 * @author jsandusky
 */
public class PointSummaryPane extends JPanel {
    
    public  static final Color BG_COLOR   = Color.WHITE;
    private static final String SPACING   = " ";
    private static final String NO_IMPORT = "<HTML><P><B><FONT COLOR=BLUE> None</FONT></B></P></HTML>";
    private static final String LOCKED    = "<HTML><P><B><FONT COLOR=BLACK> Locked</FONT></B></P></HTML>";
    private static final String UNLOCKED  = "<HTML><P><B><FONT COLOR=RED> Unlocked</FONT></B></P></HTML>";
    private static final String POINTS_STARTED  = "<HTML><P><B><FONT COLOR=BLUE> Started</FONT></B></P></HTML>";
    private static final String POINTS_NONE     = "<HTML><P><B><FONT COLOR=BLACK> None</FONT></B></P></HTML>";
    private static final String POINTS_FINISHED = "<HTML><P><B><FONT COLOR=BLACK> Finished</FONT></B></P></HTML>";
    
    
    private IPointSystem mConstraints;
    private Raid mCurrentRaid;
    
    private SummaryPane mSummaryPane;
    
    private JLabel mPointSystemLabel;
    private JLabel mRaidDateLabel;
    private JLabel mRaidInstancesLabel;
    private JLabel mImportedRaidLabel;
    private JLabel mLockStatusLabel;
    private JLabel mStatusLabel;
    private JLabel mStatusTitleLabel;
    private JLabel mImportedRaidTitleLabel;
    private JLabel mLockTitleLabel;
    private ActionListener mStatusChanger;
    
    private JToggleButton mLockButton;
    private JButton mLinkRaidButton;
    private JButton mStatusButton;
    private JButton mRaidInstancesButton;
    private ImageIcon mLockIcon;
    private Image mImageLocked;
    private Image mImageUnlocked;
    
    
    public PointSummaryPane(SummaryPane summaryPane,
                            PlayerTable playerTable, 
                            IPointSystem constraints,
                            Raid currentRaid,
                            boolean isStarted)
    {
        mSummaryPane = summaryPane;
        mConstraints = constraints;
        mCurrentRaid = currentRaid;
        
        setBackground(BG_COLOR);
        
        setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
        
        JLabel pointSystemLabel   = new JLabel("Point System:");
        JLabel raidDateLabel      = new JLabel("Raid Date:");
        JLabel raidInstancesLabel = new JLabel("Raid Instances:");
        mImportedRaidTitleLabel   = new JLabel("Linked Raid:");
        mLockTitleLabel           = new JLabel("Formulas:");
        mStatusTitleLabel         = new JLabel("Status:");
        
        Border valueBorder = BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(0, 10, 0, 5),
                BorderFactory.createMatteBorder(0, 0, 1, 0, Color.BLACK));
        mRaidDateLabel = new JLabel();
        mRaidDateLabel.setBorder(valueBorder);
        mRaidInstancesLabel = new JLabel();
        mRaidInstancesLabel.setBorder(valueBorder);
        mImportedRaidLabel = new JLabel();
        mImportedRaidLabel.setBorder(valueBorder);
        mLockStatusLabel = new JLabel();
        mLockStatusLabel.setBorder(valueBorder);
        mPointSystemLabel = new JLabel();
        mPointSystemLabel.setBorder(valueBorder);
        mStatusLabel = new JLabel();
        mStatusLabel.setBorder(valueBorder);
        
        mRaidInstancesButton = new JButton("+/-");
        mRaidInstancesButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DialogModifyInstances diag = new DialogModifyInstances(
                        AutoDKPView.getInstance().getFrame(), 
                        "Modify Raid Instances");
                diag.setVisible(true);
            }
        });
        
        mStatusButton = new JButton();

        mImageLocked = loadImage("locked.gif");
        mImageUnlocked = loadImage("unlocked.gif");
        mLockIcon = new ImageIcon(mImageLocked);
        mLockButton = new JToggleButton(mLockIcon);
        
        ActionListener lockListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mConstraints.getCurrentRaidData().isLocked()) {
                    mConstraints.getCurrentRaidData().setLocked(false);
                    mLockStatusLabel.setText(UNLOCKED);
                    mLockIcon.setImage(mImageUnlocked);
                    mLockButton.repaint();
                    updateStatusText(mConstraints.isPointsStarted(mCurrentRaid));
                    mConstraints.saveRaid(mCurrentRaid);
                    // cause general UI update
                    AutoDKPView.getInstance().reloadRaids();
                } else {
                    if (permitEntries()) {
                        mConstraints.getCurrentRaidData().setLocked(true);
                        mLockStatusLabel.setText(LOCKED);
                        mLockIcon.setImage(mImageLocked);
                        mLockButton.repaint();
                        updateStatusText(mConstraints.isPointsStarted(mCurrentRaid));
                        mConstraints.saveRaid(mCurrentRaid);
                        // cause general UI update
                        AutoDKPView.getInstance().reloadRaids();
                    }
                }
            }
        };
        mLockButton.addActionListener(lockListener);
        
        mLinkRaidButton = new JButton("Link");
        mLinkRaidButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                importRaid();
            }
        });
        
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints gdc = new GridBagConstraints();
        setLayout(gridBag);
        gdc.anchor = GridBagConstraints.NORTHWEST;
        
        int y = -1;
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(pointSystemLabel, gdc);
        add(pointSystemLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mPointSystemLabel, gdc);
        add(mPointSystemLabel);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(raidDateLabel, gdc);
        add(raidDateLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mRaidDateLabel, gdc);
        add(mRaidDateLabel);

        gdc.weighty = 1;
        
        gdc.anchor = GridBagConstraints.SOUTHWEST;
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(raidInstancesLabel, gdc);
        add(raidInstancesLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 2;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mRaidInstancesLabel, gdc);
        add(mRaidInstancesLabel);
        gdc.gridx = 3;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mRaidInstancesButton, gdc);
        add(mRaidInstancesButton);
        
        JSeparator separatorMiddle = new JSeparator(JSeparator.VERTICAL);
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 1;
        gdc.gridwidth = 4;
        gdc.weighty = 4;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(separatorMiddle, gdc);
        add(separatorMiddle);
        
        gdc.weighty = 1;
        
        gdc.anchor = GridBagConstraints.SOUTHWEST;
        
        if (!isStarted) {
            addStartStatusControls();
        } else {
            addDeleteStatusControls();
        }
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mStatusTitleLabel, gdc);
        add(mStatusTitleLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 2;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mStatusLabel, gdc);
        add(mStatusLabel);
        gdc.gridx = 3;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mStatusButton, gdc);
        add(mStatusButton);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mLockTitleLabel, gdc);
        add(mLockTitleLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 2;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mLockStatusLabel, gdc);
        add(mLockStatusLabel);
        gdc.gridx = 3;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mLockButton, gdc);
        add(mLockButton);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mImportedRaidTitleLabel, gdc);
        add(mImportedRaidTitleLabel);
        gdc.gridx = 1;
        gdc.weightx = 8;
        gdc.gridwidth = 2;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mImportedRaidLabel, gdc);
        add(mImportedRaidLabel);
        gdc.gridx = 3;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mLinkRaidButton, gdc);
        add(mLinkRaidButton);
        
        gdc.anchor = GridBagConstraints.NORTHWEST;
        JSeparator separatorBottom = new JSeparator(JSeparator.VERTICAL);
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 1;
        gdc.gridwidth = 4;
        gdc.weighty = 4;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(separatorBottom, gdc);
        add(separatorBottom);
        
        initializeForRaid();
    }
    
    private void updateStatusText(boolean isStarted) {
        if (!isStarted) {
            mStatusLabel.setText(POINTS_NONE);
        } else {
            if (mConstraints.getCurrentRaidData().isLocked()) {
                mStatusLabel.setText(POINTS_FINISHED);
            } else {
                mStatusLabel.setText(POINTS_STARTED);
            }
        }
    }
    
    private void updateStatusControls(String imagePath, ActionListener changer) {
        boolean isStarted = mConstraints.isPointsStarted(mCurrentRaid);
        updateStatusText(isStarted);
        mStatusButton.removeActionListener(mStatusChanger);
        Image imageStatus = loadImage(imagePath);
        mStatusButton.setIcon(new ImageIcon(imageStatus));
        mStatusChanger = changer;
        mStatusButton.addActionListener(mStatusChanger);
        
        mLockTitleLabel.setVisible(isStarted);
        mLockStatusLabel.setVisible(isStarted);
        mLockButton.setVisible(isStarted);
        mImportedRaidTitleLabel.setVisible(isStarted);
        mImportedRaidLabel.setVisible(isStarted);
        mLinkRaidButton.setVisible(isStarted);
    }
    
    private void addStartStatusControls() {
        ActionListener statusChanger = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // save the raid to set it started
                mConstraints.saveRaid(mCurrentRaid);
                AutoDKPView.getInstance().reloadRaids();
                // save the master raid file for future reference
                mCurrentRaid.promote();
            }
        };
        updateStatusControls("start.gif", statusChanger);
    }
    
    private void addDeleteStatusControls() {
        ActionListener statusChanger = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int result = JOptionPane.showConfirmDialog(
                        PointSummaryPane.this, 
                        "Are you sure you wish to delete points for this raid?\n" + 
                        "This cannot be undone.",
                        "Delete Raid Points",
                        JOptionPane.OK_CANCEL_OPTION,
                        JOptionPane.WARNING_MESSAGE);
                if (result == JOptionPane.YES_OPTION) {
                    // delete the raid to unstart it
                    mConstraints.deleteRaid(mCurrentRaid);
                    AutoDKPView.getInstance().reloadRaids();
                }
            }
        };
        updateStatusControls("delete.gif", statusChanger);
    }
    
    public Image loadImage(String imageName) {
        return ImageUtilities.loadJarImage(imageName);
    }
    
    private void importRaid() {
        Frame owner = null;
        Component parent = getParent();
        while (parent != null) {
            if (parent instanceof Frame) {
                owner = (Frame) parent;
                break;
            }
            parent = parent.getParent();
        }
        List raids = AutoDKPModel.getInstance().getRaidItems();
        List importableRaidsList = new ArrayList();
        for (Iterator iter=raids.iterator(); iter.hasNext();) {
            Raid raid = (Raid) iter.next();
            if (raid.getStartTime().compareTo(mCurrentRaid.getStartTime()) < 0) {
                importableRaidsList.add(raid);
            }
        }
        Raid[] importableRaids = (Raid[]) importableRaidsList.toArray(new Raid[0]);
        RaidLinker importer = new RaidLinker(owner, mConstraints, importableRaids);
        if (importer.isRaidSelectionApplied()) {
            Raid importedRaid = importer.getSelectedRaid();
            mImportedRaidLabel.setText(importedRaid.getStartTime().toString());
            IPointSystemRaidData importedRaidData = DefaultPointSystem.loadConstraintsForRaid(
                    mConstraints, 
                    importedRaid, 
                    null);
            mConstraints.importRaid(
                    importedRaid, 
                    importedRaidData, 
                    mCurrentRaid);
            AutoDKPView.getInstance().reloadRaids();
        }
    }
    
    private void initializeForRaid() {
        GeneralProperties props = GeneralProperties.getDefault();
        Date raidDate = mCurrentRaid.getStartTime();
        mRaidDateLabel.setText(SPACING + 
                props.getDateFormatProperty(props.DATE_FORMAT_LONG).format(raidDate));
        
        StringBuffer buf = new StringBuffer(SPACING);
        List instanceNames = mCurrentRaid.getInstanceNames();
        for (Iterator iter=instanceNames.iterator(); iter.hasNext();) {
            if (buf.length() != SPACING.length()) {
                buf.append(", ");
            }
            buf.append(iter.next());
        }
        String instances = buf.toString();
        if (instanceNames.size() > 0) {
            mRaidInstancesLabel.setText("<HTML><P>" + instances + "</P></HTML>");
        } else {
            mRaidInstancesLabel.setText(SPACING);
        }
        
        String importedRaidID = mConstraints.getCurrentRaidData().getImportedRaidID();
        if (importedRaidID != null && importedRaidID.length() > 0) {
            String importedRaidText = importedRaidID;
            try {
                Date date = FormatConstants.FORMAT_ID.parse(importedRaidID);
                importedRaidText = 
                    props.getDateFormatProperty(props.DATE_FORMAT_LONG).format(date);
            } catch (ParseException e) {
                // do nothing, just use id
                Logger.log(e);
            }
            mImportedRaidLabel.setText(importedRaidText);
        } else {
            mImportedRaidLabel.setText(NO_IMPORT);
        }
        
        mPointSystemLabel.setText(SPACING + mConstraints.getName());
        
        if (mConstraints.getCurrentRaidData().isLocked()) {
            mLockStatusLabel.setText(LOCKED);
            mLockIcon.setImage(mImageLocked);
            mLockButton.repaint();
        } else {
            mLockStatusLabel.setText(UNLOCKED);
            mLockIcon.setImage(mImageUnlocked);
            mLockButton.repaint();
        }
    }
    
    private boolean permitEntries() {
        // For Time DKP, we need the Raid Start and End times.
        // If the user does not wish to enter these, no Time DKP will be calculated.
        if (
                mConstraints.isTimeBased() &&
                mConstraints.getDurationPoints() > 0 &&
                mConstraints.getDurationMinutes() > 0) {
            RollCall start = mConstraints.getRollCall(RollCall.RAID_START_NAME);
            RollCall end   = mConstraints.getRollCall(RollCall.RAID_END_NAME);
            Map rollCallMap = mConstraints.getCurrentRaidData().getRollCallDateMap();
            Date startDate = (Date) rollCallMap.get(start.getName());
            Date endDate   = (Date) rollCallMap.get(end.getName());
            if (startDate == null || endDate == null) {
                String msg = null;
                if        (startDate == null && endDate == null) {
                    msg = "Both Raid Start and End times have not been entered.";
                } else if (startDate != null && endDate == null) {
                    msg = "The Raid End time has not been entered.";
                } else if (startDate == null && endDate != null) {
                    msg = "The Raid Start time has not been entered.";
                }
                int result = JOptionPane.showConfirmDialog(
                        PointSummaryPane.this,
                        msg + "\n" +
                        "All Time DKP calculations depend upon the Raid Start and End times.\n" +
                        "If you do not set both times, Time DKP points will not be awarded.\n" +
                        "\n" +
                        "Do you wish to go back and enter missing times?\n\n",
                        "No Time Information",
                        JOptionPane.YES_NO_OPTION,
                        JOptionPane.QUESTION_MESSAGE);
                if (result == JOptionPane.YES_OPTION) {
                    int timeTabIndex = mSummaryPane.indexOfTab(SummaryPane.TAB_TITLE_RAID_TIME);
                    if (timeTabIndex >= 0 && timeTabIndex < mSummaryPane.getTabCount()) {
                        mSummaryPane.setSelectedIndex(timeTabIndex);
                    }
                    return false;
                }
            }
        }
        return true;
    }
    
}
