package org.wow.autodkp.view.summary;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractCellEditor;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import org.wow.autodkp.model.FormatConstants;
import org.wow.autodkp.model.points.RollCall;
import org.wow.autodkp.view.DateControls;
import org.wow.autodkp.view.skin.AutoDKPLookAndFeelSupport;


public class RollCallDateCell extends AbstractCellEditor implements TableCellRenderer, TableCellEditor {
    
    private SummaryPane mSummaryPane;
    private JComponent mComponent;
    private JLabel mNameLabel;
    private DateControls mDateControls;
    
    protected static Border noFocusBorder = new EmptyBorder(1, 1, 1, 1); 
    private static final Border SAFE_NO_FOCUS_BORDER = new EmptyBorder(1, 1, 1, 1);

    
    public RollCallDateCell(SummaryPane summaryTab) {
        
        mSummaryPane = summaryTab;
        
        mNameLabel = new JLabel();
        mNameLabel.setOpaque(false);
        mDateControls = new DateControls();
        mDateControls.getComponent().setOpaque(false);
        
        ActionListener commitListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                stopCellEditing();
            }
        };
        mDateControls.addInputFieldCommitListener(commitListener);
        
        Box box = Box.createHorizontalBox();
        box.add(Box.createHorizontalStrut(3));
        box.add(mDateControls.getComponent());
        box.add(Box.createHorizontalStrut(5));
        box.add(mNameLabel);
        box.add(Box.createHorizontalGlue());
        box.setOpaque(false);
        box.setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
        JPanel panel = new JPanel(new BorderLayout(0, 0));
        panel.add(box, BorderLayout.CENTER);
        mComponent = panel;
    }

    
    public int getPreferredHeight() {
        return mComponent.getPreferredSize().height;
    }
    
    public Component getTableCellRendererComponent(
            JTable table, 
            Object value, 
            boolean isSelected, 
            boolean hasFocus, 
            int row, 
            int column)
    {
        if (isSelected) {
            mNameLabel.setForeground(table.getSelectionForeground());
            mDateControls.getComponent().setForeground(table.getSelectionForeground());
            mComponent.setBackground(table.getSelectionBackground());
        }
        else {
            mNameLabel.setForeground(table.getForeground());
            mDateControls.getComponent().setForeground(table.getForeground());
            Color background = 
                AutoDKPLookAndFeelSupport.getUnselectedBackgroundColor(
                        row, 
                        table.getBackground());
            mComponent.setBackground(background);
        }
        
        mNameLabel.setFont(table.getFont());
        mDateControls.getComponent().setFont(table.getFont());
        
        if (hasFocus) {
            Border border = null;
            if (isSelected) {
                border = UIManager.getBorder("Table.focusSelectedCellHighlightBorder");
            }
            if (border == null) {
                border = UIManager.getBorder("Table.focusCellHighlightBorder");
            }
            mComponent.setBorder(border);

            if (!isSelected && table.isCellEditable(row, column)) {
                Color col;
                col = UIManager.getColor("Table.focusCellForeground");
                if (col != null) {
                    mNameLabel.setForeground(col);
                    mDateControls.getComponent().setForeground(col);
                }
                col = UIManager.getColor("Table.focusCellBackground");
                if (col != null) {
                    mComponent.setBackground(col);
                }
            }
        } else {
            mComponent.setBorder(getNoFocusBorder());
        }

        
        RollCallDateItem item = (RollCallDateItem) value;
        if (!item.isRecurringItem()) {
            mNameLabel.setText(item.getName());
            mDateControls.getComponent().setVisible(true);
            mDateControls.initializeDateAndTime(item.getDate());
        } else {
            StringBuffer buf = new StringBuffer("every ");
            FormatConstants.fillSpaces(String.valueOf(item.getRecurrence()), 5, buf);
            buf.append(" minutes:   ");
            mNameLabel.setText(buf.toString() + item.getName());
            mDateControls.getComponent().setVisible(false);
        }
        
        
        if (
                item.getName().equals(RollCall.RAID_START_NAME) ||
                item.getName().equals(RollCall.RAID_END_NAME)) {
            mNameLabel.setForeground(Color.BLUE);
            Font labelFont = mNameLabel.getFont().deriveFont(Font.ITALIC | Font.BOLD);
            mNameLabel.setFont(labelFont);
        } else {
            mNameLabel.setForeground(Color.BLACK);
            Font labelFont = mNameLabel.getFont().deriveFont(Font.PLAIN);
            mNameLabel.setFont(labelFont);
        }
        
        return mComponent;
    }

    private static Border getNoFocusBorder() {
        if (System.getSecurityManager() != null) {
            return SAFE_NO_FOCUS_BORDER;
        } else {
            return noFocusBorder;
        }
    }
    
    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
        Component cellComponent = getTableCellRendererComponent(
                table, 
                value, 
                isSelected, 
                true, 
                row, 
                column);
        
        mDateControls.setControlForeground(Color.BLACK);
        if (!mDateControls.isTimeEntered()) {
            mDateControls.initializeDate(mSummaryPane.getCurrentRaid().getStartTime());
        }
        
        return cellComponent;
    }

    public Object getCellEditorValue() {
        return mDateControls.getDate();
    }
}
