package org.wow.autodkp.view.table;

import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellRenderer;

import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.skin.AutoDKPLookAndFeelSupport;

/**
 * 
 * 
 * @author jsandusky
 */
public class DateListRenderer implements TableCellRenderer {
    
    private static final Border SAFE_NO_FOCUS_BORDER = new EmptyBorder(1, 1, 1, 1);

    private JPanel mTextPanel;
    private JTextArea mTextArea;
    
    
    public DateListRenderer() {
        mTextArea = new JTextArea();
        mTextArea.setOpaque(false);
        mTextArea.setBorder(null);
        mTextPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
        mTextPanel.add(mTextArea);
    }
    
    
    public static String renderToHTML(Object value) {
        String renderedText = null;
        if (value instanceof List) {
            List dates = (List) value;
            StringBuffer buf = new StringBuffer();
            buf.append("<HTML>");
            GeneralProperties props = GeneralProperties.getDefault();
            for (Iterator iter=dates.iterator(); iter.hasNext();) {
                Date date = (Date) iter.next();
                if (date != null) {
                    buf.append(props.getDateFormatProperty(
                            props.DATE_FORMAT_SHORT).format(date));
                    buf.append("<BR>");
                }
            }
            buf.append("</HTML>");
            renderedText = buf.toString();
        }
        return renderedText;
    }
    
    private static String renderToString(Object value) {
        String renderedText = null;
        if (value instanceof List) {
            List dates = (List) value;
            StringBuffer buf = new StringBuffer();
            GeneralProperties props = GeneralProperties.getDefault();
            for (Iterator iter=dates.iterator(); iter.hasNext();) {
                Date date = (Date) iter.next();
                if (date != null) {
                    buf.append(props.getDateFormatProperty(
                            props.DATE_FORMAT_SHORT).format(date));
                    buf.append("\n");
                }
            }
            renderedText = buf.toString();
        }
        return renderedText;
    }
    
    public Component getTableCellRendererComponent(JTable table, 
                                                   Object value,
                                                   boolean isSelected, 
                                                   boolean hasFocus, 
                                                   int row, 
                                                   int column)
    {
        if (isSelected) {
            mTextArea.setForeground(table.getSelectionForeground());
            mTextPanel.setBackground(table.getSelectionBackground());
        } else {
            mTextArea.setForeground(table.getForeground());
            Color background = 
                AutoDKPLookAndFeelSupport.getUnselectedBackgroundColor(row, table.getBackground());
            mTextPanel.setBackground(background);
        }

        mTextArea.setFont(table.getFont());

        if (hasFocus) {
            Border border = null;
            if (isSelected) {
                border = UIManager.getBorder("Table.focusSelectedCellHighlightBorder");
            }
            if (border == null) {
                border = UIManager.getBorder("Table.focusCellHighlightBorder");
            }
            mTextPanel.setBorder(border);

            if (!isSelected && table.isCellEditable(row, column)) {
                Color col;
                col = UIManager.getColor("Table.focusCellForeground");
                if (col != null) {
                    mTextArea.setForeground(col);
                }
                col = UIManager.getColor("Table.focusCellBackground");
                if (col != null) {
                    mTextPanel.setBackground(col);
                }
            }
        } else {
            mTextPanel.setBorder(SAFE_NO_FOCUS_BORDER);
        }
            
            
        String text = renderToString(value);
        if (text != null) {
            mTextArea.setText(text);
        } else {
            mTextArea.setText("");
        }
        return mTextPanel;
    }
}
