package org.wow.autodkp.view.table;

import java.awt.Color;
import java.awt.Component;
import java.text.DateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.swing.JList;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.ListCellRenderer;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import org.wow.autodkp.model.Logger;
import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.filters.ItemFilter;
import org.wow.autodkp.view.html.StyleUtilities;
import org.wow.autodkp.view.skin.AutoDKPLookAndFeelSupport;
import org.wow.autodkp.view.table.players.PlayerTableModel;


/**
 * 
 * 
 * @author jsandusky
 */
public class LootRenderer implements TableCellRenderer, ListCellRenderer {
    
    private static final String SPACING = "     ";
    private static final String NEWLINE = System.getProperty("line.separator");
    private static final Border SAFE_NO_FOCUS_BORDER = new EmptyBorder(1, 1, 1, 1);
    
    private JTextPane mTextPane;
    private StyledDocument mStyledDocument;
    private ItemFilter mFilter;
    
    private Style FG_NORMAL;
    private Style FG_POOR;
    private Style FG_COMMON;
    private Style FG_UNCOMMON;
    private Style FG_RARE;
    private Style FG_EPIC;
    private Style FG_LEGENDARY;
    private Style FG_ARTIFACT;
    
    private int mOffset;
    private boolean mIsShowingLootTimes;
    
    
    public LootRenderer() {
        mTextPane = new JTextPane();
        mTextPane.setBorder(null);
        mStyledDocument = mTextPane.getStyledDocument();
        
        FG_NORMAL    = mTextPane.addStyle("FG_NORMAL",    null);
        FG_POOR      = mTextPane.addStyle("FG_POOR",      null);
        FG_COMMON    = mTextPane.addStyle("FG_COMMON",    null);
        FG_UNCOMMON  = mTextPane.addStyle("FG_UNCOMMON",  null);
        FG_RARE      = mTextPane.addStyle("FG_RARE",      null);
        FG_EPIC      = mTextPane.addStyle("FG_EPIC",      null);
        FG_LEGENDARY = mTextPane.addStyle("FG_LEGENDARY", null);
        FG_ARTIFACT  = mTextPane.addStyle("FG_ARTIFACT",  null);
        
        StyleConstants.setItalic(FG_NORMAL,        true);
        StyleConstants.setForeground(FG_NORMAL,    Color.GRAY);
        
        StyleConstants.setForeground(FG_POOR,      Loot.COLOR_POOR);
        StyleConstants.setForeground(FG_COMMON,    Loot.COLOR_COMMON);
        StyleConstants.setForeground(FG_UNCOMMON,  Loot.COLOR_UNCOMMON);
        StyleConstants.setForeground(FG_RARE,      Loot.COLOR_RARE);
        StyleConstants.setForeground(FG_EPIC,      Loot.COLOR_EPIC);
        StyleConstants.setForeground(FG_LEGENDARY, Loot.COLOR_LEGENDARY);
        StyleConstants.setForeground(FG_ARTIFACT,  Loot.COLOR_ARTIFACT);
    }
    
    
    
    private String getLootTimeForLoot(Loot loot, PlayerData playerData) {
        String lootTime = null;
        if (playerData != null) {
            GeneralProperties props = GeneralProperties.getDefault();
            DateFormat format = props.getDateFormatProperty(props.DATE_FORMAT_SHORT);
            Date date = playerData.getLootTimeForLoot(loot);
            if (date != null) {
                lootTime = format.format(date);
            }
        }
        return lootTime;
    }
    
    public void setIsShowingLootTimes(boolean isShowingLootTimes) {
        mIsShowingLootTimes = isShowingLootTimes;
    }
    
    public void setLootFilter(ItemFilter filter) {
        mFilter = filter;
    }
    
    public ItemFilter getLootFilter() {
        return mFilter;
    }
    
    public String renderToHTML(Object value, PlayerData playerData) {
        String renderedText = null;
        if (value instanceof List) {
            List loots = (List) value;
            StringBuffer buf = new StringBuffer();
            buf.append("<HTML>");
            buf.append(
                    NEWLINE + 
                    SPACING + "<TABLE CELLPADDING=0 CLASS=" + 
                    StyleUtilities.STYLE_TABLE_LOOT + ">" + NEWLINE);
            for (Iterator iter=loots.iterator(); iter.hasNext();) {
                Loot loot = (Loot) iter.next();
                if (mFilter != null && mFilter.isFiltered(loot)) {
                    continue;
                }
                buf.append(SPACING + "<TR><TD>" + NEWLINE);
                buf.append(renderLootToHTML(
                        loot.getQuality(), 
                        mIsShowingLootTimes,
                        getLootTimeForLoot(loot, playerData), 
                        loot.getName()));
                buf.append(SPACING + "</TD></TR>" + NEWLINE);
            }
            buf.append(SPACING + "</TABLE>");
            buf.append("</HTML>" + NEWLINE);
            renderedText = buf.toString();
        }
        return renderedText;
    }
    
    public static String renderLootToHTML(
            int quality, 
            boolean isShowingLootTimes, 
            String lootTime, 
            String lootName)
    {
        StringBuffer buf = new StringBuffer();
        
        buf.append(SPACING + SPACING);
        
        // Determine whether to include loot times.
        if (isShowingLootTimes && lootTime != null) {
            buf.append(
                    "<SPAN CLASS=" + StyleUtilities.STYLE_LOOT_TIME + ">" + 
                    lootTime + 
                    "&nbsp;</SPAN>");
        }
        
        buf.append("<SPAN CLASS=");
        String qualityName = LootConstants.QUALITY_KEYS[quality];
        switch (quality) {
        case LootConstants.QUALITY_POOR:      buf.append("\"loot-quality_" + qualityName + "\""); break;
        case LootConstants.QUALITY_COMMON:    buf.append("\"loot-quality_" + qualityName + "\""); break;
        case LootConstants.QUALITY_UNCOMMON:  buf.append("\"loot-quality_" + qualityName + "\""); break;
        case LootConstants.QUALITY_RARE:      buf.append("\"loot-quality_" + qualityName + "\""); break;
        case LootConstants.QUALITY_EPIC:      buf.append("\"loot-quality_" + qualityName + "\""); break;
        case LootConstants.QUALITY_LEGENDARY: buf.append("\"loot-quality_" + qualityName + "\""); break;
        case LootConstants.QUALITY_ARTIFACT:  buf.append("\"loot-quality_" + qualityName + "\""); break;
        default:
            qualityName = "none";
            buf.append("\"loot-quality_" + qualityName + "\"");
            break;
        }
        buf.append(">");
        
        buf.append(lootName);
        buf.append("</SPAN>" + "<BR>" + NEWLINE);
        return buf.toString();
    }
    
    private void renderToDocument(
            Object value, 
            PlayerData playerData, 
            boolean includeNewline)
    {
        if (value instanceof List) {
            List loots = (List) value;
            for (Iterator iter=loots.iterator(); iter.hasNext();) {
                Loot loot = (Loot) iter.next();
                if (mFilter != null && mFilter.isFiltered(loot)) {
                    continue;
                }
                renderLootToDocument(loot, getLootTimeForLoot(loot, playerData), includeNewline);
            }
        } else if (value instanceof Loot) {
            Loot loot = (Loot) value;
            renderLootToDocument(loot, getLootTimeForLoot(loot, playerData), includeNewline);
        }
    }
    
    private void renderLootToDocument(Loot loot, String lootTime, boolean includeNewline) {
        Style style = null;
        switch (loot.getQuality()) {
        case LootConstants.QUALITY_POOR:      style = FG_POOR;      break;
        case LootConstants.QUALITY_COMMON:    style = FG_COMMON;    break;
        case LootConstants.QUALITY_UNCOMMON:  style = FG_UNCOMMON;  break;
        case LootConstants.QUALITY_RARE:      style = FG_RARE;      break;
        case LootConstants.QUALITY_EPIC:      style = FG_EPIC;      break;
        case LootConstants.QUALITY_LEGENDARY: style = FG_LEGENDARY; break;
        case LootConstants.QUALITY_ARTIFACT:  style = FG_ARTIFACT;  break;
        default:
            style = FG_COMMON;
            break;
        }
        
        // Determine whether to include loot times.
        if (mIsShowingLootTimes) {
            if (lootTime != null) {
                try {
                    mStyledDocument.insertString(mOffset, lootTime + " ", FG_NORMAL);
                    mOffset += lootTime.length() + 1;
                } catch (BadLocationException e) {
                    Logger.log(e);
                }
            }
        }
        
        String newText = null;
        if (includeNewline) {
            newText = loot.getName() + NEWLINE;
        } else {
            newText = loot.getName();
        }
        try {
            mStyledDocument.insertString(mOffset, newText, style);
            mOffset += newText.length();
        } catch (BadLocationException e) {
            Logger.log(e);
        }
    }
    
    public Component getTableCellRendererComponent(JTable table, 
                                                   Object value,
                                                   boolean isSelected, 
                                                   boolean hasFocus, 
                                                   int row, 
                                                   int column)
    {
        if (isSelected) {
            mTextPane.setForeground(table.getSelectionForeground());
            mTextPane.setBackground(table.getSelectionBackground());
        } else {
            mTextPane.setForeground(table.getForeground());
            Color background = 
                AutoDKPLookAndFeelSupport.getUnselectedBackgroundColor(
                        row, 
                        table.getBackground());
            mTextPane.setBackground(background);
        }

        mTextPane.setFont(table.getFont());

        if (hasFocus) {
            Border border = null;
            if (isSelected) {
                border = UIManager.getBorder("Table.focusSelectedCellHighlightBorder");
            }
            if (border == null) {
                border = UIManager.getBorder("Table.focusCellHighlightBorder");
            }
            mTextPane.setBorder(border);

            if (!isSelected && table.isCellEditable(row, column)) {
                Color col;
                col = UIManager.getColor("Table.focusCellForeground");
                if (col != null) {
                    mTextPane.setForeground(col);
                }
                col = UIManager.getColor("Table.focusCellBackground");
                if (col != null) {
                    mTextPane.setBackground(col);
                }
            }
        } else {
            mTextPane.setBorder(SAFE_NO_FOCUS_BORDER);
        }

        PlayerData playerData = null;
        if (table.getModel() instanceof TableSorter) {
            TableSorter sorter = (TableSorter) table.getModel();
            PlayerTableModel model = (PlayerTableModel) sorter.getTableModel();
            playerData = model.getRaidPlayerDataForRow(sorter.modelIndex(row));
        }
        
        // reset text
        mOffset = 0;
        mTextPane.setText(null);
        renderToDocument(value, playerData, true);
        return mTextPane;
    }


    public Component getListCellRendererComponent(
            JList list, 
            Object value, 
            int index, 
            boolean isSelected, 
            boolean cellHasFocus)
    {
        mTextPane.setComponentOrientation(list.getComponentOrientation());
        if (isSelected) {
            mTextPane.setBackground(list.getSelectionBackground());
            mTextPane.setForeground(list.getSelectionForeground());
        } else {
            mTextPane.setForeground(list.getForeground());
            Color background = 
                AutoDKPLookAndFeelSupport.getUnselectedBackgroundColor(
                        index, 
                        list.getBackground());
            mTextPane.setBackground(background);
        }

        mTextPane.setEnabled(list.isEnabled());
        mTextPane.setFont(list.getFont());

        Border border = null;
        if (cellHasFocus) {
            if (isSelected) {
                border = UIManager.getBorder("List.focusSelectedCellHighlightBorder");
            }
            if (border == null) {
                border = UIManager.getBorder("List.focusCellHighlightBorder");
            }
        } else {
            border = SAFE_NO_FOCUS_BORDER;
        }
        mTextPane.setBorder(border);
        
        
        // reset text
        mOffset = 0;
        mTextPane.setText(null);
        renderToDocument(value, null, false);
        return mTextPane;
    }
}
