package org.wow.autodkp.view.table.customizers;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.BasicDialog;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.table.players.PlayerTable;
import org.wow.autodkp.view.table.rowheader.RowHeaderUtil;
import org.wow.autodkp.view.table.rowheader.TableRowHeaderModel;


/**
 * 
 * 
 * @author jsandusky
 */
public abstract class AbstractCustomizerEditor extends BasicDialog {

    protected PlayerTable mPlayerTable;
    protected Raid mRaid;
    protected IPointSystem mPointSystem;
    private int mTotalMinimumWidth;
    private JTable mTable;
    
    
    public AbstractCustomizerEditor(PlayerTable playerTable,
                                    Raid raid, 
                                    IPointSystem pointSystem,
                                    String title) {
        super(AutoDKPView.getInstance().getFrame(), title, true);
        addBasicSupport();
        
        mPlayerTable = playerTable;
        mRaid = raid;
        mPointSystem = pointSystem;
        
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
        JButton closeButton = new JButton("Close");
        closeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                closing();
                dispose();
            }
        });
        
        JPanel buttonPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttonPane.add(closeButton);
        buttonPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        
        GeneralProperties props = GeneralProperties.getDefault();
        props.loadWindowRect(
                this, 
                props.CUSTOMIZER_WINDOW_X,
                props.CUSTOMIZER_WINDOW_Y,
                props.CUSTOMIZER_WINDOW_WIDTH,
                props.CUSTOMIZER_WINDOW_HEIGHT,
                props.DEFAULT_FORMULA_EDITOR_WIDTH,
                props.DEFAULT_FORMULA_EDITOR_HEIGHT);
        
        setLayout(new BorderLayout());
        getContentPane().add(getMainPane(), BorderLayout.CENTER);
        getContentPane().add(buttonPane,    BorderLayout.SOUTH);
        
        addComponentListener(new ComponentAdapter() {
            public void componentResized(ComponentEvent e) {
                super.componentResized(e);
                adjustResizing();
            }
        });
        adjustResizing();
    }
    
    private void adjustResizing() {
        if (getWidth() > mTotalMinimumWidth) {
            mTable.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
        } else {
            mTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        }
    }
    
    protected void closing() {
        GeneralProperties props = GeneralProperties.getDefault();
        props.setIntegerProperty(props.CUSTOMIZER_WINDOW_X, getX());
        props.setIntegerProperty(props.CUSTOMIZER_WINDOW_Y, getY());
        props.setIntegerProperty(props.CUSTOMIZER_WINDOW_WIDTH,  getWidth());
        props.setIntegerProperty(props.CUSTOMIZER_WINDOW_HEIGHT, getHeight());
    }
    
    protected abstract AbstractDetailTableModel getDetailTableModel();
    
    protected abstract PointCheckPredicate getPointCheckPredicate();
    
    
    protected JComponent getMainPane() {
        AbstractDetailTableModel model = getDetailTableModel();
        RotatedCellRenderer headerRenderer = new RotatedCellRenderer(model);

        PointCheckEditor pointsEditor = new PointCheckEditor(getPointCheckPredicate());

        mTable = new JTable(model);
        JLabel defaultStringRenderer = (JLabel) mTable.getDefaultRenderer(String.class);
        defaultStringRenderer.setHorizontalAlignment(SwingConstants.CENTER);

        mTable.getTableHeader().setReorderingAllowed(false);
        mTable.setRowHeight(mTable.getRowHeight() + 8);
        
        JScrollPane scroller = new JScrollPane(mTable);
        scroller.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(10, 10, 0, 10), 
                scroller.getBorder()));
        
        // Use special row header utility to set a nice row header
        // in the scroll pane that contains our table.
        // -- courtesy of Maksim Kovalenko, as posted on jGuru
        RowHeaderUtil.setRowHeader(mTable, getRowHeaderModel());

        TableColumnModel columnModel = mTable.getColumnModel();
        for (int i = 0; i < columnModel.getColumnCount(); i++) {
            TableColumn column = columnModel.getColumn(i);
            column.setHeaderRenderer(headerRenderer);
            column.setCellRenderer(pointsEditor);
            column.setCellEditor(pointsEditor);
            column.setMinWidth(80);
            mTotalMinimumWidth += 80;
        }

        return scroller;
    }
    
    private TableRowHeaderModel getRowHeaderModel() {
        return new TableRowHeaderModel(mTable) {
            public Object getElementAt(int index) {
                PlayerData raidPlayerData = 
                    mPointSystem.getNonRemovedRaidMemberPlayerData()[index];
                return raidPlayerData.getPlayerName();
            }
        };
    }
}
