package org.wow.autodkp.view.table.customizers;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import org.wow.autodkp.view.skin.AutoDKPLookAndFeelSupport;

/**
 * 
 * 
 * @author jsandusky
 */
public class PointCheckEditor extends AbstractCellEditor 
implements TableCellEditor, TableCellRenderer {
    
    protected static Border noFocusBorder = new EmptyBorder(1, 1, 1, 1); 
    private JCheckBox mCheckBoxEditor;
    private JCheckBox mCheckBoxRenderer;
    private JLabel mEmptyCellRenderer;
    private PointCheckPredicate mPredicate;
    
    
    public PointCheckEditor(PointCheckPredicate predicate) {
        mPredicate = predicate;
        
        mCheckBoxEditor = new JCheckBox();
        mCheckBoxEditor.setHorizontalAlignment(SwingConstants.CENTER);
        mCheckBoxEditor.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                stopCellEditing();
            }
        });
        
        mEmptyCellRenderer = new JLabel();
        mEmptyCellRenderer.setHorizontalAlignment(SwingConstants.CENTER);
        mEmptyCellRenderer.setOpaque(true);
        
        mCheckBoxRenderer = new JCheckBox();
        mCheckBoxRenderer.setHorizontalAlignment(SwingConstants.CENTER);
    }
    

    public Object getCellEditorValue() {
        return null;
    }

    public Component getTableCellEditorComponent(JTable table, 
                                                 Object value, 
                                                 boolean isSelected, 
                                                 int row, 
                                                 int column)
    {
        mCheckBoxEditor.setForeground(table.getForeground());
        mCheckBoxEditor.setBackground(table.getBackground());
        mCheckBoxEditor.setBorder(UIManager.getBorder("Table.focusSelectedCellHighlightBorder"));
        
        mCheckBoxEditor.setFont(table.getFont());
        
        mPredicate.updatePointCheckbox(value, mCheckBoxEditor, mEmptyCellRenderer);
        
        return mCheckBoxEditor;
    }

    public Component getTableCellRendererComponent(JTable table, 
                                                   Object value, 
                                                   boolean isSelected, 
                                                   boolean hasFocus, 
                                                   int row, 
                                                   int column) {
        JComponent renderer = mCheckBoxRenderer;
        if (!mPredicate.updatePointCheckbox(value, mCheckBoxRenderer, mEmptyCellRenderer)) {
            renderer = mEmptyCellRenderer;
        }

        if (isSelected) {
            renderer.setForeground(table.getSelectionForeground());
            renderer.setBackground(table.getSelectionBackground());
        } else {
            renderer.setForeground(table.getForeground());
            Color background = 
                AutoDKPLookAndFeelSupport.getUnselectedBackgroundColor(
                        row, 
                        table.getBackground());
            renderer.setBackground(background);
        }
            
        renderer.setFont(table.getFont());

        if (hasFocus) {
            Border border = null;
            if (isSelected) {
                border = UIManager.getBorder("Table.focusSelectedCellHighlightBorder");
            }
            if (border == null) {
                border = UIManager.getBorder("Table.focusCellHighlightBorder");
            }
            renderer.setBorder(border);

            if (!isSelected && table.isCellEditable(row, column)) {
                Color col;
                col = UIManager.getColor("Table.focusCellForeground");
                if (col != null) {
                    renderer.setForeground(col);
                }
                col = UIManager.getColor("Table.focusCellBackground");
                if (col != null) {
                    renderer.setBackground(col);
                }
            }
        } else {
            renderer.setBorder(noFocusBorder);
        }
            
        return renderer;
    }
}
