package org.wow.autodkp.view.table.customizers;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.table.TableCellRenderer;

/**
 * Author not attributable, taken from Java Forums, Swing Developer Network.
 */
public class RotatedCellRenderer extends JPanel implements Icon, TableCellRenderer {

    private static final double NINETY_DEGREES = Math.toRadians(90.0);
    private JTextArea mTextComponent = new JTextArea();

    private AbstractDetailTableModel mTableModel;

    
    public RotatedCellRenderer(AbstractDetailTableModel tableModel) {
        mTableModel = tableModel;
        mTextComponent.setBorder(BorderFactory.createEmptyBorder(20, 10, 10, 10));
        mTextComponent.setOpaque(false);
        mTextComponent.setFont(new Font("Dialog", Font.BOLD, 12));
    }


    // Implementation of Icon interface
    // (especially useful with side tabs on a JTabbedPane)

    public int getIconHeight() {
        return getPreferredSize().height;
    }

    public int getIconWidth() {
        return getPreferredSize().width;
    }

    public void paintIcon(Component c, Graphics g, int x, int y) {
        Dimension d = this.getPreferredSize();
        paintHere(c, g, x, y, d.width, d.height);
    }

    // Delegate these methods to the JLabel...

    public void setText(String value) {
        mTextComponent.setText(value);
    }

    public String getText() {
        return mTextComponent.getText();
    }

    public Dimension getPreferredSize() {
        Dimension d = mTextComponent.getPreferredSize();
        return new Dimension(d.height, d.width);
    }

    public void paintComponent(Graphics g) {
        Dimension d = this.getSize();
        paintHere(this, g, 0, 0, d.width, d.height);
    }

    public void paintHere(Component c, Graphics g, int x, int y, int width, int height) {
        if (height <= 0 || width <= 0)
            return;

        mTextComponent.updateUI();
        // Paint the JLabel into an image buffer...
        BufferedImage buffer = new BufferedImage(height, width, BufferedImage.TYPE_INT_ARGB_PRE);
        Graphics2D g2 = buffer.createGraphics();
        mTextComponent.setSize(new Dimension(height, width));
        mTextComponent.paint(g2);

        // ...then apply a transform while painting the buffer into the component
        AffineTransform af = AffineTransform.getTranslateInstance(x, y + height);
        AffineTransform af2 = AffineTransform.getRotateInstance(-NINETY_DEGREES);
        af.concatenate(af2);

        ((Graphics2D) g).drawImage(buffer, af, this);
    }

    // TableCellRenderer...
    
    public Component getTableCellRendererComponent(JTable table, 
                                                   Object value, 
                                                   boolean isSelected,
                                                   boolean hasFocus, 
                                                   int row, 
                                                   int column) {
        setBorder(UIManager.getBorder("TableHeader.cellBorder"));
        
        String headerText = mTableModel.getHeaderTextForColumn(column);
        mTextComponent.setText(headerText);
        
        mTableModel.customizeTextComponentForColumn(mTextComponent, column);
        
        return this;
    }
}
