package org.wow.autodkp.view.table.customizers.bosskill;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.IPointSystemPlayerData;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.table.customizers.AbstractFormulaTableModel;
import org.wow.autodkp.view.table.players.PlayerTable;

/**
 * 
 * 
 * @author jsandusky
 */
public class BossKillsTableModel extends AbstractFormulaTableModel {
    
    private List mBossList = new ArrayList();
    
    
    public BossKillsTableModel(PlayerTable playerTable, Raid raid, IPointSystem pointSystem) {
        super(playerTable, raid, pointSystem);
        
        // obtain sorted list of bosses (by time killed)
        Map bossKillMap = raid.getBossNameToKillDateMap();
        TreeMap bossData = new TreeMap();
        int i = 0;
        for (Iterator iter=bossKillMap.keySet().iterator(); iter.hasNext(); i++) {
            String bossName = (String) iter.next();
            Date killDate = (Date) bossKillMap.get(bossName);
            bossData.put(killDate, bossName);
        }
        for (Iterator iter=bossData.values().iterator(); iter.hasNext();) {
            String bossName = (String) iter.next();
            mBossList.add(bossName);
        }
        
        mColumnCount = mRaid.getBossNameToKillDateMap().size();
    }
    

    private String getBossForColumn(int columnIndex) {
        return (String) mBossList.get(columnIndex);
    }
    
    protected String getHeaderTextForColumn(int columnIndex) {
        String bossName = getBossForColumn(columnIndex);
        Map bossKillDates = mRaid.getBossNameToKillDateMap();
        Date killDate = (Date) bossKillDates.get(bossName);
        GeneralProperties props = GeneralProperties.getDefault();
        return 
            bossName + "\n" + 
            props.getDateFormatProperty(props.DATE_FORMAT_SHORT).format(killDate);
    }
    
    public Object getValueAt(int rowIndex, int columnIndex) {
        IPointSystemPlayerData playerData = getPointSystemPlayerData(rowIndex, columnIndex);
        Map bossKillPoints = playerData.getFormula().getBossKillDetailMap();
        String bossName = getBossForColumn(columnIndex);
        return bossKillPoints.get(bossName);
    }
    
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        PlayerData raidPlayerData = mPointSystem.getNonRemovedRaidMemberPlayerData()[rowIndex];
        IPointSystemPlayerData pointData = getPointSystemPlayerData(rowIndex, columnIndex);
        String bossName = getBossForColumn(columnIndex);
        List exceptions = pointData.getBossKillExceptions();
        if (exceptions == null) {
            pointData.setBossKillExceptions(new ArrayList());
            exceptions = pointData.getBossKillExceptions();
        }
        if (exceptions.contains(bossName)) {
            exceptions.remove(bossName);
        } else {
            exceptions.add(bossName);
        }
        if        (mPointSystem.isZeroSum()) {
            mPointSystem.getCalculator().calculate();
        } else if (mPointSystem.isBossKill()) {
            mPointSystem.getCalculator().calculateLineItem(raidPlayerData, pointData);
        }
        mPlayerTable.updatePlayerRow(raidPlayerData.getPlayerName());
        mPointSystem.saveRaid(mRaid);
    }
}
