package org.wow.autodkp.view.table.customizers.time;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.text.JTextComponent;

import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.IPointSystemPlayerData;
import org.wow.autodkp.model.points.RollCall;
import org.wow.autodkp.model.points.dkp.TimePointDetail;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.table.customizers.AbstractFormulaTableModel;
import org.wow.autodkp.view.table.players.PlayerTable;

/**
 * 
 * 
 * @author jsandusky
 */
public class TimeTableModel extends AbstractFormulaTableModel {
    
    private static final Color COLOR_DURATION = new Color(204, 0, 0);
    
    private List mTimeList = new ArrayList();
    private Map mRecurrentRollCallNamesToColors = new HashMap();
    
    
    public TimeTableModel(PlayerTable playerTable, Raid raid, IPointSystem pointSystem) {
        super(playerTable, raid, pointSystem);
        
        Set recurrentRollCallNames = new HashSet();
        Map raidConstraintData = pointSystem.getCurrentRaidData().getPointSystemPlayerDataMap();
        // Just grab the first player's time details, it is the same as everyone else's.
        if (pointSystem.getNonRemovedRaidMemberPlayerData().length > 0) {
            PlayerData raidPlayerData = pointSystem.getNonRemovedRaidMemberPlayerData()[0];
            IPointSystemPlayerData pointData = 
                (IPointSystemPlayerData) raidConstraintData.get(raidPlayerData.getPlayerName());
            Map detailMap = pointData.getFormula().getTimeDetailMap();
            for (Iterator iter=detailMap.keySet().iterator(); iter.hasNext();) {
                String timeID = (String) iter.next();
                TimePointDetail detail = 
                    (TimePointDetail) detailMap.get(timeID);
                if (detail.isRollCall()) {
                    mTimeList.add(detail);
                    if (detail.hasRecurrence()) {
                        recurrentRollCallNames.add(detail.getRollCallName());
                    }
                } else {
                    mTimeList.add(detail);
                }
            }
        }
        
        Collections.sort(mTimeList);
        
        mColumnCount = mTimeList.size();
        
        // Set up rotating colors for the table column headers of
        // recurring roll calls. This makes it much easier for the user
        // to destinguish multiple recurring roll calls from another.
        Color[] recurringColors = new Color[] { 
                Color.GRAY, Color.ORANGE.darker(), Color.MAGENTA, Color.PINK.darker(), Color.CYAN.darker() };
        int i=0;
        for (Iterator iter=recurrentRollCallNames.iterator(); iter.hasNext(); i++) {
            if (i >= recurringColors.length) {
                i = 0;
            }
            Color c = recurringColors[i];
            mRecurrentRollCallNamesToColors.put(iter.next(), c);
        }
    }
    

    private TimePointDetail getTimeForColumn(int columnIndex) {
        return (TimePointDetail) mTimeList.get(columnIndex);
    }
    
    public static boolean hasTimeData(IPointSystem pointSystem) {
        Map raidConstraintData = pointSystem.getCurrentRaidData().getPointSystemPlayerDataMap();
        for (int i = 0; i < pointSystem.getNonRemovedRaidMemberPlayerData().length; i++) {
            PlayerData raidPlayerData = pointSystem.getNonRemovedRaidMemberPlayerData()[i];
            IPointSystemPlayerData pointData = 
                (IPointSystemPlayerData) raidConstraintData.get(raidPlayerData.getPlayerName());
            Map detailMap = pointData.getFormula().getTimeDetailMap();
            if (detailMap.size() > 0) {
                return true;
            }
        }
        return false;
    }
    
    protected void customizeTextComponentForColumn(
            JTextComponent component, 
            int columnIndex)
    {
        TimePointDetail detail = getTimeForColumn(columnIndex);
        if (!detail.isRollCall()) {
            // Duration
            component.setForeground(COLOR_DURATION);
            return;
        } else {
            String name = detail.getRollCallName();
            if (
                    RollCall.RAID_START_NAME.equals(name) ||
                    RollCall.RAID_END_NAME.equals(name)) {
                // Raid start/end
                component.setForeground(Color.BLUE);
                return;
            }
            if (detail.hasRecurrence()) {
                // Recurring roll calls, we shift colors
                // for each recurring roll call to better
                // distinguish them.
                Color c = (Color) mRecurrentRollCallNamesToColors.get(
                            detail.getRollCallName());
                component.setForeground(c);
                return;
            }
        }
        // Singular roll calls
        component.setForeground(Color.BLACK);
    }

    protected String getHeaderTextForColumn(int columnIndex) {
        TimePointDetail detail = getTimeForColumn(columnIndex);
        if (detail.isRollCall()) {
            GeneralProperties props = GeneralProperties.getDefault();
            if (detail.hasRecurrence()) {
                return
                    detail.getRollCallName() + " "  +
                    detail.getRecurrence()   + "\n" +
                    props.getDateFormatProperty(props.DATE_FORMAT_SHORT).format(
                            detail.getRollCallDate());
            } else {
                return
                    detail.getRollCallName() + "\n" +
                    props.getDateFormatProperty(props.DATE_FORMAT_SHORT).format(
                            detail.getRollCallDate());
            }
        } else {
            return "Duration";
        }
    }
    
    public Object getValueAt(int rowIndex, int columnIndex) {
        IPointSystemPlayerData playerData = getPointSystemPlayerData(rowIndex, columnIndex);
        Map timePoints = playerData.getFormula().getTimeDetailMap();
        TimePointDetail detail = getTimeForColumn(columnIndex);
        return timePoints.get(detail.getID());
    }
    
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        PlayerData raidPlayerData = mPointSystem.getNonRemovedRaidMemberPlayerData()[rowIndex];
        IPointSystemPlayerData pointData = getPointSystemPlayerData(rowIndex, columnIndex);
        List exceptions = pointData.getTimeExceptions();
        if (exceptions == null) {
            pointData.setTimeExceptions(new ArrayList());
            exceptions = pointData.getTimeExceptions();
        }
        TimePointDetail detail = getTimeForColumn(columnIndex);
        if (exceptions.contains(detail.getID())) {
            exceptions.remove(detail.getID());
        } else {
            exceptions.add(detail.getID());
        }
        if        (mPointSystem.isZeroSum()) {
            mPointSystem.getCalculator().calculate();
        } else if (mPointSystem.isBossKill()) {
            mPointSystem.getCalculator().calculateLineItem(raidPlayerData, pointData);
        }
        mPlayerTable.updatePlayerRow(raidPlayerData.getPlayerName());
        mPointSystem.saveRaid(mRaid);
    }
}
