package org.wow.autodkp.view.table.customizers.zerosum;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.CostItem;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.IPointSystemPlayerData;
import org.wow.autodkp.model.points.dkp.DistributionPointDetail;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.table.customizers.AbstractFormulaTableModel;
import org.wow.autodkp.view.table.players.PlayerTable;

/**
 * 
 * 
 * @author jsandusky
 */
public class DistributionsTableModel extends AbstractFormulaTableModel {
    
    private List mDistributionList = new ArrayList();
    private Map  mDetailMap = new HashMap();
    
    
    public DistributionsTableModel(PlayerTable playerTable, Raid raid, IPointSystem pointSystem) {
        super(playerTable, raid, pointSystem);
        
        // obtain sorted list of distributions (by time looted)
        Map raidConstraintData = pointSystem.getCurrentRaidData().getPointSystemPlayerDataMap();
        TreeMap lootData = new TreeMap();
        for (int i = 0; i < pointSystem.getNonRemovedRaidMemberPlayerData().length; i++) {
            PlayerData raidPlayerData = pointSystem.getNonRemovedRaidMemberPlayerData()[i];
            IPointSystemPlayerData pointData = 
                (IPointSystemPlayerData) raidConstraintData.get(raidPlayerData.getPlayerName());
            Map detailMap = pointData.getFormula().getDistributionDetailMap();
            for (Iterator iter=detailMap.keySet().iterator(); iter.hasNext();) {
                String distributionID = (String) iter.next();
                DistributionPointDetail detail = 
                    (DistributionPointDetail) detailMap.get(distributionID);
                lootData.put(detail.getDate(), distributionID);
                mDetailMap.put(distributionID, detail);
            }
        }
        for (Iterator iter=lootData.values().iterator(); iter.hasNext();) {
            String distributionID = (String) iter.next();
            mDistributionList.add(distributionID);
        }
        
        mColumnCount = mDistributionList.size();
    }
    
    public static boolean hasDistributionData(IPointSystem pointSystem) {
        Map raidConstraintData = pointSystem.getCurrentRaidData().getPointSystemPlayerDataMap();
        for (int i = 0; i < pointSystem.getNonRemovedRaidMemberPlayerData().length; i++) {
            PlayerData raidPlayerData = pointSystem.getNonRemovedRaidMemberPlayerData()[i];
            IPointSystemPlayerData pointData = 
                (IPointSystemPlayerData) raidConstraintData.get(raidPlayerData.getPlayerName());
            Map detailMap = pointData.getFormula().getDistributionDetailMap();
            if (detailMap.size() > 0) {
                return true;
            }
        }
        return false;
    }
    
    protected String getHeaderTextForColumn(int columnIndex) {
        String id = getDistributionForColumn(columnIndex);
        DistributionPointDetail detail = (DistributionPointDetail) mDetailMap.get(id);
        CostItem item = detail.getCostItem();
        GeneralProperties props = GeneralProperties.getDefault();
        if (item.isLoot()) {
            Loot loot = item.getLoot();
            return 
                loot.getName() + "\n" + 
                props.getDateFormatProperty(props.DATE_FORMAT_SHORT).format(
                        detail.getDate()) + "\n" + 
                detail.getPlayerName();
        } else {
            return 
                item.getFormulaItemText() + "\n" + 
                props.getDateFormatProperty(props.DATE_FORMAT_SHORT).format(
                        detail.getDate()) + "\n" + 
                detail.getPlayerName();
        }
    }
    
    private String getDistributionForColumn(int columnIndex) {
        return (String) mDistributionList.get(columnIndex);
    }
    
    public boolean isCellEditable(int row, int columnIndex) {
        IPointSystemPlayerData playerData = getPointSystemPlayerData(row, columnIndex);
        Map distributionPoints = playerData.getFormula().getDistributionDetailMap();
        String id = getDistributionForColumn(columnIndex);
        return distributionPoints.get(id) != null;
    }
    
    public Object getValueAt(int rowIndex, int columnIndex) {
        IPointSystemPlayerData playerData = getPointSystemPlayerData(rowIndex, columnIndex);
        Map distributionPoints = playerData.getFormula().getDistributionDetailMap();
        String id = getDistributionForColumn(columnIndex);
        return distributionPoints.get(id);
    }
    
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        PlayerData raidPlayerData = mPointSystem.getNonRemovedRaidMemberPlayerData()[rowIndex];
        IPointSystemPlayerData playerData = getPointSystemPlayerData(rowIndex, columnIndex);
        String id = getDistributionForColumn(columnIndex);
        List exceptions = playerData.getDistributionExceptions();
        if (exceptions == null) {
            playerData.setDistributionExceptions(new ArrayList());
            exceptions = playerData.getDistributionExceptions();
        }
        if (exceptions.contains(id)) {
            exceptions.remove(id);
        } else {
            exceptions.add(id);
        }
        mPlayerTable.fireTableDataChanged();
        mPlayerTable.updatePlayerRow(raidPlayerData.getPlayerName());
        mPointSystem.saveRaid(mRaid);
    }
}
