package org.wow.autodkp.view.table.players;

import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.IPointSystemPlayerData;


/**
 * 
 * 
 * @author jsandusky
 */
public class ColumnAdjustment extends AbstractPointsColumn {

    private String mAdjustmentName;
    private String mAdjustmentID;
    
    public ColumnAdjustment(IPointSystem constraints, Raid raid, String id, String name) {
        super(constraints, raid);
        mAdjustmentID = id;
        mAdjustmentName = name;
    }
    
    public String getColumnName() {
        // No alternate names for adjustment columns, 
        // because they are user-defined.
        return mAdjustmentName;
    }

    public String getDefaultColumnName() {
        return mAdjustmentName;
    }
    
    public Class getColumnClass() {
        return Float.class;
    }
    
    public int getPreferredWidth() {
        return 80;
    }

    public Object getValueAt(Object key) {
        IPointSystemPlayerData constraintData = super.getConstraintData(key);
        if (constraintData != null) {
            return new Float(constraintData.getAdjustment(mAdjustmentID));
        }
        return new Float(0);
    }

    public boolean setValueAt(Object aValue, Object key) {
        IPointSystemPlayerData pointData = super.getConstraintData(key);
        if (pointData != null) {
            try {
                float val = Float.parseFloat((String) aValue);
                pointData.setAdjustment(mAdjustmentID, val);
                if        (mPointSystem.isZeroSum()) {
                    mPointSystem.getCalculator().calculate();
                } else if (mPointSystem.isBossKill()) {
                    mPointSystem.getCalculator().calculateLineItem((PlayerData) key, pointData);
                }
                return true;
            } catch (NumberFormatException nfe) {
                // do not allow change to commit
            }
        }
        return false;
    }
    
    public boolean isEditable() {
        return true;
    }
}
