package org.wow.autodkp.view.table.players;

import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.IPointSystemPlayerData;
import org.wow.autodkp.model.points.dkp.IFormula;
import org.wow.autodkp.view.GeneralProperties;

/**
 * 
 * 
 * @author jsandusky
 */
public class ColumnFormula extends AbstractPointsColumn {

    protected static final FormulaColumnHandler HANDLER_FORMULA = new FormulaColumnHandler();
    
    
    public ColumnFormula(IPointSystem constraints, Raid raid) {
        super(constraints, raid);
        HANDLER_FORMULA.setPlayerData(constraints, raid);
    }
    
    public String getColumnName() {
        GeneralProperties props = GeneralProperties.getDefault();
        String alternateName = props.getStringProperty(props.COLUMN_TRANSLATION_FORMULA);
        if (alternateName == null || alternateName.length() == 0) {
            return getDefaultColumnName();
        }
        return alternateName;
    }

    public String getDefaultColumnName() {
        return "Formula";
    }
    
    public int getPreferredWidth() {
        return 250;
    }

    public TableCellEditor getCellEditor() {
        return HANDLER_FORMULA.getEditor();
    }

    public TableCellRenderer getCellRenderer() {
        return HANDLER_FORMULA.getRenderer();
    }
    
    public Object getValueAt(Object key) {
        IPointSystemPlayerData pointData = super.getConstraintData(key);
        if (pointData != null && pointData.getFormula() != null) {
            IFormula formula = pointData.getFormula();
            Object printObject = formula.getPrintObject();
            if (printObject == null) {
                printObject = FormulaUtilities.generateFormulaPrintObject(formula);
                formula.setPrintObject(printObject);
            }
            return printObject;
        }
        return "";
    }
    
    public boolean isEditable() {
        return !mPointSystem.getCurrentRaidData().isLocked();
    }
}
