package org.wow.autodkp.view.table.players;

import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.IPointSystemPlayerData;
import org.wow.autodkp.view.GeneralProperties;


/**
 * 
 * 
 * @author jsandusky
 */
public class ColumnIncomingTotal extends AbstractPointsColumn {

    public ColumnIncomingTotal(IPointSystem constraints, Raid raid) {
        super(constraints, raid);
    }
    
    public String getColumnName() {
        GeneralProperties props = GeneralProperties.getDefault();
        String alternateName = props.getStringProperty(props.COLUMN_TRANSLATION_PTS);
        if (alternateName == null || alternateName.length() == 0) {
            return getDefaultColumnName();
        }
        return alternateName;
    }

    public String getDefaultColumnName() {
        return "Pts";
    }
    
    public int getPreferredWidth() {
        return 80;
    }

    public Class getColumnClass() {
        return Float.class;
    }
    
    public Object getValueAt(Object key) {
        IPointSystemPlayerData constraintData = super.getConstraintData(key);
        if (constraintData != null) {
            return new Float(constraintData.getIncomingTotal());
        }
        return new Float(0);
    }
    
    public boolean setValueAt(Object aValue, Object key) {
        IPointSystemPlayerData pointData = super.getConstraintData(key);
        if (pointData != null) {
            try {
                float val = Float.parseFloat((String) aValue);
                pointData.setIncomingTotal(val);
                if        (mPointSystem.isZeroSum()) {
                    mPointSystem.getCalculator().calculate();
                } else if (mPointSystem.isBossKill()) {
                    mPointSystem.getCalculator().calculateLineItem((PlayerData) key, pointData);
                }
                return true;
            } catch (NumberFormatException nfe) {
                // do not allow change to commit
            }
        }
        return false;
    }
    
    public boolean isEditable() {
        return !mPointSystem.getCurrentRaidData().isLocked();
    }
}
