package org.wow.autodkp.view.table.players;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.CostItem;
import org.wow.autodkp.model.points.CustomFormulaItem;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.persistence.BossManager;
import org.wow.autodkp.persistence.InstanceManager;
import org.wow.autodkp.persistence.xml.XMLUtilities;
import org.wow.autodkp.view.BasicDialog;
import org.wow.autodkp.view.DateControls;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.ImageUtilities;
import org.wow.autodkp.view.SaveOnExitProperty;
import org.wow.autodkp.view.filters.BossItemFilter;
import org.wow.autodkp.view.filters.InstanceItemFilter;
import org.wow.autodkp.view.filters.ItemFilter;
import org.wow.autodkp.view.filters.NoFilter;
import org.wow.autodkp.view.filters.QualityItemFilter;
import org.wow.autodkp.view.table.LootRenderer;


/**
 * 
 * 
 * @author jsandusky
 */
public class DialogFormulaItem extends BasicDialog {
    
    private Raid mRaid;
    private PlayerData mPlayerData;
    private JLabel mStatusLabel;
    private JLabel mNameLabel;
    private JLabel mPointsLabel;
    private JLabel mDateLabel;
    private CostItem mCostItem;
    private JTextField mNameField;
    private JTextField mCostField;
    private JList mItemsList;
    private JComboBox mTypeCombo;
    private JPanel mItemsPanel;
    
    private IPointSystem mPointSystem;
    
    private DateControls mDateControls;
    
    private CustomFormulaItem mCustomFormulaItemResult;
    private Cost mLootCostResult;
    
    private static final ItemFilter NO_FILTER = new NoFilter();
    private static ItemFilter[] FILTERS1;
    private static ItemFilter[] FILTERS2;
    private JComboBox mItemFilter1;
    private JComboBox mItemFilter2;
    private Object[] mFilteredData;
    
    private DocumentListener mItemNameScroller;
    private boolean mIsNameScrollingEnabled;
    
    
    static {
        List filters = new ArrayList();
        filters.add(NO_FILTER);
        for (int i=1; i < LootConstants.QUALITY_KEYS.length; i++) {
            int quality = i;
            filters.add(new QualityItemFilter(quality));
        }
        FILTERS1 = (ItemFilter[]) filters.toArray(new ItemFilter[0]);
        
        filters.clear();
        filters.add(NO_FILTER);
        for (int i=1; i < InstanceManager.getDefault().getNames().size(); i++) {
            String key = InstanceManager.getDefault().getName(i);
            filters.add(new InstanceItemFilter(key));
        }
        for (int i=1; i < BossManager.getDefault().getNames().size(); i++) {
            String name = BossManager.getDefault().getName(i);
            filters.add(new BossItemFilter(name));
        }
        FILTERS2 = (ItemFilter[]) filters.toArray(new ItemFilter[0]);
    }
    
    
    public DialogFormulaItem(
            Raid raid,
            Dialog owner, 
            String title, 
            PlayerData playerData,
            CostItem item,
            IPointSystem pointSystem)
    {
        super(owner, title, true);
        addBasicSupport();
        
        mRaid = raid;
        mPlayerData = playerData;
        mCostItem = item;
        mPointSystem = pointSystem;
        
        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        
        ActionListener listener = null;
        if (mCostItem == null) {
            listener = getAddActionListener();
        } else {
            listener = getEditActionListener();
        }
        setLayout(new BorderLayout());
        getContentPane().add(getMainPane(),           BorderLayout.CENTER);
        getContentPane().add(getButtonPane(listener), BorderLayout.SOUTH);

        setResizable(false);
        pack();
        setLocationRelativeTo(owner);
    }
    
    protected ActionListener getAddActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mNameLabel.setForeground(Color.BLACK);
                mPointsLabel.setForeground(Color.BLACK);
                mDateLabel.setForeground(Color.BLACK);
                mStatusLabel.setText("");
                String name = mNameField.getText().trim();
                Cost cost = Cost.parseCost(mCostField.getText().trim());
                boolean isValid = true;
                if (name.length() == 0 || !XMLUtilities.isValidString(name)) {
                    isValid = false;
                    mNameLabel.setForeground(Color.RED);
                }
                if (cost == null) {
                    isValid = false;
                    mPointsLabel.setForeground(Color.RED);
                }
                Date lootDate = null;
                if (mDateControls.isTimeEntered()) {
                    lootDate = mDateControls.getDate();
                    if (lootDate == null) {
                        // incorrect date format
                        isValid = false;
                        mDateLabel.setForeground(Color.RED);
                    }
                }
                int lootType = mTypeCombo.getSelectedIndex();
                if (
                        mPointSystem.isZeroSum() && 
                        lootType > LootConstants.QUALITY_NONE && 
                        !mDateControls.isTimeEntered())
                {
                    isValid = false;
                    mDateLabel.setForeground(Color.RED);
                    mStatusLabel.setText("Zero Sum requires Loot Time");
                    // we need to resize because this text may not fit
                    pack();
                }
                if (isValid) {
                    mCustomFormulaItemResult = new CustomFormulaItem();
                    mCustomFormulaItemResult.setItemText(name);
                    mCustomFormulaItemResult.setCost(cost);
                    if (lootType > LootConstants.QUALITY_NONE) {
                        Color color = Loot.getColorFromQuality(lootType);
                        mCustomFormulaItemResult.setLootColor(color);
                    }
                    if (lootDate != null) {
                        mCustomFormulaItemResult.setLootTime(lootDate);
                    }
                    closing();
                    dispose();
                }
            }
        };
    }
    
    protected ActionListener getEditActionListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mNameLabel.setForeground(Color.BLACK);
                mPointsLabel.setForeground(Color.BLACK);
                mDateLabel.setForeground(Color.BLACK);
                mStatusLabel.setText("");
                String name = mNameField.getText().trim();
                Cost cost = Cost.parseCost(mCostField.getText().trim());
                boolean isValid = true;
                boolean fullCheck = !mCostItem.isLoot();
                if (fullCheck) {
                    if (name.length() == 0 || !XMLUtilities.isValidString(name)) {
                        isValid = false;
                        mNameLabel.setForeground(Color.RED);
                    }
                }
                if (cost == null) {
                    isValid = false;
                    mPointsLabel.setForeground(Color.RED);
                }
                Date lootDate = null;
                if (fullCheck && mDateControls.isTimeEntered()) {
                    lootDate = mDateControls.getDate();
                    if (lootDate == null) {
                        // incorrect date format
                        isValid = false;
                        mDateLabel.setForeground(Color.RED);
                    }
                }
                int lootType = mTypeCombo.getSelectedIndex();
                if (
                        fullCheck && 
                        mPointSystem.isZeroSum() && 
                        lootType > LootConstants.QUALITY_NONE &&
                        !mDateControls.isTimeEntered())
                {
                    isValid = false;
                    mDateLabel.setForeground(Color.RED);
                    mStatusLabel.setText("Zero Sum requires Loot Time");
                    // we need to resize because this text may not fit
                    pack();
                }
                if (isValid) {
                    if (mCostItem.isLoot()) {
                        mLootCostResult = cost;
                    } else {
                        mCustomFormulaItemResult = new CustomFormulaItem();
                        mCustomFormulaItemResult.setItemText(name);
                        mCustomFormulaItemResult.setCost(cost);
                        if (lootType == LootConstants.QUALITY_NONE) {
                            mCustomFormulaItemResult.setLootColor(null);
                        } else {                    
                            Color color = Loot.getColorFromQuality(lootType);
                            mCustomFormulaItemResult.setLootColor(color);
                        }
                        if (lootDate != null) {
                            mCustomFormulaItemResult.setLootTime(lootDate);
                        }
                    }
                    closing();
                    dispose();
                }
            }
        };
    }
    
    protected JComponent getMainPane() {
        mNameLabel = new JLabel("Description:");
        mNameLabel.setBorder(BorderFactory.createEmptyBorder(15, 0, 0, 5));
        mPointsLabel = new JLabel("Cost:");
        mPointsLabel.setBorder(BorderFactory.createEmptyBorder(8, 0, 0, 5));
        JLabel typeLabel = new JLabel("Loot Type:");
        typeLabel.setBorder(BorderFactory.createEmptyBorder(8, 0, 0, 5));
        mNameField = new JTextField(20);
        mCostField = new JTextField(6);
        mTypeCombo = new JComboBox(LootConstants.QUALITY_NAMES);
        mTypeCombo.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createEmptyBorder(4, 0, 0, 5),
                mTypeCombo.getBorder()));
        mTypeCombo.setSelectedIndex(LootConstants.QUALITY_NONE);
        
        mDateLabel = new JLabel("Loot Time:");
        mDateLabel.setBorder(BorderFactory.createEmptyBorder(13, 0, 0, 5));
        
        mDateControls = new DateControls();
        boolean isDateInitialized = false;
        if (mCostItem != null) {
            Date lootTime = null;
            if (mCostItem.isLoot()) {
                Loot loot = mCostItem.getLoot();
                lootTime = mPlayerData.getLootTimeForLoot(loot);
            } else {
                CustomFormulaItem customItem = mCostItem.getFormulaItem();
                lootTime = customItem.getLootTime();
            }
            if (lootTime != null) {
                mDateControls.initializeDateAndTime(lootTime);
                isDateInitialized = true;
            }
        }
        if (!isDateInitialized) {
            mDateControls.initializeDate(mRaid.getStartTime());
        }
        
        mItemsList = new JList();
        mItemsList.setCellRenderer(new LootRenderer());
        mItemsList.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    Loot loot = (Loot) mItemsList.getSelectedValue();
                    if (loot != null) {
                    	if (mIsNameScrollingEnabled) {
                        	mIsNameScrollingEnabled = false;
                            mNameField.setText(loot.getName().replaceAll("'", ""));
                            mIsNameScrollingEnabled = true;
                    	} else {
                            mNameField.setText(loot.getName().replaceAll("'", ""));
                    	}
                        Cost cost = Loot.findLootCost(mPlayerData, mPointSystem, loot);
                        if (cost != null) {
                            mCostField.setText(cost.toString());
                        }
                        mTypeCombo.setSelectedIndex(loot.getQuality());
                    }
                }
            }
        });
        
        final JScrollPane itemsScroller = new JScrollPane(mItemsList);
        
        mItemFilter1 = new JComboBox(FILTERS1);
        mItemFilter2 = new JComboBox(FILTERS2);
        ActionListener filterListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                filterData();
            }
        };
        mItemFilter1.addActionListener(filterListener);
        mItemFilter2.addActionListener(filterListener);
        
        JLabel filterLabel = new JLabel("Filters:");
        Box filterPane = Box.createHorizontalBox();
        filterPane.add(filterLabel);
        filterPane.add(Box.createHorizontalStrut(5));
        filterPane.add(mItemFilter1);
        filterPane.add(Box.createHorizontalStrut(5));
        filterPane.add(mItemFilter2);

        mItemsPanel = new JPanel(new BorderLayout(0, 5));
        mItemsPanel.setBorder(BorderFactory.createEmptyBorder(20, 5, 5, 5));
        mItemsPanel.add(filterPane,    BorderLayout.NORTH);
        mItemsPanel.add(itemsScroller, BorderLayout.CENTER);
        mItemsPanel.setVisible(false);
        
        Image browseImage = ImageUtilities.loadJarImage("browse.png");
        JButton browseButton = new JButton(new ImageIcon(browseImage));
        browseButton.setPreferredSize(new Dimension(20, 20));
        browseButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mItemsPanel.isVisible()) {
                	mItemsPanel.setVisible(false);
                    pack();
                    mNameField.requestFocusInWindow();
                } else {
                	if (mFilteredData == null) {
                		mFilteredData = AutoDKPModel.getInstance().getAllLoot().toArray();
                        mItemsList.setListData(mFilteredData);
                	}
                	mItemsPanel.setVisible(true);
                    pack();
                    mNameField.requestFocusInWindow();
                }
            }
        });
        
        // If the items list is being displayed, then as the user types
        // in the item name, the items list should scroll to that location.
        mItemNameScroller = new DocumentListener() {
			public void changedUpdate(DocumentEvent arg0) {}
			public void removeUpdate(DocumentEvent arg0) {
				scrollTo(mNameField.getText().toLowerCase());
			}
			public void insertUpdate(DocumentEvent arg0) {
				scrollTo(mNameField.getText().toLowerCase());
			}
			private void scrollTo(String text) {
				if (
						mIsNameScrollingEnabled &&
						mItemsPanel.isVisible() && 
						text.length() > 0) {
					int vertAdjust = itemsScroller.getViewport().getHeight();
					for (int i=0; i < mFilteredData.length; i++) {
						Loot loot = (Loot) mFilteredData[i];
						String lootName = loot.getName();
						if (lootName != null && lootName.toLowerCase().startsWith(text)) {
							Point p = mItemsList.indexToLocation(i);
							if (p != null) {
								mItemsList.scrollRectToVisible(
										new Rectangle(p, new Dimension(0, vertAdjust)));
								break;
							}
						}
					}
				}
			}
		};
		
        if (mCostItem == null) {
        	mIsNameScrollingEnabled = true;
            mNameField.getDocument().addDocumentListener(mItemNameScroller);
        } else {
            if (mCostItem.isLoot()) {
                Loot loot = mCostItem.getLoot();
                mNameField.setText(loot.getName());
                Cost cost = mCostItem.getCost();
                if (cost != null) {
                    mCostField.setText(cost.toString());
                }
                int quality = loot.getQuality();
                mTypeCombo.setSelectedIndex(quality);
                
                mNameField.setEditable(false);
                mCostField.setEditable(mPointSystem.isAuctionedLootValues());
                mTypeCombo.setEnabled(false);
                mDateControls.setEditable(false);
                browseButton.setEnabled(false);
                
            } else {
                CustomFormulaItem item = mCostItem.getFormulaItem();
                mNameField.setText(item.getItemText());
                mIsNameScrollingEnabled = true;
                mNameField.getDocument().addDocumentListener(mItemNameScroller);
                Cost cost = item.getCost();
                if (cost != null) {
                    mCostField.setText(cost.toString());
                }
                Color color = item.getLootColor();
                if (color != null) {
                    String colorString = Loot.getColorStringFromColor(color);
                    int quality = Loot.getQualityFromColorString(colorString);
                    mTypeCombo.setSelectedIndex(quality);
                }
            }
        }
        
        JPanel mainPane = new JPanel();
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints gdc = new GridBagConstraints();
        mainPane.setLayout(gridBag);
        gdc.anchor = GridBagConstraints.SOUTHWEST;
        
        int y = -1;
        gdc.weighty = 0;
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mNameLabel, gdc);
        mainPane.add(mNameLabel);
        gdc.gridx = 1;
        gdc.weightx = 1;
        gdc.gridwidth = 2;
        gdc.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(mNameField, gdc);
        mainPane.add(mNameField);
        gdc.gridx = 3;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(browseButton, gdc);
        mainPane.add(browseButton);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mPointsLabel, gdc);
        mainPane.add(mPointsLabel);
        gdc.gridx = 1;
        gdc.weightx = 0;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mCostField, gdc);
        mainPane.add(mCostField);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(typeLabel, gdc);
        mainPane.add(typeLabel);
        gdc.gridx = 1;
        gdc.weightx = 0;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mTypeCombo, gdc);
        mainPane.add(mTypeCombo);
        
        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 0;
        gdc.gridwidth = 1;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mDateLabel, gdc);
        mainPane.add(mDateLabel);
        gdc.gridx = 1;
        gdc.weightx = 0;
        gdc.gridwidth = 3;
        gdc.fill = GridBagConstraints.NONE;
        gridBag.setConstraints(mDateControls.getComponent(), gdc);
        mainPane.add(mDateControls.getComponent());

        gdc.gridy = ++y;
        gdc.gridx = 0;
        gdc.weightx = 1;
        gdc.weighty = 1;
        gdc.gridwidth = 4;
        gdc.fill = GridBagConstraints.BOTH;
        gridBag.setConstraints(mItemsPanel, gdc);
        mainPane.add(mItemsPanel);
        
        mainPane.setBorder(BorderFactory.createEmptyBorder(0, 10, 5, 10));
        
        GeneralProperties props = GeneralProperties.getDefault();
        Integer filter1Prop = props.getIntegerProperty(props.FORMULA_ITEM_FILTER1);
        Integer filter2Prop = props.getIntegerProperty(props.FORMULA_ITEM_FILTER2);
        String chooserShowing = props.getStringProperty(props.FORMULA_ITEM_SHOWING);
        if (chooserShowing != null) {
            if (chooserShowing.equals(XMLUtilities.IS_YES)) {
                browseButton.doClick();
            }
        }
        if (filter1Prop != null) {
            int index = filter1Prop.intValue();
            if (index < mItemFilter1.getItemCount()) {
                mItemFilter1.setSelectedIndex(index);
            }
        }
        if (filter2Prop != null) {
            int index = filter2Prop.intValue();
            if (index < mItemFilter2.getItemCount()) {
                mItemFilter2.setSelectedIndex(index);
            }
        }
        props.addSaveOnExitProperty(new SaveOnExitProperty(props.FORMULA_ITEM_FILTER1) {
            public Object getValue() { return new Integer(mItemFilter1.getSelectedIndex()); }
        });
        props.addSaveOnExitProperty(new SaveOnExitProperty(props.FORMULA_ITEM_FILTER2) {
            public Object getValue() { return new Integer(mItemFilter2.getSelectedIndex()); }
        });

        return mainPane;
    }
    
    protected JComponent getButtonPane(ActionListener actionListener) {
        JButton applyButton = new JButton();
        if (mCostItem == null) {
            applyButton.setText("Add");
        } else {
            applyButton.setText("Edit");
        }
        applyButton.addActionListener(actionListener);
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	closing();
                dispose();
            }
        });
        
        getRootPane().setDefaultButton(applyButton);
        
        mStatusLabel = new JLabel();
        mStatusLabel.setForeground(Color.RED);
        
        JPanel rightPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        rightPane.add(applyButton);
        rightPane.add(cancelButton);
        JPanel buttonPane = new JPanel(new BorderLayout());
        buttonPane.add(mStatusLabel, BorderLayout.CENTER);
        buttonPane.add(rightPane, BorderLayout.EAST);
        buttonPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        return buttonPane;
    }
    
    protected void closing() {
        GeneralProperties props = GeneralProperties.getDefault();
        props.setIntegerProperty(props.FORMULA_ITEM_FILTER1, mItemFilter1.getSelectedIndex());
        props.setIntegerProperty(props.FORMULA_ITEM_FILTER2, mItemFilter2.getSelectedIndex());
        
        props.setStringProperty(props.FORMULA_ITEM_SHOWING, 
                mItemsPanel.isVisible() ? XMLUtilities.IS_YES : XMLUtilities.IS_NO);
    }
    
    public void filterData() {
        ItemFilter filter1 = (ItemFilter) mItemFilter1.getSelectedItem();
        ItemFilter filter2 = (ItemFilter) mItemFilter2.getSelectedItem();
        if (filter1 == NO_FILTER && filter2 == NO_FILTER) {
        	mFilteredData = AutoDKPModel.getInstance().getAllLoot().toArray();
        } else {
            List filteredData = new ArrayList();
            int i = 0;
            for (Iterator iter=AutoDKPModel.getInstance().getAllLoot().iterator(); iter.hasNext(); i++) {
                Loot loot = (Loot) iter.next();
                if (!filter1.isFiltered(loot) && !filter2.isFiltered(loot)) {
                    filteredData.add(loot);
                }
            }
            mFilteredData = filteredData.toArray();
        }
        mItemsList.setListData(mFilteredData);
    }

    public CustomFormulaItem getCustomFormulaItem() {
        return mCustomFormulaItemResult;
    }
    
    public Cost getNewLootCost() {
        return mLootCostResult;
    }
}
