package org.wow.autodkp.view.table.players;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractCellEditor;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.ListCellRenderer;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellEditor;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.Cost;
import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.CostItem;
import org.wow.autodkp.model.points.CustomFormulaItem;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.model.points.IPointSystemPlayerData;
import org.wow.autodkp.model.points.dkp.IFormula;
import org.wow.autodkp.view.AutoDKPView;
import org.wow.autodkp.view.BasicDialog;
import org.wow.autodkp.view.GeneralProperties;
import org.wow.autodkp.view.ImageUtilities;
import org.wow.autodkp.view.preferences.PreferencesWindow;
import org.wow.autodkp.view.skin.AutoDKPLookAndFeelSupport;
import org.wow.autodkp.view.table.TableSorter;


/**
 * 
 * 
 * @author jsandusky
 */
public class FormulaEditor extends AbstractCellEditor implements TableCellEditor {

    private List mCostItems;
    private JList mItemsList;
    private IPointSystem mPointSystem;
    private IPointSystemPlayerData mConstraintData;
    private PlayerData mRaidPlayerData;
    private Raid mRaid;
    private ActionListener mEditListener;
    private ActionListener mDeleteListener;
    
    private JDialog mDialog;
    private JPanel mControlPane;
    private JButton mUpButton;
    private JButton mDownButton;

    private boolean mIsModified;
    private boolean mIsRaidPlayerDataModified;
    private CostItem mSelectedCostItem;
    
    
    public FormulaEditor() {
        mItemsList = new JList();
        mItemsList.setCellRenderer(new ItemCellRenderer());
        
        mItemsList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() > 1) {
                    Point point = e.getPoint();
                    int index = mItemsList.locationToIndex(point);
                    if (index >= 0) {
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                                if (mSelectedCostItem.isLoot()) {
                                    getDeleteItemListener().actionPerformed(null);
                                } else {
                                    getEditItemListener().actionPerformed(null);
                                }
                            }
                        });
                    }
                }
            }
        });
        
        mControlPane = new JPanel(new BorderLayout());
        mControlPane.setOpaque(false);
        
        mDialog = new FormulaEditorDialog();
        mDialog.setLayout(new BorderLayout());
        mDialog.setBackground(Color.WHITE);
        mDialog.add(mControlPane, BorderLayout.CENTER);
        
        JButton saveButton = new JButton("Save");
        saveButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mIsModified) {
                    List excludedIDs = new ArrayList();
                    List customItems = new ArrayList();
                    int index = 0;
                    for (Iterator iter=mCostItems.iterator(); iter.hasNext(); index++) {
                        CostItem item = (CostItem) iter.next();
                        if (item.isLoot()) {
                            String lootID = item.getLootID();
                            if (item.isFiltered()) {
                                excludedIDs.add(lootID);
                            }
                        } else {
                            CustomFormulaItem customItem = item.getFormulaItem();
                            customItem.setIndex(index);
                            customItems.add(customItem);
                        }
                    }
                    mConstraintData.setExcludedLootIDs(excludedIDs);
                    mConstraintData.setCustomFormulaItems(customItems);
                    if        (mPointSystem.isZeroSum()) {
                        mPointSystem.getCalculator().calculate();
                    } else if (mPointSystem.isBossKill()) {
                        mPointSystem.getCalculator().calculateLineItem(mRaidPlayerData, mConstraintData);
                    }
                    mPointSystem.saveRaid(mRaid);
                }
                if (mIsRaidPlayerDataModified) {
                    mRaid.promote();
                }
                if (mIsModified || mIsRaidPlayerDataModified) {
                    AutoDKPView.getInstance().reloadRaids();
                }
                saveLocation();
                mDialog.dispose();
            }
        });
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                saveLocation();
                mDialog.dispose();
            }
        });
        JPanel buttonPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttonPane.add(cancelButton);
        buttonPane.add(saveButton);
        buttonPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        mDialog.add(buttonPane, BorderLayout.SOUTH);
        
        Image upImage = ImageUtilities.loadJarImage("up.gif");
        mUpButton = new JButton(new ImageIcon(upImage));
        mUpButton.setPreferredSize(new Dimension(32, 32));
        mUpButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mSelectedCostItem != null) {
                    if (!mSelectedCostItem.isLoot()) {
                        int index = mCostItems.indexOf(mSelectedCostItem);
                        if (index > 0) {
                            mCostItems.remove(mSelectedCostItem);
                            mCostItems.add(index - 1, mSelectedCostItem);
                            mItemsList.setListData(mCostItems.toArray());
                            mItemsList.setSelectedIndex(index - 1);
                            mIsModified = true;
                        }
                    }
                }
            }
        });
        Image downImage = ImageUtilities.loadJarImage("down.gif");
        mDownButton = new JButton(new ImageIcon(downImage));
        mDownButton.setPreferredSize(new Dimension(32, 32));
        mDownButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (mSelectedCostItem != null) {
                    if (!mSelectedCostItem.isLoot()) {
                        int index = mCostItems.indexOf(mSelectedCostItem);
                        if (index < mCostItems.size() - 1) {
                            mCostItems.remove(mSelectedCostItem);
                            mCostItems.add(index + 1, mSelectedCostItem);
                            mItemsList.setListData(mCostItems.toArray());
                            mItemsList.setSelectedIndex(index + 1);
                            mIsModified = true;
                        }
                    }
                }
            }
        });
        Box orderPane = Box.createVerticalBox();
        orderPane.add(mUpButton);
        orderPane.add(Box.createVerticalStrut(5));
        orderPane.add(mDownButton);
        orderPane.setBorder(BorderFactory.createEmptyBorder(60, 5, 5, 10));
        mDialog.add(orderPane, BorderLayout.EAST);
    }
    
    private void saveLocation() {
        GeneralProperties props = GeneralProperties.getDefault();
        props.setIntegerProperty(props.FORMULA_EDITOR_WINDOW_X, mDialog.getX());
        props.setIntegerProperty(props.FORMULA_EDITOR_WINDOW_Y, mDialog.getY());
        props.setIntegerProperty(props.FORMULA_EDITOR_WINDOW_WIDTH,  mDialog.getWidth());
        props.setIntegerProperty(props.FORMULA_EDITOR_WINDOW_HEIGHT, mDialog.getHeight());
    }
    
    public void setPlayerData(IPointSystem pointSystem, Raid raid) {
        mPointSystem = pointSystem;
        mRaid = raid;
    }

    public boolean isCellEditable(EventObject anEvent) {
        if (anEvent instanceof MouseEvent) {
            if (((MouseEvent) anEvent).getClickCount() < 2) {
                return false;
            }
        }
        if (anEvent != null) {
            Object source = anEvent.getSource();
            if (source instanceof JTable) {
                JTable table = (JTable) source;
                int row = table.getSelectedRow();
                if (row >= 0) {
                    CostItem[] costItems = getCostItems(table, row);
                    if (costItems != null) {
                        // We pop open the editor without actually causing
                        // a cell edit. This is more convenient and visually
                        // appealing.
                        showEditor(table, row);
                        return false;
                    }
                }
            }
        }
        return false;
    }
    
    private CostItem[] getCostItems(JTable table, int row) {
        TableSorter sorter = (TableSorter) table.getModel();
        PlayerTableModel playerTableModel = (PlayerTableModel) sorter.getTableModel();
        Map data = mPointSystem.getCurrentRaidData().getPointSystemPlayerDataMap();
        int actualRow = sorter.modelIndex(row);
        mRaidPlayerData = playerTableModel.getRaidPlayerDataForRow(actualRow);
        mConstraintData = (IPointSystemPlayerData) data.get(mRaidPlayerData.getPlayerName());
        if (mConstraintData == null) {
            return null;
        }
        IFormula formula = mConstraintData.getFormula();
        return formula.getCostItems();
    }
    
    private void showEditor(JTable table, int row) {
        mCostItems = new ArrayList();
        CostItem[] costItems = getCostItems(table, row);
        for (int i=0; i < costItems.length; i++) {
            mCostItems.add(costItems[i]);
        }
        mItemsList.setListData(costItems);
        
        JComponent itemPane = PreferencesWindow.generatePane(
                "Cost Items", 
                getAddItemListener(),
                getEditItemListener(),
                getDeleteItemListener(),
                getItemSelectionListener(),
                mItemsList);
        
        mControlPane.removeAll();
        mControlPane.add(itemPane, BorderLayout.CENTER);
        
        GeneralProperties props = GeneralProperties.getDefault();
        props.loadWindowRect(
                mDialog, 
                props.FORMULA_EDITOR_WINDOW_X,
                props.FORMULA_EDITOR_WINDOW_Y,
                props.FORMULA_EDITOR_WINDOW_WIDTH,
                props.FORMULA_EDITOR_WINDOW_HEIGHT,
                props.DEFAULT_FORMULA_EDITOR_WIDTH,
                props.DEFAULT_FORMULA_EDITOR_HEIGHT);
        
        mDialog.setVisible(true);
    }
    
    public Component getTableCellEditorComponent(final JTable table, 
                                                 Object value,
                                                 boolean isSelected,
                                                 int row, 
                                                 int column)
    {
        // We don't actually pop open a table cell editor,
        // we just open the editor if we detect that a cell
        // should be edited.
        return null;
    }
    
    public Object getCellEditorValue() {
        return mConstraintData.getFormula().getPrintObject();
    }
    
    private ActionListener getAddItemListener() {
        return new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                DialogFormulaItem diag = new DialogFormulaItem(
                        mRaid,
                        mDialog,
                        "Add Custom Cost Item",
                        mRaidPlayerData,
                        null,
                        mPointSystem);
                diag.setVisible(true);
                CustomFormulaItem item = diag.getCustomFormulaItem();
                if (item != null) {
                    mCostItems.add(new CostItem(item));
                    mIsModified = true;
                    mItemsList.setListData(mCostItems.toArray());
                }
            }
        };
    }
    
    private ActionListener getEditItemListener() {
        if (mEditListener == null) {
            mEditListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (mSelectedCostItem != null) {
                        DialogFormulaItem diag = new DialogFormulaItem(
                                mRaid,
                                mDialog,
                                "Edit Formula Item",
                                mRaidPlayerData,
                                mSelectedCostItem,
                                mPointSystem);
                        diag.setVisible(true);
                        if (mSelectedCostItem.isLoot()) {
                            Cost cost = diag.getNewLootCost();
                            if (cost != null) {
                                Loot loot = mSelectedCostItem.getLoot();
                                mRaidPlayerData.setAuctionedLootValue(loot, cost);
                                mSelectedCostItem.setCost(cost);
                                int index = mCostItems.indexOf(mSelectedCostItem);
                                mItemsList.setListData(mCostItems.toArray());
                                mItemsList.setSelectedIndex(index);
                                mIsRaidPlayerDataModified = true;
                            }
                        } else {
                            CustomFormulaItem item = diag.getCustomFormulaItem();
                            if (item != null) {
                                int index = mCostItems.indexOf(mSelectedCostItem);
                                mCostItems.remove(mSelectedCostItem);
                                mCostItems.add(index, new CostItem(item));
                                mItemsList.setListData(mCostItems.toArray());
                                mItemsList.setSelectedIndex(index);
                                mIsModified = true;
                            }
                        }
                    }
                }
            };
        }
        return mEditListener;
    }
    
    private ActionListener getDeleteItemListener() {
        if (mDeleteListener == null) {
            mDeleteListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (mSelectedCostItem != null) {
                        if (!mSelectedCostItem.isLoot()) {
                            mCostItems.remove(mSelectedCostItem);
                            mIsModified = true;
                            mItemsList.setListData(mCostItems.toArray());
                        } else {
                            mSelectedCostItem.setFiltered(!mSelectedCostItem.isFiltered());
                            mIsModified = true;
                            mItemsList.repaint();
                        }
                    }
                }
            };
        }
        return mDeleteListener;
    }
    
    private ListSelectionListener getItemSelectionListener() {
        return new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (!e.getValueIsAdjusting()) {
                    mSelectedCostItem = (CostItem) mItemsList.getSelectedValue();
                }
            }
        };
    }
    
    
    private class ItemCellRenderer implements ListCellRenderer {
        
        protected Border noFocusBorder;
        private JCheckBox mCheck;
        private JLabel mLabel;
        
        public ItemCellRenderer() {
            mCheck = new JCheckBox();
            mCheck.setOpaque(true);
            mLabel = new JLabel();
            mLabel.setOpaque(true);
            if (noFocusBorder == null) {
                noFocusBorder = new EmptyBorder(1, 1, 1, 1);
            }
            mCheck.setBorder(noFocusBorder);
            mLabel.setBorder(noFocusBorder);
        }
        
        public Component getListCellRendererComponent(
                JList list, 
                Object value, 
                int index,
                boolean isSelected, 
                boolean cellHasFocus)
        {
            
            if (isSelected) {
                mCheck.setBackground(list.getSelectionBackground());
                mCheck.setForeground(list.getSelectionForeground());
                mLabel.setBackground(list.getSelectionBackground());
                mLabel.setForeground(list.getSelectionForeground());
            }
            else {
                mCheck.setForeground(list.getForeground());
                mLabel.setForeground(list.getForeground());
                Color background = 
                    AutoDKPLookAndFeelSupport.getUnselectedBackgroundColor(
                            index, 
                            list.getBackground());
                mCheck.setBackground(background);
                mLabel.setBackground(background);
            }

            mCheck.setFont(list.getFont());
            mLabel.setFont(list.getFont());
            mCheck.setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);
            mLabel.setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);

            CostItem costItem = (CostItem) value;
            if (costItem.isLoot()) {
                String lootID = costItem.getLootID();
                Loot loot = AutoDKPModel.getInstance().findLoot(lootID);
                if (loot != null) {
                    String nameStr = loot.getName();
                    String costStr = costItem.getCost() == null ? "" : ", " + costItem.getCost().toString();
                    mCheck.setText(nameStr + costStr);
                }
                mCheck.setSelected(!costItem.isFiltered());
                return mCheck;
            }

            CustomFormulaItem customItem = costItem.getFormulaItem();
            mLabel.setText(customItem.getItemText() + ", " + customItem.getCost().toString());
            return mLabel;
        }
    }
    
    
    private class FormulaEditorDialog extends BasicDialog {
        public FormulaEditorDialog() {
            super(
                    AutoDKPView.getInstance().getFrame(), 
                    "Formula Editor", 
                    true);
            addBasicSupport();
        }
        protected void closing() {
            saveLocation();
        }
    }
}
