package org.wow.autodkp.view.table.players;

import java.awt.Color;
import java.awt.Component;
import java.util.List;

import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import org.wow.autodkp.model.Logger;
import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.LootConstants;
import org.wow.autodkp.view.skin.AutoDKPLookAndFeelSupport;


/**
 * 
 * 
 * @author jsandusky
 */
public class FormulaRenderer implements TableCellRenderer {
    
    private static final Border SAFE_NO_FOCUS_BORDER = new EmptyBorder(1, 1, 1, 1);
    
    private JTextPane mTextPane;
    private StyledDocument mStyledDocument;
    
    private Style mMasterStyle;
    private Style mPointsStyle;
    private Style[] mStyles;
    
    private int mOffset;
    
    
    public FormulaRenderer() {
        mTextPane = new JTextPane();
        mTextPane.setBorder(null);
        mStyledDocument = mTextPane.getStyledDocument();
        
        mStyles = new Style[8];
        
        // for now, leave this null
        mMasterStyle = null;
        
        mPointsStyle = mTextPane.addStyle("POINTS", mMasterStyle);
        StyleConstants.setFontFamily(mPointsStyle, "Monospaced");
        
        mStyles[LootConstants.QUALITY_POOR]      = mTextPane.addStyle("FG_POOR",      mMasterStyle);
        mStyles[LootConstants.QUALITY_COMMON]    = mTextPane.addStyle("FG_COMMON",    mMasterStyle);
        mStyles[LootConstants.QUALITY_UNCOMMON]  = mTextPane.addStyle("FG_UNCOMMON",  mMasterStyle);
        mStyles[LootConstants.QUALITY_RARE]      = mTextPane.addStyle("FG_RARE",      mMasterStyle);
        mStyles[LootConstants.QUALITY_EPIC]      = mTextPane.addStyle("FG_EPIC",      mMasterStyle);
        mStyles[LootConstants.QUALITY_LEGENDARY] = mTextPane.addStyle("FG_LEGENDARY", mMasterStyle);
        mStyles[LootConstants.QUALITY_ARTIFACT]  = mTextPane.addStyle("FG_ARTIFACT",  mMasterStyle);
        mStyles[LootConstants.QUALITY_NONE]      = mStyles[LootConstants.QUALITY_POOR];
        
        StyleConstants.setForeground(mStyles[LootConstants.QUALITY_POOR],      Loot.COLOR_POOR);
        StyleConstants.setForeground(mStyles[LootConstants.QUALITY_COMMON],    Loot.COLOR_COMMON);
        StyleConstants.setForeground(mStyles[LootConstants.QUALITY_UNCOMMON],  Loot.COLOR_UNCOMMON);
        StyleConstants.setForeground(mStyles[LootConstants.QUALITY_RARE],      Loot.COLOR_RARE);
        StyleConstants.setForeground(mStyles[LootConstants.QUALITY_EPIC],      Loot.COLOR_EPIC);
        StyleConstants.setForeground(mStyles[LootConstants.QUALITY_LEGENDARY], Loot.COLOR_LEGENDARY);
        StyleConstants.setForeground(mStyles[LootConstants.QUALITY_ARTIFACT],  Loot.COLOR_ARTIFACT);
    }
    
    
    private void renderToDocument(Object value) {
        if (value instanceof FormulaPrintObject) {
            FormulaPrintObject printObject = (FormulaPrintObject) value;
            List printLines = printObject.getPrintLines();
            if (printLines.size() == 0) {
                return;
            }
            for (int i=0, count=printLines.size(); i < count; i++) {
                FormulaPrintLine printLine = (FormulaPrintLine) printLines.get(i);
                addText(printLine.getPoints(), mPointsStyle);
                String colorHexString = printLine.getColorHexString();
                if (colorHexString != null) {
                    int quality = Loot.getQualityFromColorString(colorHexString);
                    addText(printLine.getDescription() + "\n", mStyles[quality]);
                } else {
                    addText(printLine.getDescription() + "\n", mMasterStyle);
                }
            }
        }
    }
    
    private void addText(String newText, Style style) {
        try {
            mStyledDocument.insertString(mOffset, newText, style);
            mOffset += newText.length();
        } catch (BadLocationException e) {
            Logger.log(e);
        }
    }
    
    
    public Component getTableCellRendererComponent(JTable table, 
                                                   Object value,
                                                   boolean isSelected, 
                                                   boolean hasFocus, 
                                                   int row, 
                                                   int column)
    {
        if (isSelected) {
            mTextPane.setForeground(table.getSelectionForeground());
            mTextPane.setBackground(table.getSelectionBackground());
        } else {
            mTextPane.setForeground(table.getForeground());
            Color background = 
                AutoDKPLookAndFeelSupport.getUnselectedBackgroundColor(
                        row, 
                        table.getBackground());
            mTextPane.setBackground(background);
        }

        mTextPane.setFont(table.getFont());

        if (hasFocus) {
            Border border = null;
            if (isSelected) {
                border = UIManager.getBorder("Table.focusSelectedCellHighlightBorder");
            }
            if (border == null) {
                border = UIManager.getBorder("Table.focusCellHighlightBorder");
            }
            mTextPane.setBorder(border);

            if (!isSelected && table.isCellEditable(row, column)) {
                Color col;
                col = UIManager.getColor("Table.focusCellForeground");
                if (col != null) {
                    mTextPane.setForeground(col);
                }
                col = UIManager.getColor("Table.focusCellBackground");
                if (col != null) {
                    mTextPane.setBackground(col);
                }
            }
        } else {
            mTextPane.setBorder(SAFE_NO_FOCUS_BORDER);
        }

        
        // reset text
        mOffset = 0;
        mTextPane.setText(null);
        renderToDocument(value);
        return mTextPane;
    }
}
