package org.wow.autodkp.view.table.players;

import java.util.Iterator;
import java.util.List;

import org.wow.autodkp.model.AutoDKPModel;
import org.wow.autodkp.model.Loot;
import org.wow.autodkp.model.points.dkp.IFormula;
import org.wow.autodkp.view.html.StyleUtilities;
import org.wow.autodkp.view.table.LootRenderer;


public class FormulaUtilities {
    
    public static final String SPACING = "     ";
    public static final String NEWLINE = System.getProperty("line.separator");

    
    public static int getLineCount(IFormula formula) {
        if (formula.getLineCount() == 0) {
            return readFormulaLines(formula);
        }
        return formula.getLineCount();
    }
    
    private static int readFormulaLines(IFormula formula) {
        FormulaPrintObject printObject =
            (FormulaPrintObject) formula.getPrintObject();
        if (printObject != null) {
            return printObject.getPrintLines().size();
        }
        return 0;
    }
    

    public static Object generateFormulaPrintObject(IFormula formula) {
        FormulaPrintObject printObject = new FormulaPrintObject();
        
        List pointAdjustments = formula.getPointAdjustments();
        int count = pointAdjustments.size();
        if (count == 0) {
            return printObject;
        }
        
        List pointIDs   = formula.getPointIDs();
        List pointTypes = formula.getPointTypes();
        List pointNames = formula.getPointNames();
        for (int i = 0; i < count; i++) {
            String points = (String) pointAdjustments.get(i);
            String id     = (String) pointIDs.get(i);
            String type   = (String) pointTypes.get(i);
            String name   = (String) pointNames.get(i);
            
            FormulaPrintLine printLine = new FormulaPrintLine(points);
            printObject.addPrintLine(printLine);
            
            if (
                    id.equals(IFormula.ID_BOSS_POINTS) ||
                    id.equals(IFormula.ID_ZERO_SUM)    ||
                    id.equals(IFormula.ID_TIME_POINTS))
            {
                printLine.setIsDKP(true);
                printLine.setDescription(id);
            }
            else {
                if (id.equals(IFormula.ID_CUSTOM)) {
                    if (type != null && type.length() > 0) {
                        printLine.setColorHexString(type);
                        printLine.setDescription(name);
                    } else {
                        printLine.setDescription(name);
                    }
                } else {
                    Loot loot = AutoDKPModel.getInstance().findLoot(id);
                    if (loot != null) {
                        printLine.setColorHexString(loot.getColor());
                        printLine.setDescription(loot.getName());
                    }
                    else {
                        printLine.setDescription(id);
                    }
                }
            }
        }
        
        return printObject;
    }
    
    public static String generateHTMLString(FormulaPrintObject printObject) {
        List printLines = printObject.getPrintLines();
        
        if (printLines.size() == 0) {
            return "";
        }
        
        StringBuffer buf = new StringBuffer();
        buf.append(
                NEWLINE +
                SPACING +
                "<TABLE CELLPADDING=0 CLASS=" + StyleUtilities.STYLE_TABLE_FORMULA + ">" + 
                NEWLINE);
        for (Iterator iter=printLines.iterator(); iter.hasNext();) {
            FormulaPrintLine printLine = (FormulaPrintLine) iter.next();
            
            buf.append(
                    SPACING + 
                    "<TR CLASS=" + StyleUtilities.STYLE_TR_FORMULA_ROW + ">" +
                    NEWLINE);
            buf.append(
                    SPACING + SPACING +
                    "<TD CLASS=" + StyleUtilities.STYLE_TD_FORMULA_VALUE + ">" + 
                    printLine.getPoints() + "</TD>" + NEWLINE +
                    SPACING + SPACING +
                    "<TD>&nbsp;</TD>" + NEWLINE);
            
            if (printLine.isDKP()) {
                buf.append(
                        SPACING + SPACING +
                        "<TD CLASS=" + StyleUtilities.STYLE_TD_FORMULA_TYPE_DKP + ">" +
                        printLine.getDescription() + "</TD>" + NEWLINE);
            }
            else {
                String description = null;
                String colorHexString = printLine.getColorHexString();
                if (colorHexString != null) {
                    int quality = Loot.getQualityFromColorString(colorHexString);
                    description = LootRenderer.renderLootToHTML(
                            quality, 
                            false,
                            null, 
                            printLine.getDescription());
                } else {
                    description = printLine.getDescription();
                }
                buf.append(
                        SPACING + SPACING +
                        "<TD CLASS=" + StyleUtilities.STYLE_TD_FORMULA_TYPE_COST + ">" +
                        description + "</TD>" + NEWLINE);
            }
            buf.append(SPACING + "</TR>" + NEWLINE);

        }

        buf.append(SPACING + "</TABLE>" + NEWLINE);
        return buf.toString();
    }
}
