package org.wow.autodkp.view.table.players;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.border.EtchedBorder;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;

import org.wow.autodkp.view.table.ISortHeaderRenderer;


/**
 * 
 * 
 * @author jsandusky
 */
public class PlayerTableHeaderRenderer extends JPanel 
implements TableCellRenderer, ISortHeaderRenderer {

    private static final String TOOLTIP = 
        "<HTML><P>Click to sort only this column." + 
        "<BR>CTRL-Click to sort multiple columns.</P></HTML>";
    
    private JLabel mLabel;

    
    public PlayerTableHeaderRenderer() {
        mLabel = new JLabel();
        mLabel.setHorizontalAlignment(JLabel.CENTER);
        mLabel.setVerticalAlignment(JLabel.NORTH);
        mLabel.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
        setLayout(new BorderLayout());
        add(mLabel, BorderLayout.CENTER);
        setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
    }

    public Dimension getPreferredSize() {
        return new Dimension(80, 40);
    }

    public String getToolTipText(MouseEvent event) {
        return TOOLTIP;
    }

    public Component getTableCellRendererComponent(
            JTable table, 
            Object value, 
            boolean isSelected,
            boolean hasFocus, 
            int row, 
            int column)
    {
        if (table != null) {
            JTableHeader header = table.getTableHeader();
            if (header != null) {
                mLabel.setForeground(header.getForeground());
                mLabel.setBackground(header.getBackground());
                mLabel.setFont(header.getFont());
            }
        }
        // Value is text
        mLabel.setText((value == null) ? "" : value.toString());
        return this;
    }

    public void setHorizontalTextPosition(int position) {
        mLabel.setHorizontalTextPosition(position);
    }

    public Font getColumnFont() {
        return mLabel.getFont();
    }

    public void setIcon(Icon icon) {
        mLabel.setIcon(icon);
    }
}
