package org.wow.autodkp.view.table.players;

import java.util.List;

import javax.swing.table.AbstractTableModel;

import org.wow.autodkp.model.PlayerData;
import org.wow.autodkp.model.Raid;
import org.wow.autodkp.model.points.IPointSystem;
import org.wow.autodkp.view.table.ITableColumn;



/**
 * 
 * 
 * @author jsandusky
 */
public class PlayerTableModel extends AbstractTableModel {

    public static final int COLUMN_PLAYER      = 0;
    public static final int COLUMN_CLASS       = 1;
    public static final int COLUMN_TIME_JOIN   = 2;
    public static final int COLUMN_TIME_LEAVE  = 3;
    public static final int COLUMN_LOOT        = 4;
    public static final int COLUMN_SEPARATOR   = 5;
    public static final int COLUMN_CONSTRAINTS = 6;
    public static final int SIZE_RAID = 5;
    public static final int SIZE_DEFAULT = 6;
    
    private Raid mCurrentRaid;
    private IPointSystem mCurrentSystem;
    private PlayerData[] mPlayerData;
    private List mColumnDirectory;
    
    
    public PlayerTableModel(Raid currentRaid,
                            IPointSystem currentSystem, 
                            PlayerData[] playerData, 
                            List columnDirectory) {
        mCurrentRaid = currentRaid;
        mCurrentSystem = currentSystem;
        mPlayerData = playerData;
        mColumnDirectory = columnDirectory;
    }
    
    public List getColumnDirectory() {
        return mColumnDirectory;
    }
    
    public int getColumnCount() {
        if (mPlayerData != null) {
            int totalColumns = SIZE_RAID;
            totalColumns += mColumnDirectory.size();
            return totalColumns;
        }
        return 0;
    }

    public int getRowCount() {
        if (mPlayerData != null) {
            return mPlayerData.length;
        }
        return 0;
    }

    /**
     * Pass in the model row, not the view row.
     * The sorter can convert the view->model row index.
     */
    public PlayerData getRaidPlayerDataForRow(int rowIndex) {
        if (mPlayerData != null) {
            return mPlayerData[rowIndex];
        }
        return null;
    }
    
    public Class getColumnClass(int columnIndex) {
        if (columnIndex == COLUMN_PLAYER) {
            return String.class;
        } else if (columnIndex == COLUMN_CLASS) {
            return String.class;
        } else if (columnIndex > COLUMN_SEPARATOR) {
            int constraintsIndex = columnIndex - COLUMN_SEPARATOR;
            ITableColumn column = (ITableColumn) mColumnDirectory.get(constraintsIndex);
            return column.getColumnClass();
        }
        return super.getColumnClass(columnIndex);
    }
    
    public int updateRowForPlayer(String playerName) {
        for (int i=0; i < mPlayerData.length; i++) {
            if (mPlayerData[i].getPlayerName().equals(playerName)) {
                super.fireTableRowsUpdated(i, i);
                return i;
            }
        }
        return -1;
    }
    
    public Object getValueAt(int rowIndex, int columnIndex) {
        if (mPlayerData != null) {
            PlayerData data = mPlayerData[rowIndex];
            switch (columnIndex) {
            case COLUMN_PLAYER:
                return data.getPlayerName();
            case COLUMN_CLASS:
                if (data.getPlayerClass() != null) {
                    return data.getPlayerClass();
                }
                return "";
            case COLUMN_TIME_JOIN:
                if (data.getJoinTimes() != null) {
                    return data.getJoinTimes();
                }
                return "";
            case COLUMN_TIME_LEAVE:
                if (data.getLeaveTimes() != null) {
                    return data.getLeaveTimes();
                }
                return "";
            case COLUMN_LOOT:
                if (data.getLoots() != null) {
                    return data.getLoots();
                }
                return "";
            }
            if (columnIndex > COLUMN_SEPARATOR) {
                int constraintsIndex = columnIndex - COLUMN_SEPARATOR;
                ITableColumn column = (ITableColumn) mColumnDirectory.get(constraintsIndex);
                return column.getValueAt(data);
            }
        }
        return null;
    }
    
    public boolean isCellEditable(int row, int columnIndex) {
        if (columnIndex == COLUMN_CLASS) {
            return mCurrentRaid.isPromoted();
        } else if (columnIndex <= COLUMN_SEPARATOR) {
            return false;
        } else {
            int constraintsIndex = columnIndex - COLUMN_SEPARATOR;
            ITableColumn column = (ITableColumn) mColumnDirectory.get(constraintsIndex);
            return column.isEditable();
        }
    }
    
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        if (mPlayerData != null) {
            PlayerData data = mPlayerData[rowIndex];
            if (columnIndex == COLUMN_CLASS) {
                String newClass = ((String) aValue).trim();
                if (!newClass.equals(data.getPlayerClass())) {
                    data.setPlayerClass(newClass);
                    mCurrentRaid.promote();
                }
            }
            else if (columnIndex > COLUMN_SEPARATOR) {
                int constraintsIndex = columnIndex - COLUMN_SEPARATOR;
                ITableColumn column = 
                    (ITableColumn) mColumnDirectory.get(constraintsIndex);
                boolean isSet = column.setValueAt(aValue, data);
                if (isSet) {
                    super.fireTableRowsUpdated(rowIndex, rowIndex);
                    mCurrentSystem.saveRaid(mCurrentRaid);
                }
            }
        }
    }
}
